searchData={"items":[{"type":"module","title":"Nx","doc":"Numerical Elixir.\n\nThe `Nx` library is a collection of functions and data\ntypes to work with Numerical Elixir. This module defines\nthe main entry point for building and working with said\ndata-structures. For example, to create an n-dimensional\ntensor, do:\n\n    iex> t = Nx.tensor([[1, 2], [3, 4]])\n    iex> Nx.shape(t)\n    {2, 2}\n\n`Nx` also provides the so-called numerical definitions under\nthe `Nx.Defn` module. They are a subset of Elixir tailored for\nnumerical computations. For example, it overrides Elixir's\ndefault operators so they are tensor-aware:\n\n    defn softmax(t) do\n      Nx.exp(t) / Nx.sum(Nx.exp(t))\n    end\n\nCode inside `defn` functions can also be given to custom compilers,\nwhich can compile said functions just-in-time (JIT) to run on the\nCPU or on the GPU.","ref":"Nx.html"},{"type":"module","title":"References - Nx","doc":"Here is a general outline of the main references in this library:\n\n  * For an introduction, see our [Intro to Nx](intro-to-nx.livemd) guide\n\n  * This module provides the main API for working with tensors\n\n  * `Nx.Defn` provides numerical definitions, CPU/GPU compilation, gradients, and more\n\n  * `Nx.LinAlg` provides functions related to linear algebra\n\n  * `Nx.Constants` declares many constants commonly used in numerical code\n\nContinue reading this documentation for an overview of creating,\nbroadcasting, and accessing/slicing Nx tensors.","ref":"Nx.html#module-references"},{"type":"module","title":"Creating tensors - Nx","doc":"The main APIs for creating tensors are `tensor/2`, `from_binary/2`,\n`iota/2`, `eye/2`, `random_uniform/2`, `random_normal/2`, and\n`broadcast/3`.\n\nThe tensor types can be one of:\n\n  * unsigned integers (`u8`, `u16`, `u32`, `u64`)\n  * signed integers (`s8`, `s16`, `s32`, `s64`)\n  * floats (`f16`, `f32`, `f64`)\n  * brain floats (`bf16`)\n  * and complex numbers (`c64`, `c128`)\n\nThe types are tracked as tuples:\n\n    iex> Nx.tensor([1, 2, 3], type: {:f, 32})\n    #Nx.Tensor \n\nBut a shortcut atom notation is also available:\n\n    iex> Nx.tensor([1, 2, 3], type: :f32)\n    #Nx.Tensor \n\nThe tensor dimensions can also be named, via the `:names` option\navailable to all creation functions:\n\n    iex> Nx.iota({2, 3}, names: [:x, :y])\n    #Nx.Tensor \n\nFinally, for creating vectors and matrices, a sigil notation\nis available:\n\n    iex> import Nx, only: :sigils\n    iex> ~V[1 2 3]f32\n    #Nx.Tensor \n\n    iex> import Nx, only: :sigils\n    iex> ~M'''\n    ...> 1 2 3\n    ...> 4 5 6\n    ...> '''s32\n    #Nx.Tensor \n\nAll other APIs accept exclusively numbers or tensors, unless\nexplicitly noted otherwise.","ref":"Nx.html#module-creating-tensors"},{"type":"module","title":"Broadcasting - Nx","doc":"Broadcasting allows operations on two tensors of different shapes\nto match. For example, most often operations between tensors have\nthe same shape:\n\n    iex> a = Nx.tensor([1, 2, 3])\n    iex> b = Nx.tensor([10, 20, 30])\n    iex> Nx.add(a, b)\n    #Nx.Tensor \n\nNow let's imagine you want to multiply a large tensor of dimensions\n1000x1000x1000 by 2. If you had to create a similarly large tensor\nonly to perform this operation, it would be inefficient. Therefore,\nyou can simply multiply this large tensor by the scalar 2, and Nx\nwill propagate its dimensions at the time the operation happens,\nwithout allocating a large intermediate tensor:\n\n    iex> Nx.multiply(Nx.tensor([1, 2, 3]), 2)\n    #Nx.Tensor \n\nIn practice, broadcasting is not restricted only to scalars; it\nis a general algorithm that applies to all dimensions of a tensor.\nWhen broadcasting, `Nx` compares the shapes of the two tensors,\nstarting with the trailing ones, such that:\n\n  * If the dimensions have equal size, then they are compatible\n\n  * If one of the dimensions have size of 1, it is \"broadcast\"\n    to match the dimension of the other\n\nIn case one tensor has more dimensions than the other, the missing\ndimensions are considered to be of size one. Here are some examples\nof how broadcast would work when multiplying two tensors with the\nfollowing shapes:\n\n    s64[3] * s64\n    #=> s64[3]\n\n    s64[255][255][3] * s64[3]\n    #=> s64[255][255][3]\n\n    s64[2][1] * s[1][2]\n    #=> s64[2][2]\n\n    s64[5][1][4][1] * s64[3][4][5]\n    #=> s64[5][3][4][5]\n\nIf any of the dimensions do not match or are not 1, an error is\nraised.","ref":"Nx.html#module-broadcasting"},{"type":"module","title":"Access syntax (slicing) - Nx","doc":"Nx tensors implement Elixir's access syntax. This allows developers\nto slice tensors up and easily access sub-dimensions and values.\n\nAccess accepts integers:\n\n    iex> t = Nx.tensor([[1, 2], [3, 4]])\n    iex> t[0]\n    #Nx.Tensor \n    iex> t[1]\n    #Nx.Tensor \n    iex> t[1][1]\n    #Nx.Tensor \n\nIf a negative index is given, it accesses the element from the back:\n\n    iex> t = Nx.tensor([[1, 2], [3, 4]])\n    iex> t[-1][-1]\n    #Nx.Tensor \n\nOut of bound access will raise:\n\n    iex> Nx.tensor([1, 2])[2]\n    ** (ArgumentError) index 2 is out of bounds for axis 0 in shape {2}\n\n    iex> Nx.tensor([1, 2])[-3]\n    ** (ArgumentError) index -3 is out of bounds for axis 0 in shape {2}\n\nThe index can also be another tensor but in such cases it must be\na scalar between 0 and the dimension size. Out of bound dynamic indexes\nare always clamped to the tensor dimensions:\n\n    iex> two = Nx.tensor(2)\n    iex> t = Nx.tensor([[1, 2], [3, 4]])\n    iex> t[two][two]\n    #Nx.Tensor \n\nFor example, a `minus_one` dynamic index will be clamped to zero:\n\n    iex> minus_one = Nx.tensor(-1)\n    iex> t = Nx.tensor([[1, 2], [3, 4]])\n    iex> t[minus_one][minus_one]\n    #Nx.Tensor \n\nAccess also accepts ranges. Ranges in Elixir are inclusive:\n\n    iex> t = Nx.tensor([[1, 2], [3, 4], [5, 6], [7, 8]])\n    iex> t[0..1]\n    #Nx.Tensor \n\nRanges can receive negative positions and they will read from\nthe back. In such cases, the range step must be explicitly given\nand the right-side of the range must be equal or greater than\nthe left-side:\n\n    iex> t = Nx.tensor([[1, 2], [3, 4], [5, 6], [7, 8]])\n    iex> t[1..-2//1]\n    #Nx.Tensor \n\nAs you can see, accessing with a range does not eliminate the\naccessed axis. This means that, if you try to cascade ranges,\nyou will always be filtering the highest dimension:\n\n    iex> t = Nx.tensor([[1, 2], [3, 4], [5, 6], [7, 8]])\n    iex> t[1..-1//1] # Drop the first \"row\"\n    #Nx.Tensor \n    iex> t[1..-1//1][1..-1//1] # Drop the first \"row\" twice\n    #Nx.Tensor \n\nTherefore, if you want to slice across multiple dimensions, you can wrap\nthe ranges in a list:\n\n    iex> t = Nx.tensor([[1, 2], [3, 4], [5, 6], [7, 8]])\n    iex> t[[1..-1//1, 1..-1//1]] # Drop the first \"row\" and the first \"column\"\n    #Nx.Tensor \n\nYou can also use `..` as the full-slice range, which means you want to\nkeep a given dimension as is:\n\n    iex> t = Nx.tensor([[1, 2], [3, 4], [5, 6], [7, 8]])\n    iex> t[[.., 1..-1//1]] # Drop only the first \"column\"\n    #Nx.Tensor \n\nYou can mix both ranges and integers in the list too:\n\n    iex> t = Nx.tensor([[1, 2, 3], [4, 5, 6], [7, 8, 9], [10, 11, 12]])\n    iex> t[[1..2, 2]]\n    #Nx.Tensor \n\nIf the list has less elements than axes, the remaining dimensions\nare returned in full:\n\n    iex> t = Nx.tensor([[1, 2, 3], [4, 5, 6], [7, 8, 9], [10, 11, 12]])\n    iex> t[[1..2]]\n    #Nx.Tensor \n\nThe access syntax also pairs nicely with named tensors. By using named\ntensors, you can pass only the axis you want to slice, leaving the other\naxes intact:\n\n    iex> t = Nx.tensor([[1, 2, 3], [4, 5, 6], [7, 8, 9], [10, 11, 12]], names: [:x, :y])\n    iex> t[x: 1..2]\n    #Nx.Tensor \n    iex> t[x: 1..2, y: 0..1]\n    #Nx.Tensor \n    iex> t[x: 1, y: 0..1]\n    #Nx.Tensor \n\nFor a more complex slicing rules, including strides, you\ncan always fallback to `Nx.slice/4`.","ref":"Nx.html#module-access-syntax-slicing"},{"type":"module","title":"Backends - Nx","doc":"The `Nx` library has built-in support for multiple backends.\nA tensor is always handled by a backend, the default backend\nbeing `Nx.BinaryBackend`, which means the tensor is allocated\nas a binary within the Erlang VM.\n\nMost often backends are used to provide a completely different\nimplementation of tensor operations, often accelerated to the GPU.\nIn such cases, you want to guarantee all tensors are allocated in\nthe new backend. This can be done by configuring your runtime:\n\n    # config/runtime.exs\n    import Config\n    config :nx, default_backend: EXLA.Backend\n\nIn your notebooks and on `Mix.install/2`, you might:\n\n    Mix.install(\n      [\n        {:nx, \">= 0.0.0\"}\n      ],\n      config: [nx: [default_backend: EXLA.Backend]]\n    )\n\nOr by calling `Nx.global_default_backend/1` (less preferrable):\n\n    Nx.global_default_backend(EXLA.Backend)\n\nTo pass options to the backend, replacing `EXLA.Backend` by\n`{EXLA.Backend, client: :cuda}` or similar. See the documentation\nfor [EXLA](https://hexdocs.pm/exla) and [Torchx](https://hexdocs.pm/torchx)\nfor installation and GPU support.\n\nTo implement your own backend, check the `Nx.Tensor` behaviour.","ref":"Nx.html#module-backends"},{"type":"function","title":"Nx.abs/1","doc":"Computes the absolute value of each element in the tensor.","ref":"Nx.html#abs/1"},{"type":"function","title":"Examples - Nx.abs/1","doc":"iex> Nx.abs(Nx.tensor([-2, -1, 0, 1, 2], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#abs/1-examples"},{"type":"function","title":"Nx.acos/1","doc":"Calculates the inverse cosine of each element in the tensor.\n\nIt is equivalent to:\n\n$$acos(cos(z)) = z$$","ref":"Nx.html#acos/1"},{"type":"function","title":"Examples - Nx.acos/1","doc":"iex> Nx.acos(0.10000000149011612)\n    #Nx.Tensor \n\n    iex> Nx.acos(Nx.tensor([0.10000000149011612, 0.5, 0.8999999761581421], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#acos/1-examples"},{"type":"function","title":"Nx.acosh/1","doc":"Calculates the inverse hyperbolic cosine of each element in the tensor.\n\nIt is equivalent to:\n\n$$acosh(cosh(z)) = z$$","ref":"Nx.html#acosh/1"},{"type":"function","title":"Examples - Nx.acosh/1","doc":"iex> Nx.acosh(1)\n    #Nx.Tensor \n\n    iex> Nx.acosh(Nx.tensor([1, 2, 3], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#acosh/1-examples"},{"type":"function","title":"Nx.add/2","doc":"Element-wise addition of two tensors.\n\nIf a number is given, it is converted to a tensor.\n\nIt will broadcast tensors whenever the dimensions do\nnot match and broadcasting is possible.\n\nIf you're using `Nx.Defn.defn/2`, you can use the `+` operator\nin place of this function: `left + right`.","ref":"Nx.html#add/2"},{"type":"function","title":"Examples - Nx.add/2","doc":"#","ref":"Nx.html#add/2-examples"},{"type":"function","title":"Adding scalars - Nx.add/2","doc":"iex> Nx.add(1, 2)\n    #Nx.Tensor \n\n    iex> Nx.add(1, 2.2)\n    #Nx.Tensor \n\n#","ref":"Nx.html#add/2-adding-scalars"},{"type":"function","title":"Adding a scalar to a tensor - Nx.add/2","doc":"iex> Nx.add(Nx.tensor([1, 2, 3], names: [:data]), 1)\n    #Nx.Tensor \n\n    iex> Nx.add(1, Nx.tensor([1, 2, 3], names: [:data]))\n    #Nx.Tensor \n\nGiven a float scalar converts the tensor to a float:\n\n    iex> Nx.add(Nx.tensor([1, 2, 3], names: [:data]), 1.0)\n    #Nx.Tensor \n\n    iex> Nx.add(Nx.tensor([1.0, 2.0, 3.0], names: [:data]), 1)\n    #Nx.Tensor \n\n    iex> Nx.add(Nx.tensor([1.0, 2.0, 3.0], type: :f32, names: [:data]), 1)\n    #Nx.Tensor \n\nUnsigned tensors become signed and double their size if a\nnegative number is given:\n\n    iex> Nx.add(Nx.tensor([0, 1, 2], type: :u8, names: [:data]), -1)\n    #Nx.Tensor \n\n#","ref":"Nx.html#add/2-adding-a-scalar-to-a-tensor"},{"type":"function","title":"Adding tensors of the same shape - Nx.add/2","doc":"iex> left = Nx.tensor([[1, 2], [3, 4]], names: [:x, :y])\n    iex> right = Nx.tensor([[10, 20], [30, 40]], names: [nil, :y])\n    iex> Nx.add(left, right)\n    #Nx.Tensor \n\n#","ref":"Nx.html#add/2-adding-tensors-of-the-same-shape"},{"type":"function","title":"Adding tensors with broadcasting - Nx.add/2","doc":"iex> left = Nx.tensor([[1], [2]], names: [nil, :y])\n    iex> right = Nx.tensor([[10, 20]], names: [:x, nil])\n    iex> Nx.add(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([[10, 20]], names: [:x, nil])\n    iex> right = Nx.tensor([[1], [2]], names: [nil, :y])\n    iex> Nx.add(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([[1], [2]], names: [:x, nil])\n    iex> right = Nx.tensor([[10, 20], [30, 40]])\n    iex> Nx.add(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([[1, 2]])\n    iex> right = Nx.tensor([[10, 20], [30, 40]])\n    iex> Nx.add(left, right)\n    #Nx.Tensor","ref":"Nx.html#add/2-adding-tensors-with-broadcasting"},{"type":"function","title":"Nx.all/2","doc":"Returns a scalar tensor of value 1 if all of the\ntensor values are not zero. Otherwise the value is 0.\n\nIf the `:axes` option is given, it aggregates over\nthe given dimensions, effectively removing them.\n`axes: [0]` implies aggregating over the highest order\ndimension and so forth. If the axis is negative, then\ncounts the axis from the back. For example, `axes: [-1]`\nwill always aggregate all rows.\n\nYou may optionally set `:keep_axes` to true, which will\nretain the rank of the input tensor by setting the reduced\naxes to size 1.","ref":"Nx.html#all/2"},{"type":"function","title":"Examples - Nx.all/2","doc":"iex> Nx.all(Nx.tensor([0, 1, 2]))\n    #Nx.Tensor \n\n    iex> Nx.all(Nx.tensor([[-1, 0, 1], [2, 3, 4]], names: [:x, :y]), axes: [:x])\n    #Nx.Tensor \n\n    iex> Nx.all(Nx.tensor([[-1, 0, 1], [2, 3, 4]], names: [:x, :y]), axes: [:y])\n    #Nx.Tensor \n\n#","ref":"Nx.html#all/2-examples"},{"type":"function","title":"Keeping axes - Nx.all/2","doc":"iex> Nx.all(Nx.tensor([[-1, 0, 1], [2, 3, 4]], names: [:x, :y]), axes: [:y], keep_axes: true)\n    #Nx.Tensor \n\n#","ref":"Nx.html#all/2-keeping-axes"},{"type":"function","title":"Vectorized tensors - Nx.all/2","doc":"iex> t = Nx.vectorize(Nx.tensor([[0, 1], [1, 1]]), :x)\n    iex> Nx.all(t, axes: [0], keep_axes: true)\n    #Nx.Tensor \n\n    iex> t = Nx.vectorize(Nx.tensor([1, 0]), :x)\n    iex> Nx.all(t)\n    #Nx.Tensor","ref":"Nx.html#all/2-vectorized-tensors"},{"type":"function","title":"Nx.all_close/3","doc":"Returns a scalar tensor of value 1 if all element-wise values\nare within tolerance of b. Otherwise returns value 0.\n\nYou may set the absolute tolerance, `:atol` and relative tolerance\n`:rtol`. Given tolerances, this functions returns 1 if\n\n   absolute(a - b) <= (atol + rtol * absolute(b))\n\nis true for all elements of a and b.","ref":"Nx.html#all_close/3"},{"type":"function","title":"Options - Nx.all_close/3","doc":"* `:rtol` - relative tolerance between numbers, as described above. Defaults to 1.0e-5\n * `:atol` - absolute tolerance between numbers, as described above. Defaults to 1.0e-8\n * `:equal_nan` - if `false`, NaN will always compare as false.\n   Otherwise `NaN` will only equal `NaN`. Defaults to `false`","ref":"Nx.html#all_close/3-options"},{"type":"function","title":"Examples - Nx.all_close/3","doc":"iex> Nx.all_close(Nx.tensor([1.0e10, 1.0e-7]), Nx.tensor([1.00001e10, 1.0e-8]))\n   #Nx.Tensor \n\n   iex> Nx.all_close(Nx.tensor([1.0e-8, 1.0e-8]), Nx.tensor([1.0e-8, 1.0e-9]))\n   #Nx.Tensor \n\nAlthough `NaN` by definition isn't equal to itself, so this implementation\nalso considers all `NaN`s different from each other by default:\n\n   iex> Nx.all_close(Nx.tensor(:nan), Nx.tensor(:nan))\n   #Nx.Tensor \n\n   iex> Nx.all_close(Nx.tensor(:nan), Nx.tensor(0))\n   #Nx.Tensor \n\nWe can change this behavior with the `:equal_nan` option:\n\n   iex> t = Nx.tensor([:nan, 1])\n   iex> Nx.all_close(t, t, equal_nan: true) # nan == nan -> true\n   #Nx.Tensor \n   iex> Nx.all_close(t, t, equal_nan: false) # nan == nan -> false, default behavior\n   #Nx.Tensor \n\nInfinities behave as expected, being \"close\" to themselves but not\nto other numbers:\n\n   iex> Nx.all_close(Nx.tensor(:infinity), Nx.tensor(:infinity))\n   #Nx.Tensor \n\n   iex> Nx.all_close(Nx.tensor(:infinity), Nx.tensor(:neg_infinity))\n   #Nx.Tensor \n\n   iex> Nx.all_close(Nx.tensor(1.0e30), Nx.tensor(:infinity))\n   #Nx.Tensor","ref":"Nx.html#all_close/3-examples"},{"type":"function","title":"Vectorized tensors - Nx.all_close/3","doc":"Vectorized inputs have their vectorized axes broadcast together\nbefore calculations are performed.\n\n    iex> x = Nx.tensor([0, 1]) |> Nx.vectorize(:x)\n    iex> Nx.all_close(x, x)\n    #Nx.Tensor \n\n    iex> x = Nx.tensor([0, 1, 2]) |> Nx.vectorize(:x)\n    iex> y = Nx.tensor([0, 1]) |> Nx.vectorize(:y)\n    iex> Nx.all_close(x, y)\n    #Nx.Tensor","ref":"Nx.html#all_close/3-vectorized-tensors"},{"type":"function","title":"Nx.any/2","doc":"Returns a scalar tensor of value 1 if any of the\ntensor values are not zero. Otherwise the value is 0.\n\nIf the `:axes` option is given, it aggregates over\nthe given dimensions, effectively removing them.\n`axes: [0]` implies aggregating over the highest order\ndimension and so forth. If the axis is negative, then\ncounts the axis from the back. For example, `axes: [-1]`\nwill always aggregate all rows.\n\nYou may optionally set `:keep_axes` to true, which will\nretain the rank of the input tensor by setting the reduced\naxes to size 1.","ref":"Nx.html#any/2"},{"type":"function","title":"Examples - Nx.any/2","doc":"iex> Nx.any(Nx.tensor([0, 1, 2]))\n    #Nx.Tensor \n\n    iex> Nx.any(Nx.tensor([[0, 1, 0], [0, 1, 2]], names: [:x, :y]), axes: [:x])\n    #Nx.Tensor \n\n    iex> Nx.any(Nx.tensor([[0, 1, 0], [0, 1, 2]], names: [:x, :y]), axes: [:y])\n    #Nx.Tensor \n\n#","ref":"Nx.html#any/2-examples"},{"type":"function","title":"Keeping axes - Nx.any/2","doc":"iex> Nx.any(Nx.tensor([[0, 1, 0], [0, 1, 2]], names: [:x, :y]), axes: [:y], keep_axes: true)\n    #Nx.Tensor \n\n#","ref":"Nx.html#any/2-keeping-axes"},{"type":"function","title":"Vectorized tensors - Nx.any/2","doc":"iex> t = Nx.vectorize(Nx.tensor([[0, 1], [0, 0]]), :x)\n    iex> Nx.any(t, axes: [0], keep_axes: true)\n    #Nx.Tensor","ref":"Nx.html#any/2-vectorized-tensors"},{"type":"function","title":"Nx.argmax/2","doc":"Returns the indices of the maximum values.","ref":"Nx.html#argmax/2"},{"type":"function","title":"Options - Nx.argmax/2","doc":"* `:axis` - the axis to aggregate on. If no axis is given,\n    returns the index of the absolute maximum value in the tensor.\n\n  * `:keep_axis` - whether or not to keep the reduced axis with\n    a size of 1. Defaults to `false`.\n\n  * `:tie_break` - how to break ties. one of `:high`, or `:low`.\n    default behavior is to always return the lower index.","ref":"Nx.html#argmax/2-options"},{"type":"function","title":"Examples - Nx.argmax/2","doc":"iex> Nx.argmax(4)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[4, 2, 3], [1, -5, 3]], [[6, 2, 3], [4, 8, 3]]])\n    iex> Nx.argmax(t)\n    #Nx.Tensor \n\nIf a tensor of floats is given, it still returns integers:\n\n    iex> Nx.argmax(Nx.tensor([2.0, 4.0]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#argmax/2-examples"},{"type":"function","title":"Aggregating over an axis - Nx.argmax/2","doc":"iex> t = Nx.tensor([[[4, 2, 3], [1, -5, 3]], [[6, 2, 3], [4, 8, 3]]])\n    iex> Nx.argmax(t, axis: 0)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[4, 2, 3], [1, -5, 3]], [[6, 2, 3], [4, 8, 3]]], names: [:x, :y, :z])\n    iex> Nx.argmax(t, axis: :y)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[4, 2, 3], [1, -5, 3]], [[6, 2, 3], [4, 8, 3]]], names: [:x, :y, :z])\n    iex> Nx.argmax(t, axis: :z)\n    #Nx.Tensor \n\n#","ref":"Nx.html#argmax/2-aggregating-over-an-axis"},{"type":"function","title":"Tie breaks - Nx.argmax/2","doc":"iex> t = Nx.tensor([[[4, 2, 3], [1, -5, 3]], [[6, 2, 3], [4, 8, 3]]], names: [:x, :y, :z])\n    iex> Nx.argmax(t, tie_break: :low, axis: :y)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[4, 2, 3], [1, -5, 3]], [[6, 2, 3], [4, 8, 3]]], names: [:x, :y, :z])\n    iex> Nx.argmax(t, tie_break: :high, axis: :y)\n    #Nx.Tensor \n\n#","ref":"Nx.html#argmax/2-tie-breaks"},{"type":"function","title":"Keep axis - Nx.argmax/2","doc":"iex> t = Nx.tensor([[[4, 2, 3], [1, -5, 3]], [[6, 2, 3], [4, 8, 3]]], names: [:x, :y, :z])\n    iex> Nx.argmax(t, axis: :y, keep_axis: true)\n    #Nx.Tensor \n\n#","ref":"Nx.html#argmax/2-keep-axis"},{"type":"function","title":"Vectorized tensors - Nx.argmax/2","doc":"iex> v = Nx.tensor([[1, 2, 3], [6, 5, 4]]) |> Nx.vectorize(:x)\n    iex> Nx.argmax(v)\n    #Nx.Tensor \n    iex> Nx.argmax(v, axis: 0)\n    #Nx.Tensor \n    iex> Nx.argmax(v, keep_axis: true)\n    #Nx.Tensor","ref":"Nx.html#argmax/2-vectorized-tensors"},{"type":"function","title":"Nx.argmin/2","doc":"Returns the indices of the minimum values.","ref":"Nx.html#argmin/2"},{"type":"function","title":"Options - Nx.argmin/2","doc":"* `:axis` - the axis to aggregate on. If no axis is given,\n    returns the index of the absolute minimum value in the tensor.\n\n  * `:keep_axis` - whether or not to keep the reduced axis with\n    a size of 1. Defaults to `false`.\n\n  * `:tie_break` - how to break ties. one of `:high`, or `:low`.\n    Default behavior is to always return the lower index.","ref":"Nx.html#argmin/2-options"},{"type":"function","title":"Examples - Nx.argmin/2","doc":"iex> Nx.argmin(4)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[4, 2, 3], [1, -5, 3]], [[6, 2, 3], [4, 8, 3]]])\n    iex> Nx.argmin(t)\n    #Nx.Tensor \n\nIf a tensor of floats is given, it still returns integers:\n\n    iex> Nx.argmin(Nx.tensor([2.0, 4.0]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#argmin/2-examples"},{"type":"function","title":"Aggregating over an axis - Nx.argmin/2","doc":"iex> t = Nx.tensor([[[4, 2, 3], [1, -5, 3]], [[6, 2, 3], [4, 8, 3]]])\n    iex> Nx.argmin(t, axis: 0)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[4, 2, 3], [1, -5, 3]], [[6, 2, 3], [4, 8, 3]]], names: [:x, :y, :z])\n    iex> Nx.argmin(t, axis: 1)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[4, 2, 3], [1, -5, 3]], [[6, 2, 3], [4, 8, 3]]], names: [:x, :y, :z])\n    iex> Nx.argmin(t, axis: :z)\n    #Nx.Tensor \n\n#","ref":"Nx.html#argmin/2-aggregating-over-an-axis"},{"type":"function","title":"Tie breaks - Nx.argmin/2","doc":"iex> t = Nx.tensor([[[4, 2, 3], [1, -5, 3]], [[6, 2, 3], [4, 8, 3]]], names: [:x, :y, :z])\n    iex> Nx.argmin(t, tie_break: :low, axis: :y)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[4, 2, 3], [1, -5, 3]], [[6, 2, 3], [4, 8, 3]]], names: [:x, :y, :z])\n    iex> Nx.argmin(t, tie_break: :high, axis: :y)\n    #Nx.Tensor \n\n#","ref":"Nx.html#argmin/2-tie-breaks"},{"type":"function","title":"Keep axis - Nx.argmin/2","doc":"iex> t = Nx.tensor([[[4, 2, 3], [1, -5, 3]], [[6, 2, 3], [4, 8, 3]]], names: [:x, :y, :z])\n    iex> Nx.argmin(t, axis: :y, keep_axis: true)\n    #Nx.Tensor \n\n#","ref":"Nx.html#argmin/2-keep-axis"},{"type":"function","title":"Vectorized tensors - Nx.argmin/2","doc":"iex> v = Nx.tensor([[1, 2, 3], [6, 5, 4]]) |> Nx.vectorize(:x)\n    iex> Nx.argmin(v)\n    #Nx.Tensor \n    iex> Nx.argmin(v, axis: 0)\n    #Nx.Tensor \n    iex> Nx.argmin(v, keep_axis: true)\n    #Nx.Tensor","ref":"Nx.html#argmin/2-vectorized-tensors"},{"type":"function","title":"Nx.argsort/2","doc":"Sorts the tensor along the given axis according\nto the given direction and returns the corresponding indices\nof the original tensor in the new sorted positions.\n\nIf no axis is given, defaults to `0`.\n\nSee `take_along_axis/3` for examples on how to apply the\nresulting indices from this function.","ref":"Nx.html#argsort/2"},{"type":"function","title":"Options - Nx.argsort/2","doc":"* `:axis` - The name or number of the corresponding axis on which the sort\n    should be applied\n  * `:direction` - Can be `:asc` or `:desc`. Defaults to `:asc`\n  * `:type` - The type of the resulting tensor. Defaults to `:s64`.","ref":"Nx.html#argsort/2-options"},{"type":"function","title":"Examples - Nx.argsort/2","doc":"iex> Nx.argsort(Nx.tensor([16, 23, 42, 4, 8, 15]))\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[3, 1, 7], [2, 5, 4]], names: [:x, :y])\n    iex> Nx.argsort(t, axis: :x)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[3, 1, 7], [2, 5, 4]], names: [:x, :y])\n    iex> Nx.argsort(t, axis: :y)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[3, 1, 7], [2, 5, 4]], names: [:x, :y])\n    iex> Nx.argsort(t, axis: :y, direction: :asc, type: :u32)\n    #Nx.Tensor \n\nSame tensor sorted over different axes:\n\n    iex> t = Nx.tensor(\n    ...>   [\n    ...>     [\n    ...>       [4, 5, 2],\n    ...>       [2, 5, 3],\n    ...>       [5, 0, 2]\n    ...>     ],\n    ...>     [\n    ...>       [1, 9, 8],\n    ...>       [2, 1, 3],\n    ...>       [2, 1, 4]\n    ...>     ]\n    ...>   ],\n    ...>   names: [:x, :y, :z]\n    ...> )\n    iex> Nx.argsort(t, axis: :x)\n    #Nx.Tensor \n    iex> Nx.argsort(t, axis: :y)\n    #Nx.Tensor \n    iex> Nx.argsort(t, axis: :z)\n    #Nx.Tensor","ref":"Nx.html#argsort/2-examples"},{"type":"function","title":"Nx.as_type/2","doc":"Changes the type of a tensor.\n\nNote conversion between float and integers truncates the\nresult. Consider using `round/1`, `floor/1`, or `ceil/1`\nbefore casting from float to integer to guarantee consistent\nbehavior.\n\nCasting from a higher precision may lead to an overflow\nor underflow, which is platform and compiler dependent\nbehaviour.\n\nCasting of non-finite types to integer types are handled\nsuch as:\n\n  * negative infinity becomes the minimum value for said type\n  * positive infinity becomes the maximum value for said type\n  * nan becomes zero","ref":"Nx.html#as_type/2"},{"type":"function","title":"Examples - Nx.as_type/2","doc":"iex> Nx.as_type(Nx.tensor([0, 1, 2], names: [:data]), :f32)\n    #Nx.Tensor \n\n    iex> Nx.as_type(Nx.tensor([0.0, 1.0, 2.0], names: [:data]), :bf16)\n    #Nx.Tensor \n\n    iex> Nx.as_type(Nx.tensor([0.0, 1.0, 2.0], names: [:data]), :s64)\n    #Nx.Tensor \n\nCasting numbers as complex will return the corresponding complex with 0 imaginary component:\n\n    iex> Nx.as_type(Nx.tensor([1, -2]), :c64)\n    #Nx.Tensor \n\nCasting complex numbers will return their real parts as the target type:\n\n    iex> Nx.as_type(Nx.tensor([Complex.new(1, 2), Complex.new(0, 3), Complex.new(4, 5)]), :f64)\n    #Nx.Tensor \n\n    iex> Nx.as_type(Nx.tensor([Complex.new(-1, 2), Complex.new(-2, 3), Complex.new(3, -4)]), :s64)\n    #Nx.Tensor \n\nCasting of non-finite values to integer types convert to pre-determined\ninteger values:\n\n    iex> non_finite = Nx.tensor([:infinity, :nan, :neg_infinity])\n    iex> Nx.as_type(non_finite, :u8)\n    #Nx.Tensor \n    iex> Nx.as_type(non_finite, :s32)\n    #Nx.Tensor \n\nNon-finite values between float types are preserved:\n\n    iex> non_finite = Nx.tensor([:infinity, :nan])\n    iex> Nx.as_type(non_finite, :f64)\n    #Nx.Tensor \n    iex> Nx.as_type(non_finite, :f16)\n    #Nx.Tensor \n\nIf the input is a numerical constant instead of a tensor, this is an\nalias to `Nx.tensor(number, type: type)`. In the example below,\nnotice how precision is only lost if we pass a type which can't\nrepresent the numerical input:\n\n    iex> Nx.as_type(1.0e-128, :f32)\n    #Nx.Tensor \n    iex> Nx.as_type(1.0e-128, :f64)\n    #Nx.Tensor","ref":"Nx.html#as_type/2-examples"},{"type":"function","title":"Nx.asin/1","doc":"Calculates the inverse sine of each element in the tensor.\n\nIt is equivalent to:\n\n$$asin(sin(z)) = z$$","ref":"Nx.html#asin/1"},{"type":"function","title":"Examples - Nx.asin/1","doc":"iex> Nx.asin(0.10000000149011612)\n    #Nx.Tensor \n\n    iex> Nx.asin(Nx.tensor([0.10000000149011612, 0.5, 0.8999999761581421], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#asin/1-examples"},{"type":"function","title":"Nx.asinh/1","doc":"Calculates the inverse hyperbolic sine of each element in the tensor.\n\nIt is equivalent to:\n\n$$asinh(sinh(z)) = z$$","ref":"Nx.html#asinh/1"},{"type":"function","title":"Examples - Nx.asinh/1","doc":"iex> Nx.asinh(1)\n    #Nx.Tensor \n\n    iex> Nx.asinh(Nx.tensor([1, 2, 3], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#asinh/1-examples"},{"type":"function","title":"Nx.atan2/2","doc":"Element-wise arc tangent of two tensors.\n\nIf a number is given, it is converted to a tensor.\n\nIt always returns a float tensor. If any of the input\ntensors are not float, they are converted to f32.\n\nIt will broadcast tensors whenever the dimensions do\nnot match and broadcasting is possible.","ref":"Nx.html#atan2/2"},{"type":"function","title":"Examples - Nx.atan2/2","doc":"#","ref":"Nx.html#atan2/2-examples"},{"type":"function","title":"Arc tangent between scalars - Nx.atan2/2","doc":"iex> Nx.atan2(1, 2)\n    #Nx.Tensor \n\n#","ref":"Nx.html#atan2/2-arc-tangent-between-scalars"},{"type":"function","title":"Arc tangent between tensors and scalars - Nx.atan2/2","doc":"iex> Nx.atan2(Nx.tensor([1, 2, 3], names: [:data]), 1)\n    #Nx.Tensor \n\n    iex> Nx.atan2(1, Nx.tensor([1.0, 2.0, 3.0], names: [:data]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#atan2/2-arc-tangent-between-tensors-and-scalars"},{"type":"function","title":"Arc tangent between tensors - Nx.atan2/2","doc":"iex> neg_and_pos_zero_columns = Nx.tensor([[-0.0], [0.0]], type: :f64)\n    iex> neg_and_pos_zero_rows = Nx.tensor([-0.0, 0.0], type: :f64)\n    iex> Nx.atan2(neg_and_pos_zero_columns, neg_and_pos_zero_rows)\n    #Nx.Tensor","ref":"Nx.html#atan2/2-arc-tangent-between-tensors"},{"type":"function","title":"Nx.atan/1","doc":"Calculates the inverse tangent of each element in the tensor.\n\nIt is equivalent to:\n\n$$atan(tan(z)) = z$$","ref":"Nx.html#atan/1"},{"type":"function","title":"Examples - Nx.atan/1","doc":"iex> Nx.atan(0.10000000149011612)\n    #Nx.Tensor \n\n    iex> Nx.atan(Nx.tensor([0.10000000149011612, 0.5, 0.8999999761581421], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#atan/1-examples"},{"type":"function","title":"Nx.atanh/1","doc":"Calculates the inverse hyperbolic tangent of each element in the tensor.\n\nIt is equivalent to:\n\n$$atanh(tanh(z)) = z$$","ref":"Nx.html#atanh/1"},{"type":"function","title":"Examples - Nx.atanh/1","doc":"iex> Nx.atanh(0.10000000149011612)\n    #Nx.Tensor \n\n    iex> Nx.atanh(Nx.tensor([0.10000000149011612, 0.5, 0.8999999761581421], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#atanh/1-examples"},{"type":"function","title":"Nx.axes/1","doc":"Returns all of the axes in a tensor.\n\nIf a shape is given, it returns the axes for the given shape.","ref":"Nx.html#axes/1"},{"type":"function","title":"Examples - Nx.axes/1","doc":"iex> Nx.axes(Nx.tensor([[1, 2, 3], [4, 5, 6]]))\n    [0, 1]\n\n    iex> Nx.axes(1)\n    []\n\n    iex> Nx.axes({1, 2, 3})\n    [0, 1, 2]","ref":"Nx.html#axes/1-examples"},{"type":"function","title":"Nx.axis_index/2","doc":"Returns the index of the given axis in the tensor.","ref":"Nx.html#axis_index/2"},{"type":"function","title":"Examples - Nx.axis_index/2","doc":"iex> Nx.axis_index(Nx.iota({100, 10, 20}), 0)\n    0\n\n    iex> Nx.axis_index(Nx.iota({100, 10, 20}), -1)\n    2\n\n    iex> Nx.axis_index(Nx.iota({100, 10, 20}, names: [:batch, :x, :y]), :x)\n    1","ref":"Nx.html#axis_index/2-examples"},{"type":"function","title":"Error cases - Nx.axis_index/2","doc":"iex> Nx.axis_index(Nx.iota({100, 10, 20}), 3)\n    ** (ArgumentError) given axis (3) invalid for shape with rank 3\n\n    iex> Nx.axis_index(Nx.iota({100, 10, 20}, names: [:batch, :x, :y]), :z)\n    ** (ArgumentError) name :z not found in tensor with names [:batch, :x, :y]","ref":"Nx.html#axis_index/2-error-cases"},{"type":"function","title":"Nx.axis_size/2","doc":"Returns the size of a given axis of a tensor.\n\nIt accepts either an atom as the name or an integer as the axis.\nIt raises if the axis/name does not exist.","ref":"Nx.html#axis_size/2"},{"type":"function","title":"Examples - Nx.axis_size/2","doc":"iex> Nx.axis_size(Nx.iota({100, 10, 20}), 0)\n    100\n\n    iex> Nx.axis_size(Nx.iota({100, 10, 20}, names: [:batch, :x, :y]), :y)\n    20","ref":"Nx.html#axis_size/2-examples"},{"type":"function","title":"Nx.backend_copy/2","doc":"Copies data to the given backend.\n\nIf a backend is not given, `Nx.Tensor` is used, which means\nthe given tensor backend will pick the most appropriate\nbackend to copy the data to.\n\nNote this function keeps the data in the original backend.\nTherefore, use this function with care, as it may duplicate\nlarge amounts of data across backends. Generally speaking,\nyou may want to use `backend_transfer/2`, unless you explicitly\nwant to copy the data.\n\nNote:\n\n* `Nx.default_backend/1` does not affect the behaviour of\nthis function.\n* This function cannot be used in `defn`.","ref":"Nx.html#backend_copy/2"},{"type":"function","title":"Examples - Nx.backend_copy/2","doc":"iex> Nx.backend_copy(Nx.tensor([[1, 2, 3], [4, 5, 6]]))\n  #Nx.Tensor","ref":"Nx.html#backend_copy/2-examples"},{"type":"function","title":"Nx.backend_deallocate/1","doc":"Deallocates data in a device.\n\nIt returns either `:ok` or `:already_deallocated`.\n\nNote: This function cannot be used in `defn`.","ref":"Nx.html#backend_deallocate/1"},{"type":"function","title":"Nx.backend_transfer/2","doc":"Transfers data to the given backend.\n\nThis operation can be seen as an equivalent to `backend_copy/3`\nfollowed by a `backend_deallocate/1` on the initial tensor:\n\n    new_tensor = Nx.backend_copy(old_tensor, new_backend)\n    Nx.backend_deallocate(old_tensor)\n\nIf a backend is not given, `Nx.Tensor` is used, which means\nthe given tensor backend will pick the most appropriate\nbackend to transfer to.\n\nFor Elixir's builtin tensor, transferring to another backend\nwill call `new_backend.from_binary(tensor, binary, opts)`.\nTransferring from a mutable backend, such as GPU memory,\nimplies the data is copied from the GPU to the Erlang VM\nand then deallocated from the device.\n\nNote:\n\n* `Nx.default_backend/1` does not affect the behaviour of this function.\n* This function cannot be used in `defn`.","ref":"Nx.html#backend_transfer/2"},{"type":"function","title":"Examples - Nx.backend_transfer/2","doc":"Transfer a tensor to an EXLA device backend, stored in the GPU:\n\n    device_tensor = Nx.backend_transfer(tensor, {EXLA.Backend, client: :cuda})\n\nTransfer the device tensor back to an Elixir tensor:\n\n    tensor = Nx.backend_transfer(device_tensor)","ref":"Nx.html#backend_transfer/2-examples"},{"type":"function","title":"Nx.bf16/1","doc":"Short-hand function for creating tensor of type `bf16`.\n\nThis is just an alias for `Nx.tensor(tensor, type: bf16)`.","ref":"Nx.html#bf16/1"},{"type":"function","title":"Nx.bitcast/2","doc":"Changes the type of a tensor, using a bitcast.\n\nThe width of input tensor's type must match the width\nof the output type. `bitcast/1` does not change the\nunderlying tensor data, but instead changes how\nthe tensor data is viewed.\n\nMachines with different floating-point representations\nwill give different results.\n\nFor complex numbers, the last axis will change in size\ndepending on whether you are upcasting or downcasting.","ref":"Nx.html#bitcast/2"},{"type":"function","title":"Examples - Nx.bitcast/2","doc":"iex> t = Nx.bitcast(Nx.tensor([0, 0, 0], names: [:data], type: :s32), :f32)\n    #Nx.Tensor \n    iex> Nx.bitcast(t, :s32)\n    #Nx.Tensor \n\n    iex> t = Nx.vectorize(Nx.tensor([[0, -1], [1, -2], [2, -3]], type: :s8), :x)\n    #Nx.Tensor \n    iex> Nx.bitcast(t, :u8)\n    #Nx.Tensor","ref":"Nx.html#bitcast/2-examples"},{"type":"function","title":"Error cases - Nx.bitcast/2","doc":"iex> Nx.bitcast(Nx.tensor([0, 1, 2], names: [:data], type: :s16), :f32)\n    ** (ArgumentError) input type width must match new type width, got input type {:s, 16} and output type {:f, 32}\n\n    iex> Nx.bitcast(Nx.tensor([0], type: :c64), :s64)\n    ** (ArgumentError) Nx.bitcast/2 does not support complex inputs\n\n    iex> Nx.bitcast(Nx.tensor([0], type: :s64), :c64)\n    ** (ArgumentError) Nx.bitcast/2 does not support complex inputs","ref":"Nx.html#bitcast/2-error-cases"},{"type":"function","title":"Nx.bitwise_and/2","doc":"Element-wise bitwise AND of two tensors.\n\nOnly integer tensors are supported. If a float or\ncomplex tensor is given, an error is raised.\n\nIt will broadcast tensors whenever the dimensions do\nnot match and broadcasting is possible.\n\nIf you're using `Nx.Defn.defn/2`, you can use the `&&&` operator\nin place of this function: `left &&& right`.\n\nIt does not support short-circuiting.","ref":"Nx.html#bitwise_and/2"},{"type":"function","title":"Examples - Nx.bitwise_and/2","doc":"#","ref":"Nx.html#bitwise_and/2-examples"},{"type":"function","title":"bitwise and between scalars - Nx.bitwise_and/2","doc":"iex> Nx.bitwise_and(1, 0)\n    #Nx.Tensor \n\n#","ref":"Nx.html#bitwise_and/2-bitwise-and-between-scalars"},{"type":"function","title":"bitwise and between tensors and scalars - Nx.bitwise_and/2","doc":"iex> Nx.bitwise_and(Nx.tensor([0, 1, 2], names: [:data]), 1)\n    #Nx.Tensor \n\n    iex> Nx.bitwise_and(Nx.tensor([0, -1, -2], names: [:data]), -1)\n    #Nx.Tensor \n\n#","ref":"Nx.html#bitwise_and/2-bitwise-and-between-tensors-and-scalars"},{"type":"function","title":"bitwise and between tensors - Nx.bitwise_and/2","doc":"iex> Nx.bitwise_and(Nx.tensor([0, 0, 1, 1], names: [:data]), Nx.tensor([0, 1, 0, 1]))\n    #Nx.Tensor","ref":"Nx.html#bitwise_and/2-bitwise-and-between-tensors"},{"type":"function","title":"Error cases - Nx.bitwise_and/2","doc":"iex> Nx.bitwise_and(Nx.tensor([0, 0, 1, 1]), 1.0)\n    ** (ArgumentError) bitwise operators expect integer tensors as inputs and outputs an integer tensor, got: {:f, 32}","ref":"Nx.html#bitwise_and/2-error-cases"},{"type":"function","title":"Nx.bitwise_not/1","doc":"Applies bitwise not to each element in the tensor.\n\nIf you're using `Nx.Defn.defn/2`, you can use the `~~~` operator\nin place of this function: `~~~tensor`.","ref":"Nx.html#bitwise_not/1"},{"type":"function","title":"Examples - Nx.bitwise_not/1","doc":"iex> Nx.bitwise_not(1)\n    #Nx.Tensor \n\n    iex> Nx.bitwise_not(Nx.tensor([-1, 0, 1], type: :s8, names: [:x]))\n    #Nx.Tensor \n\n    iex> Nx.bitwise_not(Nx.tensor([0, 1, 254, 255], type: :u8, names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#bitwise_not/1-examples"},{"type":"function","title":"Error cases - Nx.bitwise_not/1","doc":"iex> Nx.bitwise_not(Nx.tensor([0.0, 1.0]))\n    ** (ArgumentError) bitwise operators expect integer tensors as inputs and outputs an integer tensor, got: {:f, 32}","ref":"Nx.html#bitwise_not/1-error-cases"},{"type":"function","title":"Nx.bitwise_or/2","doc":"Element-wise bitwise OR of two tensors.\n\nOnly integer tensors are supported. If a float or\ncomplex tensor is given, an error is raised.\n\nIt will broadcast tensors whenever the dimensions do\nnot match and broadcasting is possible.\n\nIf you're using `Nx.Defn.defn/2`, you can use the `|||` operator\nin place of this function: `left ||| right`.\n\nIt does not support short-circuiting.","ref":"Nx.html#bitwise_or/2"},{"type":"function","title":"Examples - Nx.bitwise_or/2","doc":"#","ref":"Nx.html#bitwise_or/2-examples"},{"type":"function","title":"bitwise or between scalars - Nx.bitwise_or/2","doc":"iex> Nx.bitwise_or(1, 0)\n    #Nx.Tensor \n\n#","ref":"Nx.html#bitwise_or/2-bitwise-or-between-scalars"},{"type":"function","title":"bitwise or between tensors and scalars - Nx.bitwise_or/2","doc":"iex> Nx.bitwise_or(Nx.tensor([0, 1, 2], names: [:data]), 1)\n    #Nx.Tensor \n\n    iex> Nx.bitwise_or(Nx.tensor([0, -1, -2], names: [:data]), -1)\n    #Nx.Tensor \n\n#","ref":"Nx.html#bitwise_or/2-bitwise-or-between-tensors-and-scalars"},{"type":"function","title":"bitwise or between tensors - Nx.bitwise_or/2","doc":"iex> Nx.bitwise_or(Nx.tensor([0, 0, 1, 1], names: [:data]), Nx.tensor([0, 1, 0, 1], names: [:data]))\n    #Nx.Tensor","ref":"Nx.html#bitwise_or/2-bitwise-or-between-tensors"},{"type":"function","title":"Error cases - Nx.bitwise_or/2","doc":"iex> Nx.bitwise_or(Nx.tensor([0, 0, 1, 1]), 1.0)\n    ** (ArgumentError) bitwise operators expect integer tensors as inputs and outputs an integer tensor, got: {:f, 32}","ref":"Nx.html#bitwise_or/2-error-cases"},{"type":"function","title":"Nx.bitwise_xor/2","doc":"Element-wise bitwise XOR of two tensors.\n\nOnly integer tensors are supported. If a float or complex\ntensor is given, an error is raised.\n\nIt will broadcast tensors whenever the dimensions do\nnot match and broadcasting is possible.","ref":"Nx.html#bitwise_xor/2"},{"type":"function","title":"Examples - Nx.bitwise_xor/2","doc":"#","ref":"Nx.html#bitwise_xor/2-examples"},{"type":"function","title":"Bitwise xor between scalars - Nx.bitwise_xor/2","doc":"iex> Nx.bitwise_xor(1, 0)\n    #Nx.Tensor \n\n#","ref":"Nx.html#bitwise_xor/2-bitwise-xor-between-scalars"},{"type":"function","title":"Bitwise xor and between tensors and scalars - Nx.bitwise_xor/2","doc":"iex> Nx.bitwise_xor(Nx.tensor([1, 2, 3], names: [:data]), 2)\n    #Nx.Tensor \n\n    iex> Nx.bitwise_xor(Nx.tensor([-1, -2, -3], names: [:data]), 2)\n    #Nx.Tensor \n\n#","ref":"Nx.html#bitwise_xor/2-bitwise-xor-and-between-tensors-and-scalars"},{"type":"function","title":"Bitwise xor between tensors - Nx.bitwise_xor/2","doc":"iex> Nx.bitwise_xor(Nx.tensor([0, 0, 1, 1]), Nx.tensor([0, 1, 0, 1], names: [:data]))\n    #Nx.Tensor","ref":"Nx.html#bitwise_xor/2-bitwise-xor-between-tensors"},{"type":"function","title":"Error cases - Nx.bitwise_xor/2","doc":"iex> Nx.bitwise_xor(Nx.tensor([0, 0, 1, 1]), 1.0)\n    ** (ArgumentError) bitwise operators expect integer tensors as inputs and outputs an integer tensor, got: {:f, 32}","ref":"Nx.html#bitwise_xor/2-error-cases"},{"type":"function","title":"Nx.broadcast/3","doc":"Broadcasts `tensor` to the given `broadcast_shape`.\n\nThe new shape is either a tuple or a tensor which we will\nretrieve the current shape from. The broadcast shape must\nbe of equal or higher rank than the current shape.\n\nAn optional `:axes` can be given to customize how broadcasting\nhappens. `axes` must be a list with the same length as the\ntensor shape. Each `axis` in the list maps to the dimension\nin the broadcast shape that must match. For example, an axis\nof `[1, 2]` says the 0 dimension of the tensor matches to\nthe 1 dimension of the broadcast shape and the 1 dimension\nof the tensor matches the 2 dimension of the broadcast shape.\nEach matching dimension must either be 1, for implicit\nbroadcasting, or match the dimension in the broadcast shape.\n\nBroadcasting is destructive with respect to names. You can\noptionally provide new `:names` for the new tensor. If you\npass a tensor with named dimensions, the new tensor will\ninherit names from that tensor.","ref":"Nx.html#broadcast/3"},{"type":"function","title":"Examples - Nx.broadcast/3","doc":"#","ref":"Nx.html#broadcast/3-examples"},{"type":"function","title":"Without axes - Nx.broadcast/3","doc":"iex> Nx.broadcast(1, {1, 2, 3})\n    #Nx.Tensor \n\n    iex> Nx.broadcast(Nx.tensor([[1], [2]], names: [:x, :y]), Nx.tensor([[10, 20], [30, 40]], names: [:i, :j]))\n    #Nx.Tensor \n\n    iex> Nx.broadcast(Nx.tensor([[1, 2]], names: [:x, :y]), Nx.tensor([[10, 20], [30, 40]], names: [:i, :j]))\n    #Nx.Tensor \n\nNote that, even if there is no broadcasting because the\nshape is the same, names are discarded if none are given:\n\n    iex> Nx.broadcast(Nx.iota({2, 2}, names: [:x, :y]), {2, 2})\n    #Nx.Tensor \n\n    iex> Nx.broadcast(Nx.iota({2, 2}, names: [:x, :y]), {2, 2}, names: [:i, :j])\n    #Nx.Tensor \n\n#","ref":"Nx.html#broadcast/3-without-axes"},{"type":"function","title":"With axes - Nx.broadcast/3","doc":"Using the default broadcast rules, we cannot broadcast a\ntensor of shape (3) to the shape (3, 2), because the lower\ndimensions must match. But with `Nx.broadcast/3` we can\nconfigure how the dimensions match:\n\n    iex> t = Nx.tensor([1, 2, 3])\n    iex> Nx.broadcast(t, {3, 2}, axes: [0], names: [:x, :y])\n    #Nx.Tensor \n\nOr a more complex example:\n\n    iex> t = Nx.tensor([1, 2, 3])\n    iex> Nx.broadcast(t, {2, 3, 2}, axes: [1], names: [:x, :y, :z])\n    #Nx.Tensor","ref":"Nx.html#broadcast/3-with-axes"},{"type":"function","title":"Vectorized tensors - Nx.broadcast/3","doc":"Vectorized axes remain unchanged, and normal broadcast rules apply otherwise.\n\n    iex> a = Nx.tensor([[[1, 2, 3]], [[4, 5, 6]]]) |> Nx.vectorize(:x)\n    iex> Nx.broadcast(a, {2, 3})\n    #Nx.Tensor \n\nFor tensors as shapes, the broadcast will only take the shape in consideration.\n\n    iex> a = Nx.tensor([[1, 2, 3], [4, 5, 6]]) |> Nx.vectorize(:x)\n    #Nx.Tensor \n    iex> b = Nx.tensor([[[1, 2, 3], [4, 5, 6]]], names: [nil, nil, :y]) |> Nx.vectorize(:a)\n    #Nx.Tensor \n    iex> Nx.broadcast(a, b, axes: [1], names: [:i, :j])\n    #Nx.Tensor","ref":"Nx.html#broadcast/3-vectorized-tensors"},{"type":"function","title":"Nx.broadcast_vectors/2","doc":"Broadcasts vectorized axes, ensuring they end up with the same final size.\n\nThe inner shape is unchanged for each tensor.\nThe order of the vectorized axes is determined by order of appearance in the input list.","ref":"Nx.html#broadcast_vectors/2"},{"type":"function","title":"Options - Nx.broadcast_vectors/2","doc":"* `:align_ranks` - boolean that indicates whether the inner\n    shapes should be aligned to the maximum rank of the inputs.\n    That is, 1-sized leading dimensions are added so\n    that all tensors have the same rank in the output.\n    This only applies in case one of the inputs is vectorized.","ref":"Nx.html#broadcast_vectors/2-options"},{"type":"function","title":"Examples - Nx.broadcast_vectors/2","doc":"iex> x = Nx.tensor([1, 2]) |> Nx.vectorize(:x)\n    #Nx.Tensor \n    iex> xy = Nx.tensor([[[5]], [[6]]]) |> Nx.vectorize(:y) |> Nx.vectorize(:x)\n    #Nx.Tensor \n    iex> [broadcast_x, broadcast_xy] = Nx.broadcast_vectors([x, xy], align_ranks: true)\n    iex> broadcast_x\n    #Nx.Tensor \n    iex> broadcast_xy\n    #Nx.Tensor \n    iex> [broadcast_xy, broadcast_x] = Nx.broadcast_vectors([xy, x])\n    iex> broadcast_x\n    #Nx.Tensor \n    iex> broadcast_xy\n    #Nx.Tensor","ref":"Nx.html#broadcast_vectors/2-examples"},{"type":"function","title":"Nx.byte_size/1","doc":"Returns the byte size of the data in the tensor\ncomputed from its shape and type.","ref":"Nx.html#byte_size/1"},{"type":"function","title":"Examples - Nx.byte_size/1","doc":"iex> Nx.byte_size(Nx.tensor([[1, 2, 3], [4, 5, 6]]))\n    48\n    iex> Nx.byte_size(Nx.tensor([[1.0, 2.0, 3.0], [4.0, 5.0, 6.0]]))\n    24\n    iex> Nx.byte_size(Nx.tensor([[1, 2, 3], [4, 5, 6]], type: :u8))\n    6\n    iex> Nx.byte_size(1)\n    8\n\nVectorized tensors account for all elements\n\n    iex> Nx.byte_size(Nx.tensor([[1, 2], [3, 4]]) |> Nx.vectorize(:x))\n    32","ref":"Nx.html#byte_size/1-examples"},{"type":"function","title":"Nx.cbrt/1","doc":"Calculates the cube root of each element in the tensor.\n\nIt is equivalent to:\n\n$$cbrt(z) = z^{\\frac{1}{3}}$$","ref":"Nx.html#cbrt/1"},{"type":"function","title":"Examples - Nx.cbrt/1","doc":"iex> Nx.cbrt(1)\n    #Nx.Tensor \n\n    iex> Nx.cbrt(Nx.tensor([1, 2, 3], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#cbrt/1-examples"},{"type":"function","title":"Nx.ceil/1","doc":"Calculates the ceil of each element in the tensor.\n\nIf a non-floating tensor is given, it is returned as is.\nIf a floating tensor is given, then we apply the operation,\nbut keep its type.","ref":"Nx.html#ceil/1"},{"type":"function","title":"Examples - Nx.ceil/1","doc":"iex> Nx.ceil(Nx.tensor([-1, 0, 1], names: [:x]))\n    #Nx.Tensor \n\n    iex> Nx.ceil(Nx.tensor([-1.5, -0.5, 0.5, 1.5], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#ceil/1-examples"},{"type":"function","title":"Nx.clip/3","doc":"Clips the values of the tensor on the closed\ninterval `[min, max]`.\n\nYou can pass a tensor to `min` or `max` as long\nas the tensor has a scalar shape.","ref":"Nx.html#clip/3"},{"type":"function","title":"Examples - Nx.clip/3","doc":"iex> t = Nx.tensor([[1, 2, 3], [4, 5, 6]], names: [:x, :y])\n    iex> Nx.clip(t, 2, 4)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[1, 2, 3], [4, 5, 6]], names: [:x, :y])\n    iex> Nx.clip(t, 2.0, 3)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[1, 2, 3], [4, 5, 6]], names: [:x, :y])\n    iex> Nx.clip(t, Nx.tensor(2.0), Nx.max(1.0, 3.0))\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[1.0, 2.0, 3.0], [4.0, 5.0, 6.0]], names: [:x, :y])\n    iex> Nx.clip(t, 2, 6.0)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[1.0, 2.0, 3.0], [4.0, 5.0, 6.0]], type: :f32, names: [:x, :y])\n    iex> Nx.clip(t, 1, 4)\n    #Nx.Tensor","ref":"Nx.html#clip/3-examples"},{"type":"function","title":"Vectorized tensors - Nx.clip/3","doc":"Only the main input tensor is allowed to be vectorized. `min` and `max` threshold tensors\nmust be unvectorized scalar tensors.\n\n    iex> t = Nx.tensor([[1.0, 2.0, 3.0], [4.0, 5.0, 6.0]], type: :f32, names: [nil, :y]) |> Nx.vectorize(:x)\n    iex> Nx.clip(t, 1, 4)\n    #Nx.Tensor","ref":"Nx.html#clip/3-vectorized-tensors"},{"type":"function","title":"Nx.compatible?/2","doc":"Checks if two tensors have the same shape, type, and compatible names.\n\nThe data in the tensor is ignored.\n\nNote: This function cannot be used in `defn`.","ref":"Nx.html#compatible?/2"},{"type":"function","title":"Examples - Nx.compatible?/2","doc":"iex> Nx.compatible?(Nx.iota({3, 2}), Nx.iota({3, 2}))\n    true\n\n    iex> Nx.compatible?(Nx.iota({3, 2}), Nx.iota({3, 2}, names: [:rows, :columns]))\n    true\n\n    iex> Nx.compatible?(\n    ...>   Nx.iota({3, 2}, names: [:rows, nil]),\n    ...>   Nx.iota({3, 2}, names: [nil, :columns])\n    ...> )\n    true\n\n    iex> Nx.compatible?(\n    ...>   Nx.iota({3, 2}, names: [:foo, :bar]),\n    ...>   Nx.iota({3, 2}, names: [:rows, :columns])\n    ...> )\n    false\n\n    iex> Nx.compatible?(Nx.iota({3, 2}), Nx.iota({2, 3}))\n    false\n\n    iex> Nx.compatible?(Nx.iota({2, 2}), Nx.iota({2, 2}, type: :f32))\n    false\n\nUsing collections:\n\n    iex> Nx.compatible?({Nx.iota({3, 2}), {1, 2}}, {Nx.iota({3, 2}), {3, 4}})\n    true\n\n    iex> Nx.compatible?(%{foo: Nx.iota({3, 2})}, %{foo: Nx.iota({3, 2})})\n    true\n\n    iex> Nx.compatible?(%{foo: Nx.iota({3, 2})}, %{bar: Nx.iota({3, 2})})\n    false","ref":"Nx.html#compatible?/2-examples"},{"type":"function","title":"Vectorized tensors - Nx.compatible?/2","doc":"Same compatibility criteria applies to vectorized tensors, but there's\nthe additional requirement that vectorized axes must be the same in both\ntensors.\n\n    iex> Nx.compatible?(Nx.tensor([1, 2]) |> Nx.vectorize(:x), Nx.tensor([3, 4]) |> Nx.vectorize(:x))\n    true\n    iex> Nx.compatible?(Nx.tensor([1, 2, 3]) |> Nx.vectorize(:x), Nx.tensor([1, 2]) |> Nx.vectorize(:x))\n    false\n    iex> Nx.compatible?(Nx.tensor([1]) |> Nx.vectorize(:x), Nx.tensor([1, 2]) |> Nx.vectorize(:y))\n    false","ref":"Nx.html#compatible?/2-vectorized-tensors"},{"type":"function","title":"Nx.complex/2","doc":"Constructs a complex tensor from two equally-shaped tensors.\n\nDoes not accept complex tensors as inputs.","ref":"Nx.html#complex/2"},{"type":"function","title":"Examples - Nx.complex/2","doc":"iex> Nx.complex(Nx.tensor(1), Nx.tensor(2))\n    #Nx.Tensor \n\n    iex> Nx.complex(Nx.tensor([1, 2]), Nx.tensor([3, 4]))\n    #Nx.Tensor","ref":"Nx.html#complex/2-examples"},{"type":"function","title":"Nx.concatenate/2","doc":"Concatenates tensors along the given axis.\n\nTensors can be a tuple or any `Nx.Container` or `Nx.LazyContainer`.\nThis means you can easily concatenate all columns in a dataframe\nand other data structures. For convenience, this function also allows\na list of tensors to be given, which may be common outside of `defn`.\n\nIf no axis is provided, defaults to 0. All tensors must have the same\nrank and all of their axis except the concatenated one must match.\n\nIf tensors with mixed types are given, the types will\nbe merged to a higher type and all of the tensors will\nbe cast to the higher type before concatenating.\nIf tensors are named, the names must match.","ref":"Nx.html#concatenate/2"},{"type":"function","title":"Examples - Nx.concatenate/2","doc":"Giving a single tensor is a no-op:\n\n    iex> Nx.concatenate([Nx.tensor([1, 2, 3])])\n    #Nx.Tensor \n\nMultiple tensors are concatented:\n\n    iex> Nx.concatenate([Nx.tensor([1, 2, 3]), Nx.tensor([4, 5, 6])])\n    #Nx.Tensor \n\nTypes are merged and names must match:\n\n    iex> t1 = Nx.iota({2, 2, 2}, names: [:x, :y, :z], type: :f32)\n    iex> t2 = Nx.iota({1, 2, 2}, names: [:x, :y, :z], type: :u8)\n    iex> t3 = Nx.iota({1, 2, 2}, names: [:x, :y, :z], type: :s64)\n    iex> Nx.concatenate([t1, t2, t3], axis: :x)\n    #Nx.Tensor \n\nAnd you can pick a different axis:\n\n    iex> t1 = Nx.iota({1, 3, 2}, names: [:x, :y, :z])\n    iex> t2 = Nx.iota({1, 1, 2}, names: [:x, :y, :z])\n    iex> t3 = Nx.iota({1, 2, 2}, names: [:x, :y, :z])\n    iex> Nx.concatenate([t1, t2, t3], axis: :y)\n    #Nx.Tensor \n\nYou can also pass any container (or lazy container) as first argument\nand they are recursively traversed:\n\n    iex> Nx.concatenate({Nx.tensor([1, 2]), {Nx.tensor([3, 4]), Nx.tensor([5, 6])}})\n    #Nx.Tensor","ref":"Nx.html#concatenate/2-examples"},{"type":"function","title":"Vectorized tensors - Nx.concatenate/2","doc":"If vectorized tensors are given, they are all broadcasted throughout the\nvectorized axes before concatenation. Normal concatenation rules still apply\nto the inner shapes.\n\n    iex> x = Nx.tensor([[1, 2]]) |> Nx.vectorize(:x)\n    iex> y = Nx.tensor([[3, 4], [5, 6]]) |> Nx.vectorize(:y)\n    iex> z = Nx.tensor([[10], [11]]) |> Nx.vectorize(:x)\n    iex> Nx.concatenate({x, y, z})\n    #Nx.Tensor","ref":"Nx.html#concatenate/2-vectorized-tensors"},{"type":"function","title":"Error cases - Nx.concatenate/2","doc":"Shapes must have the same rank and match on the non-concatenating axis.\n\nFor example, the tensors below work if we concatenate on axis 1, but not on axis 0:\n\n    iex> t1 = Nx.iota({1, 2, 3})\n    iex> t2 = Nx.iota({1, 1, 3})\n    iex> result = Nx.concatenate([t1, t2], axis: 1)\n    iex> Nx.shape(result)\n    {1, 3, 3}\n    iex> Nx.concatenate([t1, t2], axis: 0)\n    ** (ArgumentError) expected all shapes to match {*, 2, 3}, got unmatching shape: {1, 1, 3}\n\nIf the ranks are different, it doesn't work, regardless of the axis choice:\n\n    iex> t1 = Nx.iota({1, 2, 3})\n    iex> t2 = Nx.iota({1, 1})\n    iex> Nx.concatenate([t1, t2])\n    ** (ArgumentError) expected all shapes to match {*, 2, 3}, got unmatching shape: {1, 1}","ref":"Nx.html#concatenate/2-error-cases"},{"type":"function","title":"Nx.conjugate/1","doc":"Calculates the complex conjugate of each element in the tensor.\n\nIf $$z = a + bi = r e^\\theta$$, $$conjugate(z) = z^* = a - bi =  r e^{-\\theta}$$","ref":"Nx.html#conjugate/1"},{"type":"function","title":"Examples - Nx.conjugate/1","doc":"iex> Nx.conjugate(Complex.new(1, 2))\n     #Nx.Tensor \n\n     iex> Nx.conjugate(1)\n     #Nx.Tensor \n\n     iex> Nx.conjugate(Nx.tensor([Complex.new(1, 2), Complex.new(2, -4)]))\n     #Nx.Tensor","ref":"Nx.html#conjugate/1-examples"},{"type":"function","title":"Nx.conv/3","doc":"Computes an n-D convolution (where `n >= 3`) as used in neural networks.\n\nThis function can be thought of as sliding an n-D\nkernel across the input, producing a new tensor that\nhas the same number of elements as the number of valid\nwindows in the input tensor. Each element is the result\nof summing the element-wise products in the window across\neach input channel.\n\nThe ranks of both `input` and `kernel` must match. By\ndefault, both `input` and `kernel` are expected to have shapes\nof the following form:\n\n  * `input` - `{batch_size, input_channels, input_d0, ..., input_dn}`\n  * `kernel` - `{output_channels, input_channels, kernel_d0, ..., kernel_dn}`\n\nWhere `input_d0...input_dn` and `kernel_d0...kernel_dn` represent\nan arbitrary number of spatial dimensions. You can alter this configuration\nusing one of the `*_permutation` configuration options. Permutations\nare input, kernel, and output specifications for the layout of the\nconvolution. For example, if your input tensor is configured with\n\"channels last\", you can specify the input permutation with:\n\n    Nx.conv(img, kernel, input_permutation: [0, 3, 1, 2])\n\nPermutations expect configurations that specify the location of\ndimensions in the following orders:\n\n  * `input_permutation` - `[batch_dim, input_channel_dim, ...spatial_dims...]`\n  * `kernel_permutation` - `[output_channel_dim, input_channel_dim, ...spatial_dims...]`\n  * `output_permutation` - `[batch_dim, output_channel_dim, ...spatial_dims...]`\n\nUsing named tensors, it's a bit easier to see how permutations\nhelp you configure the convolution. Given input tensor with names\n`[:batch, :height, :width, :channels]` (channels last) and kernel\ntensor with names `[:input, :output, :height, :width]`, you can\nconfigure the convolution with the following permutations:\n\n    Nx.conv(img, kernel,\n      input_permutation: [:batch, :channels, :height, :width],\n      kernel_permutation: [:output, :input, :height, :width],\n      output_permutation: [:batch, :channels, :height, :width]\n    )\n\nNotice that `output_permutation` is normalized with respect to\nthe input permutation names. We cannot guarantee that every\npermutation is supported in every backend or compiler.\n\nTo configure how the window slides along the input tensor, you\ncan specify `:strides`. `:strides` must be a positive integer\nor tuple of positive integers for each spatial dimension\nin the input and kernel. For each spatial dimension, the\nwindow will slide by the configuration specified in `:strides`.\nAs an example, for a 2-D convolution with `strides: [2, 1]`,\nthe window will slide 2 positions along the first spatial\ndimension until it reaches the end of the dimension and then\n1 position along the second spatial dimension.\n\nYou may specify a padding configuration using `:padding`,\nwhich will zero-pad the input tensor. Acceptable padding\nconfigurations are:\n\n  * `:valid` - no padding\n  * `:same` - pad input spatial dimensions such that they\n  will remain unchanged in the output tensor\n  * `[{d0_hi, d0_lo}, ..., {dn_hi, dn_lo}]` - a general padding\n  configuration of edge high and edge low padding values. You\n  may only specify padding for the edges of spatial dimensions\n  of the input tensor. Padding values may be negative.\n\nYou can dilate convolutions by setting `:input_dilation` or\n`:kernel_dilation`. Both `:input_dilation` and `:kernel_dilation`\nmust either be positive integers or tuples of positive integers\nfor each spatial dimension in the input and kernel tensors. Dilations\ncan be thought of as applying `dilation - 1` interior padding to the\ninput or kernel tensor.\n\nYou can split both the input and kernel tensor into feature groups\nusing `:feature_group_size`. This will split both the input and kernel\ntensor channels and compute a grouped convolution. The size of the\nkernel input feature channels times the size of the feature group must\nmatch the size of the input tensor feature channels. Additionally,\nthe size of the kernel output feature channels must be evenly divisible\nby the group size.\n\nYou can also split the input tensor along the batch dimension by\nspecifying `:batch_group_size`. This will compute a grouped convolution\nin the same way as with `:feature_group_size`, however, the input\ntensor will be split into groups along the batch dimension.","ref":"Nx.html#conv/3"},{"type":"function","title":"Examples - Nx.conv/3","doc":"iex> left = Nx.iota({9})\n    iex> left = Nx.reshape(left, {1, 1, 3, 3})\n    iex> right = Nx.iota({4})\n    iex> right = Nx.reshape(right, {4, 1, 1, 1})\n    iex> Nx.conv(left, right, strides: [1, 1])\n    #Nx.Tensor \n\n    iex> left = Nx.iota({9})\n    iex> left = Nx.reshape(left, {1, 1, 3, 3})\n    iex> right = Nx.iota({8})\n    iex> right = Nx.reshape(right, {4, 1, 2, 1})\n    iex> Nx.conv(left, right, strides: 2, padding: :same, kernel_dilation: [2, 1])\n    #Nx.Tensor \n\nComplex tensors are also supported:\n\n    iex> left = Nx.tensor([[[Complex.new(1, 1), 2, Complex.new(3, -3)]]])\n    iex> right = Nx.tensor([[[1, Complex.new(0, 2), Complex.new(0, 3)]]])\n    iex> Nx.conv(left, right, padding: [{2, 2}])\n    #Nx.Tensor","ref":"Nx.html#conv/3-examples"},{"type":"function","title":"Nx.cos/1","doc":"Calculates the cosine of each element in the tensor.\n\nIt is equivalent to:\n\n$$cos(z) = \\frac{e^{iz} + e^{-iz}}{2}$$","ref":"Nx.html#cos/1"},{"type":"function","title":"Examples - Nx.cos/1","doc":"iex> Nx.cos(1)\n    #Nx.Tensor \n\n    iex> Nx.cos(Nx.tensor([1, 2, 3], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#cos/1-examples"},{"type":"function","title":"Nx.cosh/1","doc":"Calculates the hyperbolic cosine of each element in the tensor.\n\nIt is equivalent to:\n\n$$cosh(z) = \\frac{e^z + e^{-z}}{2}$$","ref":"Nx.html#cosh/1"},{"type":"function","title":"Examples - Nx.cosh/1","doc":"iex> Nx.cosh(1)\n    #Nx.Tensor \n\n    iex> Nx.cosh(Nx.tensor([1, 2, 3], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#cosh/1-examples"},{"type":"function","title":"Nx.count_leading_zeros/1","doc":"Counts the number of leading zeros of each element in the tensor.","ref":"Nx.html#count_leading_zeros/1"},{"type":"function","title":"Examples - Nx.count_leading_zeros/1","doc":"iex> Nx.count_leading_zeros(1)\n    #Nx.Tensor \n\n    iex> Nx.count_leading_zeros(-1)\n    #Nx.Tensor \n\n    iex> Nx.count_leading_zeros(Nx.tensor([0, 0xF, 0xFF, 0xFFFF], names: [:x]))\n    #Nx.Tensor \n\n    iex> Nx.count_leading_zeros(Nx.tensor([0xF000000000000000, 0x0F00000000000000], names: [:x]))\n    #Nx.Tensor \n\n    iex> Nx.count_leading_zeros(Nx.tensor([0, 0xF, 0xFF, 0xFFFF], type: :s32, names: [:x]))\n    #Nx.Tensor \n\n    iex> Nx.count_leading_zeros(Nx.tensor([0, 0xF, 0xFF, 0xFFFF], type: :s16, names: [:x]))\n    #Nx.Tensor \n\n    iex> Nx.count_leading_zeros(Nx.tensor([0, 1, 2, 4, 8, 16, 32, 64, -1, -128], type: :s8, names: [:x]))\n    #Nx.Tensor \n\n    iex> Nx.count_leading_zeros(Nx.tensor([0, 1, 2, 4, 8, 16, 32, 64, 128], type: :u8, names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#count_leading_zeros/1-examples"},{"type":"function","title":"Error cases - Nx.count_leading_zeros/1","doc":"iex> Nx.count_leading_zeros(Nx.tensor([0.0, 1.0]))\n    ** (ArgumentError) bitwise operators expect integer tensors as inputs and outputs an integer tensor, got: {:f, 32}","ref":"Nx.html#count_leading_zeros/1-error-cases"},{"type":"function","title":"Nx.covariance/2","doc":"A shortcut to `covariance/3` with either `opts` or `mean` as second argument.","ref":"Nx.html#covariance/2"},{"type":"function","title":"Nx.covariance/3","doc":"Computes the covariance matrix of the input tensor.\n\nThe covariance of two random variables X and Y is calculated as $Cov(X, Y) = \\frac{1}{N}\\sum_{i=0}^{N-1}{(X_i - \\overline{X}) * (Y_i - \\overline{Y})}$.\n\nThe tensor must be at least of rank 2, with shape `{n, d}`. Any additional\ndimension will be treated as batch dimensions.\n\nThe column mean can be provided as the second argument and it must be\na tensor of shape `{..., d}`, where the batch shape is broadcastable with\nthat of the input tensor. If not provided, the mean is estimated using `Nx.mean/2`.\n\nIf the `:ddof` (delta degrees of freedom) option is given, the divisor `n - ddof`\nis used for the sum of the products.","ref":"Nx.html#covariance/3"},{"type":"function","title":"Examples - Nx.covariance/3","doc":"iex> Nx.covariance(Nx.tensor([[1, 2], [3, 4], [5, 6]]))\n    #Nx.Tensor \n\n    iex> Nx.covariance(Nx.tensor([[[1, 2], [3, 4], [5, 6]], [[7, 8], [9, 10], [11, 12]]]))\n    #Nx.Tensor \n\n    iex> Nx.covariance(Nx.tensor([[1, 2], [3, 4], [5, 6]]), ddof: 1)\n    #Nx.Tensor \n\n    iex> Nx.covariance(Nx.tensor([[1, 2], [3, 4], [5, 6]]), Nx.tensor([4, 3]))\n    #Nx.Tensor","ref":"Nx.html#covariance/3-examples"},{"type":"function","title":"Nx.cumulative_max/2","doc":"Returns the cumulative maximum of elements along an axis.","ref":"Nx.html#cumulative_max/2"},{"type":"function","title":"Options - Nx.cumulative_max/2","doc":"* `:axis` - the axis to compare elements along. Defaults to `0`\n  * `:reverse` - whether to perform accumulation in the opposite direction. Defaults to `false`","ref":"Nx.html#cumulative_max/2-options"},{"type":"function","title":"Examples - Nx.cumulative_max/2","doc":"iex> Nx.cumulative_max(Nx.tensor([3, 4, 2, 1]))\n    #Nx.Tensor \n\n    iex> Nx.cumulative_max(Nx.tensor([[2, 3, 1], [1, 3, 2], [2, 1, 3]]), axis: 0)\n    #Nx.Tensor \n\n    iex> Nx.cumulative_max(Nx.tensor([[2, 3, 1], [1, 3, 2], [2, 1, 3]]), axis: 1)\n    #Nx.Tensor \n\n    iex> Nx.cumulative_max(Nx.tensor([[2, 3, 1], [1, 3, 2], [2, 1, 3]]), axis: 0, reverse: true)\n    #Nx.Tensor \n\n    iex> Nx.cumulative_max(Nx.tensor([[2, 3, 1], [1, 3, 2], [2, 1, 3]]), axis: 1, reverse: true)\n    #Nx.Tensor","ref":"Nx.html#cumulative_max/2-examples"},{"type":"function","title":"Vectorized axes - Nx.cumulative_max/2","doc":"Works the same as if the accumulation was to happen over a list of tensors.\n`:axis` refers to the non-vectorized shape.\n\n    iex> Nx.cumulative_max(Nx.tensor([[2, 3, 1], [1, 3, 2], [2, 1, 3]]) |> Nx.vectorize(:x), axis: 0)\n    #Nx.Tensor","ref":"Nx.html#cumulative_max/2-vectorized-axes"},{"type":"function","title":"Nx.cumulative_min/2","doc":"Returns the cumulative minimum of elements along an axis.","ref":"Nx.html#cumulative_min/2"},{"type":"function","title":"Options - Nx.cumulative_min/2","doc":"* `:axis` - the axis to compare elements along. Defaults to `0`\n  * `:reverse` - whether to perform accumulation in the opposite direction. Defaults to `false`","ref":"Nx.html#cumulative_min/2-options"},{"type":"function","title":"Examples - Nx.cumulative_min/2","doc":"iex> Nx.cumulative_min(Nx.tensor([3, 4, 2, 1]))\n    #Nx.Tensor \n\n    iex> Nx.cumulative_min(Nx.tensor([[2, 3, 1], [1, 3, 2], [2, 1, 3]]), axis: 0)\n    #Nx.Tensor \n\n    iex> Nx.cumulative_min(Nx.tensor([[2, 3, 1], [1, 3, 2], [2, 1, 3]]), axis: 1)\n    #Nx.Tensor \n\n    iex> Nx.cumulative_min(Nx.tensor([[2, 3, 1], [1, 3, 2], [2, 1, 3]]), axis: 0, reverse: true)\n    #Nx.Tensor \n\n    iex> Nx.cumulative_min(Nx.tensor([[2, 3, 1], [1, 3, 2], [2, 1, 3]]), axis: 1, reverse: true)\n    #Nx.Tensor","ref":"Nx.html#cumulative_min/2-examples"},{"type":"function","title":"Vectorized axes - Nx.cumulative_min/2","doc":"Works the same as if the accumulation was to happen over a list of tensors.\n`:axis` refers to the non-vectorized shape.\n\n    iex> Nx.cumulative_min(Nx.tensor([[2, 3, 1], [1, 3, 2], [2, 1, 3]]) |> Nx.vectorize(:x), axis: 0)\n    #Nx.Tensor","ref":"Nx.html#cumulative_min/2-vectorized-axes"},{"type":"function","title":"Nx.cumulative_product/2","doc":"Returns the cumulative product of elements along an axis.","ref":"Nx.html#cumulative_product/2"},{"type":"function","title":"Options - Nx.cumulative_product/2","doc":"* `:axis` - the axis to multiply elements along. Defaults to `0`\n  * `:reverse` - whether to perform accumulation in the opposite direction. Defaults to `false`","ref":"Nx.html#cumulative_product/2-options"},{"type":"function","title":"Examples - Nx.cumulative_product/2","doc":"iex> Nx.cumulative_product(Nx.tensor([1, 2, 3, 4]))\n    #Nx.Tensor \n\n    iex> Nx.cumulative_product(Nx.iota({3, 3}), axis: 0)\n    #Nx.Tensor \n\n    iex> Nx.cumulative_product(Nx.iota({3, 3}), axis: 1)\n    #Nx.Tensor \n\n    iex> Nx.cumulative_product(Nx.iota({3, 3}), axis: 0, reverse: true)\n    #Nx.Tensor \n\n    iex> Nx.cumulative_product(Nx.iota({3, 3}), axis: 1, reverse: true)\n    #Nx.Tensor","ref":"Nx.html#cumulative_product/2-examples"},{"type":"function","title":"Vectorized axes - Nx.cumulative_product/2","doc":"Works the same as if the accumulation was to happen over a list of tensors.\n`:axis` refers to the non-vectorized shape.\n\n    iex> Nx.cumulative_product(Nx.tensor([[2, 3, 0], [1, 3, 2], [2, 1, 3]]) |> Nx.vectorize(:x), axis: 0)\n    #Nx.Tensor","ref":"Nx.html#cumulative_product/2-vectorized-axes"},{"type":"function","title":"Nx.cumulative_sum/2","doc":"Returns the cumulative sum of elements along an axis.","ref":"Nx.html#cumulative_sum/2"},{"type":"function","title":"Options - Nx.cumulative_sum/2","doc":"* `:axis` - the axis to sum elements along. Defaults to `0`\n  * `:reverse` - whether to perform accumulation in the opposite direction. Defaults to `false`","ref":"Nx.html#cumulative_sum/2-options"},{"type":"function","title":"Examples - Nx.cumulative_sum/2","doc":"iex> Nx.cumulative_sum(Nx.tensor([1, 2, 3, 4]))\n    #Nx.Tensor \n\n    iex> Nx.cumulative_sum(Nx.iota({3, 3}), axis: 0)\n    #Nx.Tensor \n\n    iex> Nx.cumulative_sum(Nx.iota({3, 3}), axis: 1)\n    #Nx.Tensor \n\n    iex> Nx.cumulative_sum(Nx.iota({3, 3}), axis: 0, reverse: true)\n    #Nx.Tensor \n\n    iex> Nx.cumulative_sum(Nx.iota({3, 3}), axis: 1, reverse: true)\n    #Nx.Tensor","ref":"Nx.html#cumulative_sum/2-examples"},{"type":"function","title":"Vectorized axes - Nx.cumulative_sum/2","doc":"Works the same as if the accumulation was to happen over a list of tensors.\n`:axis` refers to the non-vectorized shape.\n\n    iex> Nx.cumulative_sum(Nx.tensor([[2, 3, 1], [1, 3, 2], [2, 1, 3]]) |> Nx.vectorize(:x), axis: 0)\n    #Nx.Tensor","ref":"Nx.html#cumulative_sum/2-vectorized-axes"},{"type":"function","title":"Nx.default_backend/0","doc":"Gets the default backend for the current process.\n\nNote: This function cannot be used in `defn`.","ref":"Nx.html#default_backend/0"},{"type":"function","title":"Nx.default_backend/1","doc":"Sets the given `backend` as default in the **current process**.\n\nThe default backend is stored only in the process dictionary.\nThis means if you start a separate process, such as `Task`,\nthe default backend must be set on the new process too.\n\nDue to this reason, this function is mostly used for scripting\nand testing. In your applications, you must prefer to set the\nbackend in your config files:\n\n    config :nx, :default_backend, {EXLA.Backend, device: :cuda}\n\nIn your notebooks and on `Mix.install/2`, you might:\n\n    Mix.install(\n      [\n        {:nx, \">= 0.0.0\"}\n      ],\n      config: [nx: [default_backend: {EXLA.Backend, device: :cuda}]]\n    )\n\nOr use `Nx.global_default_backend/1` as it changes the\ndefault backend on all processes.\n\nThe function returns the value that was previously set as backend.\n\nNote: This function cannot be used in `defn`.","ref":"Nx.html#default_backend/1"},{"type":"function","title":"Examples - Nx.default_backend/1","doc":"Nx.default_backend({EXLA.Backend, device: :cuda})\n    #=> {Nx.BinaryBackend, []}","ref":"Nx.html#default_backend/1-examples"},{"type":"function","title":"Nx.deserialize/2","doc":"Deserializes a serialized representation of a tensor or a container\nwith the given options.\n\nIt is the opposite of `Nx.serialize/2`.\n\nNote: This function cannot be used in `defn`.","ref":"Nx.html#deserialize/2"},{"type":"function","title":"Examples - Nx.deserialize/2","doc":"iex> a = Nx.tensor([1, 2, 3])\n    iex> serialized_a = Nx.serialize(a)\n    iex> Nx.deserialize(serialized_a)\n    #Nx.Tensor \n\n    iex> container = {Nx.vectorize(Nx.tensor([1, 2, 3]), :x), %{b: Nx.tensor([4, 5, 6])}}\n    iex> serialized_container = Nx.serialize(container)\n    iex> {a, %{b: b}} = Nx.deserialize(serialized_container)\n    iex> a\n    #Nx.Tensor \n    iex> b\n    #Nx.Tensor","ref":"Nx.html#deserialize/2-examples"},{"type":"function","title":"Nx.devectorize/2","doc":"Transforms a vectorized tensor back into a regular tensor.","ref":"Nx.html#devectorize/2"},{"type":"function","title":"Options - Nx.devectorize/2","doc":"* `:keep_names` - a boolean indicating whether\n    vectorized axes' names should be turned into the new\n    axes' names. Defaults to `true`.","ref":"Nx.html#devectorize/2-options"},{"type":"function","title":"Examples - Nx.devectorize/2","doc":"iex> t = Nx.iota({1, 2, 3}) |> Nx.vectorize(:x) |> Nx.vectorize(:y)\n    #Nx.Tensor \n    iex> Nx.devectorize(t)\n    #Nx.Tensor \n    iex> Nx.devectorize(t, keep_names: false)\n    #Nx.Tensor","ref":"Nx.html#devectorize/2-examples"},{"type":"function","title":"Containers - Nx.devectorize/2","doc":"Containers are also supported:\n\n    iex> input = {1, %{a: Nx.iota({3}, vectorized_axes: [x: 1])}}\n    iex> {t1, %{a: t2}} = Nx.devectorize(input)\n    iex> t1\n    #Nx.Tensor \n    iex> t2\n    #Nx.Tensor","ref":"Nx.html#devectorize/2-containers"},{"type":"function","title":"Nx.diff/2","doc":"Calculate the n-th discrete difference along the given axis.\n\nThe first difference is given by $out_i = a_{i+1} - a_i$ along the given axis,\nhigher differences are calculated by using `diff` recursively.","ref":"Nx.html#diff/2"},{"type":"function","title":"Options - Nx.diff/2","doc":"* `:order` - the number of times to perform the difference. Defaults to `1`\n  * `:axis` - the axis to perform the difference along. Defaults to `-1`","ref":"Nx.html#diff/2-options"},{"type":"function","title":"Examples - Nx.diff/2","doc":"iex> Nx.diff(Nx.tensor([1, 2, 4, 7, 0]))\n    #Nx.Tensor \n\n    iex> Nx.diff(Nx.tensor([1, 2, 4, 7, 0]), order: 2)\n    #Nx.Tensor \n\n    iex> Nx.diff(Nx.tensor([[1, 3, 6, 10], [0, 5, 6, 8]]))\n    #Nx.Tensor \n\n    iex> Nx.diff(Nx.tensor([[1, 3, 6, 10], [0, 5, 6, 8]]), axis: 0)\n    #Nx.Tensor \n\n    iex> Nx.diff(Nx.tensor([1, 2, 4, 7, 0]), order: 0)\n    #Nx.Tensor \n\n    iex> Nx.diff(Nx.tensor([1, 2, 4, 7, 0]), order: -1)\n    ** (ArgumentError) order must be non-negative but got: -1","ref":"Nx.html#diff/2-examples"},{"type":"function","title":"Nx.divide/2","doc":"Element-wise division of two tensors.\n\nIf a number is given, it is converted to a tensor.\n\nIt always returns a float tensor. If any of the input\ntensors are not float, they are converted to f32.\nDivision by zero raises, but it may trigger undefined\nbehaviour on some compilers.\n\nIt will broadcast tensors whenever the dimensions do\nnot match and broadcasting is possible.\n\nIf you're using `Nx.Defn.defn/2`, you can use the `/` operator\nin place of this function: `left / right`.","ref":"Nx.html#divide/2"},{"type":"function","title":"Examples - Nx.divide/2","doc":"#","ref":"Nx.html#divide/2-examples"},{"type":"function","title":"Dividing scalars - Nx.divide/2","doc":"iex> Nx.divide(1, 2)\n    #Nx.Tensor \n\n#","ref":"Nx.html#divide/2-dividing-scalars"},{"type":"function","title":"Dividing tensors and scalars - Nx.divide/2","doc":"iex> Nx.divide(Nx.tensor([1, 2, 3], names: [:data]), 1)\n    #Nx.Tensor \n\n    iex> Nx.divide(1, Nx.tensor([1.0, 2.0, 3.0], names: [:data]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#divide/2-dividing-tensors-and-scalars"},{"type":"function","title":"Dividing tensors - Nx.divide/2","doc":"iex> left = Nx.tensor([[1], [2]], names: [:x, nil])\n    iex> right = Nx.tensor([[10, 20]], names: [nil, :y])\n    iex> Nx.divide(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([[1], [2]], type: :s8)\n    iex> right = Nx.tensor([[10, 20]], type: :s8, names: [:x, :y])\n    iex> Nx.divide(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([[1], [2]], type: :f32, names: [:x, nil])\n    iex> right = Nx.tensor([[10, 20]], type: :f32, names: [nil, :y])\n    iex> Nx.divide(left, right)\n    #Nx.Tensor","ref":"Nx.html#divide/2-dividing-tensors"},{"type":"function","title":"Nx.dot/2","doc":"Returns the dot product of two tensors.\n\nGiven `a` and `b`, computes the dot product according to\nthe following rules:\n\n  * If both `a` and `b` are scalars, it is equivalent to `a * b`.\n\n  * If `a` is a scalar and `b` is a tensor, it is equivalent to `Nx.multiply(a, b)`.\n\n  * If `a` is a tensor and `b` is a scalar, it is equivalent to `Nx.multiply(a, b)`.\n\n  * If both `a` and `b` are 1-D tensors (vectors), it is the sum of the element-wise\n    product between `a` and `b`. The lengths of `a` and `b` must be equal.\n\n  * If both `a` and `b` are 2-D tensors (matrices), it is equivalent to matrix-multiplication.\n\n  * If either `a` or `b` is a 1-D tensor, and the other is an n-D tensor, it is the\n    sum of the element-wise product along the last axis of `a` or `b`. The length of the\n    1-D tensor must match the last dimension of the n-D tensor.\n\n  * If `a` is an n-D tensor and `b` is an m-D tensor, it is the sum of the element-wise\n    product along the last axis of `a` and the second-to-last axis of `b`. The last dimension\n    of `a` must match the second-to-last dimension of `b`.\n\nFor a more general `dot` function where you control which axes contract,\nsee `dot/4`.","ref":"Nx.html#dot/2"},{"type":"function","title":"Examples - Nx.dot/2","doc":"#","ref":"Nx.html#dot/2-examples"},{"type":"function","title":"Dot product of scalars - Nx.dot/2","doc":"iex> Nx.dot(5, 5)\n    #Nx.Tensor \n\n    iex> Nx.dot(-2.0, 5.0)\n    #Nx.Tensor \n\n    iex> Nx.dot(2, 2.0)\n    #Nx.Tensor \n\n#","ref":"Nx.html#dot/2-dot-product-of-scalars"},{"type":"function","title":"Dot product of vectors - Nx.dot/2","doc":"iex> Nx.dot(Nx.tensor([1, 2, 3]), Nx.tensor([4, 5, 6]))\n    #Nx.Tensor \n\n    iex> Nx.dot(Nx.tensor([2.0, 4.0, 3.0, 5.0]), Nx.tensor([1.0, 2.0, 3.0, 4.0]))\n    #Nx.Tensor \n\n    iex> Nx.dot(Nx.tensor([1.0, 2.0, 3.0]), Nx.tensor([1, 2, 3]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#dot/2-dot-product-of-vectors"},{"type":"function","title":"Dot product of matrices - Nx.dot/2","doc":"iex> left = Nx.tensor([[1, 2, 3], [4, 5, 6]], names: [:i, :j])\n    iex> right = Nx.tensor([[7, 8], [9, 10], [11, 12]], names: [:x, :y])\n    iex> Nx.dot(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([[10.0, 13.0, 14.0, 15.0], [59.0, 20.0, 10.0, 30.0]], names: [:i, :j])\n    iex> right = Nx.tensor([[2.0, 4.0], [5.0, 1.0], [6.0, 8.0], [9.0, 10.0]], names: [:x, :y])\n    iex> Nx.dot(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([[1, 2, 3], [4, 5, 6]], names: [:i, :j])\n    iex> right = Nx.tensor([[7.0, 8.0], [9.0, 10.0], [11.0, 12.0]], names: [:x, :y])\n    iex> Nx.dot(left, right)\n    #Nx.Tensor \n\n#","ref":"Nx.html#dot/2-dot-product-of-matrices"},{"type":"function","title":"Dot product of vector and n-d tensor - Nx.dot/2","doc":"iex> left = Nx.tensor([[[1, 2], [3, 4]], [[5, 6], [7, 8]]], names: [:i, :j, :k])\n    iex> right = Nx.tensor([5, 10], names: [:x])\n    iex> Nx.dot(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([5, 10], names: [:x])\n    iex> right = Nx.tensor([[1, 2, 3], [4, 5, 6]], names: [:i, :j])\n    iex> Nx.dot(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([[[[[1.0, 2.0], [3.0, 4.0]], [[5.0, 6.0], [7.0, 8.0]]]]], names: [:shard, :batch, :x, :y, :z])\n    iex> right = Nx.tensor([2.0, 2.0], names: [:data])\n    iex> Nx.dot(left, right)\n    #Nx.Tensor \n\n#","ref":"Nx.html#dot/2-dot-product-of-vector-and-n-d-tensor"},{"type":"function","title":"Dot product of n-D and m-D tensor - Nx.dot/2","doc":"iex> left = Nx.tensor([[[1, 2, 3], [4, 5, 6], [7, 8, 9]], [[1, 2, 3], [4, 5, 6], [7, 8, 9]]], names: [:x, :y, :z])\n    iex> right = Nx.tensor([[[1, 2, 3], [3, 4, 5], [5, 6, 7]]], names: [:i, :j, :k])\n    iex> Nx.dot(left, right)\n    #Nx.Tensor","ref":"Nx.html#dot/2-dot-product-of-n-d-and-m-d-tensor"},{"type":"function","title":"Vectorized tensors - Nx.dot/2","doc":"Vectorized axes are treated as batched axes, much like\n`dot/6` behaves with non-vectorized tensors.\n\n    iex> t1 = Nx.tensor([[1, 2], [3, 4]]) |> Nx.vectorize(:x)\n    iex> t2 = Nx.tensor([[10, 20], [30, 40]]) |> Nx.vectorize(:x)\n    iex> Nx.dot(t1, t2)\n    #Nx.Tensor \n\n    iex> t1 = Nx.tensor([1, 2]) |> Nx.vectorize(:x)\n    iex> t2 = Nx.tensor([[10, 20]]) |> Nx.vectorize(:y)\n    iex> Nx.dot(t1, t2)\n    #Nx.Tensor","ref":"Nx.html#dot/2-vectorized-tensors"},{"type":"function","title":"Error cases - Nx.dot/2","doc":"iex> Nx.dot(Nx.tensor([1, 2, 3]), Nx.tensor([1, 2]))\n    ** (ArgumentError) dot/zip expects shapes to be compatible, dimension 0 of left-side (3) does not equal dimension 0 of right-side (2)","ref":"Nx.html#dot/2-error-cases"},{"type":"function","title":"Nx.dot/4","doc":"Computes the generalized dot product between two tensors, given\nthe contracting axes.\n\nThis is equivalent to calling `Nx.dot/6` with no batching dimensions:\n\n    Nx.dot(t1, contract_axes1, [], t2, contract_axes2, [])","ref":"Nx.html#dot/4"},{"type":"function","title":"Examples - Nx.dot/4","doc":"iex> t1 = Nx.tensor([[1, 2], [3, 4]], names: [:x, :y])\n    iex> t2 = Nx.tensor([[10, 20], [30, 40]], names: [:height, :width])\n    iex> Nx.dot(t1, [0], t2, [0])\n    #Nx.Tensor \n\n    iex> t1 = Nx.tensor([[0.0, 1.0, 2.0], [3.0, 4.0, 5.0]])\n    iex> t2 = Nx.tensor([[0.0, 1.0], [2.0, 3.0], [4.0, 5.0]])\n    iex> Nx.dot(t1, [0, 1], t2, [1, 0])\n    #Nx.Tensor","ref":"Nx.html#dot/4-examples"},{"type":"function","title":"Vectorized tensors - Nx.dot/4","doc":"The contracting axes refer to the tensors' shapes\nand do not apply to the vectorized axes:\n\n    iex> t1 = Nx.tensor([[[1, 1], [2, 2]], [[1, 1], [1, 1]]]) |> Nx.vectorize(:x)\n    iex> t2 = Nx.tensor([[1, 2], [3, 4]])\n    iex> Nx.dot(t1, [0], t2, [0])\n    #Nx.Tensor \n    iex> Nx.dot(t1, [1], t2, [0])\n    #Nx.Tensor","ref":"Nx.html#dot/4-vectorized-tensors"},{"type":"function","title":"Nx.dot/6","doc":"Computes the generalized dot product between two tensors, given\nthe contracting and batch axes.\n\nThe dot product is computed by multiplying the values from `t1`\ngiven by `contract_axes1` against the values from `t2` given by\n`contract_axes2`, considering batch axes of `batch_axes1` and\n`batch_axes2`. For instance, the first axis in `contract_axes1`\nwill be matched against the first axis in `contract_axes2` and\nso on. The axes given by `contract_axes1` and `contract_axes2`\nare effectively removed from the final tensor, which is why they\nare often called the contraction axes.\n\nIf no contracting axes are given, the final product works like\n`Nx.outer/2`.\n\nSpecifying batch axes will compute a vectorized dot product\nalong the given batch dimensions. The length of `batch_axes1`\nand `batch_axes2` must match. Additionally, `batch_axes1` and\n`batch_axes2` must be a list of successive dimension numbers,\nwhere each batch axis matches the dimension of the corresponding\nbatch axis in the other input.\n\nThe contracting axes must be dot-product compatible and the\nbatch dimensions must always have the same number of elements.","ref":"Nx.html#dot/6"},{"type":"function","title":"Examples - Nx.dot/6","doc":"#","ref":"Nx.html#dot/6-examples"},{"type":"function","title":"Contracting along axes - Nx.dot/6","doc":"iex> t1 = Nx.tensor([[1, 2], [3, 4]], names: [:x, :y])\n    iex> t2 = Nx.tensor([[10, 20], [30, 40]], names: [:height, :width])\n    iex> Nx.dot(t1, [0], [], t2, [0], [])\n    #Nx.Tensor \n    iex> Nx.dot(t1, [0], [], t2, [1], [])\n    #Nx.Tensor \n    iex> Nx.dot(t1, [1], [], t2, [0], [])\n    #Nx.Tensor \n    iex> Nx.dot(t1, [1], [], t2, [1], [])\n    #Nx.Tensor \n    iex> Nx.dot(t1, [0, 1], [], t2, [0, 1], [])\n    #Nx.Tensor \n\nIf no axes are given, it works like `outer/2`:\n\n    iex> t1 = Nx.tensor([[1, 2], [3, 4]])\n    iex> t2 = Nx.tensor([[10, 20], [30, 40]])\n    iex> Nx.dot(t1, [], [], t2, [], [])\n    #Nx.Tensor \n\n#","ref":"Nx.html#dot/6-contracting-along-axes"},{"type":"function","title":"Dot product between two batched tensors - Nx.dot/6","doc":"iex> u = Nx.tensor([[[1]], [[2]]])\n    iex> v = Nx.tensor([[[3]], [[4]]])\n    iex> Nx.dot(u, [2], [0], v, [2], [0])\n    #Nx.Tensor \n\n    iex> u = Nx.tensor([[[1, 1]], [[2, 2]]])\n    iex> v = Nx.tensor([[[3], [3]], [[4], [4]]])\n    iex> Nx.dot(u, [2], [0], v, [1], [0])\n    #Nx.Tensor","ref":"Nx.html#dot/6-dot-product-between-two-batched-tensors"},{"type":"function","title":"Vectorized tensors - Nx.dot/6","doc":"If you already have vectorized axes, they will be automatically\nadded to the batched axes of `dot/6`. Input axes must refer to\nthe tensor shape, and offsets due to vectorized axes are\nhandled internally.\n\nRewriting the previous example with vectorization:\n\n    iex> u = Nx.tensor([[[1, 1]], [[2, 2]]]) |> Nx.vectorize(:x)\n    iex> v = Nx.tensor([[[3], [3]], [[4], [4]]]) |> Nx.vectorize(:x)\n    iex> Nx.dot(u, [1], [], v, [0], []) # note that axes refer to the inner shapes\n    #Nx.Tensor \n\n  Because the batch axes are now empty, we can use `dot/4` to be more concise.\n\n    Nx.dot(u, [1], v, [0])\n\n  However, we can go even further. Since we are contracting the last axis of\n  `u` with the first axis of `v`, we can rely on `dot/2` to achieve the same\n  result.\n\n      Nx.dot(u, v)","ref":"Nx.html#dot/6-vectorized-tensors"},{"type":"function","title":"Error cases - Nx.dot/6","doc":"iex> u = Nx.tensor([[[1, 1]], [[2, 2]]])\n    iex> v = Nx.tensor([[[3], [3]], [[4], [4]]])\n    iex> Nx.dot(u, [2], [0], v, [1], [])\n    ** (ArgumentError) right tensor must be batched if left tensor is batched\n\n    iex> u = Nx.tensor([[[1, 1]], [[2, 2]]])\n    iex> v = Nx.tensor([[[3], [3]], [[4], [4]]])\n    iex> Nx.dot(u, [2], [], v, [1], [0])\n    ** (ArgumentError) left tensor must be batched if right tensor is batched\n\n    iex> u = Nx.tensor([[[1, 1]], [[2, 2]]])\n    iex> v = Nx.tensor([[[3], [3]], [[4], [4]]])\n    iex> Nx.dot(u, [2], [1], v, [1], [0])\n    ** (ArgumentError) invalid dot batch axis for the left tensor, batch axes must be successive dimensions starting from 0, got [1]\n\n    iex> u = Nx.tensor([[[1, 1]], [[2, 2]]])\n    iex> v = Nx.tensor([[[3], [3]], [[4], [4]]])\n    iex> Nx.dot(u, [2], [0], v, [1], [1])\n    ** (ArgumentError) invalid dot batch axis for the right tensor, batch axes must be successive dimensions starting from 0, got [1]\n\n    iex> u = Nx.tensor([[[1, 1]], [[2, 2]]])\n    iex> v = Nx.tensor([[[3], [3]], [[4], [4]]])\n    iex> Nx.dot(u, [0], [0], v, [1], [0])\n    ** (ArgumentError) dot batch axes for left tensor ([0]) cannot be in contract axes ([0])\n\n    iex> u = Nx.tensor([[[1, 1]], [[2, 2]]])\n    iex> v = Nx.tensor([[[3], [3]], [[4], [4]]])\n    iex> Nx.dot(u, [2], [0], v, [0], [0])\n    ** (ArgumentError) dot batch axes for right tensor ([0]) cannot be in contract axes ([0])","ref":"Nx.html#dot/6-error-cases"},{"type":"function","title":"Nx.equal/2","doc":"Element-wise equality comparison of two tensors.\n\nIf a number is given, it is converted to a tensor.\n\nIt will broadcast tensors whenever the dimensions do\nnot match and broadcasting is possible.\n\nIf you're using `Nx.Defn.defn/2`, you can use the `==` operator\nin place of this function: `left == right`.","ref":"Nx.html#equal/2"},{"type":"function","title":"Examples - Nx.equal/2","doc":"#","ref":"Nx.html#equal/2-examples"},{"type":"function","title":"Comparison of scalars - Nx.equal/2","doc":"iex> Nx.equal(1, 2)\n    #Nx.Tensor \n\n#","ref":"Nx.html#equal/2-comparison-of-scalars"},{"type":"function","title":"Comparison of tensors and scalars - Nx.equal/2","doc":"iex> Nx.equal(1, Nx.tensor([1, 2, 3], names: [:data]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#equal/2-comparison-of-tensors-and-scalars"},{"type":"function","title":"Comparison of tensors - Nx.equal/2","doc":"iex> left = Nx.tensor([1, 2, 3], names: [:data])\n    iex> right = Nx.tensor([1, 2, 5])\n    iex> Nx.equal(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([[1.0, 2.0, 3.0], [4.0, 5.0, 6.0]], names: [:x, nil])\n    iex> right = Nx.tensor([1, 2, 3])\n    iex> Nx.equal(left, right)\n    #Nx.Tensor","ref":"Nx.html#equal/2-comparison-of-tensors"},{"type":"function","title":"Nx.erf/1","doc":"Calculates the error function of each element in the tensor.\n\nIt is equivalent to:\n\n$$erf(z) = \\frac{2}{\\sqrt{\\pi}} \\int_{0}^{z} e^{-t^2}dt$$","ref":"Nx.html#erf/1"},{"type":"function","title":"Examples - Nx.erf/1","doc":"iex> Nx.erf(1)\n    #Nx.Tensor \n\n    iex> Nx.erf(Nx.tensor([1, 2, 3], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#erf/1-examples"},{"type":"function","title":"Nx.erf_inv/1","doc":"Calculates the inverse error function of each element in the tensor.\n\nIt is equivalent to:\n\n$$erf\\text{\\textunderscore}inv(erf(z)) = z$$","ref":"Nx.html#erf_inv/1"},{"type":"function","title":"Examples - Nx.erf_inv/1","doc":"iex> Nx.erf_inv(0.10000000149011612)\n    #Nx.Tensor \n\n    iex> Nx.erf_inv(Nx.tensor([0.10000000149011612, 0.5, 0.8999999761581421], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#erf_inv/1-examples"},{"type":"function","title":"Nx.erfc/1","doc":"Calculates the one minus error function of each element in the tensor.\n\nIt is equivalent to:\n\n$$erfc(z) = 1 - erf(z)$$","ref":"Nx.html#erfc/1"},{"type":"function","title":"Examples - Nx.erfc/1","doc":"iex> Nx.erfc(1)\n    #Nx.Tensor \n\n    iex> Nx.erfc(Nx.tensor([1, 2, 3], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#erfc/1-examples"},{"type":"function","title":"Nx.exp/1","doc":"Calculates the exponential of each element in the tensor.\n\nIt is equivalent to:\n\n$$exp(z) = e^z$$","ref":"Nx.html#exp/1"},{"type":"function","title":"Examples - Nx.exp/1","doc":"iex> Nx.exp(1)\n    #Nx.Tensor \n\n    iex> Nx.exp(Nx.tensor([1, 2, 3], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#exp/1-examples"},{"type":"function","title":"Nx.expm1/1","doc":"Calculates the exponential minus one of each element in the tensor.\n\nIt is equivalent to:\n\n$$expm1(z) = e^z - 1$$","ref":"Nx.html#expm1/1"},{"type":"function","title":"Examples - Nx.expm1/1","doc":"iex> Nx.expm1(1)\n    #Nx.Tensor \n\n    iex> Nx.expm1(Nx.tensor([1, 2, 3], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#expm1/1-examples"},{"type":"function","title":"Nx.eye/2","doc":"Creates the identity matrix of size `n`.","ref":"Nx.html#eye/2"},{"type":"function","title":"Options - Nx.eye/2","doc":"* `:type` - the type of the tensor\n\n  * `:names` - the names of the tensor dimensions\n\n  * `:backend` - the backend to allocate the tensor on. It is either\n    an atom or a tuple in the shape `{backend, options}`. This option\n    is ignored inside `defn`\n\n  * `:vectorized_axes` - a keyword list of `axis_name: axis_size`.\n    If given, the resulting tensor will be vectorized accordingly.\n    Vectorization is not supported via tensor inputs.","ref":"Nx.html#eye/2-options"},{"type":"function","title":"Examples - Nx.eye/2","doc":"iex> Nx.eye(2)\n    #Nx.Tensor \n\n    iex> Nx.eye(3, type: :f32, names: [:height, :width])\n    #Nx.Tensor \n\nThe first argument can also be a shape of a matrix:\n\n    iex> Nx.eye({1, 2})\n    #Nx.Tensor \n\nThe shape can also represent a tensor batch. In this case,\nthe last two axes will represent the same identity matrix.\n\n    iex> Nx.eye({2, 4, 3})\n    #Nx.Tensor","ref":"Nx.html#eye/2-examples"},{"type":"function","title":"Vectorized tensors - Nx.eye/2","doc":"If given, vectorized axes, are added as leading dimensions to the tensor,\neffectively broadcasting the base shape along them.\n\n    iex> Nx.eye({3}, vectorized_axes: [x: 1, y: 2])\n    #Nx.Tensor \n\n    iex> Nx.eye({2, 3}, vectorized_axes: [x: 2])\n    #Nx.Tensor","ref":"Nx.html#eye/2-vectorized-tensors"},{"type":"function","title":"Nx.f16/1","doc":"Short-hand function for creating tensor of type `f16`.\n\nThis is just an alias for `Nx.tensor(tensor, type: f16)`.","ref":"Nx.html#f16/1"},{"type":"function","title":"Nx.f32/1","doc":"Short-hand function for creating tensor of type `f32`.\n\nThis is just an alias for `Nx.tensor(tensor, type: f32)`.","ref":"Nx.html#f32/1"},{"type":"function","title":"Nx.f64/1","doc":"Short-hand function for creating tensor of type `f64`.\n\nThis is just an alias for `Nx.tensor(tensor, type: f64)`.","ref":"Nx.html#f64/1"},{"type":"function","title":"Nx.fft/2","doc":"Calculates the DFT of the given tensor.","ref":"Nx.html#fft/2"},{"type":"function","title":"Options - Nx.fft/2","doc":"* `:eps` - Threshold which backends can use for cleaning-up results. Defaults to `1.0e-10`.\n  * `:length` - Either a positive integer or `:power_of_two`. Will pad or slice the tensor\n    accordingly. `:power_of_two` will automatically pad to the next power of two.","ref":"Nx.html#fft/2-options"},{"type":"function","title":"Examples - Nx.fft/2","doc":"iex> Nx.fft(Nx.tensor([1, 1, 0, 0]))\n    #Nx.Tensor \n\n    iex> Nx.fft(Nx.tensor([1, 1, 1, 0, 1, 1]))\n    #Nx.Tensor \n\nPadding and slicing can be introduced through `:length`:\n\n    iex> Nx.fft(Nx.tensor([1, 1]), length: 4)\n    #Nx.Tensor \n\n    iex> Nx.fft(Nx.tensor([1, 1, 0]), length: :power_of_two)\n    #Nx.Tensor \n\n    iex> Nx.fft(Nx.tensor([1, 1, 0, 0, 2, 3]), length: 4)\n    #Nx.Tensor \n\nIf an N-dimensional tensor is passed, the DFT is applied to its last axis:\n\n    iex> Nx.fft(Nx.tensor([[1, 1, 0, 0, 2, 3], [1, 0, 0, 0, 2, 3]]), length: 4)\n    #Nx.Tensor","ref":"Nx.html#fft/2-examples"},{"type":"function","title":"Vectorized tensors - Nx.fft/2","doc":"Vectorized tensors work the same as N-dimensional tensors\n\n    iex> tensor = Nx.tensor([[1, 1, 0, 0, 2, 3], [1, 0, 0, 0, 2, 3]]) |> Nx.vectorize(:x)\n    iex> Nx.fft(tensor, length: 4)\n    #Nx.Tensor","ref":"Nx.html#fft/2-vectorized-tensors"},{"type":"function","title":"Error Cases - Nx.fft/2","doc":"iex> Nx.fft(Nx.tensor([1, 1]), length: :invalid)\n    ** (RuntimeError) expected an integer or :power_of_two as length, got: :invalid","ref":"Nx.html#fft/2-error-cases"},{"type":"function","title":"Nx.flat_size/1","doc":"Returns the number of elements in the tensor (including vectorized axes).\n\nSee also: `size/1`","ref":"Nx.html#flat_size/1"},{"type":"function","title":"Examples - Nx.flat_size/1","doc":"iex> Nx.flat_size(Nx.tensor([[1, 2, 3], [4, 5, 6]]))\n    6\n\n    iex> Nx.flat_size(10)\n    1\n\n    iex> t = Nx.iota({4, 3, 2})\n    iex> v1 = Nx.vectorize(t, :x)\n    iex> Nx.flat_size(v1)\n    24\n    iex> Nx.flat_size(Nx.vectorize(v1, :y))\n    24","ref":"Nx.html#flat_size/1-examples"},{"type":"function","title":"Nx.flatten/2","doc":"Flattens a n-dimensional tensor to a 1-dimensional tensor.\n\nFlattening only changes the tensor metadata, it doesn't\ncopy the underlying structure.\n\nFlatten is a destructive operation with respect to names.","ref":"Nx.html#flatten/2"},{"type":"function","title":"Examples - Nx.flatten/2","doc":"iex> t = Nx.iota({2, 2, 2, 2})\n    #Nx.Tensor \n    iex> Nx.flatten(t)\n    #Nx.Tensor \n\nAnd if the tensor is already 1-dimensional:\n\n    iex> t = Nx.iota({16})\n    #Nx.Tensor \n    iex> Nx.flatten(t)\n    #Nx.Tensor \n\nYou may also pass `:axes` to `Nx.flatten/2`, to specify which consecutive\naxes to flatten:\n\n    iex> t = Nx.iota({1, 2, 3})\n    #Nx.Tensor \n    iex> Nx.flatten(t, axes: [1, 2])\n    #Nx.Tensor \n\n`:axes` must be consecutive, otherwise it will raise:\n\n    iex> t = Nx.iota({1, 2, 3})\n    #Nx.Tensor \n    iex> Nx.flatten(t, axes: [0, 2])\n    ** (ArgumentError) flatten axes must be consecutive","ref":"Nx.html#flatten/2-examples"},{"type":"function","title":"Vectorized tensors - Nx.flatten/2","doc":"Only the inner shape is flattened, leaving vectorized axes untouched.\n\n    iex> t = Nx.iota({1, 3, 2, 2}) |> Nx.vectorize(:x) |> Nx.vectorize(:y)\n    iex> Nx.flatten(t)\n    #Nx.Tensor","ref":"Nx.html#flatten/2-vectorized-tensors"},{"type":"function","title":"Nx.floor/1","doc":"Calculates the floor of each element in the tensor.\n\nIf a non-floating tensor is given, it is returned as is.\nIf a floating tensor is given, then we apply the operation,\nbut keep its type.","ref":"Nx.html#floor/1"},{"type":"function","title":"Examples - Nx.floor/1","doc":"iex> Nx.floor(Nx.tensor([-1, 0, 1], names: [:x]))\n    #Nx.Tensor \n\n    iex> Nx.floor(Nx.tensor([-1.5, -0.5, 0.5, 1.5], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#floor/1-examples"},{"type":"function","title":"Nx.from_binary/3","doc":"Creates a one-dimensional tensor from a `binary` with the given `type`.\n\nIf the binary size does not match its type, an error is raised.","ref":"Nx.html#from_binary/3"},{"type":"function","title":"Examples - Nx.from_binary/3","doc":"iex> Nx.from_binary( >, :s8)\n    #Nx.Tensor \n\nThe atom notation for types is also supported:\n\n    iex> Nx.from_binary( >, :f64)\n    #Nx.Tensor \n\nAn error is raised for incompatible sizes:\n\n    iex> Nx.from_binary( >, :f64)\n    ** (ArgumentError) binary does not match the given size","ref":"Nx.html#from_binary/3-examples"},{"type":"function","title":"Options - Nx.from_binary/3","doc":"* `:backend` - the backend to allocate the tensor on. It is either\n    an atom or a tuple in the shape `{backend, options}`. This option\n    is ignored inside `defn`","ref":"Nx.html#from_binary/3-options"},{"type":"function","title":"Nx.gather/3","doc":"Builds a new tensor by taking individual values from the original\ntensor at the given indices.","ref":"Nx.html#gather/3"},{"type":"function","title":"Options - Nx.gather/3","doc":"* `:axes` - controls which dimensions the indexes apply to.\n    It must be a sorted list of axes and be of the same size\n    as the second (last) dimension of the indexes tensor.\n    It defaults to the leading axes of the tensor.","ref":"Nx.html#gather/3-options"},{"type":"function","title":"Examples - Nx.gather/3","doc":"#","ref":"Nx.html#gather/3-examples"},{"type":"function","title":"Gathering scalars - Nx.gather/3","doc":"iex> t = Nx.tensor([[1, 2], [3, 4]])\n    iex> Nx.gather(t, Nx.tensor([[1, 1], [0, 1], [1, 0]]))\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[1, 2], [3, 4]])\n    iex> Nx.gather(t, Nx.tensor([[[1, 1], [0, 0]], [[1, 0], [0, 1]]]))\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[1, 2], [11, 12]], [[101, 102], [111, 112]]])\n    iex> Nx.gather(t, Nx.tensor([[0, 0, 0], [0, 1, 1], [1, 1, 1]]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#gather/3-gathering-scalars"},{"type":"function","title":"Gathering subsets - Nx.gather/3","doc":"iex> t = Nx.tensor([[1, 2, 3], [3, 4, 5]])\n    iex> Nx.gather(t, Nx.tensor([[1], [0], [1]]))\n    #Nx.Tensor \n\nThe `:axes` option controls which dimensions the indexes point to,\nthis can be useful, for example, to access columns instead of rows:\n\n    iex> t = Nx.tensor([[[1, 2, 3]], [[4, 5, 6]]])\n    iex> Nx.gather(t, Nx.tensor([[1], [0]]), axes: [2])\n    #Nx.Tensor","ref":"Nx.html#gather/3-gathering-subsets"},{"type":"function","title":"Vectorized tensors - Nx.gather/3","doc":"`tensor` and `indices` have their vectorized axes broadcast together,\nand then the operation takes place normally, with `:axis` and `indices`\nhaving their values in reference to the input shape.\n\n    iex> t = Nx.tensor([[[1, 2], [11, 12]], [[101, 102], [111, 112]]]) |> Nx.vectorize(:x)\n    iex> idx = Nx.tensor([[[0, 0], [0, 1]], [[1, 0], [1, 1]]]) |> Nx.vectorize(:x)\n    iex> Nx.gather(t, idx)\n    #Nx.Tensor \n\nAnd with vectorized broadcasting:\n\n    iex> t = Nx.tensor([[[1, 2], [11, 12]], [[101, 102], [111, 112]]]) |> Nx.vectorize(:x)\n    iex> idx = Nx.tensor([[[0, 0], [0, 1]], [[1, 0], [1, 1]]]) |> Nx.vectorize(:y)\n    iex> Nx.gather(t, idx)\n    #Nx.Tensor","ref":"Nx.html#gather/3-vectorized-tensors"},{"type":"function","title":"Error cases - Nx.gather/3","doc":"iex> Nx.gather(Nx.tensor([[1, 2], [3, 4]]), Nx.tensor([[0, 0]], type: :f32))\n    ** (ArgumentError) indices must be an integer tensor, got {:f, 32}","ref":"Nx.html#gather/3-error-cases"},{"type":"function","title":"Nx.global_default_backend/1","doc":"Sets the default backend globally.\n\nYou must avoid calling this function at runtime. It is mostly\nuseful during scripts or code notebooks to set a default.\n\nIf you need to configure a global default backend in your\napplications, it is generally preferred to do so in your\n`config/*.exs` files:\n\n    config :nx, :default_backend, {EXLA.Backend, []}\n\nIn your notebooks and on `Mix.install/2`, you might:\n\n    Mix.install(\n      [\n        {:nx, \">= 0.0.0\"}\n      ],\n      config: [nx: [default_backend: {EXLA.Backend, device: :cuda}]]\n    )\n\nThe function returns the value that was previously set as global backend.","ref":"Nx.html#global_default_backend/1"},{"type":"function","title":"Nx.greater/2","doc":"Element-wise greater than comparison of two tensors.\n\nIf a number is given, it is converted to a tensor.\n\nIt will broadcast tensors whenever the dimensions do\nnot match and broadcasting is possible.\n\nIf you're using `Nx.Defn.defn/2`, you can use the `>` operator\nin place of this function: `left > right`.","ref":"Nx.html#greater/2"},{"type":"function","title":"Examples - Nx.greater/2","doc":"#","ref":"Nx.html#greater/2-examples"},{"type":"function","title":"Comparison of scalars - Nx.greater/2","doc":"iex> Nx.greater(1, 2)\n    #Nx.Tensor \n\n#","ref":"Nx.html#greater/2-comparison-of-scalars"},{"type":"function","title":"Comparison of tensors and scalars - Nx.greater/2","doc":"iex> Nx.greater(1, Nx.tensor([1, 2, 3], names: [:data]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#greater/2-comparison-of-tensors-and-scalars"},{"type":"function","title":"Comparison of tensors - Nx.greater/2","doc":"iex> left = Nx.tensor([1, 2, 3], names: [:data])\n    iex> right = Nx.tensor([1, 2, 2])\n    iex> Nx.greater(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([[1.0, 2.0, 3.0], [4.0, 5.0, 6.0]], names: [:x, :y])\n    iex> right = Nx.tensor([1, 2, 3])\n    iex> Nx.greater(left, right)\n    #Nx.Tensor","ref":"Nx.html#greater/2-comparison-of-tensors"},{"type":"function","title":"Nx.greater_equal/2","doc":"Element-wise greater than or equal comparison of two tensors.\n\nIf a number is given, it is converted to a tensor.\n\nIt will broadcast tensors whenever the dimensions do\nnot match and broadcasting is possible.\n\nIf you're using `Nx.Defn.defn/2`, you can use the `>=` operator\nin place of this function: `left >= right`.","ref":"Nx.html#greater_equal/2"},{"type":"function","title":"Examples - Nx.greater_equal/2","doc":"#","ref":"Nx.html#greater_equal/2-examples"},{"type":"function","title":"Comparison of scalars - Nx.greater_equal/2","doc":"iex> Nx.greater_equal(1, 2)\n    #Nx.Tensor \n\n#","ref":"Nx.html#greater_equal/2-comparison-of-scalars"},{"type":"function","title":"Comparison of tensors and scalars - Nx.greater_equal/2","doc":"iex> Nx.greater_equal(1, Nx.tensor([1, 2, 3], names: [:data]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#greater_equal/2-comparison-of-tensors-and-scalars"},{"type":"function","title":"Comparison of tensors - Nx.greater_equal/2","doc":"iex> left = Nx.tensor([1, 2, 3], names: [:data])\n    iex> right = Nx.tensor([1, 2, 2])\n    iex> Nx.greater_equal(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([[1.0, 2.0, 3.0], [4.0, 5.0, 6.0]], names: [:x, :y])\n    iex> right = Nx.tensor([1, 2, 3])\n    iex> Nx.greater_equal(left, right)\n    #Nx.Tensor","ref":"Nx.html#greater_equal/2-comparison-of-tensors"},{"type":"function","title":"Nx.ifft/2","doc":"Calculates the Inverse DFT of the given tensor.","ref":"Nx.html#ifft/2"},{"type":"function","title":"Options - Nx.ifft/2","doc":"* `:eps` - Threshold which backends can use for cleaning-up results. Defaults to `1.0e-10`.\n  * `:length` - Either a positive integer or `:power_of_two`. Will pad or slice the tensor\n    accordingly. `:power_of_two` will automatically pad to the next power of two.","ref":"Nx.html#ifft/2-options"},{"type":"function","title":"Examples - Nx.ifft/2","doc":"iex> Nx.ifft(Nx.tensor([2, Complex.new(1, -1), 0, Complex.new(1, 1)]))\n    #Nx.Tensor \n\n    iex> Nx.ifft(Nx.tensor([5, 1, -1, 1, -1, 1]))\n    #Nx.Tensor \n\nPadding and slicing can be introduced through `:length`:\n\n    iex> Nx.ifft(Nx.tensor([1, 1]), length: 4)\n    #Nx.Tensor \n\n    iex> Nx.ifft(Nx.tensor([1, 1, 0]), length: :power_of_two)\n    #Nx.Tensor \n\n    iex> Nx.ifft(Nx.tensor([1, 1, 0, 0, 2, 3]), length: 4)\n    #Nx.Tensor \n\nIf an N-dimensional tensor is passed, the Inverse DFT is applied to its last axis:\n\n    iex> Nx.ifft(Nx.tensor([[1, 1, 0, 0, 2, 3], [1, 0, 0, 0, 2, 3]]), length: 4)\n    #Nx.Tensor","ref":"Nx.html#ifft/2-examples"},{"type":"function","title":"Vectorized tensors - Nx.ifft/2","doc":"Vectorized tensors work the same as N-dimensional tensors\n\n    iex> tensor = Nx.tensor([[1, 1, 0, 0, 2, 3], [1, 0, 0, 0, 2, 3]]) |> Nx.vectorize(:x)\n    iex> Nx.ifft(tensor, length: 4)\n    #Nx.Tensor","ref":"Nx.html#ifft/2-vectorized-tensors"},{"type":"function","title":"Error Cases - Nx.ifft/2","doc":"iex> Nx.ifft(Nx.tensor([1, 1]), length: :invalid)\n    ** (RuntimeError) expected an integer or :power_of_two as length, got: :invalid","ref":"Nx.html#ifft/2-error-cases"},{"type":"function","title":"Nx.imag/1","doc":"Returns the imaginary component of each entry in a complex tensor\nas a floating point tensor.","ref":"Nx.html#imag/1"},{"type":"function","title":"Examples - Nx.imag/1","doc":"iex> Nx.imag(Complex.new(1, 2))\n    #Nx.Tensor \n\n    iex> Nx.imag(Nx.tensor(1))\n    #Nx.Tensor \n\n    iex> Nx.imag(Nx.tensor(1, type: :bf16))\n    #Nx.Tensor \n\n    iex> Nx.imag(Nx.tensor([Complex.new(1, 2), Complex.new(2, -4)]))\n    #Nx.Tensor","ref":"Nx.html#imag/1-examples"},{"type":"function","title":"Nx.indexed_add/4","doc":"Performs an indexed `add` operation on the `target` tensor,\nadding the `updates` into the corresponding `indices` positions.\n\nThis operation is the grad for `gather/2` and gather-like operations such as\n`take/3` and `take_along_axis/3`.\n\n`indices` must be a fully qualified tensor of shape `{n, Nx.rank(target)}`, with `n`\nbeing an arbitrary number of indices, while `updates` must have a compatible `{n}` shape.\n\nSee also: `indexed_add/3`, `gather/2`, `take/3`, `take_along_axis/3`","ref":"Nx.html#indexed_add/4"},{"type":"function","title":"Options - Nx.indexed_add/4","doc":"* `:axes` - controls which dimensions the indexes apply to.\n    It must be a sorted list of axes and be of the same size\n    as the second (last) dimension of the indexes tensor.\n    It defaults to the leading axes of the tensor.","ref":"Nx.html#indexed_add/4-options"},{"type":"function","title":"Examples - Nx.indexed_add/4","doc":"#","ref":"Nx.html#indexed_add/4-examples"},{"type":"function","title":"Adding a single entry as a scalar - Nx.indexed_add/4","doc":"As a shorthand notation, rank-1 indices can be used for updating a single entry:\n\n      iex> Nx.indexed_add(Nx.tensor([[1], [2]]), Nx.tensor([1, 0]), 8)\n      #Nx.Tensor \n\n#","ref":"Nx.html#indexed_add/4-adding-a-single-entry-as-a-scalar"},{"type":"function","title":"Adding multiple scalar entries - Nx.indexed_add/4","doc":"iex> t = Nx.iota({1, 2, 3})\n    #Nx.Tensor \n    iex> indices = Nx.tensor([[0, 0, 0], [0, 1, 1], [0, 0, 0], [0, 0, 2], [0, 1, 2]])\n    iex> updates = Nx.tensor([1, 3, 1, -2, 5])\n    iex> Nx.indexed_add(t, indices, updates)\n    #Nx.Tensor \n\n#","ref":"Nx.html#indexed_add/4-adding-multiple-scalar-entries"},{"type":"function","title":"Type promotions - Nx.indexed_add/4","doc":"Type promotions should happen automatically, with the resulting type being the combination\nof the `target` type and the `updates` type.\n\n    iex> Nx.indexed_add(Nx.tensor([1.0]), Nx.tensor([[0], [0]]), Nx.tensor([1, 1]))\n    #Nx.Tensor \n\n    iex> Nx.indexed_add(Nx.tensor([1]), Nx.tensor([[0], [0]]), Nx.tensor([1.0, 1.0]))\n    #Nx.Tensor \n\n    iex> Nx.indexed_add(Nx.tensor([1], type: :s32), Nx.tensor([[0], [0]]), Nx.tensor([1, 1], type: :s64))\n    #Nx.Tensor","ref":"Nx.html#indexed_add/4-type-promotions"},{"type":"function","title":"Vectorized tensors - Nx.indexed_add/4","doc":"All of the inputs can be vectorized. The function will broadcast along the vectorized axes\nbefore calculating the results.\n\n    iex> x = Nx.tensor([[0, 10], [10, 20]]) |> Nx.vectorize(:x)\n    iex> idx = Nx.tensor([[[0], [0]], [[0], [1]], [[1], [1]]]) |> Nx.vectorize(:y)\n    iex> Nx.indexed_add(x, idx, Nx.tensor([1, 1]))\n    #Nx.Tensor","ref":"Nx.html#indexed_add/4-vectorized-tensors"},{"type":"function","title":"Error cases - Nx.indexed_add/4","doc":"iex> Nx.indexed_add(Nx.tensor([[1], [2]]), Nx.tensor([[[1, 2, 3]]]), Nx.tensor([0]))\n    ** (ArgumentError) indices must be a rank 1 or 2 tensor, got: 3\n\n    iex> Nx.indexed_add(Nx.tensor([[1], [2]]), Nx.tensor([[1, 2]]), Nx.tensor([0, 1]))\n    ** (ArgumentError) expected the leading axis of indices ({1, 2}) and leading axis of updates ({2}) to match\n\n    iex> Nx.indexed_add(Nx.tensor([[1, 2, 3]]), Nx.tensor([[0]]), Nx.tensor([[1, 2, 3, 4, 5]]))\n    ** (ArgumentError) axis (1) of updates ({1, 5}) must be less than or equal to the axis (1) of {1, 3})","ref":"Nx.html#indexed_add/4-error-cases"},{"type":"function","title":"Nx.indexed_put/4","doc":"Puts individual values from `updates` into the given tensor at the corresponding `indices`.\n\n`indices` must be a fully qualified tensor of shape `{n, i}`, with `n` being an arbitrary\nnumber of indices, while `updates` must have a compatible `{n, ...j}` shape, such that\n`i + j = rank(tensor)`.\n\nIn case of repeating indices, the result is non-determinstic, since the operation happens\nin parallel when running on devices such as the GPU.\n\nSee also: `indexed_add/3`, `put_slice/3`.","ref":"Nx.html#indexed_put/4"},{"type":"function","title":"Options - Nx.indexed_put/4","doc":"* `:axes` - controls which dimensions the indexes apply to.\n    It must be a sorted list of axes and be of the same size\n    as the second (last) dimension of the indexes tensor.\n    It defaults to the leading axes of the tensor.","ref":"Nx.html#indexed_put/4-options"},{"type":"function","title":"Examples - Nx.indexed_put/4","doc":"#","ref":"Nx.html#indexed_put/4-examples"},{"type":"function","title":"Storing a single entry as a scalar - Nx.indexed_put/4","doc":"As a shorthand notation, rank-1 indices can be used for updating a single entry:\n\n    iex> Nx.indexed_put(Nx.tensor([[1], [2]]), Nx.tensor([1, 0]), 10)\n    #Nx.Tensor \n\n#","ref":"Nx.html#indexed_put/4-storing-a-single-entry-as-a-scalar"},{"type":"function","title":"Storing multiple scalar entries scalars - Nx.indexed_put/4","doc":"iex> Nx.indexed_put(Nx.tensor([0, 0, 0]), Nx.tensor([[1], [2]]), Nx.tensor([2, 4]))\n    #Nx.Tensor \n\n    iex> Nx.indexed_put(Nx.tensor([0, 0, 0]), Nx.tensor([[1], [2]]), Nx.tensor([3, 4]))\n    #Nx.Tensor \n\n    iex> t = Nx.iota({1, 2, 3})\n    #Nx.Tensor \n    iex> indices = Nx.tensor([[0, 0, 0], [0, 1, 1], [0, 0, 2]])\n    iex> updates = Nx.tensor([1, 3, -2])\n    iex> Nx.indexed_put(t, indices, updates)\n    #Nx.Tensor \n\n#","ref":"Nx.html#indexed_put/4-storing-multiple-scalar-entries-scalars"},{"type":"function","title":"Storing non-scalars on a given dimension - Nx.indexed_put/4","doc":"iex> t = Nx.iota({1, 3, 2})\n      #Nx.Tensor \n      iex> indices = Nx.tensor([[0, 0], [0, 2]])\n      iex> updates = Nx.tensor([[0, 10], [40, 50]])\n      iex> Nx.indexed_put(t, indices, updates)\n      #Nx.Tensor \n\nThe `:axes` option controls which dimensions the indexes apply to.\nIt must be a sorted list of axes. All non-listed axes are taken as\nthe update dimensions:\n\n    iex> t = Nx.iota({1, 2, 3})\n    #Nx.Tensor \n    iex> indices = Nx.tensor([[0, 0], [0, 2]])\n    iex> updates = Nx.tensor([[0, 30], [20, 50]])\n    iex> Nx.indexed_put(t, indices, updates, axes: [0, 2])\n    #Nx.Tensor \n\n#","ref":"Nx.html#indexed_put/4-storing-non-scalars-on-a-given-dimension"},{"type":"function","title":"Type promotions - Nx.indexed_put/4","doc":"Type promotions should happen automatically, with the resulting type being the combination\nof the `target` type and the `updates` type.\n\n    iex> Nx.indexed_put(Nx.tensor([1.0]), Nx.tensor([[0]]), Nx.tensor([3]))\n    #Nx.Tensor \n\n    iex> Nx.indexed_put(Nx.tensor([1]), Nx.tensor([[0]]), Nx.tensor([3.0]))\n    #Nx.Tensor \n\n    iex> Nx.indexed_put(Nx.tensor([1], type: :s32), Nx.tensor([[0]]), Nx.tensor([3], type: :s64))\n    #Nx.Tensor","ref":"Nx.html#indexed_put/4-type-promotions"},{"type":"function","title":"Vectorized tensors - Nx.indexed_put/4","doc":"All of the inputs can be vectorized. The function will broadcast along the vectorized axes\nbefore calculating the results.\n\n    iex> x = Nx.tensor([[0, 10], [10, 20]]) |> Nx.vectorize(:x)\n    iex> idx = Nx.tensor([[[0], [0]], [[0], [1]], [[1], [1]]]) |> Nx.vectorize(:y)\n    iex> Nx.indexed_put(x, idx, Nx.tensor([1, 1]))\n    #Nx.Tensor","ref":"Nx.html#indexed_put/4-vectorized-tensors"},{"type":"function","title":"Error cases - Nx.indexed_put/4","doc":"iex> Nx.indexed_put(Nx.tensor([[1], [2]]), Nx.tensor([[[1, 2, 3]]]), Nx.tensor([0]))\n    ** (ArgumentError) indices must be a rank 1 or 2 tensor, got: 3\n\n    iex> Nx.indexed_put(Nx.tensor([[1], [2]]), Nx.tensor([[1, 2]]), Nx.tensor([0, 1]))\n    ** (ArgumentError) expected the leading axis of indices ({1, 2}) and leading axis of updates ({2}) to match\n\n    iex> Nx.indexed_put(Nx.iota({1, 2, 3}), Nx.tensor([[0, 1, 2]]), 10, axes: [1, 0, 2])\n    ** (ArgumentError) :axes must be an ordered list","ref":"Nx.html#indexed_put/4-error-cases"},{"type":"function","title":"Nx.iota/2","doc":"Creates a tensor with the given shape which increments\nalong the provided axis. You may optionally provide dimension\nnames.\n\nIf no axis is provided, index counts up at each element.\n\nIf a tensor or a number are given, the shape and names are taken from the tensor.","ref":"Nx.html#iota/2"},{"type":"function","title":"Options - Nx.iota/2","doc":"* `:type` - the type of the tensor\n\n  * `:axis` - an axis to repeat the iota over\n\n  * `:names` - the names of the tensor dimensions\n\n  * `:backend` - the backend to allocate the tensor on. It is either\n    an atom or a tuple in the shape `{backend, options}`. This option\n    is ignored inside `defn`\n\n  * `:vectorized_axes` - a keyword list of `axis_name: axis_size`.\n    If given, the resulting tensor will be vectorized accordingly.\n    Vectorization is not supported via tensor inputs.","ref":"Nx.html#iota/2-options"},{"type":"function","title":"Examples - Nx.iota/2","doc":"iex> Nx.iota({})\n    #Nx.Tensor \n\n    iex> Nx.iota({5})\n    #Nx.Tensor \n\n    iex> Nx.iota({3, 2, 3}, names: [:batch, :height, :width])\n    #Nx.Tensor \n\n    iex> Nx.iota({3, 3}, axis: 1, names: [:batch, nil])\n    #Nx.Tensor \n\n    iex> Nx.iota({3, 3}, axis: -1)\n    #Nx.Tensor \n\n    iex> Nx.iota({3, 4, 3}, axis: 0, type: :f64)\n    #Nx.Tensor \n\n    iex> Nx.iota({1, 3, 2}, axis: 2)\n    #Nx.Tensor \n\n    iex> Nx.iota({2, 3}, axis: 0, vectorized_axes: [x: 1, y: 2])\n    #Nx.Tensor","ref":"Nx.html#iota/2-examples"},{"type":"function","title":"Nx.is_infinity/1","doc":"Determines if each element in `tensor` is `Inf` or `-Inf`.\n\nFor complex tensors, if either of the components is infinity,\nthe entry is deemed infinity as well.","ref":"Nx.html#is_infinity/1"},{"type":"function","title":"Examples - Nx.is_infinity/1","doc":"iex> Nx.is_infinity(Nx.tensor([:infinity, :nan, :neg_infinity, 1, 0]))\n    #Nx.Tensor \n\n    iex> Nx.is_infinity(Nx.tensor([:infinity, 1, Complex.new(0, :infinity), :neg_infinity]))\n    #Nx.Tensor \n\n    iex> Nx.is_infinity(Nx.tensor([1, 0]))\n    #Nx.Tensor","ref":"Nx.html#is_infinity/1-examples"},{"type":"function","title":"Nx.is_nan/1","doc":"Determines if each element in `tensor` is a `NaN`.\n\nFor complex tensors, if either of the components is `NaN`,\nthe entry is deemed `NaN` as well.","ref":"Nx.html#is_nan/1"},{"type":"function","title":"Examples - Nx.is_nan/1","doc":"iex> Nx.is_nan(Nx.tensor([:nan, 1, 0]))\n    #Nx.Tensor \n\n    iex> Nx.is_nan(Nx.tensor([:nan, :infinity, Complex.new(0, :nan)]))\n    #Nx.Tensor \n\n    iex> Nx.is_nan(Nx.tensor([1, 0]))\n    #Nx.Tensor","ref":"Nx.html#is_nan/1-examples"},{"type":"macro","title":"Nx.is_tensor/1","doc":"Checks whether the value is a valid numerical value.\n\nReturns true if the value is a `number`, a non-finite atom (like `:infinity`),\na `Complex` number or an `Nx.Tensor`.\n\nSee also: `t:t/0`","ref":"Nx.html#is_tensor/1"},{"type":"function","title":"Nx.left_shift/2","doc":"Element-wise left shift of two tensors.\n\nOnly integer tensors are supported. If a float or complex\ntensor is given, an error is raised. If the right side\nis negative, it will raise, but it may trigger undefined\nbehaviour on some compilers.\n\nIt will broadcast tensors whenever the dimensions do\nnot match and broadcasting is possible. If the number of\nshifts are negative, Nx's default backend will raise,\nbut it may trigger undefined behaviour in other backends.\n\nIf you're using `Nx.Defn.defn/2`, you can use the `<<<` operator\nin place of this function: `left <<< right`.","ref":"Nx.html#left_shift/2"},{"type":"function","title":"Examples - Nx.left_shift/2","doc":"#","ref":"Nx.html#left_shift/2-examples"},{"type":"function","title":"Left shift between scalars - Nx.left_shift/2","doc":"iex> Nx.left_shift(1, 0)\n    #Nx.Tensor \n\n#","ref":"Nx.html#left_shift/2-left-shift-between-scalars"},{"type":"function","title":"Left shift between tensors and scalars - Nx.left_shift/2","doc":"iex> Nx.left_shift(Nx.tensor([1, 2, 3], names: [:data]), 2)\n    #Nx.Tensor \n\n#","ref":"Nx.html#left_shift/2-left-shift-between-tensors-and-scalars"},{"type":"function","title":"Left shift between tensors - Nx.left_shift/2","doc":"iex> left = Nx.tensor([1, 1, -1, -1], names: [:data])\n    iex> right = Nx.tensor([1, 2, 3, 4], names: [:data])\n    iex> Nx.left_shift(left, right)\n    #Nx.Tensor","ref":"Nx.html#left_shift/2-left-shift-between-tensors"},{"type":"function","title":"Error cases - Nx.left_shift/2","doc":"iex> Nx.left_shift(Nx.tensor([0, 0, 1, 1]), 1.0)\n    ** (ArgumentError) bitwise operators expect integer tensors as inputs and outputs an integer tensor, got: {:f, 32}","ref":"Nx.html#left_shift/2-error-cases"},{"type":"function","title":"Nx.less/2","doc":"Element-wise less than comparison of two tensors.\n\nIf a number is given, it is converted to a tensor.\n\nIt will broadcast tensors whenever the dimensions do\nnot match and broadcasting is possible.\n\nIf you're using `Nx.Defn.defn/2`, you can use the `<` operator\nin place of this function: `left < right`.","ref":"Nx.html#less/2"},{"type":"function","title":"Examples - Nx.less/2","doc":"#","ref":"Nx.html#less/2-examples"},{"type":"function","title":"Comparison of scalars - Nx.less/2","doc":"iex> Nx.less(1, 2)\n    #Nx.Tensor \n\n#","ref":"Nx.html#less/2-comparison-of-scalars"},{"type":"function","title":"Comparison of tensors and scalars - Nx.less/2","doc":"iex> Nx.less(1, Nx.tensor([1, 2, 3], names: [:data]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#less/2-comparison-of-tensors-and-scalars"},{"type":"function","title":"Comparison of tensors - Nx.less/2","doc":"iex> Nx.less(Nx.tensor([1, 2, 1]), Nx.tensor([1, 2, 2], names: [:data]))\n    #Nx.Tensor \n\n    iex> Nx.less(Nx.tensor([[1.0, 2.0, 3.0], [4.0, 2.0, 1.0]], names: [:x, :y]), Nx.tensor([1, 2, 3]))\n    #Nx.Tensor","ref":"Nx.html#less/2-comparison-of-tensors"},{"type":"function","title":"Nx.less_equal/2","doc":"Element-wise less than or equal comparison of two tensors.\n\nIf a number is given, it is converted to a tensor.\n\nIt will broadcast tensors whenever the dimensions do\nnot match and broadcasting is possible.\n\nIf you're using `Nx.Defn.defn/2`, you can use the `<=` operator\nin place of this function: `left <= right`.","ref":"Nx.html#less_equal/2"},{"type":"function","title":"Examples - Nx.less_equal/2","doc":"#","ref":"Nx.html#less_equal/2-examples"},{"type":"function","title":"Comparison of scalars - Nx.less_equal/2","doc":"iex> Nx.less_equal(1, 2)\n    #Nx.Tensor \n\n#","ref":"Nx.html#less_equal/2-comparison-of-scalars"},{"type":"function","title":"Comparison of tensors and scalars - Nx.less_equal/2","doc":"iex> Nx.less_equal(1, Nx.tensor([1, 2, 3], names: [:data]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#less_equal/2-comparison-of-tensors-and-scalars"},{"type":"function","title":"Comparison of tensors - Nx.less_equal/2","doc":"iex> left = Nx.tensor([1, 2, 3], names: [:data])\n    iex> right = Nx.tensor([1, 2, 2])\n    iex> Nx.less_equal(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([[1.0, 2.0, 3.0], [4.0, 5.0, 6.0]])\n    iex> right = Nx.tensor([1, 2, 3], names: [:y])\n    iex> Nx.less_equal(left, right)\n    #Nx.Tensor","ref":"Nx.html#less_equal/2-comparison-of-tensors"},{"type":"function","title":"Nx.linspace/3","doc":"Creates a tensor of shape `{n}` with linearly spaced samples between `start` and `stop`.","ref":"Nx.html#linspace/3"},{"type":"function","title":"Options - Nx.linspace/3","doc":"* `:n` - The number of samples in the tensor.\n  * `:name` - Optional name for the output axis.\n  * `:type` - Optional type for the output. Defaults to `{:f, 32}`\n  * `:endpoint` - Boolean that indicates whether to include `stop`\n    as the last point in the output. Defaults to `true`","ref":"Nx.html#linspace/3-options"},{"type":"function","title":"Examples - Nx.linspace/3","doc":"iex> Nx.linspace(5, 8, n: 5)\n    #Nx.Tensor \n\n    iex> Nx.linspace(0, 10, n: 5, endpoint: false, name: :x)\n    #Nx.Tensor \n\nFor integer types, the results might not be what's expected.\nWhen `endpoint: true` (the default), the step is given by\n`step = (stop - start) / (n - 1)`, which means that instead\nof a step of `3` in the example below, we get a step close to\n`3.42`. The results are calculated first and only cast in the\nend, so that the `:endpoint` condition is respected.\n\n    iex> Nx.linspace(0, 24, n: 8, type: {:u, 8}, endpoint: true)\n    #Nx.Tensor \n\n    iex> Nx.linspace(0, 24, n: 8, type: {:s, 64}, endpoint: false)\n    #Nx.Tensor \n\nOne can also pass two higher order tensors with the same shape `{j, k, ...}`, in which case\nthe output will be of shape `{j, k, ..., n}`.\n\n  iex> Nx.linspace(Nx.tensor([[[0, 10]]]), Nx.tensor([[[10, 100]]]), n: 10, name: :samples, type: {:u, 8})\n  #Nx.Tensor","ref":"Nx.html#linspace/3-examples"},{"type":"function","title":"Vectorized tensors - Nx.linspace/3","doc":"iex> Nx.linspace(0, Nx.vectorize(Nx.tensor([10, 20]), :x), n: 5)\n    #Nx.Tensor \n\n    iex> start = Nx.vectorize(Nx.tensor([0, 1]), :x)\n    iex> stop = Nx.vectorize(Nx.tensor([10, 20]), :y)\n    iex> Nx.linspace(start, stop, n: 5)\n    #Nx.Tensor \n\n    iex> start = Nx.vectorize(Nx.tensor([0, 1]), :x)\n    iex> stop = Nx.vectorize(Nx.tensor([10, 10]), :x)\n    iex> Nx.linspace(start, stop, n: 5)\n    #Nx.Tensor","ref":"Nx.html#linspace/3-vectorized-tensors"},{"type":"function","title":"Error cases - Nx.linspace/3","doc":"iex> Nx.linspace(0, 24, n: 1.0)\n    ** (ArgumentError) expected n to be a non-negative integer, got: 1.0\n\n    iex> Nx.linspace(Nx.tensor([[0, 1]]), Nx.tensor([1, 2, 3]), n: 2)\n    ** (ArgumentError) expected start and stop to have the same shape. Got shapes {1, 2} and {3}","ref":"Nx.html#linspace/3-error-cases"},{"type":"function","title":"Nx.load_numpy!/1","doc":"Loads a `.npy` file into a tensor.\n\nAn `.npy` file stores a single array created from Python's\nNumPy library. This function can be useful for loading data\noriginally created or intended to be loaded from NumPy into\nElixir.\n\nThis function will raise if the archive or any of its contents\nare invalid.\n\nNote: This function cannot be used in `defn`.","ref":"Nx.html#load_numpy!/1"},{"type":"function","title":"Examples - Nx.load_numpy!/1","doc":"\"array.npy\"\n    |> File.read!()\n    |> Nx.load_numpy!()\n    #=>\n    #Nx.Tensor","ref":"Nx.html#load_numpy!/1-examples"},{"type":"function","title":"Nx.load_numpy_archive!/1","doc":"Loads a `.npz` archive into a list of tensors.\n\nAn `.npz` file is a zipped, possibly compressed\narchive containing multiple `.npy` files.\n\nIt returns a list of two elements tuples, where\nthe tensor name is first and the serialized tensor\nis second. The list is returned in the same order\nas in the archive. Use `Map.new/1` afterwards if\nyou want to access the list elements by name.\n\nIt will raise if the archive or any of its contents\nare invalid.\n\nNote: This function cannot be used in `defn`.","ref":"Nx.html#load_numpy_archive!/1"},{"type":"function","title":"Examples - Nx.load_numpy_archive!/1","doc":"\"archive.npz\"\n    |> File.read!()\n    |> Nx.load_numpy_archive!()\n    #=>\n    [\n      {\"foo\",\n       #Nx.Tensor },\n      {\"bar\",\n       #Nx.Tensor }\n    ]","ref":"Nx.html#load_numpy_archive!/1-examples"},{"type":"function","title":"Nx.log1p/1","doc":"Calculates the natural log plus one of each element in the tensor.\n\nIt is equivalent to:\n\n$$log1p(z) = log(z + 1)$$","ref":"Nx.html#log1p/1"},{"type":"function","title":"Examples - Nx.log1p/1","doc":"iex> Nx.log1p(1)\n    #Nx.Tensor \n\n    iex> Nx.log1p(Nx.tensor([1, 2, 3], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#log1p/1-examples"},{"type":"function","title":"Nx.log2/1","doc":"Calculates the element-wise logarithm of a tensor with base 2.","ref":"Nx.html#log2/1"},{"type":"function","title":"Examples - Nx.log2/1","doc":"iex> Nx.log2(2)\n    #Nx.Tensor \n\n    iex> Nx.log2(Nx.tensor([1, 2, 4, 8]))\n    #Nx.Tensor","ref":"Nx.html#log2/1-examples"},{"type":"function","title":"Nx.log10/1","doc":"Calculates the element-wise logarithm of a tensor with base 10.","ref":"Nx.html#log10/1"},{"type":"function","title":"Examples - Nx.log10/1","doc":"iex> Nx.log10(10)\n    #Nx.Tensor \n\n    iex> Nx.log10(Nx.tensor([1, 10, 100, 1000]))\n    #Nx.Tensor","ref":"Nx.html#log10/1-examples"},{"type":"function","title":"Nx.log/1","doc":"Calculates the natural log of each element in the tensor.\n\nIt is equivalent to:\n\n$$log(z) = ln(z),\\quad \\text{if z} \\in \\Reals$$\n\n$$log(z) = ln(r) + i\\theta,\\quad\\text{if }z = re^{i\\theta} \\in \\Complex$$","ref":"Nx.html#log/1"},{"type":"function","title":"Examples - Nx.log/1","doc":"iex> Nx.log(1)\n    #Nx.Tensor \n\n    iex> Nx.log(Nx.tensor([1, 2, 3], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#log/1-examples"},{"type":"function","title":"Nx.log/2","doc":"Calculates the element-wise logarithm of a tensor with given `base`.","ref":"Nx.html#log/2"},{"type":"function","title":"Examples - Nx.log/2","doc":"iex> Nx.log(2, 2)\n    #Nx.Tensor \n\n    iex> Nx.log(Nx.tensor([3, 9, 27, 81]), 3)\n    #Nx.Tensor","ref":"Nx.html#log/2-examples"},{"type":"function","title":"Nx.logical_and/2","doc":"Element-wise logical and of two tensors.\n\nZero is considered false, any other number is considered\ntrue.\n\nIt will broadcast tensors whenever the dimensions do\nnot match and broadcasting is possible.\n\nIf you're using `Nx.Defn.defn/2`, you can use the `and` operator\nin place of this function: `left and right`.","ref":"Nx.html#logical_and/2"},{"type":"function","title":"Examples - Nx.logical_and/2","doc":"iex> Nx.logical_and(1, Nx.tensor([-1, 0, 1], names: [:data]))\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([-1, 0, 1], names: [:data])\n    iex> right = Nx.tensor([[-1], [0], [1]])\n    iex> Nx.logical_and(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([-1.0, 0.0, 1.0], names: [:data])\n    iex> right = Nx.tensor([[-1], [0], [1]])\n    iex> Nx.logical_and(left, right)\n    #Nx.Tensor","ref":"Nx.html#logical_and/2-examples"},{"type":"function","title":"Nx.logical_not/1","doc":"Element-wise logical not a tensor.\n\nZero is considered false, any other number is considered\ntrue.\n\nIf you're using `Nx.Defn.defn/2`, you can use the `not` operator\nin place of this function: `not tensor`.","ref":"Nx.html#logical_not/1"},{"type":"function","title":"Examples - Nx.logical_not/1","doc":"iex> Nx.logical_not(Nx.tensor([-1, 0, 1], names: [:data]))\n    #Nx.Tensor \n\n    iex> Nx.logical_not(Nx.tensor([-1.0, 0.0, 1.0], names: [:data]))\n    #Nx.Tensor","ref":"Nx.html#logical_not/1-examples"},{"type":"function","title":"Nx.logical_or/2","doc":"Element-wise logical or of two tensors.\n\nZero is considered false, any other number is considered\ntrue.\n\nIt will broadcast tensors whenever the dimensions do\nnot match and broadcasting is possible.\n\nIf you're using `Nx.Defn.defn/2`, you can use the `or` operator\nin place of this function: `left or right`.","ref":"Nx.html#logical_or/2"},{"type":"function","title":"Examples - Nx.logical_or/2","doc":"iex> Nx.logical_or(0, Nx.tensor([-1, 0, 1], names: [:data]))\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([-1, 0, 1], names: [:data])\n    iex> right = Nx.tensor([[-1], [0], [1]])\n    iex> Nx.logical_or(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([-1.0, 0.0, 1.0], names: [:data])\n    iex> right = Nx.tensor([[-1], [0], [1]])\n    iex> Nx.logical_or(left, right)\n    #Nx.Tensor","ref":"Nx.html#logical_or/2-examples"},{"type":"function","title":"Nx.logical_xor/2","doc":"Element-wise logical xor of two tensors.\n\nZero is considered false, any other number is considered\ntrue.\n\nIt will broadcast tensors whenever the dimensions do\nnot match and broadcasting is possible.","ref":"Nx.html#logical_xor/2"},{"type":"function","title":"Examples - Nx.logical_xor/2","doc":"iex> Nx.logical_xor(0, Nx.tensor([-1, 0, 1], names: [:data]))\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([-1, 0, 1], names: [:data])\n    iex> right = Nx.tensor([[-1], [0], [1]])\n    iex> Nx.logical_xor(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([-1.0, 0.0, 1.0], names: [:data])\n    iex> right = Nx.tensor([[-1], [0], [1]])\n    iex> Nx.logical_xor(left, right)\n    #Nx.Tensor","ref":"Nx.html#logical_xor/2-examples"},{"type":"function","title":"Nx.logsumexp/2","doc":"Returns the logarithm of the sum of the exponentials of tensor elements.\n\nIf the `:axes` option is given, it aggregates over\nthe given dimensions, effectively removing them.\n`axes: [0]` implies aggregating over the highest order\ndimension and so forth. If the axis is negative, then\ncounts the axis from the back. For example, `axes: [-1]`\nwill always aggregate all rows.\n\nYou may optionally set `:keep_axes` to true, which will\nretain the rank of the input tensor by setting the reduced\naxes to size 1.\n\nExponentials can be scaled before summation by multiplying\nthem with `:exp_scaling_factor` option. It must be of the same shape\nas the input tensor or broadcastable to it.","ref":"Nx.html#logsumexp/2"},{"type":"function","title":"Examples - Nx.logsumexp/2","doc":"iex> Nx.logsumexp(Nx.tensor([1, 2, 3, 4, 5, 6]))\n    #Nx.Tensor \n\n    iex> Nx.logsumexp(Nx.tensor([1, 2, 3, 4, 5, 6]), exp_scaling_factor: 0.5)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([1, 2, 3, 4, 5, 6])\n    iex> a = Nx.tensor([-1, -1, -1, 1, 1, 1])\n    iex> Nx.logsumexp(t, exp_scaling_factor: a)\n    #Nx.Tensor \n\n    iex> Nx.logsumexp(Nx.tensor([[1, 2], [3, 4], [5, 6]]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#logsumexp/2-examples"},{"type":"function","title":"Aggregating over an axis - Nx.logsumexp/2","doc":"iex> t = Nx.tensor([[1, 2], [3, 4], [5, 6]], names: [:x, :y])\n    iex> Nx.logsumexp(t, axes: [:x])\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[1, 2], [3, 4], [5, 6]], names: [:x, :y])\n    iex> Nx.logsumexp(t, axes: [:y])\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[1, 2], [3, 4]], [[5, 6], [7, 8]]], names: [:x, :y, :z])\n    iex> Nx.logsumexp(t, axes: [:x, :z])\n    #Nx.Tensor \n\n#","ref":"Nx.html#logsumexp/2-aggregating-over-an-axis"},{"type":"function","title":"Keeping axes - Nx.logsumexp/2","doc":"iex> t = Nx.tensor([[[1, 2], [3, 4]], [[5, 6], [7, 8]]], names: [:x, :y, :z])\n    iex> Nx.logsumexp(t, axes: [:x, :z], keep_axes: true)\n    #Nx.Tensor \n\n#","ref":"Nx.html#logsumexp/2-keeping-axes"},{"type":"function","title":"Vectorized tensors - Nx.logsumexp/2","doc":"iex> t = Nx.vectorize(Nx.tensor([[1, 2], [3, 4], [5, 6]]), :x)\n    iex> Nx.logsumexp(t, axes: [0], keep_axes: true)\n    #Nx.Tensor","ref":"Nx.html#logsumexp/2-vectorized-tensors"},{"type":"function","title":"Nx.make_diagonal/2","doc":"Creates a diagonal tensor from a 1D tensor.\n\nConverse of `take_diagonal/2`.\n\nThe returned tensor will be a square matrix of dimensions equal\nto the size of the tensor. If an offset is given, the absolute value\nof the offset is added to the matrix dimensions sizes.","ref":"Nx.html#make_diagonal/2"},{"type":"function","title":"Options - Nx.make_diagonal/2","doc":"* `:offset` - offset used for making the diagonal.\n    Use offset > 0 for diagonals above the main diagonal,\n    and offset < 0 for diagonals below the main diagonal.\n    Defaults to 0.","ref":"Nx.html#make_diagonal/2-options"},{"type":"function","title":"Examples - Nx.make_diagonal/2","doc":"Given a 1D tensor:\n\n    iex> Nx.make_diagonal(Nx.tensor([1, 2, 3, 4]))\n    #Nx.Tensor \n\n  Given a 1D tensor with an offset:\n\n    iex> Nx.make_diagonal(Nx.tensor([1, 2, 3]), offset: 1)\n    #Nx.Tensor \n\n    iex> Nx.make_diagonal(Nx.tensor([1, 2, 3]), offset: -1)\n    #Nx.Tensor \n\n  You can also have offsets with an abs greater than the tensor length:\n\n    iex> Nx.make_diagonal(Nx.tensor([1, 2, 3]), offset: -4)\n    #Nx.Tensor \n\n    iex> Nx.make_diagonal(Nx.tensor([1, 2, 3]), offset: 4)\n    #Nx.Tensor \n\n#","ref":"Nx.html#make_diagonal/2-examples"},{"type":"function","title":"Vectorized tensors - Nx.make_diagonal/2","doc":"iex> t = Nx.vectorize(Nx.tensor([[1, 2], [3, 4]]), :x)\n    iex> Nx.make_diagonal(t, offset: 1)\n    #Nx.Tensor \n    iex> Nx.make_diagonal(t, offset: -1)\n    #Nx.Tensor","ref":"Nx.html#make_diagonal/2-vectorized-tensors"},{"type":"function","title":"Error cases - Nx.make_diagonal/2","doc":"iex> Nx.make_diagonal(Nx.tensor([[0, 0], [0, 1]]))\n    ** (ArgumentError) make_diagonal/2 expects tensor of rank 1, got tensor of rank: 2","ref":"Nx.html#make_diagonal/2-error-cases"},{"type":"function","title":"Nx.map/3","doc":"Maps the given scalar function over the entire\ntensor.\n\nThe type of the returned tensor will be of the same type\nas the input tensor, unless the `:type` option is given.\nTherefore, you may need to explicitly cast the tensor to\navoid errors. For example, if you have an integer tensor\nand you convert it to a float, as below, it will fail:\n\n    tensor = Nx.tensor([[1, 2, 3], [4, 5, 6]]),\n    Nx.map(tensor, fn x -> Nx.multiply(x, 1.0) end)\n\nYou need to explicitly pass the output type in such cases:\n\n    iex> tensor = Nx.tensor([[1, 2, 3], [4, 5, 6]])\n    iex> Nx.map(tensor, [type: :f32], fn x -> Nx.multiply(x, 1.0) end)\n    #Nx.Tensor","ref":"Nx.html#map/3"},{"type":"function","title":"Limitations - Nx.map/3","doc":"Given this function relies on anonymous functions, it\nmay not be available or efficient on all Nx backends.\nTherefore, you should avoid using `map/2` whenever possible\nand use other functions in the `Nx` module to achieve the\ndesired result.\n\nInside `defn`, consider using `Nx.Defn.Kernel.while/4` instead.","ref":"Nx.html#map/3-limitations"},{"type":"function","title":"Examples - Nx.map/3","doc":"iex> Nx.map(Nx.tensor([[1, 2, 3], [4, 5, 6]]), fn x -> Nx.add(x, 1) end)\n    #Nx.Tensor \n\n    iex> Nx.map(Nx.tensor(1), fn x -> Nx.add(x, 1) end)\n    #Nx.Tensor \n\n    iex> Nx.map(Nx.tensor([[1, 2, 3], [4, 5, 6]]), [type: :f64], fn x -> Nx.add(x, 1) end)\n    #Nx.Tensor","ref":"Nx.html#map/3-examples"},{"type":"function","title":"Vectorized tensors - Nx.map/3","doc":"`map/3` behaves the same as with non-vectorized tensors, applying\n`fun` in an element-wise fashion.\n\n    iex> Nx.map(Nx.tensor([[1, 2, 3], [4, 5, 6]]) |> Nx.vectorize(:x), [type: :f64], &Nx.add(&1, 1))\n    #Nx.Tensor","ref":"Nx.html#map/3-vectorized-tensors"},{"type":"function","title":"Nx.max/2","doc":"Element-wise maximum of two tensors.\n\nIf a number is given, it is converted to a tensor.\n\nIt will broadcast tensors whenever the dimensions do\nnot match and broadcasting is possible.\n\nIf you're using `Nx.Defn.defn/2`, you can use the `max/2` function\nin place of this function: `max(left, right)`.","ref":"Nx.html#max/2"},{"type":"function","title":"Examples - Nx.max/2","doc":"#","ref":"Nx.html#max/2-examples"},{"type":"function","title":"Max between scalars - Nx.max/2","doc":"iex> Nx.max(1, 2)\n    #Nx.Tensor \n\n#","ref":"Nx.html#max/2-max-between-scalars"},{"type":"function","title":"Max between tensors and scalars - Nx.max/2","doc":"iex> Nx.max(Nx.tensor([1, 2, 3], names: [:data]), 1)\n    #Nx.Tensor \n\n    iex> Nx.max(1, Nx.tensor([1.0, 2.0, 3.0], names: [:data]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#max/2-max-between-tensors-and-scalars"},{"type":"function","title":"Max between tensors - Nx.max/2","doc":"iex> left = Nx.tensor([[1], [2]], names: [:x, :y])\n    iex> right = Nx.tensor([[10, 20]])\n    iex> Nx.max(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([[1], [2]], type: :s8, names: [:x, nil])\n    iex> right = Nx.tensor([[10, 20]], type: :s8)\n    iex> Nx.max(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([[1], [2]], type: :f32, names: [:x, nil])\n    iex> right = Nx.tensor([[10, 20]], type: :f32, names: [nil, :y])\n    iex> Nx.max(left, right)\n    #Nx.Tensor","ref":"Nx.html#max/2-max-between-tensors"},{"type":"function","title":"Nx.mean/2","doc":"Returns the mean for the tensor.\n\nIf the `:axes` option is given, it aggregates over\nthat dimension, effectively removing it. `axes: [0]`\nimplies aggregating over the highest order dimension\nand so forth. If the axis is negative, then counts\nthe axis from the back. For example, `axes: [-1]` will\nalways aggregate all rows.\n\nYou may optionally set `:keep_axes` to true, which will\nretain the rank of the input tensor by setting the averaged\naxes to size 1.","ref":"Nx.html#mean/2"},{"type":"function","title":"Examples - Nx.mean/2","doc":"iex> Nx.mean(Nx.tensor(42))\n    #Nx.Tensor \n\n    iex> Nx.mean(Nx.tensor([1, 2, 3]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#mean/2-examples"},{"type":"function","title":"Aggregating over an axis - Nx.mean/2","doc":"iex> Nx.mean(Nx.tensor([1, 2, 3]), axes: [0])\n    #Nx.Tensor \n\n    iex> Nx.mean(Nx.tensor([1, 2, 3], type: :u8, names: [:x]), axes: [:x])\n    #Nx.Tensor \n\n    iex> t = Nx.tensor(Nx.iota({2, 2, 3}), names: [:x, :y, :z])\n    iex> Nx.mean(t, axes: [:x])\n    #Nx.Tensor \n\n    iex> t = Nx.tensor(Nx.iota({2, 2, 3}), names: [:x, :y, :z])\n    iex> Nx.mean(t, axes: [:x, :z])\n    #Nx.Tensor \n\n    iex> t = Nx.tensor(Nx.iota({2, 2, 3}), names: [:x, :y, :z])\n    iex> Nx.mean(t, axes: [-1])\n    #Nx.Tensor \n\n#","ref":"Nx.html#mean/2-aggregating-over-an-axis"},{"type":"function","title":"Keeping axes - Nx.mean/2","doc":"iex> t = Nx.tensor(Nx.iota({2, 2, 3}), names: [:x, :y, :z])\n    iex> Nx.mean(t, axes: [-1], keep_axes: true)\n    #Nx.Tensor","ref":"Nx.html#mean/2-keeping-axes"},{"type":"function","title":"Vectorized tensors - Nx.mean/2","doc":"iex> t = Nx.iota({2, 5}, vectorized_axes: [x: 2])\n    iex> Nx.mean(t)\n    #Nx.Tensor \n    iex> Nx.mean(t, axes: [0])\n    #Nx.Tensor \n    iex> Nx.mean(t, axes: [1])\n    #Nx.Tensor","ref":"Nx.html#mean/2-vectorized-tensors"},{"type":"function","title":"Nx.median/2","doc":"Returns the median for the tensor.\n\nThe median is the value in the middle of a data set.\n\nIf the `:axis` option is given, it aggregates over\nthat dimension, effectively removing it. `axis: 0`\nimplies aggregating over the highest order dimension\nand so forth. If the axis is negative, then the axis will\nbe counted from the back. For example, `axis: -1` will\nalways aggregate over the last dimension.\n\nYou may optionally set `:keep_axis` to true, which will\nretain the rank of the input tensor by setting the reduced\naxis to size 1.","ref":"Nx.html#median/2"},{"type":"function","title":"Examples - Nx.median/2","doc":"iex> Nx.median(Nx.tensor(42))\n    #Nx.Tensor \n\n    iex> Nx.median(Nx.tensor([1, 2, 3]))\n    #Nx.Tensor \n\n    iex> Nx.median(Nx.tensor([1, 2]))\n    #Nx.Tensor \n\n    iex> Nx.median(Nx.iota({2, 3, 3}))\n    #Nx.Tensor \n\n#","ref":"Nx.html#median/2-examples"},{"type":"function","title":"Aggregating over an axis - Nx.median/2","doc":"iex> Nx.median(Nx.tensor([[1, 2, 3], [4, 5, 6]], names: [:x, :y]), axis: 0)\n    #Nx.Tensor \n\n    iex> Nx.median(Nx.tensor([[1, 2, 3], [4, 5, 6]], names: [:x, :y]), axis: :y)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor(Nx.iota({2, 2, 3}), names: [:x, :y, :z])\n    iex> Nx.median(t, axis: :x)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[1, 2, 2], [3, 4, 2]], [[4, 5, 2], [7, 9, 2]]])\n    iex> Nx.median(t, axis: -1)\n    #Nx.Tensor \n\n#","ref":"Nx.html#median/2-aggregating-over-an-axis"},{"type":"function","title":"Keeping axis - Nx.median/2","doc":"iex> t = Nx.tensor([[[1, 2, 2], [3, 4, 2]], [[4, 5, 2], [7, 9, 2]]])\n    iex> Nx.median(t, axis: -1, keep_axis: true)\n    #Nx.Tensor \n\n#","ref":"Nx.html#median/2-keeping-axis"},{"type":"function","title":"Vectorized tensors - Nx.median/2","doc":"For vectorized inputs, `:axis` refers to the\nnon-vectorized shape:\n\n    iex> Nx.median(Nx.tensor([[1, 2, 3], [4, 5, 6]]) |> Nx.vectorize(:x), axis: 0)\n    #Nx.Tensor","ref":"Nx.html#median/2-vectorized-tensors"},{"type":"function","title":"Nx.min/2","doc":"Element-wise minimum of two tensors.\n\nIf a number is given, it is converted to a tensor.\n\nIt will broadcast tensors whenever the dimensions do\nnot match and broadcasting is possible.\n\nIf you're using `Nx.Defn.defn/2`, you can use the `min/2` function\nin place of this function: `min(left, right)`.","ref":"Nx.html#min/2"},{"type":"function","title":"Examples - Nx.min/2","doc":"#","ref":"Nx.html#min/2-examples"},{"type":"function","title":"Min between scalars - Nx.min/2","doc":"iex> Nx.min(1, 2)\n    #Nx.Tensor \n\n#","ref":"Nx.html#min/2-min-between-scalars"},{"type":"function","title":"Min between tensors and scalars - Nx.min/2","doc":"iex> Nx.min(Nx.tensor([1, 2, 3], names: [:data]), 1)\n    #Nx.Tensor \n\n    iex> Nx.min(1, Nx.tensor([1.0, 2.0, 3.0], names: [:data]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#min/2-min-between-tensors-and-scalars"},{"type":"function","title":"Min between tensors - Nx.min/2","doc":"iex> left = Nx.tensor([[1], [2]], names: [:x, nil])\n    iex> right = Nx.tensor([[10, 20]])\n    iex> Nx.min(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([[1], [2]], type: :s8, names: [:x, :y])\n    iex> right = Nx.tensor([[10, 20]], type: :s8)\n    iex> Nx.min(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([[1], [2]], type: :f32, names: [:x, nil])\n    iex> right = Nx.tensor([[10, 20]], type: :f32, names: [nil, :y])\n    iex> Nx.min(left, right)\n    #Nx.Tensor","ref":"Nx.html#min/2-min-between-tensors"},{"type":"function","title":"Nx.mode/2","doc":"Returns the mode of a tensor.\n\nThe mode is the value that appears most often.\n\nIf the `:axis` option is given, it aggregates over\nthat dimension, effectively removing it. `axis: 0`\nimplies aggregating over the highest order dimension\nand so forth. If the axis is negative, then the axis will\nbe counted from the back. For example, `axis: -1` will\nalways aggregate over the last dimension.\n\nYou may optionally set `:keep_axis` to true, which will\nretain the rank of the input tensor by setting the reduced\naxis to size 1.","ref":"Nx.html#mode/2"},{"type":"function","title":"Examples - Nx.mode/2","doc":"iex> Nx.mode(Nx.tensor(42))\n    #Nx.Tensor \n\n    iex> Nx.mode(Nx.tensor([[1]]))\n    #Nx.Tensor \n\n    iex> Nx.mode(Nx.tensor([1, 2, 2, 3, 5]))\n    #Nx.Tensor \n\n    iex> Nx.mode(Nx.tensor([[1, 2, 2, 3, 5], [1, 1, 76, 8, 1]]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#mode/2-examples"},{"type":"function","title":"Aggregating over an axis - Nx.mode/2","doc":"iex> Nx.mode(Nx.tensor([[1, 2, 2, 3, 5], [1, 1, 76, 8, 1]]), axis: 0)\n    #Nx.Tensor \n\n    iex> Nx.mode(Nx.tensor([[1, 2, 2, 3, 5], [1, 1, 76, 8, 1]]), axis: 1)\n    #Nx.Tensor \n\n    iex> Nx.mode(Nx.tensor([[[1]]]), axis: 1)\n    #Nx.Tensor \n\n#","ref":"Nx.html#mode/2-aggregating-over-an-axis"},{"type":"function","title":"Keeping axis - Nx.mode/2","doc":"iex> Nx.mode(Nx.tensor([[1, 2, 2, 3, 5], [1, 1, 76, 8, 1]]), axis: 1, keep_axis: true)\n    #Nx.Tensor \n\n#","ref":"Nx.html#mode/2-keeping-axis"},{"type":"function","title":"Vectorized tensors - Nx.mode/2","doc":"For vectorized tensors, `:axis` refers to the non-vectorized shape:\n\n    iex> t = Nx.tensor([[[1, 2, 2, 3, 5], [1, 1, 76, 8, 1]], [[1, 2, 2, 2, 5], [5, 2, 2, 2, 1]]]) |> Nx.vectorize(:x)\n    iex> Nx.mode(t, axis: 0)\n    #Nx.Tensor \n    iex> Nx.mode(t, axis: 1)\n    #Nx.Tensor","ref":"Nx.html#mode/2-vectorized-tensors"},{"type":"function","title":"Nx.multiply/2","doc":"Element-wise multiplication of two tensors.\n\nIf a number is given, it is converted to a tensor.\n\nIt will broadcast tensors whenever the dimensions do\nnot match and broadcasting is possible.\n\nIf you're using `Nx.Defn.defn/2`, you can use the `*` operator\noperator in place of this function as `left * right`.","ref":"Nx.html#multiply/2"},{"type":"function","title":"Examples - Nx.multiply/2","doc":"#","ref":"Nx.html#multiply/2-examples"},{"type":"function","title":"Multiplying scalars - Nx.multiply/2","doc":"iex> Nx.multiply(1, 2)\n    #Nx.Tensor \n\n#","ref":"Nx.html#multiply/2-multiplying-scalars"},{"type":"function","title":"Multiplying tensors and scalars - Nx.multiply/2","doc":"iex> Nx.multiply(Nx.tensor([1, 2, 3], names: [:data]), 1)\n    #Nx.Tensor \n\n    iex> Nx.multiply(1, Nx.tensor([1.0, 2.0, 3.0], names: [:data]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#multiply/2-multiplying-tensors-and-scalars"},{"type":"function","title":"Multiplying tensors - Nx.multiply/2","doc":"iex> left = Nx.tensor([[1], [2]], names: [:x, :y])\n    iex> right = Nx.tensor([[10, 20]], names: [:x, :y])\n    iex> Nx.multiply(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([[1], [2]], type: :s8, names: [:x, nil])\n    iex> right = Nx.tensor([[10, 20]], type: :s8, names: [nil, :y])\n    iex> Nx.multiply(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([[1], [2]], type: :f32, names: [nil, :y])\n    iex> right = Nx.tensor([[10, 20]], type: :f32, names: [:x, nil])\n    iex> Nx.multiply(left, right)\n    #Nx.Tensor","ref":"Nx.html#multiply/2-multiplying-tensors"},{"type":"function","title":"Nx.names/1","doc":"Returns all of the names in a tensor.","ref":"Nx.html#names/1"},{"type":"function","title":"Examples - Nx.names/1","doc":"iex> Nx.names(Nx.tensor([[1, 2, 3], [4, 5, 6]], names: [:batch, :data]))\n    [:batch, :data]\n\n    iex> Nx.names(Nx.tensor([1, 2, 3]))\n    [nil]\n\n    iex> Nx.names(5)\n    []","ref":"Nx.html#names/1-examples"},{"type":"function","title":"Nx.negate/1","doc":"Negates each element in the tensor.\n\nIf you're using `Nx.Defn.defn/2`, you can use the `-` unary operator\nin place of this function: `-tensor`.","ref":"Nx.html#negate/1"},{"type":"function","title":"Examples - Nx.negate/1","doc":"iex> Nx.negate(1)\n    #Nx.Tensor \n\n    iex> Nx.negate(Nx.tensor([-1, 0, 1]))\n    #Nx.Tensor \n\n    iex> Nx.negate(Nx.tensor([1.0, 2.0, 3.0], type: :f32))\n    #Nx.Tensor \n\nIf an unsigned tensor is given, it works as `bitwise_not`:\n\n    iex> Nx.negate(Nx.tensor([0, 1, 2], type: :u8, names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#negate/1-examples"},{"type":"function","title":"Nx.new_axis/3","doc":"Adds a new `axis` of size 1 with optional `name`.","ref":"Nx.html#new_axis/3"},{"type":"function","title":"Examples - Nx.new_axis/3","doc":"iex> t = Nx.tensor([[1, 2, 3], [4, 5, 6]])\n    iex> Nx.new_axis(t, 0, :new)\n    #Nx.Tensor \n    iex> Nx.new_axis(t, 1, :new)\n    #Nx.Tensor \n    iex> Nx.new_axis(t, 2, :new)\n    #Nx.Tensor \n\nAxis can also be negative, which will start from the back:\n\n    iex> t = Nx.tensor([[1, 2, 3], [4, 5, 6]])\n    iex> Nx.new_axis(t, -1, :new)\n    #Nx.Tensor","ref":"Nx.html#new_axis/3-examples"},{"type":"function","title":"Vectorized tensors - Nx.new_axis/3","doc":"Similarly to `reshape/2`, vectorized tensors will have their\nvectors unchanged. The examples below show that the new axes\nonly affect the tensor shape.\n\n    iex> t = Nx.tensor([1]) |> Nx.vectorize(:x)\n    #Nx.Tensor \n    iex> t = Nx.new_axis(t, -1, :new)\n    #Nx.Tensor \n    iex> Nx.new_axis(t, 0)\n    #Nx.Tensor","ref":"Nx.html#new_axis/3-vectorized-tensors"},{"type":"function","title":"Nx.not_equal/2","doc":"Element-wise not-equal comparison of two tensors.\n\nIf a number is given, it is converted to a tensor.\n\nIt will broadcast tensors whenever the dimensions do\nnot match and broadcasting is possible.\n\nIf you're using `Nx.Defn.defn/2`, you can use the `!=` operator\nin place of this function: `left != right`.","ref":"Nx.html#not_equal/2"},{"type":"function","title":"Examples - Nx.not_equal/2","doc":"#","ref":"Nx.html#not_equal/2-examples"},{"type":"function","title":"Comparison of scalars - Nx.not_equal/2","doc":"iex> Nx.not_equal(1, 2)\n    #Nx.Tensor \n\n#","ref":"Nx.html#not_equal/2-comparison-of-scalars"},{"type":"function","title":"Comparison of tensor and scalar - Nx.not_equal/2","doc":"iex> Nx.not_equal(Nx.tensor([1, 2, 3], names: [:data]), Nx.tensor(1))\n    #Nx.Tensor \n\n#","ref":"Nx.html#not_equal/2-comparison-of-tensor-and-scalar"},{"type":"function","title":"Comparison of tensors - Nx.not_equal/2","doc":"iex> left = Nx.tensor([1, 1, 2])\n    iex> right = Nx.tensor([1, 2, 3], names: [:data])\n    iex> Nx.not_equal(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([[1, 4, 2], [4, 5, 6]], names: [:x, :y])\n    iex> right = Nx.tensor([[1, 3, 2], [4, 2, 1]], names: [:x, :y])\n    iex> Nx.not_equal(left, right)\n    #Nx.Tensor","ref":"Nx.html#not_equal/2-comparison-of-tensors"},{"type":"function","title":"Nx.outer/2","doc":"Computes the outer product of two tensors.\n\nThe output is always a two-dimensional tensor.","ref":"Nx.html#outer/2"},{"type":"function","title":"Examples - Nx.outer/2","doc":"iex> Nx.outer(Nx.tensor([1, 2, 3], names: [:x]), 100)\n    #Nx.Tensor \n\n    iex> Nx.outer(Nx.tensor([1, 2, 3], names: [:x]), Nx.tensor([10, 20], names: [:y]))\n    #Nx.Tensor \n\n    iex> Nx.outer(Nx.tensor([[1, 2], [3, 4]], names: [:x, :y]), Nx.tensor([10, 20, 30], names: [:z]))\n    #Nx.Tensor","ref":"Nx.html#outer/2-examples"},{"type":"function","title":"Vectorized tensors - Nx.outer/2","doc":"Because `outer/2` is built on top of other\n\n    iex> x = Nx.tensor([[1, 2, 3], [0, -1, -2]], names: [nil, :a]) |> Nx.vectorize(:x)\n    iex> y = Nx.tensor([[10, 20], [-10, -20]], names: [nil, :b]) |> Nx.vectorize(:y)\n    iex> Nx.outer(x, y)\n    #Nx.Tensor","ref":"Nx.html#outer/2-vectorized-tensors"},{"type":"function","title":"Nx.pad/3","doc":"Pads a tensor with a given value.\n\nYou must specify a padding configuration. A padding\nconfiguration is a list of tuples consisting of\n`{pad_width_low, pad_width_high, pad_width_interior}`\nfor each dimension in the input tensor. The padding\nconfiguration must be of the same length as the tensor shape.\n\nPadding widths can be negative. If they are negative,\nthe tensor is clipped on either end according to the\npadding width. Interior padding widths cannot be negative.\n\nSee also: `reflect/2`","ref":"Nx.html#pad/3"},{"type":"function","title":"Examples - Nx.pad/3","doc":"iex> Nx.pad(Nx.tensor(1), 0, [])\n    #Nx.Tensor \n\n    iex> Nx.pad(Nx.tensor([1, 2, 3], names: [:data]), 0, [{1, 1, 0}])\n    #Nx.Tensor \n\n    iex> Nx.pad(Nx.tensor([[1, 2, 3], [4, 5, 6]]), 0, [{0, 0, 1}, {0, 0, 1}])\n    #Nx.Tensor \n\n    iex> Nx.pad(Nx.tensor([[1, 2, 3], [4, 5, 6]]), 0, [{1, 1, 0}, {1, 1, 0}])\n    #Nx.Tensor \n\n    iex> tensor = Nx.tensor([[[1, 2], [3, 4]], [[5, 6], [7, 8]]])\n    iex> Nx.pad(tensor, 0, [{0, 2, 0}, {1, 1, 0}, {1, 0, 0}])\n    #Nx.Tensor \n\n    iex> tensor = Nx.tensor([[[1, 2], [3, 4]], [[5, 6], [7, 8]]])\n    iex> Nx.pad(tensor, 0, [{1, 0, 0}, {1, 1, 0}, {0, 1, 0}])\n    #Nx.Tensor \n\n    iex> tensor = Nx.tensor([[[1.0, 2.0], [3.0, 4.0]], [[5.0, 6.0], [7.0, 8.0]]])\n    iex> Nx.pad(tensor, 0.0, [{1, 2, 0}, {1, 0, 0}, {0, 1, 0}])\n    #Nx.Tensor \n\n    iex> Nx.pad(Nx.tensor([0, 1, 2, 3, 0]), 0, [{-1, -1, 0}])\n    #Nx.Tensor \n\n    iex> tensor = Nx.tensor([\n    ...>   [[0, 0, 0], [0, 0, 0], [0, 0, 0], [0, 0, 0]],\n    ...>   [[0, 0, 0], [1, 2, 0], [3, 4, 0], [0, 0, 0]],\n    ...>   [[0, 0, 0], [5, 6, 0], [7, 8, 0], [0, 0, 0]]\n    ...> ])\n    iex> Nx.pad(tensor, 0, [{-1, 0, 0}, {-1, -1, 0}, {0, -1, 0}])\n    #Nx.Tensor \n\n    iex> tensor = Nx.tensor([[0, 1, 2, 3], [0, 4, 5, 6]])\n    iex> Nx.pad(tensor, 0, [{0, 0, 0}, {-1, 1, 0}])\n    #Nx.Tensor \n\n    iex> tensor = Nx.tensor([[0, 1, 2], [3, 4, 5]], type: :f32)\n    iex> Nx.pad(tensor, 0, [{-1, 2, 0}, {1, -1, 0}])\n    #Nx.Tensor","ref":"Nx.html#pad/3-examples"},{"type":"function","title":"Vectorized tensors - Nx.pad/3","doc":"Like with the non-vectorized case, `pad_value` must be a non-vectorized scalar tensor.\nVectorized axes remain unchanged.\n\n    iex> t = Nx.tensor([[1], [2], [3]], names: [nil, :data]) |> Nx.vectorize(:x)\n    iex> Nx.pad(t, 0, [{1, 1, 0}])\n    #Nx.Tensor","ref":"Nx.html#pad/3-vectorized-tensors"},{"type":"function","title":"Nx.phase/1","doc":"Calculates the complex phase angle of each element in the tensor.\n$$phase(z) = atan2(b, a), z = a + bi \\in \\Complex$$","ref":"Nx.html#phase/1"},{"type":"function","title":"Examples - Nx.phase/1","doc":"iex> Nx.phase(Complex.new(1, 2))\n     #Nx.Tensor \n\n     iex> Nx.phase(1)\n     #Nx.Tensor \n\n     iex> import Nx, only: [sigil_V: 2]\n     iex> Nx.phase(~V[1+2i -2+1i])\n     #Nx.Tensor","ref":"Nx.html#phase/1-examples"},{"type":"function","title":"Nx.population_count/1","doc":"Computes the bitwise population count of each element in the tensor.","ref":"Nx.html#population_count/1"},{"type":"function","title":"Examples - Nx.population_count/1","doc":"iex> Nx.population_count(1)\n    #Nx.Tensor \n\n    iex> Nx.population_count(-128)\n    #Nx.Tensor \n\n    iex> Nx.population_count(Nx.tensor([0, 1, 254, 255], names: [:x]))\n    #Nx.Tensor \n\n    iex> Nx.population_count(Nx.tensor([0, 1, 126, 127, -1, -127, -128], type: :s8, names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#population_count/1-examples"},{"type":"function","title":"Error cases - Nx.population_count/1","doc":"iex> Nx.population_count(Nx.tensor([0.0, 1.0]))\n    ** (ArgumentError) bitwise operators expect integer tensors as inputs and outputs an integer tensor, got: {:f, 32}","ref":"Nx.html#population_count/1-error-cases"},{"type":"function","title":"Nx.pow/2","doc":"Element-wise power of two tensors.\n\nIf a number is given, it is converted to a tensor.\n\nIt will broadcast tensors whenever the dimensions do\nnot match and broadcasting is possible.\n\nIf both tensors are integers and the exponent is\nnegative, it will raise, but it may trigger undefined\nbehaviour on some compilers.","ref":"Nx.html#pow/2"},{"type":"function","title":"Examples - Nx.pow/2","doc":"#","ref":"Nx.html#pow/2-examples"},{"type":"function","title":"Power of scalars - Nx.pow/2","doc":"iex> Nx.pow(2, 4)\n    #Nx.Tensor \n\n#","ref":"Nx.html#pow/2-power-of-scalars"},{"type":"function","title":"Power of tensors and scalars - Nx.pow/2","doc":"iex> Nx.pow(Nx.tensor([1, 2, 3], names: [:data]), 2)\n    #Nx.Tensor \n\n    iex> Nx.pow(2, Nx.tensor([1.0, 2.0, 3.0], names: [:data]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#pow/2-power-of-tensors-and-scalars"},{"type":"function","title":"Power of tensors - Nx.pow/2","doc":"iex> Nx.pow(Nx.tensor([[2], [3]], names: [:x, nil]), Nx.tensor([[4, 5]], names: [nil, :y]))\n    #Nx.Tensor","ref":"Nx.html#pow/2-power-of-tensors"},{"type":"function","title":"Nx.product/2","doc":"Returns the product for the tensor.\n\nIf the `:axes` option is given, it aggregates over\nthe given dimensions, effectively removing them.\n`axes: [0]` implies aggregating over the highest order\ndimension and so forth. If the axis is negative, then\ncounts the axis from the back. For example, `axes: [-1]`\nwill always aggregate all rows.\n\nYou may optionally set `:keep_axes` to true, which will\nretain the rank of the input tensor by setting the multiplied\naxes to size 1.","ref":"Nx.html#product/2"},{"type":"function","title":"Examples - Nx.product/2","doc":"By default the product always returns a scalar:\n\n    iex> Nx.product(Nx.tensor(42))\n    #Nx.Tensor \n\n    iex> Nx.product(Nx.tensor([1, 2, 3]))\n    #Nx.Tensor \n\n    iex> Nx.product(Nx.tensor([[1.0, 2.0], [3.0, 4.0]]))\n    #Nx.Tensor \n\nGiving a tensor with low precision casts it to a higher\nprecision to make sure the sum does not overflow:\n\n    iex> Nx.product(Nx.tensor([[10, 20], [30, 40]], type: :u8, names: [:x, :y]))\n    #Nx.Tensor \n\n    iex> Nx.product(Nx.tensor([[10, 20], [30, 40]], type: :s8, names: [:x, :y]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#product/2-examples"},{"type":"function","title":"Aggregating over an axis - Nx.product/2","doc":"iex> Nx.product(Nx.tensor([1, 2, 3]), axes: [0])\n    #Nx.Tensor \n\nSame tensor over different axes combinations:\n\n    iex> t = Nx.iota({2, 2, 3}, names: [:x, :y, :z])\n    iex> Nx.product(t, axes: [:x])\n    #Nx.Tensor \n    iex> Nx.product(t, axes: [:y])\n    #Nx.Tensor \n    iex> Nx.product(t, axes: [:x, :z])\n    #Nx.Tensor \n    iex> Nx.product(t, axes: [:z])\n    #Nx.Tensor \n    iex> Nx.product(t, axes: [-3])\n    #Nx.Tensor \n\n#","ref":"Nx.html#product/2-aggregating-over-an-axis"},{"type":"function","title":"Keeping axes - Nx.product/2","doc":"iex> t = Nx.iota({2, 2, 3}, names: [:x, :y, :z])\n    iex> Nx.product(t, axes: [:z], keep_axes: true)\n    #Nx.Tensor \n\n#","ref":"Nx.html#product/2-keeping-axes"},{"type":"function","title":"Vectorized tensors - Nx.product/2","doc":"iex> t = Nx.vectorize(Nx.tensor([[1, 2], [3, 4]]), :x)\n    iex> Nx.product(t, axes: [0], keep_axes: true)\n    #Nx.Tensor \n\n#","ref":"Nx.html#product/2-vectorized-tensors"},{"type":"function","title":"Errors - Nx.product/2","doc":"iex> Nx.product(Nx.tensor([[1, 2]]), axes: [2])\n    ** (ArgumentError) given axis (2) invalid for shape with rank 2","ref":"Nx.html#product/2-errors"},{"type":"function","title":"Nx.put_diagonal/3","doc":"Puts the individual values from a 1D diagonal into the diagonal indices\nof the given 2D tensor.\n\nSee also: `take_diagonal/2`, `make_diagonal/2`.","ref":"Nx.html#put_diagonal/3"},{"type":"function","title":"Examples - Nx.put_diagonal/3","doc":"Given a 2D tensor and a 1D diagonal:\n\n    iex> t = Nx.broadcast(0, {4, 4})\n    #Nx.Tensor \n    iex> Nx.put_diagonal(t, Nx.tensor([1, 2, 3, 4]))\n    #Nx.Tensor \n\n    iex> t = Nx.broadcast(0, {4, 3})\n    #Nx.Tensor \n    iex> Nx.put_diagonal(t, Nx.tensor([1, 2, 3]))\n    #Nx.Tensor \n\nGiven a 2D tensor and a 1D diagonal with a positive offset:\n\n    iex> Nx.put_diagonal(Nx.broadcast(0, {4, 4}), Nx.tensor([1, 2, 3]), offset: 1)\n    #Nx.Tensor \n\n    iex> Nx.put_diagonal(Nx.broadcast(0, {4, 3}), Nx.tensor([1, 2]), offset: 1)\n    #Nx.Tensor \n\nGiven a 2D tensor and a 1D diagonal with a negative offset:\n\n    iex> Nx.put_diagonal(Nx.broadcast(0, {4, 4}), Nx.tensor([1, 2, 3]), offset: -1)\n    #Nx.Tensor \n\n    iex> Nx.put_diagonal(Nx.broadcast(0, {4, 3}), Nx.tensor([1, 2, 3]), offset: -1)\n    #Nx.Tensor","ref":"Nx.html#put_diagonal/3-examples"},{"type":"function","title":"Options - Nx.put_diagonal/3","doc":"* `:offset` - offset used for putting the diagonal.\n    Use offset > 0 for diagonals above the main diagonal,\n    and offset < 0 for diagonals below the main diagonal.\n    Defaults to 0.","ref":"Nx.html#put_diagonal/3-options"},{"type":"function","title":"Error cases - Nx.put_diagonal/3","doc":"Given an invalid tensor:\n\n    iex> Nx.put_diagonal(Nx.iota({3, 3, 3}), Nx.iota({3}))\n    ** (ArgumentError) put_diagonal/3 expects tensor of rank 2, got tensor of rank: 3\n\nGiven invalid diagonals:\n\n    iex> Nx.put_diagonal(Nx.iota({3, 3}), Nx.iota({3, 3}))\n    ** (ArgumentError) put_diagonal/3 expects diagonal of rank 1, got tensor of rank: 2\n\n    iex> Nx.put_diagonal(Nx.iota({3, 3}), Nx.iota({2}))\n    ** (ArgumentError) expected diagonal tensor of length: 3, got diagonal tensor of length: 2\n\n    iex> Nx.put_diagonal(Nx.iota({3, 3}), Nx.iota({3}), offset: 1)\n    ** (ArgumentError) expected diagonal tensor of length: 2, got diagonal tensor of length: 3\n\nGiven invalid offsets:\n\n    iex> Nx.put_diagonal(Nx.iota({3, 3}), Nx.iota({3}), offset: 4)\n    ** (ArgumentError) offset must be less than length of axis 1 when positive, got: 4\n\n    iex> Nx.put_diagonal(Nx.iota({3, 3}), Nx.iota({3}), offset: -3)\n    ** (ArgumentError) absolute value of offset must be less than length of axis 0 when negative, got: -3","ref":"Nx.html#put_diagonal/3-error-cases"},{"type":"function","title":"Nx.put_slice/3","doc":"Puts the given `slice` into the given `tensor` at the given\n`start_indices`.\n\nThe given slice must be of the same rank as tensor. Each axis\nmust be less than or equal to the size to the equivalent axis\nin the tensor.\n\nThe number of elements in `start_indices` should match the\nrank of the tensor.\n\nSee also: `indexed_add/3`, `indexed_put/3`.","ref":"Nx.html#put_slice/3"},{"type":"function","title":"Examples - Nx.put_slice/3","doc":"iex> t = Nx.tensor([0, 1, 2, 3, 4])\n    iex> Nx.put_slice(t, [2], Nx.tensor([5, 6]))\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[1, 2, 3], [4, 5, 6]])\n    iex> Nx.put_slice(t, [0, 1], Nx.tensor([[7, 8], [9, 10]]))\n    #Nx.Tensor \n\nSimilar to `slice/3`, dynamic start indexes are also supported:\n\n    iex> t = Nx.tensor([[1, 2, 3], [4, 5, 6]])\n    iex> Nx.put_slice(t, [Nx.tensor(0), Nx.tensor(1)], Nx.tensor([[10.0, 11.0]]))\n    #Nx.Tensor \n\nAlso similar to `slice/3`, if `start_index + slice_dimension > dimension`,\nthe start index will be clipped in order to put the whole slice:\n\n    iex> t = Nx.tensor([[1, 2, 3], [4, 5, 6]])\n    iex> Nx.put_slice(t, [1, 1], Nx.tensor([[7, 8], [9, 10]]))\n    #Nx.Tensor","ref":"Nx.html#put_slice/3-examples"},{"type":"function","title":"Vectorized tensors - Nx.put_slice/3","doc":"The both tensor to be sliced and the slices can be vectorized,\nbut indices must be non-vectorized.\n\n    iex> t = Nx.tensor([[1, 2, 3, 4], [5, 6, 7, 8]]) |> Nx.vectorize(:x)\n    iex> slice = Nx.tensor([[10, 20], [30, 40]]) |> Nx.vectorize(:y)\n    iex> Nx.put_slice(t, [2], slice)\n    #Nx.Tensor","ref":"Nx.html#put_slice/3-vectorized-tensors"},{"type":"function","title":"Nx.quotient/2","doc":"Element-wise integer division of two tensors.\n\nIf a number is given, it is converted to a tensor.\n\nIt always returns an integer tensor. Input tensors and\nnumbers must be integer types. Division by zero raises,\nbut it may trigger undefined behaviour on some compilers.\n\nIt will broadcast tensors whenever the dimensions do\nnot match and broadcasting is possible.","ref":"Nx.html#quotient/2"},{"type":"function","title":"Caveat for `grad` - Nx.quotient/2","doc":"The `grad` operation is not supported for `quotient/2`.\nSince integer division is, by definition, a closed operation\nfor the set of integers and grad involves floating points,\n`grad` is undefined.\n\nIf you need to support gradients, you might consider using\nfloor division, but beware of precision errors caused by\nfloating points:\n\n    a |> Nx.divide(b) |> Nx.floor()","ref":"Nx.html#quotient/2-caveat-for-grad"},{"type":"function","title":"Examples - Nx.quotient/2","doc":"#","ref":"Nx.html#quotient/2-examples"},{"type":"function","title":"Integer dividing scalars - Nx.quotient/2","doc":"iex> Nx.quotient(11, 2)\n    #Nx.Tensor \n\n#","ref":"Nx.html#quotient/2-integer-dividing-scalars"},{"type":"function","title":"Integer dividing tensors and scalars - Nx.quotient/2","doc":"iex> Nx.quotient(Nx.tensor([2, 4, 5], names: [:data]), 2)\n    #Nx.Tensor \n\n    iex> Nx.quotient(10, Nx.tensor([1, 2, 3], names: [:data]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#quotient/2-integer-dividing-tensors-and-scalars"},{"type":"function","title":"Dividing tensors - Nx.quotient/2","doc":"iex> left = Nx.tensor([[10, 20]], names: [nil, :y])\n    iex> right = Nx.tensor([[1], [2]], names: [:x, nil])\n    iex> Nx.quotient(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([[10, 20]], type: :s8, names: [:x, :y])\n    iex> right = Nx.tensor([[1], [2]], type: :s8)\n    iex> Nx.quotient(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([[10, 20]], type: :u8, names: [:x, :y])\n    iex> right = Nx.tensor([[1], [2]], type: :u32)\n    iex> Nx.quotient(left, right)\n    #Nx.Tensor","ref":"Nx.html#quotient/2-dividing-tensors"},{"type":"function","title":"Nx.rank/1","doc":"Returns the rank of a tensor.\n\nIf a tuple is given as a shape, it computes the rank\nof the given tuple.","ref":"Nx.html#rank/1"},{"type":"function","title":"Examples - Nx.rank/1","doc":"iex> Nx.rank(Nx.tensor(1))\n    0\n\n    iex> Nx.rank(Nx.tensor([[1, 2, 3], [4, 5, 6]]))\n    2\n\n    iex> Nx.rank(1)\n    0\n\n    iex> Nx.rank({1, 2, 3})\n    3","ref":"Nx.html#rank/1-examples"},{"type":"function","title":"Nx.real/1","doc":"Returns the real component of each entry in a complex tensor\nas a floating point tensor.","ref":"Nx.html#real/1"},{"type":"function","title":"Examples - Nx.real/1","doc":"iex> Nx.real(Complex.new(1, 2))\n    #Nx.Tensor \n\n    iex> Nx.real(Nx.tensor(1))\n    #Nx.Tensor \n\n    iex> Nx.real(Nx.tensor(1, type: :bf16))\n    #Nx.Tensor \n\n    iex> Nx.real(Nx.tensor([Complex.new(1, 2), Complex.new(2, -4)]))\n    #Nx.Tensor","ref":"Nx.html#real/1-examples"},{"type":"function","title":"Nx.reduce/4","doc":"Reduces over a tensor with the given accumulator.\n\nThe given `fun` will receive two tensors and it must\nreturn the reduced value.\n\nThe tensor may be reduced in parallel and the reducer\nfunction can be called with arguments in any order, the\ninitial accumulator may be given multiples, and it may\nbe non-deterministic. Therefore, the reduction function\nshould be associative (or as close as possible to\nassociativity considered floats themselves are not\nstrictly associative).\n\nBy default, it reduces all dimensions of the tensor and\nreturn a scalar. If the `:axes` option is given, it\naggregates over multiple dimensions, effectively removing\nthem. `axes: [0]` implies aggregating over the highest\norder dimension and so forth. If the axis is negative,\nthen counts the axis from the back. For example,\n`axes: [-1]` will always aggregate all rows.\n\nThe type of the returned tensor will be computed based on\nthe given tensor and the initial value. For example,\na tensor of integers with a float accumulator will be\ncast to float, as done by most binary operators. You can\nalso pass a `:type` option to change this behaviour.\n\nYou may optionally set `:keep_axes` to true, which will\nretain the rank of the input tensor by setting the reduced\naxes to size 1.","ref":"Nx.html#reduce/4"},{"type":"function","title":"Limitations - Nx.reduce/4","doc":"Given this function relies on anonymous functions, it\nmay not be available or efficient on all Nx backends.\nTherefore, you should avoid using `reduce/4` whenever\npossible. Instead, use functions `sum/2`, `reduce_max/2`,\n`all/1`, and so forth.\n\nInside `defn`, consider using `Nx.Defn.Kernel.while/4` instead.","ref":"Nx.html#reduce/4-limitations"},{"type":"function","title":"Examples - Nx.reduce/4","doc":"iex> Nx.reduce(Nx.tensor(42), 0, fn x, y -> Nx.add(x, y) end)\n    #Nx.Tensor \n\n    iex> Nx.reduce(Nx.tensor([1, 2, 3]), 0, fn x, y -> Nx.add(x, y) end)\n    #Nx.Tensor \n\n    iex> Nx.reduce(Nx.tensor([[1.0, 2.0], [3.0, 4.0]]), 0, fn x, y -> Nx.add(x, y) end)\n    #Nx.Tensor \n\n#","ref":"Nx.html#reduce/4-examples"},{"type":"function","title":"Aggregating over axes - Nx.reduce/4","doc":"iex> t = Nx.tensor([1, 2, 3], names: [:x])\n    iex> Nx.reduce(t, 0, [axes: [:x]], fn x, y -> Nx.add(x, y) end)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[1, 2, 3], [4, 5, 6]], [[7, 8, 9], [10, 11, 12]]], names: [:x, :y, :z])\n    iex> Nx.reduce(t, 0, [axes: [:x]], fn x, y -> Nx.add(x, y) end)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[1, 2, 3], [4, 5, 6]], [[7, 8, 9], [10, 11, 12]]], names: [:x, :y, :z])\n    iex> Nx.reduce(t, 0, [axes: [:y]], fn x, y -> Nx.add(x, y) end)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[1, 2, 3], [4, 5, 6]], [[7, 8, 9], [10, 11, 12]]], names: [:x, :y, :z])\n    iex> Nx.reduce(t, 0, [axes: [:x, 2]], fn x, y -> Nx.add(x, y) end)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[1, 2, 3], [4, 5, 6]], [[7, 8, 9], [10, 11, 12]]], names: [:x, :y, :z])\n    iex> Nx.reduce(t, 0, [axes: [-1]], fn x, y -> Nx.add(x, y) end)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[1, 2, 3], [4, 5, 6]], [[7, 8, 9], [10, 11, 12]]], names: [:x, :y, :z])\n    iex> Nx.reduce(t, 0, [axes: [:x]], fn x, y -> Nx.add(x, y) end)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[1, 2, 3], [4, 5, 6]], [[7, 8, 9], [10, 11, 12]]], names: [:x, :y, :z])\n    iex> Nx.reduce(t, 0, [axes: [:x], keep_axes: true], fn x, y -> Nx.add(x, y) end)\n    #Nx.Tensor","ref":"Nx.html#reduce/4-aggregating-over-axes"},{"type":"function","title":"Vectorized tensors - Nx.reduce/4","doc":"Only `tensor` can be vectorized. Normal behavior of `reduce/4`\nis applied to each corresponding entry. `:axes` refers to the\nnon-vectorized shape.\n\n    iex> t = Nx.tensor([[[1, 2, 3], [4, 5, 6]], [[10, 20, 30], [40, 50, 60]]]) |> Nx.vectorize(:x)\n    iex> Nx.reduce(t, 10, [axes: [1]], &Nx.add/2)\n    #Nx.Tensor","ref":"Nx.html#reduce/4-vectorized-tensors"},{"type":"function","title":"Nx.reduce_max/2","doc":"Returns the maximum values of the tensor.\n\nIf the `:axes` option is given, it aggregates over\nthe given dimensions, effectively removing them.\n`axes: [0]` implies aggregating over the highest order\ndimension and so forth. If the axis is negative, then\ncounts the axis from the back. For example, `axes: [-1]`\nwill always aggregate all rows.\n\nYou may optionally set `:keep_axes` to true, which will\nretain the rank of the input tensor by setting the reduced\naxes to size 1.","ref":"Nx.html#reduce_max/2"},{"type":"function","title":"Examples - Nx.reduce_max/2","doc":"iex> Nx.reduce_max(Nx.tensor(42))\n    #Nx.Tensor \n\n    iex> Nx.reduce_max(Nx.tensor(42.0))\n    #Nx.Tensor \n\n    iex> Nx.reduce_max(Nx.tensor([1, 2, 3]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#reduce_max/2-examples"},{"type":"function","title":"Aggregating over an axis - Nx.reduce_max/2","doc":"iex> t = Nx.tensor([[3, 1, 4], [2, 1, 1]], names: [:x, :y])\n    iex> Nx.reduce_max(t, axes: [:x])\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[3, 1, 4], [2, 1, 1]], names: [:x, :y])\n    iex> Nx.reduce_max(t, axes: [:y])\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[1, 2], [4, 5]], [[2, 4], [3, 8]]], names: [:x, :y, :z])\n    iex> Nx.reduce_max(t, axes: [:x, :z])\n    #Nx.Tensor \n\n#","ref":"Nx.html#reduce_max/2-aggregating-over-an-axis"},{"type":"function","title":"Keeping axes - Nx.reduce_max/2","doc":"iex> t = Nx.tensor([[[1, 2], [4, 5]], [[2, 4], [3, 8]]], names: [:x, :y, :z])\n    iex> Nx.reduce_max(t, axes: [:x, :z], keep_axes: true)\n    #Nx.Tensor \n\n#","ref":"Nx.html#reduce_max/2-keeping-axes"},{"type":"function","title":"Vectorized tensors - Nx.reduce_max/2","doc":"iex> t = Nx.vectorize(Nx.tensor([[1, 2], [3, 4]]), :x)\n    iex> Nx.reduce_max(t, axes: [0], keep_axes: true)\n    #Nx.Tensor","ref":"Nx.html#reduce_max/2-vectorized-tensors"},{"type":"function","title":"Nx.reduce_min/2","doc":"Returns the minimum values of the tensor.\n\nIf the `:axes` option is given, it aggregates over\nthe given dimensions, effectively removing them.\n`axes: [0]` implies aggregating over the highest order\ndimension and so forth. If the axis is negative, then\ncounts the axis from the back. For example, `axes: [-1]`\nwill always aggregate all rows.\n\nYou may optionally set `:keep_axes` to true, which will\nretain the rank of the input tensor by setting the reduced\naxes to size 1.","ref":"Nx.html#reduce_min/2"},{"type":"function","title":"Examples - Nx.reduce_min/2","doc":"iex> Nx.reduce_min(Nx.tensor(42))\n    #Nx.Tensor \n\n    iex> Nx.reduce_min(Nx.tensor(42.0))\n    #Nx.Tensor \n\n    iex> Nx.reduce_min(Nx.tensor([1, 2, 3]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#reduce_min/2-examples"},{"type":"function","title":"Aggregating over an axis - Nx.reduce_min/2","doc":"iex> t = Nx.tensor([[3, 1, 4], [2, 1, 1]], names: [:x, :y])\n    iex> Nx.reduce_min(t, axes: [:x])\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[3, 1, 4], [2, 1, 1]], names: [:x, :y])\n    iex> Nx.reduce_min(t, axes: [:y])\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[1, 2], [4, 5]], [[2, 4], [3, 8]]], names: [:x, :y, :z])\n    iex> Nx.reduce_min(t, axes: [:x, :z])\n    #Nx.Tensor \n\n#","ref":"Nx.html#reduce_min/2-aggregating-over-an-axis"},{"type":"function","title":"Keeping axes - Nx.reduce_min/2","doc":"iex> t = Nx.tensor([[[1, 2], [4, 5]], [[2, 4], [3, 8]]], names: [:x, :y, :z])\n    iex> Nx.reduce_min(t, axes: [:x, :z], keep_axes: true)\n    #Nx.Tensor \n\n#","ref":"Nx.html#reduce_min/2-keeping-axes"},{"type":"function","title":"Vectorized tensors - Nx.reduce_min/2","doc":"iex> t = Nx.vectorize(Nx.tensor([[1, 2], [3, 4]]), :x)\n    iex> Nx.reduce_min(t, axes: [0], keep_axes: true)\n    #Nx.Tensor","ref":"Nx.html#reduce_min/2-vectorized-tensors"},{"type":"function","title":"Nx.reflect/2","doc":"Pads a tensor of rank 1 or greater along the given axes through periodic reflections.","ref":"Nx.html#reflect/2"},{"type":"function","title":"Options - Nx.reflect/2","doc":"* `:padding_config` - A list of tuples in the format `{pre, post}`,\n    which specify the length (0 or greater) of the reflection before and\n    after the tensor along a each axis.\n\nSee also: `pad/3`","ref":"Nx.html#reflect/2-options"},{"type":"function","title":"Examples - Nx.reflect/2","doc":"iex> Nx.reflect(Nx.tensor([0, 1, 2]), padding_config: [{3, 1}])\n    #Nx.Tensor \n\n    iex> Nx.reflect(Nx.tensor([[0, 1, 2], [3, 4, 5]], names: [:x, :y]), padding_config: [{2, 0}, {2, 1}])\n    #Nx.Tensor","ref":"Nx.html#reflect/2-examples"},{"type":"function","title":"Nx.remainder/2","doc":"Element-wise remainder of two tensors.\n\nIf a number is given, it is converted to a tensor.\n\nIt will broadcast tensors whenever the dimensions do\nnot match and broadcasting is possible.\n\nIf you're using `Nx.Defn.defn/2`, you can use the `rem/2` function\nin place of this function: `rem(left, right)`.","ref":"Nx.html#remainder/2"},{"type":"function","title":"Examples - Nx.remainder/2","doc":"#","ref":"Nx.html#remainder/2-examples"},{"type":"function","title":"Remainder of scalars - Nx.remainder/2","doc":"iex> Nx.remainder(1, 2)\n    #Nx.Tensor \n\n#","ref":"Nx.html#remainder/2-remainder-of-scalars"},{"type":"function","title":"Remainder of tensors and scalars - Nx.remainder/2","doc":"iex> Nx.remainder(Nx.tensor([1, 2, 3], names: [:data]), 2)\n    #Nx.Tensor \n\n    iex> Nx.remainder(2, Nx.tensor([1.0, 2.0, 3.0], names: [:data]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#remainder/2-remainder-of-tensors-and-scalars"},{"type":"function","title":"Remainder of tensors - Nx.remainder/2","doc":"iex> left = Nx.tensor([[10], [20]], names: [:x, :y])\n    iex> right = Nx.tensor([[3, 4]], names: [nil, :y])\n    iex> Nx.remainder(left, right)\n    #Nx.Tensor \n\n#","ref":"Nx.html#remainder/2-remainder-of-tensors"},{"type":"function","title":"Remainder involving negative values - Nx.remainder/2","doc":"If given a negative value as the right operand, the operation\nwill return the negative image of the remainder.\n\nFor the example below, note that in modulo-10, adding 20 shouldn't\nchange the result, but in this case it does because the sign changes.\n\n    iex> left = Nx.tensor(-11, type: :s8)\n    iex> right = Nx.tensor(10, type: :u8)\n    iex> Nx.remainder(left, right)\n    #Nx.Tensor \n    iex> Nx.remainder(Nx.add(left, Nx.tensor(20, type: :s8)), right)\n    #Nx.Tensor \n    iex> positive_left = Nx.tensor(9, type: :u8)\n    iex> Nx.remainder(positive_left, right)\n    #Nx.Tensor \n    iex> Nx.remainder(Nx.add(positive_left, Nx.tensor(20, type: :u8)), right)\n    #Nx.Tensor","ref":"Nx.html#remainder/2-remainder-involving-negative-values"},{"type":"function","title":"Nx.rename/2","doc":"Adds (or overrides) the given names to the tensor.","ref":"Nx.html#rename/2"},{"type":"function","title":"Examples - Nx.rename/2","doc":"iex> Nx.rename(Nx.iota({2, 3}), [:foo, :bar])\n    #Nx.Tensor","ref":"Nx.html#rename/2-examples"},{"type":"function","title":"Vectorized tensors - Nx.rename/2","doc":"Only the inner axis names are renamed. New names must not overlap with\nvectorized names.\n\n    iex> t = Nx.tensor([[1], [2], [3]], names: [nil, :y]) |> Nx.vectorize(:x)\n    iex> Nx.rename(t, [:a])\n    #Nx.Tensor \n    iex> Nx.rename(t, [:x])\n    ** (ArgumentError) name :x is already a name for a vectorized axis","ref":"Nx.html#rename/2-vectorized-tensors"},{"type":"function","title":"Nx.reshape/3","doc":"Changes the shape of a tensor.\n\nThe new shape is either a tuple or a tensor which we will\nretrieve the current shape from. The shapes must be compatible:\nthe product of each dimension in the shape must be equal.\n\nYou may specify one of the dimensions as `:auto`. Nx will compute\nthe size of the dimension based on the original shape and new shape.\n\nReshaping only changes the tensor metadata, it doesn't copy\nthe underlying structure.\n\nReshape is a destructive operation with respect to names. You\ncan optionally provide `:names` for each of the dimensions\nin the reshaped tensor. If you do not provide `:names`, they\nwill be taken from the tensor the shape is taken from or\nall of the dimension names will be set to `nil`.","ref":"Nx.html#reshape/3"},{"type":"function","title":"Examples - Nx.reshape/3","doc":"iex> t = Nx.tensor([1, 2, 3, 4], names: [:x])\n    iex> Nx.reshape(t, {2, 2}, names: [:x, :y])\n    #Nx.Tensor \n\nThe shape can also be an existing tensor:\n\n    iex> shape = Nx.tensor([[0], [0], [0], [0]], names: [:x, :y])\n    iex> Nx.reshape(Nx.tensor([1, 2, 3, 4]), shape)\n    #Nx.Tensor \n\nEven a scalar can be transformed into a 3-dimensional tensor:\n\n    iex> t = Nx.tensor(1)\n    iex> Nx.reshape(t, {1, 1, 1}, names: [:x, :y, :z])\n    #Nx.Tensor \n\nYou can use `:auto` to infer dimension sizes. This is useful when you\ndon't know the size some dimension should be ahead of time:\n\n    iex> t = Nx.tensor([[1, 2, 3], [4, 5, 6]])\n    iex> Nx.reshape(t, {:auto, 2}, names: [:x, :y])\n    #Nx.Tensor","ref":"Nx.html#reshape/3-examples"},{"type":"function","title":"Vectorized tensors - Nx.reshape/3","doc":"Vectorized tensors have their inner shapes changed, keeping vectors unchanged.\n\n    iex> t = Nx.tensor([[[1, 2, 3], [4, 5, 6]]]) |> Nx.vectorize(:x)\n    iex> t.shape\n    {2, 3}\n    iex> Nx.reshape(t, {3, 2})\n    #Nx.Tensor","ref":"Nx.html#reshape/3-vectorized-tensors"},{"type":"function","title":"Nx.reshape_vectors/2","doc":"Reshapes input tensors so that they are all vectorized with the same vectors.\n\nFor vectors with the same name to be compatible, they need to either\nhave the same size or one must be of size 1.","ref":"Nx.html#reshape_vectors/2"},{"type":"function","title":"Options - Nx.reshape_vectors/2","doc":"* `:align_ranks` - boolean that indicates whether the inner\n    shapes should be aligned to the maximum rank of the inputs.\n    That is, 1-sized leading dimensions are added so\n    that all tensors have the same rank in the output.\n    This only applies in case one of the inputs is vectorized.","ref":"Nx.html#reshape_vectors/2-options"},{"type":"function","title":"Examples - Nx.reshape_vectors/2","doc":"Two vectors of the same name are compatible if they have the same sizes or if either has size 1.\n\n    iex> x = Nx.tensor([1, 2, 3]) |> Nx.vectorize(:x)\n    iex> xy = Nx.tensor([[[5]], [[6]]]) |> Nx.vectorize(:y) |> Nx.vectorize(:x)\n    iex> [x, xy] = Nx.reshape_vectors([x, xy])\n    iex> x.vectorized_axes\n    [x: 3, y: 1]\n    iex> xy.vectorized_axes\n    [x: 1, y: 2]\n\nThe resulting tensors will all present the combined vectors in the\nsame order in which each unique vector appears in the input.\nThe example below shows how this behaves for a pair of tensors.\n\n    iex> x = Nx.tensor([1, 2, 3]) |> Nx.vectorize(:x)\n    iex> y = Nx.tensor([4]) |> Nx.vectorize(:y)\n    iex> [xv, yv] = Nx.reshape_vectors([x, y])\n    iex> xv.vectorized_axes\n    [x: 3, y: 1]\n    iex> yv.vectorized_axes\n    [x: 1, y: 1]\n    iex> [yv, xv] = Nx.reshape_vectors([y, x])\n    iex> xv.vectorized_axes\n    [y: 1, x: 3]\n    iex> yv.vectorized_axes\n    [y: 1, x: 1]\n\nThe `:align_ranks` option controls whether the resulting tensors should end up\nwith the same rank, which helps with broadcasting in some cases.\n\n    iex> x = 1\n    iex> y = Nx.tensor([[[1], [1]], [[2], [2]], [[3], [3]]]) |> Nx.vectorize(:y)\n    iex> [xv, yv] = Nx.reshape_vectors([x, y])\n    iex> xv\n    #Nx.Tensor \n    iex> yv\n    #Nx.Tensor \n    iex> [xv, _yv] = Nx.reshape_vectors([x, y], align_ranks: true)\n    iex> xv\n    #Nx.Tensor","ref":"Nx.html#reshape_vectors/2-examples"},{"type":"function","title":"Nx.revectorize/3","doc":"Changes the disposition of the vectorized axes of a tensor or `Nx.Container`.\n\nThis function is basically a short-hand for:\n\n    tensor\n    |> Nx.devectorize(keep_names: false)\n    |> Nx.reshape(vectorized_sizes ++ target_shape, names: target_names)\n    |> Nx.vectorize(vectorized_names)\n\nAccepts the `target_axes` keyword list where the total size must match the current total\nsize of the vectorized axes.\n\nBetween `target_axes` and the `:target_shape` option, there can be at most one `:auto` entry.\n\n#","ref":"Nx.html#revectorize/3"},{"type":"function","title":"Options - Nx.revectorize/3","doc":"* `:target_shape` - the (non-vectorized) output shape.\n  * `:target_names` - the names for the output shape.\n\n#","ref":"Nx.html#revectorize/3-options"},{"type":"function","title":"Examples - Nx.revectorize/3","doc":"iex> t = Nx.iota({1}, vectorized_axes: [x: 2, y: 3, z: 4])\n    iex> t2 = Nx.revectorize(t, x: 12, y: :auto)\n    iex> t2.vectorized_axes\n    [x: 12, y: 2]\n    iex> t3 = Nx.revectorize(t, a: :auto)\n    iex> t3.vectorized_axes\n    [a: 24]\n\nAlso works on containers. Note that the revectorization happens on a per-entry basis.\n\n    iex> t1  = Nx.iota({1}, vectorized_axes: [x: 2, y: 3])\n    iex> t2 = Nx.iota({1}, vectorized_axes: [x: 2, y: 1])\n    iex> {r1, r2} = Nx.revectorize({t1, t2}, a: :auto)\n    iex> r1.vectorized_axes\n    [a: 6]\n    iex> r2.vectorized_axes\n    [a: 2]\n\nThis function is useful for when you need to introduce a temporary custom axis to ease calculations.\nThe example below shows how to manipulate your vectorized tensor for that objective.\n\n    iex> t = Nx.iota({2, 2, 2}) |> Nx.vectorize(x: 2, y: 2)\n    #Nx.Tensor \n    iex> Nx.revectorize(t, temp: :auto, x: 2) # Note that if we don't pass `:target_shape`, `:auto` will only act upon the vectorized axes\n    #Nx.Tensor \n    iex> revec = Nx.revectorize(t, [temp: :auto, x: 2], target_shape: {})\n    #Nx.Tensor \n    iex> Nx.revectorize(revec, [new_vec: 2], target_shape: {1, 4}, target_names: [:x, :last])\n    #Nx.Tensor \n\nNote how in the last example the `:x` name could be reused in various positions\n(both vectorized and non-vectorized), because `revectorize/2` ensures that the\nnames are rewritten at each call.","ref":"Nx.html#revectorize/3-examples"},{"type":"function","title":"Nx.reverse/2","doc":"Reverses the tensor in the given dimensions.\n\nIf no axes are provided, reverses every axis.\n\nYou can pass either names or numbers for the reverse\ndimensions. Dimensions must be unique, but they do not\nhave to be successive.","ref":"Nx.html#reverse/2"},{"type":"function","title":"Examples - Nx.reverse/2","doc":"iex> Nx.reverse(Nx.tensor([1, 2, 3]))\n    #Nx.Tensor \n\n    iex> Nx.reverse(Nx.tensor([[1, 2, 3], [4, 5, 6]]))\n    #Nx.Tensor \n\n    iex> Nx.reverse(Nx.tensor([1, 2, 3], names: [:x]), axes: [:x])\n    #Nx.Tensor \n\n    iex> Nx.reverse(Nx.tensor([[1, 2, 3], [4, 5, 6]], names: [:x, :y]), axes: [:x])\n    #Nx.Tensor \n\n    iex> Nx.reverse(Nx.tensor([[1, 2, 3], [4, 5, 6]], names: [:x, :y]), axes: [:y])\n    #Nx.Tensor \n\n    iex> Nx.reverse(Nx.iota({2, 2, 2}, type: :f32, names: [:x, :y, :z]), axes: [:x, :z])\n    #Nx.Tensor \n\n#","ref":"Nx.html#reverse/2-examples"},{"type":"function","title":"Vectorized tensors - Nx.reverse/2","doc":"For vectorized tensors, the `:axes` refer to the non-vectorized part.\nVectorized axes will always remain unchanged.\n\n    iex> v = Nx.vectorize(Nx.iota({1, 2, 3}), :x)\n    #Nx.Tensor \n    iex> Nx.reverse(v)\n    #Nx.Tensor \n    iex> Nx.reverse(v, axes: [1])\n    #Nx.Tensor","ref":"Nx.html#reverse/2-vectorized-tensors"},{"type":"function","title":"Nx.right_shift/2","doc":"Element-wise right shift of two tensors.\n\nOnly integer tensors are supported. If a float or complex\ntensor is given, an error is raised. If the right side\nis negative, it will raise, but it may trigger undefined\nbehaviour on some compilers.\n\nIt performs an arithmetic shift if the tensor is made of\nsigned integers, it performs a logical shift otherwise.\nIn other words, it preserves the sign for signed integers.\n\nIt will broadcast tensors whenever the dimensions do\nnot match and broadcasting is possible. If the number of\nshifts are negative, Nx's default backend will raise,\nbut it may trigger undefined behaviour in other backends.\n\nIf you're using `Nx.Defn.defn/2`, you can use the `>>>` operator\nin place of this function: `left >>> right`.","ref":"Nx.html#right_shift/2"},{"type":"function","title":"Examples - Nx.right_shift/2","doc":"#","ref":"Nx.html#right_shift/2-examples"},{"type":"function","title":"Right shift between scalars - Nx.right_shift/2","doc":"iex> Nx.right_shift(1, 0)\n    #Nx.Tensor \n\n#","ref":"Nx.html#right_shift/2-right-shift-between-scalars"},{"type":"function","title":"Right shift between tensors and scalars - Nx.right_shift/2","doc":"iex> Nx.right_shift(Nx.tensor([2, 4, 8], names: [:data]), 2)\n    #Nx.Tensor \n\n#","ref":"Nx.html#right_shift/2-right-shift-between-tensors-and-scalars"},{"type":"function","title":"Right shift between tensors - Nx.right_shift/2","doc":"iex> left = Nx.tensor([16, 32, -64, -128], names: [:data])\n    iex> right = Nx.tensor([1, 2, 3, 4])\n    iex> Nx.right_shift(left, right)\n    #Nx.Tensor","ref":"Nx.html#right_shift/2-right-shift-between-tensors"},{"type":"function","title":"Error cases - Nx.right_shift/2","doc":"iex> Nx.right_shift(Nx.tensor([0, 0, 1, 1]), 1.0)\n    ** (ArgumentError) bitwise operators expect integer tensors as inputs and outputs an integer tensor, got: {:f, 32}","ref":"Nx.html#right_shift/2-error-cases"},{"type":"function","title":"Nx.round/1","doc":"Calculates the round (away from zero) of each element in the tensor.\n\nIf a non-floating tensor is given, it is returned as is.\nIf a floating tensor is given, then we apply the operation,\nbut keep its type.","ref":"Nx.html#round/1"},{"type":"function","title":"Examples - Nx.round/1","doc":"iex> Nx.round(Nx.tensor([-1, 0, 1], names: [:x]))\n    #Nx.Tensor \n\n    iex> Nx.round(Nx.tensor([-1.5, -0.5, 0.5, 1.5], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#round/1-examples"},{"type":"function","title":"Nx.rsqrt/1","doc":"Calculates the reverse square root of each element in the tensor.\n\nIt is equivalent to:\n\n$$rsqrt(z) = \\frac{1}{\\sqrt{z}}$$","ref":"Nx.html#rsqrt/1"},{"type":"function","title":"Examples - Nx.rsqrt/1","doc":"iex> Nx.rsqrt(1)\n    #Nx.Tensor \n\n    iex> Nx.rsqrt(Nx.tensor([1, 2, 3], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#rsqrt/1-examples"},{"type":"function","title":"Nx.s8/1","doc":"Short-hand function for creating tensor of type `s8`.\n\nThis is just an alias for `Nx.tensor(tensor, type: s8)`.","ref":"Nx.html#s8/1"},{"type":"function","title":"Nx.s16/1","doc":"Short-hand function for creating tensor of type `s16`.\n\nThis is just an alias for `Nx.tensor(tensor, type: s16)`.","ref":"Nx.html#s16/1"},{"type":"function","title":"Nx.s32/1","doc":"Short-hand function for creating tensor of type `s32`.\n\nThis is just an alias for `Nx.tensor(tensor, type: s32)`.","ref":"Nx.html#s32/1"},{"type":"function","title":"Nx.s64/1","doc":"Short-hand function for creating tensor of type `s64`.\n\nThis is just an alias for `Nx.tensor(tensor, type: s64)`.","ref":"Nx.html#s64/1"},{"type":"function","title":"Nx.select/3","doc":"Constructs a tensor from two tensors, based on a predicate.\n\nThe resulting tensor is built by evaluating each element of\n`pred` and returning either the corresponding element from\n`on_true` or `on_false`.\n\n`pred` must either be `1` or `0` or a tensor of predicates\nwith a shape that matches the largest shape between `s1` or `s2`.\n\nIf the shape of `on_true` or `on_false` do not match the shape of\n`pred`, attempts to broadcast both so they match the shape of `pred`.","ref":"Nx.html#select/3"},{"type":"function","title":"Examples - Nx.select/3","doc":"When the first argument is a scalar:\n\n    iex> Nx.select(1, Nx.tensor([1, 2, 3], names: [:x]), Nx.tensor([4, 5, 6], names: [:x]))\n    #Nx.Tensor \n\n    iex> Nx.select(0, Nx.tensor([1, 2, 3], names: [:y]), Nx.tensor([4, 5, 6], names: [:y]))\n    #Nx.Tensor \n\n    iex> Nx.select(0, Nx.tensor([[1, 2]], names: [:x, :y]), Nx.tensor([[3], [4]], names: [:x, :y]))\n    #Nx.Tensor \n\nWhen the first argument is a tensor:\n\n    iex> Nx.select(Nx.tensor([0, 1, 0], names: [:x]), Nx.tensor([1, 2, 3], names: [:y]), Nx.tensor([4, 5, 6], names: [:z]))\n    #Nx.Tensor \n\n    iex> x = Nx.tensor([2, 4, 6], names: [:x])\n    iex> y = Nx.tensor([3, 2, 1])\n    iex> Nx.select(Nx.greater(x, y), Nx.tensor([2, 4, 6], names: [:i]), Nx.tensor([1, 3, 5], names: [:j]))\n    #Nx.Tensor \n\n    iex> x = Nx.tensor([2, 4, 6, 8, 10], names: [:x])\n    iex> y = Nx.tensor([1, 6, 2, 11, 2], names: [:x])\n    iex> Nx.select(Nx.greater(x, y), Nx.tensor(2), Nx.tensor([1, 3, 5, 7, 9], names: [:x]))\n    #Nx.Tensor \n\nIf the tensor has other values, any non-zero value is considered true:\n\n    iex> Nx.select(Nx.tensor([0, 1, 2], type: :u8), Nx.tensor([0, 0, 0]), Nx.tensor([1, 1, 1]))\n    #Nx.Tensor \n\n    iex> Nx.select(Nx.tensor([0, 1, 0]), Nx.tensor([1, 1, 1]), Nx.tensor([2.0, 2.0, 2.0]))\n    #Nx.Tensor","ref":"Nx.html#select/3-examples"},{"type":"function","title":"Vectorized tensors - Nx.select/3","doc":"Vectorized and non-vectorized tensors can be mixed-and-matched on all three inputs.\n\n    iex> pred = Nx.tensor([[0, 1, 0], [1, 1, 0]]) |> Nx.vectorize(:x)\n    iex> on_true = 1\n    iex> on_false = Nx.tensor([2, 3]) |> Nx.vectorize(:y)\n    iex> Nx.select(pred, on_true, on_false)\n    #Nx.Tensor \n\n  In the next example, notice that even though the `pred` input\n  is scalar, because we're dealing with vectorized inputs, some\n  broadcasting still occurs.\n\n    iex> pred = 1\n    iex> on_true = Nx.tensor([1, 2, 3]) |> Nx.vectorize(:x)\n    iex> on_false = Nx.tensor([4, 5]) |> Nx.vectorize(:y)\n    iex> Nx.select(pred, on_true, on_false)\n    #Nx.Tensor \n\nFinally, broadcasting will also occur if more than one input share\nthe same vectorized axes, but one of them presents size 1\n\n    iex> pred = Nx.tensor([1, 0, 0]) |> Nx.vectorize(:x)\n    iex> on_true = Nx.tensor([[2]]) |> Nx.vectorize(:x) |> Nx.vectorize(:y)\n    iex> on_false = Nx.tensor([3, 4]) |> Nx.vectorize(:y)\n    iex> Nx.select(pred, on_true, on_false)\n    #Nx.Tensor","ref":"Nx.html#select/3-vectorized-tensors"},{"type":"function","title":"Nx.serialize/2","doc":"Serializes the given tensor or container of tensors to iodata.\n\nYou may pass any tensor or `Nx.Container` to serialization.\nOpposite to other functions in this module, `Nx.LazyContainer`\ncannot be serialized and they must be explicitly converted\nto tensors before (that's because lazy containers do not preserve\ntheir shape).\n\n`opts` controls the serialization options. For example, you can choose\nto compress the given tensor or container of tensors by passing a\ncompression level:\n\n    Nx.serialize(tensor, compressed: 9)\n\nCompression level corresponds to compression options in `:erlang.term_to_iovec/2`.\n\n`iodata` is a list of binaries that can be written to any io device,\nsuch as a file or a socket. You can ensure the result is a binary by\ncalling `IO.iodata_to_binary/1`.\n\nNote: This function cannot be used in `defn`.","ref":"Nx.html#serialize/2"},{"type":"function","title":"Examples - Nx.serialize/2","doc":"iex> a = Nx.tensor([1, 2, 3])\n    iex> serialized_a = Nx.serialize(a)\n    iex> Nx.deserialize(serialized_a)\n    #Nx.Tensor \n\n    iex> container = {Nx.tensor([1, 2, 3]), %{b: Nx.tensor([4, 5, 6])}}\n    iex> serialized_container = Nx.serialize(container)\n    iex> {a, %{b: b}} = Nx.deserialize(serialized_container)\n    iex> a\n    #Nx.Tensor \n    iex> b\n    #Nx.Tensor","ref":"Nx.html#serialize/2-examples"},{"type":"function","title":"Nx.shape/1","doc":"Returns the shape of the tensor as a tuple.\n\nThe size of this tuple gives the rank of the tensor.\n\nIf a shape as a tuple is given, it returns the shape itself.","ref":"Nx.html#shape/1"},{"type":"function","title":"Examples - Nx.shape/1","doc":"iex> Nx.shape(Nx.tensor(1))\n    {}\n\n    iex> Nx.shape(Nx.tensor([[1, 2, 3], [4, 5, 6]]))\n    {2, 3}\n\n    iex> Nx.shape(1)\n    {}\n\n    iex> Nx.shape({1, 2, 3})\n    {1, 2, 3}","ref":"Nx.html#shape/1-examples"},{"type":"macro","title":"Nx.sigil_M/2","doc":"A convenient `~M` sigil for building matrices (two-dimensional tensors).","ref":"Nx.html#sigil_M/2"},{"type":"macro","title":"Examples - Nx.sigil_M/2","doc":"Before using sigils, you must first import them:\n\n    import Nx, only: :sigils\n\nThen you use the sigil to create matrices. The sigil:\n\n    ~M \n\nIs equivalent to:\n\n    Nx.tensor([\n      [-1, 0, 0, 1],\n      [0, 2, 0, 0],\n      [0, 0, 3, 0],\n      [0, 0, 0, 4]\n    ])\n\nIf the tensor has any complex type, it defaults to c64.\nIf the tensor has any float type, it defaults to f32.\nOtherwise, it is s64. You can specify the tensor type\nas a sigil modifier:\n\n    iex> import Nx, only: :sigils\n    iex> ~M[0.1 0.2 0.3 0.4]f16\n    #Nx.Tensor \n    iex> ~M[1+1i 2-2.0i -3]\n    #Nx.Tensor \n    iex> ~M[1 Inf NaN]\n    #Nx.Tensor \n    iex> ~M[1i Inf NaN]\n    #Nx.Tensor \n    iex> ~M[1i Inf+2i NaN-Infi]\n    #Nx.Tensor","ref":"Nx.html#sigil_M/2-examples"},{"type":"macro","title":"Nx.sigil_V/2","doc":"A convenient `~V` sigil for building vectors (one-dimensional tensors).","ref":"Nx.html#sigil_V/2"},{"type":"macro","title":"Examples - Nx.sigil_V/2","doc":"Before using sigils, you must first import them:\n\n    import Nx, only: :sigils\n\nThen you use the sigil to create vectors. The sigil:\n\n    ~V[-1 0 0 1]\n\nIs equivalent to:\n\n    Nx.tensor([-1, 0, 0, 1])\n\nIf the tensor has any complex type, it defaults to c64.\nIf the tensor has any float type, it defaults to f32.\nOtherwise, it is s64. You can specify the tensor type\nas a sigil modifier:\n\n    iex> import Nx, only: :sigils\n    iex> ~V[0.1 0.2 0.3 0.4]f16\n    #Nx.Tensor \n    iex> ~V[1+1i 2-2.0i -3]\n    #Nx.Tensor \n    iex> ~V[1 Inf NaN]\n    #Nx.Tensor \n    iex> ~V[1i Inf NaN]\n    #Nx.Tensor \n    iex> ~V[1i Inf+2i NaN-Infi]\n    #Nx.Tensor","ref":"Nx.html#sigil_V/2-examples"},{"type":"function","title":"Nx.sigmoid/1","doc":"Calculates the sigmoid of each element in the tensor.\n\nIt is equivalent to:\n\n$$sigmoid(z) = \\frac{1}{1 + e^{-z}}$$","ref":"Nx.html#sigmoid/1"},{"type":"function","title":"Examples - Nx.sigmoid/1","doc":"iex> Nx.sigmoid(1)\n    #Nx.Tensor \n\n    iex> Nx.sigmoid(Nx.tensor([1, 2, 3], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#sigmoid/1-examples"},{"type":"function","title":"Nx.sign/1","doc":"Computes the sign of each element in the tensor.\n\nIf a number is less than zero, it returns -1.\nIf a number is more than zero, it returns 1.\nOtherwise it returns zero (which may either be\npositive or negative for floats).","ref":"Nx.html#sign/1"},{"type":"function","title":"Examples - Nx.sign/1","doc":"iex> Nx.sign(Nx.tensor([-2, -1, 0, 1, 2], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#sign/1-examples"},{"type":"function","title":"Nx.sin/1","doc":"Calculates the sine of each element in the tensor.\n\nIt is equivalent to:\n\n$$sin(z) = \\frac{e^{iz} - e^{-iz}}{2i}$$","ref":"Nx.html#sin/1"},{"type":"function","title":"Examples - Nx.sin/1","doc":"iex> Nx.sin(1)\n    #Nx.Tensor \n\n    iex> Nx.sin(Nx.tensor([1, 2, 3], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#sin/1-examples"},{"type":"function","title":"Nx.sinh/1","doc":"Calculates the hyperbolic sine of each element in the tensor.\n\nIt is equivalent to:\n\n$$sinh(z) = \\frac{e^z - e^{-z}}{2}$$","ref":"Nx.html#sinh/1"},{"type":"function","title":"Examples - Nx.sinh/1","doc":"iex> Nx.sinh(1)\n    #Nx.Tensor \n\n    iex> Nx.sinh(Nx.tensor([1, 2, 3], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#sinh/1-examples"},{"type":"function","title":"Nx.size/1","doc":"Returns the number of elements in the tensor.\n\nIf a tuple is given, it returns the number of elements in a tensor with that shape.\nVectorized tensors will not include vectorized axes sizes. See `flat_size/1`.","ref":"Nx.html#size/1"},{"type":"function","title":"Examples - Nx.size/1","doc":"iex> Nx.size(Nx.tensor([[1, 2, 3], [4, 5, 6]]))\n    6\n\n    iex> Nx.size(1)\n    1\n\n    iex> Nx.size({1, 2, 3, 2})\n    12\n\n    iex> Nx.size(Nx.vectorize(Nx.iota({4, 3, 2}), :x))\n    6","ref":"Nx.html#size/1-examples"},{"type":"function","title":"Nx.slice/4","doc":"Slices a tensor from `start_indices` with `lengths`.\n\nYou can optionally provide a `stride` to specify the amount\nof stride in each dimension.\n\nBoth start indices and lengths must match the rank of the\ninput tensor shape. All start indexes must be greater than\nor equal to zero. All lengths must be strictly greater than\nzero. If `start_index + length` exceeds the tensor dimension,\nthe `start_index` will be clipped in order to guarantee the\n`length` is the requested one. See the \"Clipping\" section below.\n\nIt is possible for `start_indices` to be a list of tensors.\nHowever, `lengths` must always be a list of integers. If you\nwant to specify a tensor as the list of indices, see `take/3`.\n\nIf the `:strides` is given, it must be strictly greater than zero.\nThe resulting tensor will have the shape of `length` unless\n`:strides` are given.\n\nIt is not possible to slice in reverse. See `gather/2`,\n`slice_along_axis/4`, `take/3`, and `take_along_axis/3` for other ways\nto retrieve values from a tensor.","ref":"Nx.html#slice/4"},{"type":"function","title":"Examples - Nx.slice/4","doc":"iex> Nx.slice(Nx.tensor([1, 2, 3, 4, 5, 6]), [0], [3])\n    #Nx.Tensor \n\n    iex> Nx.slice(Nx.tensor([1, 2, 3, 4, 5, 6]), [0], [6], strides: [2])\n    #Nx.Tensor \n\n    iex> Nx.slice(Nx.tensor([[1, 2], [3, 4], [5, 6]]), [0, 0], [3, 2], strides: [2, 1])\n    #Nx.Tensor \n\nStrides can also be a number that applies to all dimensions:\n\n    iex> t = Nx.tensor([[1, 2], [3, 4], [5, 6]])\n    iex> Nx.slice(t, [0, 0], [3, 2], strides: 2)\n    #Nx.Tensor \n\nA more complex example:\n\n    iex> t = Nx.iota({900})\n    iex> t = Nx.reshape(t, {2, 15, 30})\n    iex> Nx.slice(t, [0, 4, 11], [2, 3, 9], strides: [2, 1, 3])\n    #Nx.Tensor","ref":"Nx.html#slice/4-examples"},{"type":"function","title":"Tensors as `start_indices` - Nx.slice/4","doc":"The `start_indices` list can be made of scalar tensors:\n\n    iex> Nx.slice(Nx.tensor([[1, 2, 3], [4, 5, 6]]), [Nx.tensor(1), Nx.tensor(2)], [1, 1])\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([\n    ...>   [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],\n    ...>   [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],\n    ...>   [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],\n    ...>   [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],\n    ...>   [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],\n    ...>   [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0]\n    ...> ])\n    iex> Nx.slice(t, [Nx.tensor(0), Nx.tensor(0)], [6, 7], strides: [5, 3])\n    #Nx.Tensor","ref":"Nx.html#slice/4-tensors-as-start_indices"},{"type":"function","title":"Clipping - Nx.slice/4","doc":"`slice/3` will always guarantee the return tensor has the\ngiven `lengths`. See the following example:\n\n    iex> Nx.slice(Nx.iota({3, 3}), [2, 2], [1, 1])\n    #Nx.Tensor \n\nIn the example above, `start_index + length   Nx.slice(Nx.iota({3, 3}), [2, 2], [2, 2])\n    #Nx.Tensor \n\nThis also applies when the start index is given by tensors:\n\n    iex> Nx.slice(Nx.iota({3, 3}), [Nx.tensor(2), Nx.tensor(2)], [2, 2])\n    #Nx.Tensor","ref":"Nx.html#slice/4-clipping"},{"type":"function","title":"Vectorized tensors - Nx.slice/4","doc":"Both the tensor to be sliced and the indices can be vectorized.\n\n    iex> Nx.slice(Nx.iota({3, 3}, vectorized_axes: [x: 2]), [0, Nx.tensor(1)], [2, 2])\n    #Nx.Tensor \n\n    iex> idx = Nx.tensor([0, 1, 10]) |> Nx.vectorize(:i)\n    iex> Nx.slice(Nx.iota({3, 3}), [0, idx], [2, 2])\n    #Nx.Tensor","ref":"Nx.html#slice/4-vectorized-tensors"},{"type":"function","title":"Error cases - Nx.slice/4","doc":"iex> Nx.slice(Nx.tensor([[1, 2, 3], [4, 5, 6]]), [Nx.tensor([1, 2]), Nx.tensor(1)], [1, 1])\n    ** (ArgumentError) index must be scalar, got shape {2} for axis 0\n\n    iex> Nx.slice(Nx.tensor([[1, 2, 3], [4, 5, 6]]), [Nx.tensor(1.0), Nx.tensor(0)], [1, 1])\n    ** (ArgumentError) index must be integer type, got {:f, 32} for axis 0","ref":"Nx.html#slice/4-error-cases"},{"type":"function","title":"Nx.slice_along_axis/4","doc":"Slices a tensor along the given axis.\n\nYou can optionally provide a `stride` to specify the amount\nof stride in along the given dimension.\n\nStart index must be greater than or equal to zero. It can be an\ninteger or a scalar tensor. Length must be strictly greater than\nzero. `start_index + length` must not exceed the respective tensor\ndimension.\n\nThe axis will be normalized with the dimensions and names of the\ngiven tensor.\n\nIf the `:strides` is given, it must be strictly greater than zero.\n\nIt is not possible to slice in reverse. See `gather/2`, `slice/3`,\n`take/3`, and `take_along_axis/3` for other ways to retrieve values\nfrom a tensor.","ref":"Nx.html#slice_along_axis/4"},{"type":"function","title":"Options - Nx.slice_along_axis/4","doc":"* `:axis` - The axis along which to take the values from. Defaults to `0`.\n  * `:strides` - The stride to slice the axis along of. Defaults to `1`.","ref":"Nx.html#slice_along_axis/4-options"},{"type":"function","title":"Examples - Nx.slice_along_axis/4","doc":"iex> Nx.slice_along_axis(Nx.iota({5, 2}), 1, 2, axis: 0)\n    #Nx.Tensor \n\n    iex> Nx.slice_along_axis(Nx.iota({2, 5}), 1, 2, axis: 1)\n    #Nx.Tensor \n\n    iex> Nx.slice_along_axis(Nx.iota({2, 5}, names: [:x, :y]), 0, 1, axis: :x)\n    #Nx.Tensor \n\n    iex> Nx.slice_along_axis(Nx.iota({2, 5}, names: [:x, :y]), Nx.tensor(0), 1, axis: :x)\n    #Nx.Tensor \n\n    iex> Nx.slice_along_axis(Nx.iota({2, 5}), 0, 3, axis: -1, strides: 2)\n    #Nx.Tensor","ref":"Nx.html#slice_along_axis/4-examples"},{"type":"function","title":"Vectorized tensors - Nx.slice_along_axis/4","doc":"Slices are taken over each vectorized entry.\nThe `start_index` cannot be vectorized.\n\n    iex> t = Nx.iota({2, 5}, vectorized_axes: [x: 2])\n    iex> Nx.slice_along_axis(t, 0, 3, axis: 1, strides: 2)\n    #Nx.Tensor","ref":"Nx.html#slice_along_axis/4-vectorized-tensors"},{"type":"function","title":"Nx.sort/2","doc":"Sorts the tensor along the given axis according\nto the given direction.\n\nIf no axis is given, defaults to `0`.\n\n#","ref":"Nx.html#sort/2"},{"type":"function","title":"Options - Nx.sort/2","doc":"* `:axis` - The name or number of the corresponding axis on which the sort\n    should be applied\n  * `:direction` - Can be `:asc` or `:desc`. Defaults to `:asc`","ref":"Nx.html#sort/2-options"},{"type":"function","title":"Examples - Nx.sort/2","doc":"iex> Nx.sort(Nx.tensor([16, 23, 42, 4, 8, 15]))\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[3, 1, 7], [2, 5, 4]], names: [:x, :y])\n    iex> Nx.sort(t, axis: :x)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[3, 1, 7], [2, 5, 4]], names: [:x, :y])\n    iex> Nx.sort(t, axis: :y)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[3, 1, 7], [2, 5, 4]], names: [:x, :y])\n    iex> Nx.sort(t, axis: :y, direction: :asc)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor(\n    ...>   [\n    ...>     [[4, 5], [2, 5], [5, 0]],\n    ...>     [[1, 9], [2, 1], [2, 1]],\n    ...>     [[0, -1], [-1, 0], [0, -1]],\n    ...>     [[-1, 0], [0, -1], [-1, 0]]\n    ...>   ],\n    ...>   names: [:x, :y, :z]\n    ...> )\n    iex> Nx.sort(t, axis: :x)\n    #Nx.Tensor \n\nSame tensor sorted over different axes:\n\n    iex> t = Nx.tensor(\n    ...>   [\n    ...>     [\n    ...>       [4, 5, 2],\n    ...>       [2, 5, 3],\n    ...>       [5, 0, 2]\n    ...>     ],\n    ...>     [\n    ...>       [1, 9, 8],\n    ...>       [2, 1, 3],\n    ...>       [2, 1, 4]\n    ...>     ]\n    ...>   ],\n    ...>   names: [:x, :y, :z]\n    ...> )\n    iex> Nx.sort(t, axis: :x)\n    #Nx.Tensor \n    iex> Nx.sort(t, axis: :y)\n    #Nx.Tensor \n    iex> Nx.sort(t, axis: :z)\n    #Nx.Tensor","ref":"Nx.html#sort/2-examples"},{"type":"function","title":"Nx.split/3","doc":"Split a tensor into train and test subsets.\n\n`split` must be defined so that there are no empty result tensors.\nThis means that `split` must be:\n\n  * an integer such that `0 < split` and `split < axis_size`\n  * a float such that `0.0 < split` and `ceil(axis_size * split) < axis_size`","ref":"Nx.html#split/3"},{"type":"function","title":"Options - Nx.split/3","doc":"* `:axis` - The axis along which to split the tensor. Defaults to `0`.","ref":"Nx.html#split/3-options"},{"type":"function","title":"Examples - Nx.split/3","doc":"All examples will operate on the same tensor so that it's easier to compare different configurations.\n\n    iex> t = Nx.tensor([[0, 1, 2, 3], [4, 5, 6, 7], [8, 9, 10, 11]])\n    iex> {left, right} = Nx.split(t, 2, axis: 0)\n    iex> left\n    #Nx.Tensor \n    iex> right\n    #Nx.Tensor \n    iex> {left, right} = Nx.split(t, 2, axis: 1)\n    iex> left\n    #Nx.Tensor \n    iex> right\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[0, 1, 2, 3], [4, 5, 6, 7], [8, 9, 10, 11]])\n    iex> {left, right} = Nx.split(t, 0.5, axis: 0)\n    iex> left\n    #Nx.Tensor \n    iex> right\n    #Nx.Tensor \n    iex> {left, right} = Nx.split(t, 0.75, axis: 1)\n    iex> left\n    #Nx.Tensor \n    iex> right\n    #Nx.Tensor \n\nNegative indices are also accepted, in the same fashion as `Enum.split/2`.\n\n    iex> t = Nx.tensor([1, 2, 3, 4])\n    iex> {left, right} = Nx.split(t, -1)\n    iex> left\n    #Nx.Tensor \n    iex> right\n    #Nx.Tensor","ref":"Nx.html#split/3-examples"},{"type":"function","title":"Nx.sqrt/1","doc":"Calculates the square root of each element in the tensor.\n\nIt is equivalent to:\n\n$$sqrt(z) = \\sqrt{z}$$","ref":"Nx.html#sqrt/1"},{"type":"function","title":"Examples - Nx.sqrt/1","doc":"iex> Nx.sqrt(1)\n    #Nx.Tensor \n\n    iex> Nx.sqrt(Nx.tensor([1, 2, 3], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#sqrt/1-examples"},{"type":"function","title":"Nx.squeeze/2","doc":"Squeezes the given size `1` dimensions out of the tensor.\n\nIf no axes are given, squeezes all size `1` dimensions\nfrom the tensor.\n\nWhile this is equivalent to a reshape which eliminates\nthe size `1` axes, squeeze preserves important information\nabout which axes were squeezed out which can then be used\nlater on in transformations.","ref":"Nx.html#squeeze/2"},{"type":"function","title":"Examples - Nx.squeeze/2","doc":"iex> Nx.squeeze(Nx.tensor([[[[[1]]]]]))\n    #Nx.Tensor \n\n    iex> Nx.squeeze(Nx.tensor([[[[1]]], [[[2]]]], names: [:x, :y, :z, :i]))\n    #Nx.Tensor \n\n    iex> Nx.squeeze(Nx.tensor([[1, 2, 3]], names: [:x, :y]), axes: [:x])\n    #Nx.Tensor \n\n    iex> Nx.squeeze(Nx.tensor([[1], [2]], names: [:x, :y]), axes: [:y])\n    #Nx.Tensor","ref":"Nx.html#squeeze/2-examples"},{"type":"function","title":"Vectorized tensors - Nx.squeeze/2","doc":"`squeeze/2` operates on the tensor's shape, leaving vectorized axes untouched.\n\n    iex> t = Nx.tensor([[[[[1], [2], [3]]]]]) |> Nx.vectorize(:x)\n    #Nx.Tensor \n    iex> Nx.squeeze(t)\n    #Nx.Tensor \n    iex> Nx.squeeze(t, axes: [0, 1])\n    #Nx.Tensor","ref":"Nx.html#squeeze/2-vectorized-tensors"},{"type":"function","title":"Error cases - Nx.squeeze/2","doc":"iex> Nx.squeeze(Nx.tensor([[1, 2, 3], [4, 5, 6]]), axes: [1])\n    ** (ArgumentError) cannot squeeze dimensions whose sizes are not 1, got 3 for dimension 1\n\n    iex> Nx.squeeze(Nx.tensor([[[[[1]]]]]), axes: [0, 0])\n    ** (ArgumentError) axes [0, 0] must be unique integers between 0 and 4","ref":"Nx.html#squeeze/2-error-cases"},{"type":"function","title":"Nx.stack/2","doc":"Stacks a list of tensors with the same shape along a new axis.\n\nTensors can be a tuple or any `Nx.Container` or `Nx.LazyContainer`.\nThis means you can easily concatenate all columns in a dataframe\nand other data structures. For convenience, this function also allows\na list of tensors to be given, which may be common outside of `defn`.\n\nIf no axis is provided, defaults to 0. All tensors must have the same\nshape.\n\nIf tensors with mixed types are given, the types will\nbe merged to a higher type and all of the tensors will\nbe cast to the higher type before concatenating.\nIf tensors are named, the names must match.\n\n#","ref":"Nx.html#stack/2"},{"type":"function","title":"Options - Nx.stack/2","doc":"* `:axis` - optional index of the axis along which the tensors are stacked. Defaults to 0.\n  * `:name` - optional name for the added dimension. Defaults to an unnamed axis.","ref":"Nx.html#stack/2-options"},{"type":"function","title":"Examples - Nx.stack/2","doc":"Stacking always creates a new dimension:\n\n    iex> Nx.stack([1, 2, 3])\n    #Nx.Tensor \n\n    iex> Nx.stack([Nx.tensor([1, 2, 3]), Nx.tensor([4, 5, 6])])\n    #Nx.Tensor \n\nThe axis option can be given:\n\n    iex> t1 = Nx.iota({2, 1, 4})\n    iex> t2 = Nx.iota({2, 1, 4})\n    iex> t3 = Nx.iota({2, 1, 4})\n    iex> Nx.stack([t1, t2, t3], axis: -1)\n    #Nx.Tensor \n\nAnd a name can be given for the new dimension:\n\n    iex> Nx.stack([Nx.tensor(1), Nx.tensor(2)], name: :x)\n    #Nx.Tensor \n\nYou can also pass any container (or lazy container) as first argument\nand they are recursively traversed:\n\n    iex> Nx.stack({Nx.tensor([1, 2]), {Nx.tensor([3, 4]), Nx.tensor([5, 6])}})\n    #Nx.Tensor","ref":"Nx.html#stack/2-examples"},{"type":"function","title":"Nx.standard_deviation/2","doc":"Finds the standard deviation of a tensor.\n\nThe standard deviation is taken as the square root of the variance.\nIf the `:ddof` (delta degrees of freedom) option is given, the divisor\n`n - ddof` is used to calculate the variance. See `variance/2`.","ref":"Nx.html#standard_deviation/2"},{"type":"function","title":"Examples - Nx.standard_deviation/2","doc":"iex> Nx.standard_deviation(Nx.tensor([[1, 2], [3, 4]]))\n    #Nx.Tensor \n\n    iex> Nx.standard_deviation(Nx.tensor([[1, 2], [3, 4]]), ddof: 1)\n    #Nx.Tensor \n\n    iex> Nx.standard_deviation(Nx.tensor([[1, 2], [10, 20]]), axes: [0])\n    #Nx.Tensor \n\n    iex> Nx.standard_deviation(Nx.tensor([[1, 2], [10, 20]]), axes: [1])\n    #Nx.Tensor \n\n    iex> Nx.standard_deviation(Nx.tensor([[1, 2], [10, 20]]), axes: [0], ddof: 1)\n    #Nx.Tensor \n\n    iex> Nx.standard_deviation(Nx.tensor([[1, 2], [10, 20]]), axes: [1], ddof: 1)\n    #Nx.Tensor \n\n#","ref":"Nx.html#standard_deviation/2-examples"},{"type":"function","title":"Keeping axes - Nx.standard_deviation/2","doc":"iex> Nx.standard_deviation(Nx.tensor([[1, 2], [10, 20]]), keep_axes: true)\n    #Nx.Tensor \n\n#","ref":"Nx.html#standard_deviation/2-keeping-axes"},{"type":"function","title":"Vectorized tensors - Nx.standard_deviation/2","doc":"iex> Nx.standard_deviation(Nx.tensor([[1, 2], [0, 4]]) |> Nx.vectorize(:x))\n    #Nx.Tensor","ref":"Nx.html#standard_deviation/2-vectorized-tensors"},{"type":"function","title":"Nx.subtract/2","doc":"Element-wise subtraction of two tensors.\n\nIf a number is given, it is converted to a tensor.\n\nIt will broadcast tensors whenever the dimensions do\nnot match and broadcasting is possible.\n\nIf you're using `Nx.Defn.defn/2`, you can use the `-` operator\nin place of this function: `left - right`.","ref":"Nx.html#subtract/2"},{"type":"function","title":"Examples - Nx.subtract/2","doc":"#","ref":"Nx.html#subtract/2-examples"},{"type":"function","title":"Subtracting scalars - Nx.subtract/2","doc":"iex> Nx.subtract(1, 2)\n    #Nx.Tensor \n\n#","ref":"Nx.html#subtract/2-subtracting-scalars"},{"type":"function","title":"Subtracting tensors and scalars - Nx.subtract/2","doc":"iex> Nx.subtract(Nx.tensor([1, 2, 3], names: [:data]), 1)\n    #Nx.Tensor \n\n    iex> Nx.subtract(1, Nx.tensor([1.0, 2.0, 3.0], names: [:data]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#subtract/2-subtracting-tensors-and-scalars"},{"type":"function","title":"Subtracting tensors - Nx.subtract/2","doc":"iex> left = Nx.tensor([[1], [2]], names: [:x, :y])\n    iex> right = Nx.tensor([[10, 20]], names: [:x, :y])\n    iex> Nx.subtract(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([[1], [2]], type: :s8, names: [:x, nil])\n    iex> right = Nx.tensor([[10, 20]], type: :s8, names: [nil, :y])\n    iex> Nx.subtract(left, right)\n    #Nx.Tensor \n\n    iex> left = Nx.tensor([[1], [2]], type: :f32, names: [nil, :y])\n    iex> right = Nx.tensor([[10, 20]], type: :f32, names: [:x, nil])\n    iex> Nx.subtract(left, right)\n    #Nx.Tensor","ref":"Nx.html#subtract/2-subtracting-tensors"},{"type":"function","title":"Nx.sum/2","doc":"Returns the sum for the tensor.\n\nIf the `:axes` option is given, it aggregates over\nthe given dimensions, effectively removing them.\n`axes: [0]` implies aggregating over the highest order\ndimension and so forth. If the axis is negative, then\ncounts the axis from the back. For example, `axes: [-1]`\nwill always aggregate all rows.\n\nYou may optionally set `:keep_axes` to true, which will\nretain the rank of the input tensor by setting the summed\naxes to size 1.","ref":"Nx.html#sum/2"},{"type":"function","title":"Examples - Nx.sum/2","doc":"By default the sum always returns a scalar:\n\n    iex> Nx.sum(Nx.tensor(42))\n    #Nx.Tensor \n\n    iex> Nx.sum(Nx.tensor([1, 2, 3]))\n    #Nx.Tensor \n\n    iex> Nx.sum(Nx.tensor([[1.0, 2.0], [3.0, 4.0]]))\n    #Nx.Tensor \n\nGiving a tensor with low precision casts it to a higher\nprecision to make sure the sum does not overflow:\n\n    iex> Nx.sum(Nx.tensor([[101, 102], [103, 104]], type: :s8))\n    #Nx.Tensor \n\n    iex> Nx.sum(Nx.tensor([[101, 102], [103, 104]], type: :s16))\n    #Nx.Tensor \n\n#","ref":"Nx.html#sum/2-examples"},{"type":"function","title":"Aggregating over an axis - Nx.sum/2","doc":"iex> Nx.sum(Nx.tensor([1, 2, 3]), axes: [0])\n    #Nx.Tensor \n\nSame tensor over different axes combinations:\n\n    iex> t = Nx.iota({2, 2, 3}, names: [:x, :y, :z])\n    iex> Nx.sum(t, axes: [:x])\n    #Nx.Tensor \n    iex> Nx.sum(t, axes: [:y])\n    #Nx.Tensor \n    iex> Nx.sum(t, axes: [:z])\n    #Nx.Tensor \n    iex> Nx.sum(t, axes: [:x, :z])\n    #Nx.Tensor \n    iex> Nx.sum(t, axes: [-3])\n    #Nx.Tensor \n\n#","ref":"Nx.html#sum/2-aggregating-over-an-axis"},{"type":"function","title":"Keeping axes - Nx.sum/2","doc":"iex> t = Nx.tensor([[1, 2], [3, 4]], names: [:x, :y])\n    iex> Nx.sum(t, axes: [:x], keep_axes: true)\n    #Nx.Tensor \n\n#","ref":"Nx.html#sum/2-keeping-axes"},{"type":"function","title":"Vectorized tensors - Nx.sum/2","doc":"iex> t = Nx.tensor([[[[1, 2]], [[3, 4]]], [[[5, 6]], [[7, 8]]]]) |> Nx.vectorize(:x) |> Nx.vectorize(:y)\n    #Nx.Tensor \n    iex> Nx.sum(t)\n    #Nx.Tensor \n    iex> Nx.sum(t, axes: [0])\n    #Nx.Tensor \n\n#","ref":"Nx.html#sum/2-vectorized-tensors"},{"type":"function","title":"Errors - Nx.sum/2","doc":"iex> Nx.sum(Nx.tensor([[1, 2]]), axes: [2])\n    ** (ArgumentError) given axis (2) invalid for shape with rank 2","ref":"Nx.html#sum/2-errors"},{"type":"function","title":"Nx.take/3","doc":"Takes and concatenates slices along an axis.\n\nIntuitively speaking, `take/3` reorders tensor slices along\nthe given axis based on the given indices, possibly duplicating\nand removing slices.\n\nPassing a multi-dimensional indices tensor only affects the\nresulting shape. Specifically, the given axis in the input shape\ngets replaced with the indices shape.\n\nSee `gather/2`, `slice/3`, `slice_along_axis/4`, and `take_along_axis/3`\nfor other ways to retrieve values from a tensor.","ref":"Nx.html#take/3"},{"type":"function","title":"Options - Nx.take/3","doc":"* `:axis` - an axis to take tensor slices over. Defaults to 0.","ref":"Nx.html#take/3-options"},{"type":"function","title":"Examples - Nx.take/3","doc":"iex> t = Nx.tensor([[1, 2], [3, 4]])\n    iex> Nx.take(t, Nx.tensor([1, 0, 1]))\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[1, 2], [3, 4]])\n    iex> Nx.take(t, Nx.tensor([1, 0, 1]), axis: 1)\n    #Nx.Tensor \n\n\n    iex> t = Nx.tensor([[1, 2], [3, 4]], names: [:x, :y])\n    iex> Nx.take(t, Nx.tensor([1, 0, 1]), axis: :y)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[1, 2], [11, 12]], [[101, 102], [111, 112]]])\n    iex> Nx.take(t, Nx.tensor([1, 0, 1]), axis: 1)\n    #Nx.Tensor \n\nMulti-dimensional indices tensor:\n\n    iex> t = Nx.tensor([[1, 2], [11, 12]])\n    iex> Nx.take(t, Nx.tensor([[0, 0], [1, 1], [0, 0]]), axis: 1)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[1, 2], [11, 12]], [[101, 102], [111, 112]]])\n    iex> Nx.take(t, Nx.tensor([[0, 0, 0], [1, 1, 1], [0, 0, 0]]), axis: 1)\n    #Nx.Tensor","ref":"Nx.html#take/3-examples"},{"type":"function","title":"Vectorized tensors - Nx.take/3","doc":"`tensor` and `indices` have their vectorized axes broadcast together,\nand then the operation takes place normally, with `:axis` and `indices`\nhaving their values in reference to the input shape.\n\n    iex> t = Nx.tensor([[1, 2], [11, 12]])\n    iex> idx = Nx.tensor([0, 1, 0]) |> Nx.vectorize(:x)\n    iex> Nx.take(t, idx)\n    #Nx.Tensor \n    iex> t = Nx.tensor([[[1, 2]], [[11, 12]]]) |> Nx.vectorize(:x)\n    iex> idx = Nx.tensor([0, 1])\n    iex> Nx.take(t, idx, axis: 1)\n    #Nx.Tensor \n\nIn case both inputs are vectorized, they will be broadcasted\ntogether before calculations are performed:\n\n    iex> t = Nx.tensor([[1, 2], [11, 12]]) |> Nx.vectorize(:x)\n    iex> idx = Nx.tensor([0, 1, 0]) |> Nx.vectorize(:y)\n    iex> Nx.take(t, idx)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[1, 2], [11, 12]]) |> Nx.vectorize(:x)\n    iex> idx = Nx.tensor([[0, 1, 0], [0, 1, 1]]) |> Nx.vectorize(:x)\n    iex> Nx.take(t, idx)\n    #Nx.Tensor","ref":"Nx.html#take/3-vectorized-tensors"},{"type":"function","title":"Error cases - Nx.take/3","doc":"iex> Nx.take(Nx.tensor([[1, 2], [3, 4]]), Nx.tensor([1, 0, 1], type: :f32))\n    ** (ArgumentError) indices must be an integer tensor, got {:f, 32}","ref":"Nx.html#take/3-error-cases"},{"type":"function","title":"Nx.take_along_axis/3","doc":"Takes the values from a tensor given an `indices` tensor, along the specified axis.\n\nThe `indices` shape must be the same as the `tensor`'s shape, with the exception for\nthe `axis` dimension, which can have arbitrary size. The returned tensor will have the\nsame shape as the `indices` tensor.\n\nSee `gather/2`, `slice/3`, `slice_along_axis/4`, and `take/3` for other ways to retrieve\nvalues from a tensor.","ref":"Nx.html#take_along_axis/3"},{"type":"function","title":"Options - Nx.take_along_axis/3","doc":"* `:axis` - The axis along which to take the values from. Defaults to `0`.","ref":"Nx.html#take_along_axis/3-options"},{"type":"function","title":"Examples - Nx.take_along_axis/3","doc":"iex> t = Nx.tensor([[1, 2, 3], [4, 5, 6]])\n    iex> Nx.take_along_axis(t, Nx.tensor([[0, 0, 2, 2, 1, 1], [2, 2, 1, 1, 0, 0]]), axis: 1)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[1, 2, 3], [4, 5, 6]])\n    iex> Nx.take_along_axis(t, Nx.tensor([[0, 1, 1], [1, 0, 0], [0, 1, 0]]), axis: 0)\n    #Nx.Tensor \n\nThe indices returned from `Nx.argsort/2` can be used with `Nx.take_along_axis/3` to\nproduce the sorted tensor (or to sort more tensors according to the same criteria).\n\n    iex> tensor = Nx.tensor([[[1, 2], [3, 4], [5, 6]]])\n    #Nx.Tensor \n    iex> idx1 = Nx.argsort(tensor, axis: 1, direction: :desc)\n    #Nx.Tensor \n    iex> Nx.take_along_axis(tensor, idx1, axis: 1)\n    #Nx.Tensor \n    iex> idx2 = Nx.argsort(tensor, axis: 2, direction: :desc)\n    #Nx.Tensor \n    iex> Nx.take_along_axis(tensor, idx2, axis: 2)\n    #Nx.Tensor","ref":"Nx.html#take_along_axis/3-examples"},{"type":"function","title":"Vectorized tensors - Nx.take_along_axis/3","doc":"`tensor` and `indices` have their vectorized axes broadcast together,\nand then the operation takes place normally, with `:axis` and `indices`\nhaving their values in reference to the input shape.\n\n    iex> t = Nx.tensor([[[1, 2, 3]], [[4, 5, 6]]]) |> Nx.vectorize(:x)\n    iex> idx = Nx.tensor([[[0, 0, 2, 1]], [[2, 1, 0, 0]]]) |> Nx.vectorize(:x)\n    iex> Nx.take_along_axis(t, idx, axis: 1)\n    #Nx.Tensor \n\nIn the example below, we have broadcasting throughout the vectorized axes\n\n    iex> t = Nx.tensor([[1, 2, 3], [4, 5, 6]]) |> Nx.vectorize(:x)\n    iex> idx = Nx.tensor([[0, 0, 2, 1], [2, 1, 0, 0]]) |> Nx.vectorize(:y)\n    iex> Nx.take_along_axis(t, idx, axis: 0)\n    #Nx.Tensor","ref":"Nx.html#take_along_axis/3-vectorized-tensors"},{"type":"function","title":"Error cases - Nx.take_along_axis/3","doc":"iex> tensor = Nx.iota({3, 3})\n    iex> idx = Nx.tensor([[2.0], [1.0], [2.0]], type: :f32)\n    iex> Nx.take_along_axis(tensor, idx, axis: 1)\n    ** (ArgumentError) indices must be an integer tensor, got {:f, 32}","ref":"Nx.html#take_along_axis/3-error-cases"},{"type":"function","title":"Nx.take_diagonal/2","doc":"Extracts the diagonal of batched matrices.\n\nConverse of `make_diagonal/2`.","ref":"Nx.html#take_diagonal/2"},{"type":"function","title":"Examples - Nx.take_diagonal/2","doc":"Given a matrix without offset:\n\n    iex> Nx.take_diagonal(Nx.tensor([\n    ...> [0, 1, 2],\n    ...> [3, 4, 5],\n    ...> [6, 7, 8]\n    ...> ]))\n    #Nx.Tensor \n\nAnd if given a matrix along with an offset:\n\n    iex> Nx.take_diagonal(Nx.iota({3, 3}), offset: 1)\n    #Nx.Tensor \n\n    iex> Nx.take_diagonal(Nx.iota({3, 3}), offset: -1)\n    #Nx.Tensor \n\nGiven batched matrix:\n\n    iex> Nx.take_diagonal(Nx.iota({3, 2, 2}))\n    #Nx.Tensor \n\n    iex> Nx.take_diagonal(Nx.iota({3, 2, 2}), offset: -1)\n    #Nx.Tensor","ref":"Nx.html#take_diagonal/2-examples"},{"type":"function","title":"Options - Nx.take_diagonal/2","doc":"* `:offset` - offset used for extracting the diagonal.\n    Use offset > 0 for diagonals above the main diagonal,\n    and offset < 0 for diagonals below the main diagonal.\n    Defaults to 0.","ref":"Nx.html#take_diagonal/2-options"},{"type":"function","title":"Error cases - Nx.take_diagonal/2","doc":"iex> Nx.take_diagonal(Nx.tensor([0, 1, 2]))\n    ** (ArgumentError) take_diagonal/2 expects tensor of rank 2 or higher, got tensor of rank: 1\n\n    iex> Nx.take_diagonal(Nx.iota({3, 3}), offset: 3)\n    ** (ArgumentError) offset must be less than length of axis 1 when positive, got: 3\n\n    iex> Nx.take_diagonal(Nx.iota({3, 3}), offset: -4)\n    ** (ArgumentError) absolute value of offset must be less than length of axis 0 when negative, got: -4","ref":"Nx.html#take_diagonal/2-error-cases"},{"type":"function","title":"Nx.tan/1","doc":"Calculates the tangent of each element in the tensor.\n\nIt is equivalent to:\n\n$$tan(z) = \\frac{sin(z)}{cos(z)}$$","ref":"Nx.html#tan/1"},{"type":"function","title":"Examples - Nx.tan/1","doc":"iex> Nx.tan(1)\n    #Nx.Tensor \n\n    iex> Nx.tan(Nx.tensor([1, 2, 3], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#tan/1-examples"},{"type":"function","title":"Nx.tanh/1","doc":"Calculates the hyperbolic tangent of each element in the tensor.\n\nIt is equivalent to:\n\n$$sinh(z) = \\frac{e^z - e^{-z}}{e^z + e^{-z}}$$","ref":"Nx.html#tanh/1"},{"type":"function","title":"Examples - Nx.tanh/1","doc":"iex> Nx.tanh(1)\n    #Nx.Tensor \n\n    iex> Nx.tanh(Nx.tensor([1, 2, 3], names: [:x]))\n    #Nx.Tensor","ref":"Nx.html#tanh/1-examples"},{"type":"function","title":"Nx.template/3","doc":"Creates a tensor template.\n\nYou can't perform any operation on this tensor.\nIt exists exclusively to define APIs that say\na tensor with a certain type, shape, and names\nis expected in the future.","ref":"Nx.html#template/3"},{"type":"function","title":"Examples - Nx.template/3","doc":"iex> Nx.template({2, 3}, :f32)\n    #Nx.Tensor \n\n    iex> Nx.template({2, 3}, {:f, 32}, names: [:rows, :columns])\n    #Nx.Tensor \n\nAlthough note it is impossible to perform any operation on a tensor template:\n\n    iex> t = Nx.template({2, 3}, {:f, 32}, names: [:rows, :columns])\n    iex> Nx.abs(t)\n    ** (RuntimeError) cannot perform operations on a Nx.TemplateBackend tensor\n\nTo convert existing tensors to templates, use `to_template/1`.","ref":"Nx.html#template/3-examples"},{"type":"function","title":"Nx.tensor/2","doc":"Builds a tensor.\n\nThe argument must be one of:\n\n  * a tensor\n  * a number (which means the tensor is scalar/zero-dimensional)\n  * a boolean (also scalar/zero-dimensional)\n  * an arbitrarily nested list of numbers and booleans\n\nIf a new tensor has to be allocated, it will be allocated in\n`Nx.default_backend/0`, unless the `:backend` option is given,\nwhich overrides the default one.","ref":"Nx.html#tensor/2"},{"type":"function","title":"Examples - Nx.tensor/2","doc":"A number returns a tensor of zero dimensions:\n\n    iex> Nx.tensor(0)\n    #Nx.Tensor \n\n    iex> Nx.tensor(1.0)\n    #Nx.Tensor \n\nGiving a list returns a vector (a one-dimensional tensor):\n\n    iex> Nx.tensor([1, 2, 3])\n    #Nx.Tensor \n\n    iex> Nx.tensor([1.2, 2.3, 3.4, 4.5])\n    #Nx.Tensor \n\nThe type can be explicitly given. Integers and floats\nbigger than the given size overflow:\n\n    iex> Nx.tensor([300, 301, 302], type: :s8)\n    #Nx.Tensor \n\nMixed types give higher priority to floats:\n\n    iex> Nx.tensor([1, 2, 3.0])\n    #Nx.Tensor \n\nBoolean values are also accepted, where `true` is\nconverted to `1` and `false` to `0`, with the type\nbeing inferred as `{:u, 8}`\n\n    iex> Nx.tensor(true)\n    #Nx.Tensor \n\n    iex> Nx.tensor(false)\n    #Nx.Tensor \n\n    iex> Nx.tensor([true, false])\n    #Nx.Tensor \n\nMulti-dimensional tensors are also possible:\n\n    iex> Nx.tensor([[1, 2, 3], [4, 5, 6]])\n    #Nx.Tensor \n\n    iex> Nx.tensor([[1, 2], [3, 4], [5, 6]])\n    #Nx.Tensor \n\n    iex> Nx.tensor([[[1, 2], [3, 4], [5, 6]], [[-1, -2], [-3, -4], [-5, -6]]])\n    #Nx.Tensor","ref":"Nx.html#tensor/2-examples"},{"type":"function","title":"Floats and complex numbers - Nx.tensor/2","doc":"Besides single-precision (32 bits), floats can also have\nhalf-precision (16) or double-precision (64):\n\n    iex> Nx.tensor([1, 2, 3], type: :f16)\n    #Nx.Tensor \n\n    iex> Nx.tensor([1, 2, 3], type: :f64)\n    #Nx.Tensor \n\nBrain-floating points are also supported:\n\n    iex> Nx.tensor([1, 2, 3], type: :bf16)\n    #Nx.Tensor \n\nIn all cases, the non-finite values negative infinity (-Inf),\ninfinity (Inf), and \"not a number\" (NaN) can be represented by\nthe atoms `:neg_infinity`, `:infinity`, and `:nan` respectively:\n\n    iex> Nx.tensor([:neg_infinity, :nan, :infinity])\n    #Nx.Tensor \n\nFinally, complex numbers are also supported in tensors:\n\n    iex> Nx.tensor(Complex.new(1, -1))\n    #Nx.Tensor","ref":"Nx.html#tensor/2-floats-and-complex-numbers"},{"type":"function","title":"Naming dimensions - Nx.tensor/2","doc":"You can provide names for tensor dimensions. Names are atoms:\n\n    iex> Nx.tensor([[1, 2, 3], [4, 5, 6]], names: [:x, :y])\n    #Nx.Tensor \n\nNames make your code more expressive:\n\n    iex> Nx.tensor([[[1, 2, 3], [4, 5, 6], [7, 8, 9]]], names: [:batch, :height, :width])\n    #Nx.Tensor \n\nYou can also leave dimension names as `nil`:\n\n    iex> Nx.tensor([[[1, 2, 3], [4, 5, 6], [7, 8, 9]]], names: [:batch, nil, nil])\n    #Nx.Tensor \n\nHowever, you must provide a name for every dimension in the tensor:\n\n    iex> Nx.tensor([[[1, 2, 3], [4, 5, 6], [7, 8, 9]]], names: [:batch])\n    ** (ArgumentError) invalid names for tensor of rank 3, when specifying names every dimension must have a name or be nil","ref":"Nx.html#tensor/2-naming-dimensions"},{"type":"function","title":"Tensors - Nx.tensor/2","doc":"Tensors can also be given as inputs:\n\n    iex> Nx.tensor(Nx.tensor([1, 2, 3]))\n    #Nx.Tensor \n\nIf the `:backend` and `:type` options are given, the tensor will\ncompared against those values and raise in case of mismatch:\n\n    iex> Nx.tensor(Nx.tensor([1, 2, 3]), type: :f32)\n    ** (ArgumentError) Nx.tensor/2 expects a tensor with type :f32 but it was given a tensor of type {:s, 64}\n\nThe `:backend` option will check only against the backend name\nand not specific backend configuration such as device and client.\nIn case the backend differs, it will also raise.\n\nThe names in the given tensor are always discarded but Nx will raise\nin case the tensor already has names that conflict with the assigned ones:\n\n    iex> Nx.tensor(Nx.tensor([1, 2, 3]), names: [:row])\n    #Nx.Tensor \n\n    iex> Nx.tensor(Nx.tensor([1, 2, 3], names: [:column]))\n    #Nx.Tensor \n\n    iex> Nx.tensor(Nx.tensor([1, 2, 3], names: [:column]), names: [:row])\n    ** (ArgumentError)  cannot merge name :column on axis 0 with name :row on axis 0","ref":"Nx.html#tensor/2-tensors"},{"type":"function","title":"Options - Nx.tensor/2","doc":"* `:type` - sets the type of the tensor. If one is not given,\n    one is automatically inferred based on the input.\n\n  * `:names` - dimension names. If you wish to specify dimension\n    names you must specify a name for every dimension in the tensor.\n    Only `nil` and atoms are supported as dimension names.\n\n  * `:backend` - the backend to allocate the tensor on. It is either\n    an atom or a tuple in the shape `{backend, options}`. It defaults\n    to `Nx.default_backend/0` for new tensors","ref":"Nx.html#tensor/2-options"},{"type":"function","title":"Nx.tile/2","doc":"Creates a new tensor by repeating the input tensor\nalong the given axes.\n\nIf the `tensor` has less dimensions than the repetitions given,\nthe tensor will grow in dimensionality.\n\nIf the `tensor` has more dimensions than the repetitions given,\ntiling is done from the rightmost dimensions (i.e. if the input\nshape is `{1,2,3}` and `repetitions = [2]`, the result is the same\nas if `repetitions = [1,1,2]`).","ref":"Nx.html#tile/2"},{"type":"function","title":"Examples - Nx.tile/2","doc":"iex> a = Nx.tensor([0, 1, 2])\n    iex> Nx.tile(a, [2])\n    #Nx.Tensor \n    iex> Nx.tile(a, [1, 2])\n    #Nx.Tensor \n    iex> Nx.tile(a, [2, 2])\n    #Nx.Tensor \n    iex> Nx.tile(a, [2, 1])\n    #Nx.Tensor \n    iex> Nx.tile(a, [2, 1, 2])\n    #Nx.Tensor \n\n    iex> b = Nx.tensor([[1,2],[3,4]])\n    iex> Nx.tile(b, [2])\n    #Nx.Tensor \n    iex> Nx.tile(b, [2, 1])\n    #Nx.Tensor \n    iex> Nx.tile(b, [1, 2])\n    #Nx.Tensor \n\n    iex> c = Nx.tensor([1,2,3,4])\n    iex> Nx.tile(c, [4,1])\n    #Nx.Tensor","ref":"Nx.html#tile/2-examples"},{"type":"function","title":"Vectorized tensors - Nx.tile/2","doc":"Like `reshape/2`, `tile/2` works on the shape, leaving vectors untouched.\n\n    iex> t = Nx.vectorize(Nx.tensor([[1, 2, 3], [4, 5, 6]]), :x)\n    iex> Nx.tile(t, [1, 3, 1])\n    #Nx.Tensor","ref":"Nx.html#tile/2-vectorized-tensors"},{"type":"function","title":"Error cases - Nx.tile/2","doc":"iex> Nx.tile(Nx.tensor([1,2]), 1.0)\n    ** (ArgumentError) repetitions must be a list of integers, got: 1.0\n\n    iex> Nx.tile(Nx.tensor([1,2]), [1, 1.0])\n    ** (ArgumentError) repetitions must be a list of integers, got: [1, 1.0]\n\n    iex> Nx.tile(Nx.tensor([1,2]), nil)\n    ** (ArgumentError) repetitions must be a list of integers, got: nil","ref":"Nx.html#tile/2-error-cases"},{"type":"function","title":"Nx.to_batched/3","doc":"Converts the underlying tensor to a stream of tensor batches.\n\nThe first dimension (axis 0) is divided by `batch_size`.\nIn case the dimension cannot be evenly divided by\n`batch_size`, you may specify what to do with leftover\ndata using `:leftover`. `:leftover` must be one of `:repeat`\nor `:discard`. `:repeat` repeats the first `n` values to\nmake the last batch match the desired batch size. `:discard`\ndiscards excess elements.\n\nNote: This function cannot be used in `defn`.","ref":"Nx.html#to_batched/3"},{"type":"function","title":"Examples - Nx.to_batched/3","doc":"In the examples below we immediately pipe to `Enum.to_list/1`\nfor convenience, but in practice you want to lazily traverse\nthe batches to avoid allocating multiple tensors at once in\ncertain backends:\n\n    iex> [first, second] = Nx.to_batched(Nx.iota({2, 2, 2}), 1) |> Enum.to_list()\n    iex> first\n    #Nx.Tensor \n    iex> second\n    #Nx.Tensor \n\nIf the batch size would result in uneven batches, you can repeat or discard excess data.\nBy default, we repeat:\n\n    iex> [first, second, third] = Nx.to_batched(Nx.iota({5, 2}, names: [:x, :y]), 2) |> Enum.to_list()\n    iex> first\n    #Nx.Tensor \n    iex> second\n    #Nx.Tensor \n    iex> third\n    #Nx.Tensor \n\nBut you can also discard:\n\n    iex> [first, second] = Nx.to_batched(Nx.iota({5, 2}, names: [:x, :y]), 2, leftover: :discard) |> Enum.to_list()\n    iex> first\n    #Nx.Tensor \n    iex> second\n    #Nx.Tensor","ref":"Nx.html#to_batched/3-examples"},{"type":"function","title":"Vectorized tensors - Nx.to_batched/3","doc":"Similarly to `to_list/1` and `to_binary/1`, `to_batched/2` will\nignore vectorization to perform calculations. Because the output\nstill contains tensors, however, they will still be vectorized.\n\n    iex> t = Nx.iota({2, 2, 2}) |> Nx.vectorize(x: 2)\n    iex> [first, second] = Nx.to_batched(t, 1) |> Enum.to_list()\n    iex> first\n    #Nx.Tensor \n    iex> second\n    #Nx.Tensor \n\n    iex> t = Nx.iota({2, 2, 2}) |> Nx.vectorize(x: 2, y: 2)\n    iex> [first, second] = Nx.to_batched(t, 1) |> Enum.to_list()\n    iex> first\n    #Nx.Tensor \n    iex> second\n    #Nx.Tensor \n\nSame rules about uneven batches still apply:\n\n    iex> t = Nx.iota({5, 2}, names: [:x, :y]) |> Nx.vectorize(:x)\n    iex> [first, second, third] = Nx.to_batched(t, 2) |> Enum.to_list()\n    iex> first\n    #Nx.Tensor \n    iex> second\n    #Nx.Tensor \n    iex> third\n    #Nx.Tensor \n\nBecause we're dealing with vectorized tensors, a vectorized\nscalar tensor can also be batched.\n\n    iex> t = Nx.tensor([1, 2, 3]) |> Nx.vectorize(:x)\n    iex> [first, second] = t |> Nx.to_batched(2) |> Enum.to_list()\n    iex> first\n    #Nx.Tensor \n    iex> second\n    #Nx.Tensor","ref":"Nx.html#to_batched/3-vectorized-tensors"},{"type":"function","title":"Nx.to_binary/2","doc":"Returns the underlying tensor as a binary.\n\n**Warning**: converting a tensor to a binary can\npotentially be a very expensive operation, as it\nmay copy a GPU tensor fully to the machine memory.\n\nIt returns the in-memory binary representation of\nthe tensor in a row-major fashion. The binary is\nin the system endianness, which has to be taken into\naccount if the binary is meant to be serialized to\nother systems.\n\nNote: This function cannot be used in `defn`.","ref":"Nx.html#to_binary/2"},{"type":"function","title":"Options - Nx.to_binary/2","doc":"* `:limit` - limit the number of entries represented in the binary","ref":"Nx.html#to_binary/2-options"},{"type":"function","title":"Examples - Nx.to_binary/2","doc":"iex> Nx.to_binary(1)\n     >\n\n    iex> Nx.to_binary(Nx.tensor([1.0, 2.0, 3.0]))\n     >\n\n    iex> Nx.to_binary(Nx.tensor([1.0, 2.0, 3.0]), limit: 2)\n     >\n\n#","ref":"Nx.html#to_binary/2-examples"},{"type":"function","title":"Vectorized tensors - Nx.to_binary/2","doc":"`to_binary/2` disregards the vectorized axes before calculating the data to be returned:\n\n    iex> Nx.to_binary(Nx.vectorize(Nx.tensor([[1, 2], [3, 4]]), :x))\n     >\n\n    iex> Nx.to_binary(Nx.vectorize(Nx.tensor([1, 2, 3]), :x), limit: 2)\n     >","ref":"Nx.html#to_binary/2-vectorized-tensors"},{"type":"function","title":"Nx.to_flat_list/2","doc":"Returns the underlying tensor as a flat list.\n\nNegative infinity (-Inf), infinity (Inf), and \"not a number\" (NaN)\nwill be represented by the atoms `:neg_infinity`, `:infinity`, and\n`:nan` respectively.\n\nNote: This function cannot be used in `defn`.","ref":"Nx.html#to_flat_list/2"},{"type":"function","title":"Examples - Nx.to_flat_list/2","doc":"iex> Nx.to_flat_list(1)\n    [1]\n\n    iex> Nx.to_flat_list(Nx.tensor([1.0, 2.0, 3.0]))\n    [1.0, 2.0, 3.0]\n\n    iex> Nx.to_flat_list(Nx.tensor([1.0, 2.0, 3.0]), limit: 2)\n    [1.0, 2.0]\n\nNon-finite numbers are returned as atoms:\n\n    iex> t = Nx.tensor([:neg_infinity, :nan, :infinity])\n    iex> Nx.to_flat_list(t)\n    [:neg_infinity, :nan, :infinity]\n\n#","ref":"Nx.html#to_flat_list/2-examples"},{"type":"function","title":"Vectorized tensors - Nx.to_flat_list/2","doc":"`to_flat_list/2` disregards the vectorized axes before calculating the data to be returned.\nLike `to_binary/1`, `:limit` refers to the flattened devectorized data.\n\n    iex> t = Nx.vectorize(Nx.tensor([[1], [2], [3], [4]]), :x)\n    iex> Nx.to_flat_list(t)\n    [1, 2, 3, 4]\n    iex> Nx.to_flat_list(t, limit: 2)\n    [1, 2]","ref":"Nx.html#to_flat_list/2-vectorized-tensors"},{"type":"function","title":"Nx.to_heatmap/2","doc":"Returns a heatmap struct with the tensor data.\n\nOn terminals, coloring is done via ANSI colors. If ANSI\nis not enabled, the tensor is normalized to show numbers\nbetween 0 and 9.","ref":"Nx.html#to_heatmap/2"},{"type":"function","title":"Terminal coloring - Nx.to_heatmap/2","doc":"Coloring is enabled by default on most Unix terminals.\nIt is also available on Windows consoles from Windows\n10, although it must be explicitly enabled for the current\nuser in the registry by running the following command:\n\n    reg add HKCU\\Console /v VirtualTerminalLevel /t REG_DWORD /d 1\n\nAfter running the command above, you must restart your current\nconsole.","ref":"Nx.html#to_heatmap/2-terminal-coloring"},{"type":"function","title":"Options - Nx.to_heatmap/2","doc":"* `:ansi_enabled` - forces ansi to be enabled or disabled.\n    Defaults to `IO.ANSI.enabled?/0`\n\n  * `:ansi_whitespace` - which whitespace character to use when\n    printing. By default it uses `\"\\u3000\"`, which is a full-width\n    whitespace which often prints more precise shapes","ref":"Nx.html#to_heatmap/2-options"},{"type":"function","title":"Nx.to_list/1","doc":"Converts the tensor into a list reflecting its structure.\n\nNegative infinity (-Inf), infinity (Inf), and \"not a number\" (NaN)\nwill be represented by the atoms `:neg_infinity`, `:infinity`, and\n`:nan` respectively.\n\nIt raises if a scalar tensor is given, use `to_number/1` instead.\n\nNote: This function cannot be used in `defn`.","ref":"Nx.html#to_list/1"},{"type":"function","title":"Examples - Nx.to_list/1","doc":"iex> Nx.iota({2, 3}) |> Nx.to_list()\n    [\n      [0, 1, 2],\n      [3, 4, 5]\n    ]\n\n    iex> Nx.tensor(123) |> Nx.to_list()\n    ** (ArgumentError) cannot convert a scalar tensor to a list, got: #Nx.Tensor \n\n#","ref":"Nx.html#to_list/1-examples"},{"type":"function","title":"Vectorized tensors - Nx.to_list/1","doc":"`to_list/1` disregards the vectorized axes before calculating the data to be returned.\nThe special case below shows that a vectorized tensor with inner scalar shape will\nstill be converted to a list accordingly:\n\n    iex> %{shape: {}} = t = Nx.vectorize(Nx.tensor([1, 2, 3]), :x)\n    iex> Nx.to_list(t) # recall that normally, shape == {} would raise!\n    [1, 2, 3]","ref":"Nx.html#to_list/1-vectorized-tensors"},{"type":"function","title":"Nx.to_number/1","doc":"Returns the underlying tensor as a number.\n\nNegative infinity (-Inf), infinity (Inf), and \"not a number\" (NaN)\nwill be represented by the atoms `:neg_infinity`, `:infinity`, and\n`:nan` respectively.\n\nIf the tensor has a dimension or is vectorized, it raises.\n\nNote: This function cannot be used in `defn`.","ref":"Nx.html#to_number/1"},{"type":"function","title":"Examples - Nx.to_number/1","doc":"iex> Nx.to_number(1)\n    1\n\n    iex> Nx.to_number(Nx.tensor([1.0, 2.0, 3.0]))\n    ** (ArgumentError) cannot convert tensor of shape {3} to number\n\n    iex> Nx.to_number(Nx.vectorize(Nx.tensor([1]), :x))\n    ** (ArgumentError) cannot convert vectorized tensor with axes [x: 1] and shape {} to number","ref":"Nx.html#to_number/1-examples"},{"type":"function","title":"Nx.to_template/1","doc":"Converts a tensor (or tuples and maps of tensors) to tensor templates.\n\nTemplates are useful when you need to pass types and shapes to\noperations and the data is not yet available.\n\nFor convenience, this function accepts tensors and any container\n(such as maps and tuples as defined by the `Nx.LazyContainer` protocol)\nand recursively converts all tensors to templates.","ref":"Nx.html#to_template/1"},{"type":"function","title":"Examples - Nx.to_template/1","doc":"iex> Nx.iota({2, 3}) |> Nx.to_template()\n    #Nx.Tensor \n\n    iex> {int, float} = Nx.to_template({1, 2.0})\n    iex> int\n    #Nx.Tensor \n    iex> float\n    #Nx.Tensor \n\nAlthough note it is impossible to perform any operation on a tensor template:\n\n    iex> t = Nx.iota({2, 3}) |> Nx.to_template()\n    iex> Nx.abs(t)\n    ** (RuntimeError) cannot perform operations on a Nx.TemplateBackend tensor\n\nTo build a template from scratch, use `template/3`.","ref":"Nx.html#to_template/1-examples"},{"type":"function","title":"Nx.to_tensor/1","doc":"Converts a data structure into a tensor.\n\nThis function only converts types which are automatically\ncast to tensors throughout Nx API: numbers, complex numbers,\ntensors themselves, and implementations of `Nx.LazyContainer`\n(and `Nx.Container`).\n\nIf your goal is to create tensors from lists, see `tensor/2`.\nIf you want to create a tensor from binary, see `from_binary/3`.\nIf you want to convert a data structure with several tensors at\nonce into a single one, see `stack/2` or `concatenate/2` instead.","ref":"Nx.html#to_tensor/1"},{"type":"function","title":"Nx.top_k/2","doc":"Returns a tuple of `{values, indices}` for the top `k`\nvalues in last dimension of the tensor.\n\n`:k` is an option and must be at least 1, and less than\nor equal to the size of the last dimension of the tensor.\nIt defaults to `1`.","ref":"Nx.html#top_k/2"},{"type":"function","title":"Examples - Nx.top_k/2","doc":"iex> a = Nx.tensor([1, 2, 3, 4, 5])\n    iex> {values, indices} = Nx.top_k(a, k: 2)\n    iex> values\n    #Nx.Tensor \n    iex> indices\n    #Nx.Tensor \n\n`:k` defaults to 1:\n\n    iex> a = Nx.tensor([[1.0, 2.0, 3.0], [4.0, 5.0, 6.0]])\n    iex> {values, indices} = Nx.top_k(a)\n    iex> values\n    #Nx.Tensor \n    iex> indices\n    #Nx.Tensor","ref":"Nx.html#top_k/2-examples"},{"type":"function","title":"Error cases - Nx.top_k/2","doc":"iex> a = Nx.tensor([1, 2, 3, 4, 5])\n    iex> Nx.top_k(a, k: 6)\n    ** (ArgumentError) top_k input last axis size must be greater than or equal to k, got size=5 and k=6\n\n    iex> a = Nx.tensor(1)\n    iex> Nx.top_k(a, k: 1)\n    ** (ArgumentError) top_k input must have at least rank 1","ref":"Nx.html#top_k/2-error-cases"},{"type":"function","title":"Nx.transpose/2","doc":"Transposes a tensor to the given `axes`.\n\nIf no axes are given, the default behavior is to\nreverse the order of the original tensor's axes.\n\nThe axes is a list of integers or dimension names\ncontaining how the new dimensions must be ordered.\nThe highest dimension is zero.","ref":"Nx.html#transpose/2"},{"type":"function","title":"Examples - Nx.transpose/2","doc":"iex> Nx.transpose(Nx.tensor(1))\n    #Nx.Tensor \n\n    iex> Nx.transpose(Nx.iota({2, 3, 4}, names: [:x, :y, :z]))\n    #Nx.Tensor \n\n    iex> Nx.transpose(Nx.tensor(1), axes: [])\n    #Nx.Tensor \n\n    iex> Nx.transpose(Nx.iota({2, 3, 4}, names: [:batch, :x, :y]), axes: [2, 1, :batch])\n    #Nx.Tensor \n\n    iex> Nx.transpose(Nx.iota({2, 3, 4}, names: [:batch, :x, :y]), axes: [:y, :batch, :x])\n    #Nx.Tensor \n\n    iex> Nx.transpose(Nx.iota({2, 3, 4}, names: [:batch, :x, :y]), axes: [:batch, :y, :x])\n    #Nx.Tensor \n\n#","ref":"Nx.html#transpose/2-examples"},{"type":"function","title":"Vectorized tensors - Nx.transpose/2","doc":"For vectorized tensors, transpose will manipulate the inner shape only,\nkeeping the order of vectorized axes the same.\n\n    iex> v = Nx.vectorize(Nx.iota({1, 2, 3}), :x)\n    #Nx.Tensor \n    iex> Nx.transpose(v)\n    #Nx.Tensor \n    iex> Nx.transpose(v, axes: [1, 0])\n    #Nx.Tensor \n\n#","ref":"Nx.html#transpose/2-vectorized-tensors"},{"type":"function","title":"Errors - Nx.transpose/2","doc":"iex> Nx.transpose(Nx.iota({2, 2}, names: [:batch, :x]), axes: [:batch])\n    ** (ArgumentError) expected length of permutation (1) to match rank of shape (2)\n\n    iex> Nx.transpose(Nx.iota({2, 2}), axes: [1, 2])\n    ** (ArgumentError) given axis (2) invalid for shape with rank 2","ref":"Nx.html#transpose/2-errors"},{"type":"function","title":"Nx.tri/3","doc":"An array with ones at and below the given diagonal and zeros elsewhere.","ref":"Nx.html#tri/3"},{"type":"function","title":"Options - Nx.tri/3","doc":"* `k` - The diagonal above which to zero elements. Default: 0.","ref":"Nx.html#tri/3-options"},{"type":"function","title":"Examples - Nx.tri/3","doc":"iex> tensor = Nx.tensor([[1, 2, 3], [4, 5, 6], [7, 8, 9]])\n    iex> {num_rows, num_cols} = Nx.shape(tensor)\n    iex> Nx.tri(num_rows, num_cols)\n    #Nx.Tensor \n\n    iex> tensor = Nx.tensor([[1, 2, 3], [4, 5, 6], [7, 8, 9]])\n    iex> {num_rows, num_cols} = Nx.shape(tensor)\n    iex> Nx.tri(num_rows, num_cols, k: 1)\n    #Nx.Tensor","ref":"Nx.html#tri/3-examples"},{"type":"function","title":"Nx.tril/2","doc":"Lower triangle of a matrix.","ref":"Nx.html#tril/2"},{"type":"function","title":"Options - Nx.tril/2","doc":"* `k` - The diagonal above which to zero elements. Default: 0.","ref":"Nx.html#tril/2-options"},{"type":"function","title":"Examples - Nx.tril/2","doc":"iex> Nx.tril(Nx.tensor([[1, 2, 3], [4, 5, 6], [7, 8, 9]]))\n    #Nx.Tensor \n\n    iex> Nx.tril(Nx.tensor([[1, 2, 3], [4, 5, 6], [7, 8, 9]]), k: 1)\n    #Nx.Tensor \n\n    iex> Nx.tril(Nx.iota({2, 3, 4}))\n    #Nx.Tensor \n\n    iex> Nx.tril(Nx.iota({6}))\n    ** (ArgumentError) tril/2 expects a tensor with at least 2 dimensions, got: #Nx.Tensor","ref":"Nx.html#tril/2-examples"},{"type":"function","title":"Nx.triu/2","doc":"Upper triangle of an array.","ref":"Nx.html#triu/2"},{"type":"function","title":"Options - Nx.triu/2","doc":"* `k` - The diagonal below which to zero elements. Default: 0.","ref":"Nx.html#triu/2-options"},{"type":"function","title":"Examples - Nx.triu/2","doc":"iex> Nx.triu(Nx.tensor([[1, 2, 3], [4, 5, 6], [7, 8, 9]]))\n    #Nx.Tensor \n\n    iex> Nx.triu(Nx.tensor([[1, 2, 3], [4, 5, 6], [7, 8, 9]]), k: 1)\n    #Nx.Tensor \n\n    iex> Nx.triu(Nx.iota({2, 3, 4}))\n    #Nx.Tensor \n\n    iex> Nx.triu(Nx.iota({6}))\n    ** (ArgumentError) triu/2 expects a tensor with at least 2 dimensions, got: #Nx.Tensor","ref":"Nx.html#triu/2-examples"},{"type":"function","title":"Nx.type/1","doc":"Returns the type of the tensor.\n\nSee `Nx.Type` for more information.","ref":"Nx.html#type/1"},{"type":"function","title":"Examples - Nx.type/1","doc":"iex> Nx.type(Nx.tensor([1, 2, 3]))\n    {:s, 64}\n\n    iex> Nx.type(Nx.tensor([1, 2, 3], type: :f32))\n    {:f, 32}\n\n    iex> Nx.type(1)\n    {:s, 64}\n\n    iex> Nx.type(1.0)\n    {:f, 32}","ref":"Nx.html#type/1-examples"},{"type":"function","title":"Nx.u8/1","doc":"Short-hand function for creating tensor of type `u8`.\n\nThis is just an alias for `Nx.tensor(tensor, type: u8)`.","ref":"Nx.html#u8/1"},{"type":"function","title":"Nx.u16/1","doc":"Short-hand function for creating tensor of type `u16`.\n\nThis is just an alias for `Nx.tensor(tensor, type: u16)`.","ref":"Nx.html#u16/1"},{"type":"function","title":"Nx.u32/1","doc":"Short-hand function for creating tensor of type `u32`.\n\nThis is just an alias for `Nx.tensor(tensor, type: u32)`.","ref":"Nx.html#u32/1"},{"type":"function","title":"Nx.u64/1","doc":"Short-hand function for creating tensor of type `u64`.\n\nThis is just an alias for `Nx.tensor(tensor, type: u64)`.","ref":"Nx.html#u64/1"},{"type":"function","title":"Nx.variance/2","doc":"Finds the variance of a tensor.\n\nThe variance is the average of the squared deviations from the mean.\nThe mean is typically calculated as `sum(tensor) / n`, where `n` is the total\nof elements. If, however, `:ddof` (delta degrees of freedom) is specified, the\ndivisor `n - ddof` is used instead.","ref":"Nx.html#variance/2"},{"type":"function","title":"Examples - Nx.variance/2","doc":"iex> Nx.variance(Nx.tensor([[1, 2], [3, 4]]))\n    #Nx.Tensor \n\n    iex> Nx.variance(Nx.tensor([[1, 2], [3, 4]]), ddof: 1)\n    #Nx.Tensor \n\n    iex> Nx.variance(Nx.tensor([[1, 2], [3, 4]]), axes: [0])\n    #Nx.Tensor \n\n    iex> Nx.variance(Nx.tensor([[1, 2], [3, 4]]), axes: [1])\n    #Nx.Tensor \n\n    iex> Nx.variance(Nx.tensor([[1, 2], [3, 4]]), axes: [0], ddof: 1)\n    #Nx.Tensor \n\n    iex> Nx.variance(Nx.tensor([[1, 2], [3, 4]]), axes: [1], ddof: 1)\n    #Nx.Tensor \n\n#","ref":"Nx.html#variance/2-examples"},{"type":"function","title":"Keeping axes - Nx.variance/2","doc":"iex> Nx.variance(Nx.tensor([[1, 2], [3, 4]]), axes: [1], keep_axes: true)\n    #Nx.Tensor \n\n#","ref":"Nx.html#variance/2-keeping-axes"},{"type":"function","title":"Vectorized tensors - Nx.variance/2","doc":"iex> Nx.variance(Nx.tensor([[1, 2], [0, 4]]) |> Nx.vectorize(:x))\n    #Nx.Tensor","ref":"Nx.html#variance/2-vectorized-tensors"},{"type":"function","title":"Nx.vectorize/2","doc":"Transforms a tensor into a vectorized tensor.\n\nEach vectorization removes the leading axes from the shape and appends them to\nthe `:vectorized_axes` list for the tensor.\n\nThe vectorization specification can be a list of atoms or `{atom, pos_integer}`\npairs. If a single atom is given, it behaves as a single-element list.\nThe atom names the vectorized axes. If a pair is given, we also verify\nthat the given size matches the size of the to-be-vectorized axis.\n\nIn the examples below, we discuss in more detail how a vectorized tensor works.","ref":"Nx.html#vectorize/2"},{"type":"function","title":"Examples - Nx.vectorize/2","doc":"In this first example, we turn a `{2, 3}`-shaped tensor into a vectorized tensor\nwith 1 vectorized axes and rank 1 shape, `{3}`, and then into a vectorized tensor\nwith 2 vectorized axes and rank 0 shape.\n\n    iex> t = Nx.iota({2, 3})\n    iex> vectorized = Nx.vectorize(t, :first)\n    #Nx.Tensor \n    iex> Nx.vectorize(vectorized, :second)\n    #Nx.Tensor \n\nYou can also vectorize multiple axes at once by passing a list,\nas seen in the examples below. The first example doesn't validate\nsizes. The second ensures the second axis has size `3`.\n\n    iex> t = Nx.iota({2, 3})\n    iex> v1 = Nx.vectorize(t, [:first, :second])\n    #Nx.Tensor \n    iex> v2 = Nx.vectorize(t, [:first, second: 3])\n    iex> v1 == v2\n    true\n\nA vectorized tensor can be thought of as a tensor that signals\nto Nx that any operation applied on it must instead be applied\nto each individual entry for the vectorized axis.\nNested vectorizations just apply this idea recursively, ultimately\napplying the operation to each non-vectorized entry.\n\nIn the following example, notice that you don't need to have the\nsecond argument shaped in a way that can be broadcasted, because\nvectorization handles that automatically.\n\nIn the example below, shape `{4}` isn't broadcast-compatible with `{2}`:\n\n    iex> Nx.add(Nx.tensor([4, 3, 2, 1]), Nx.tensor([0, 1]))\n    ** (ArgumentError) cannot broadcast tensor of dimensions {4} to {2}\n\nIf we want to add the two tensors, normally we would need to reshape\nto signal which axis are broadcasted together:\n\n    iex> left = Nx.tensor([4, 3, 2, 1]) |> Nx.reshape({4, 1})\n    iex> right = Nx.tensor([0, 1]) |> Nx.reshape({1, 2})\n    iex> Nx.add(left, right)\n    #Nx.Tensor \n\nHowever, it `vectorize/1` simplifies this process. We can instead\nsignal that each entry on the `left` tensor will be treated as an\nindividual tensor, effectively forcing the same broadcast to happen.\nIn fact, you can think of the following code as a series of\nadditions between tensors of shapes `{}` and `{2}` respectively.\n\n    iex> vectorized = Nx.vectorize(Nx.tensor([4, 3, 2, 1]), :x)\n    #Nx.Tensor \n    iex> Nx.add(vectorized, Nx.tensor([0, 1]))\n    #Nx.Tensor","ref":"Nx.html#vectorize/2-examples"},{"type":"function","title":"Containers - Nx.vectorize/2","doc":"Containers are also supported:\n\n    iex> input = {Nx.tensor([1]), %{a: Nx.tensor([2])}}\n    iex> {t1, %{a: t2}} = Nx.vectorize(input, x: 1)\n    iex> t1\n    #Nx.Tensor \n    iex> t2\n    #Nx.Tensor","ref":"Nx.html#vectorize/2-containers"},{"type":"function","title":"Error cases - Nx.vectorize/2","doc":"iex> Nx.vectorize(Nx.tensor(1), :x)\n    ** (ArgumentError) cannot vectorize tensor of rank 0\n\n    iex> Nx.vectorize(Nx.tensor([1]), [:x, :y])\n    ** (ArgumentError) number of vectorized axes must not be greater than the shape size\n\n    iex> Nx.vectorize(Nx.tensor([1]), [x: 2])\n    ** (ArgumentError) expected vectorized axis :x to have size 2, got 1\n\n    iex> Nx.vectorize(Nx.tensor([[1]]), [:x, \"y\"])\n    ** (ArgumentError) expected vectorized axis specification to be an atom or a tuple of {atom, pos_integer}, got: \"y\"\n\n    iex> Nx.vectorize(Nx.tensor([[1]], names: [:x, :y]), [:y])\n    ** (ArgumentError) cannot use name :y for new vectorized axes because there's already an axis with the same name\n\n    iex> t = Nx.vectorize(Nx.tensor([[1]]), :x)\n    iex> Nx.vectorize(t, :x)\n    ** (ArgumentError) cannot use name :x for new vectorized axes because there's already a vectorized axis with the same name","ref":"Nx.html#vectorize/2-error-cases"},{"type":"function","title":"Nx.weighted_mean/3","doc":"Returns the weighted mean for the tensor and the weights.\n\nIf the `:axes` option is given, it aggregates over\nthose dimensions, effectively removing them. `axes: [0]`\nimplies aggregating over the highest order dimension\nand so forth. If the axes are negative, then the axes will\nbe counted from the back. For example, `axes: [-1]` will\nalways aggregate over the last dimension.\n\nYou may optionally set `:keep_axes` to true, which will\nretain the rank of the input tensor by setting the averaged\naxes to size 1.","ref":"Nx.html#weighted_mean/3"},{"type":"function","title":"Examples - Nx.weighted_mean/3","doc":"iex> Nx.weighted_mean(Nx.tensor(42), Nx.tensor(2))\n    #Nx.Tensor \n\n    iex> Nx.weighted_mean(Nx.tensor([1, 2, 3]), Nx.tensor([3, 2, 1]))\n    #Nx.Tensor \n\n#","ref":"Nx.html#weighted_mean/3-examples"},{"type":"function","title":"Aggregating over axes - Nx.weighted_mean/3","doc":"iex> Nx.weighted_mean(Nx.tensor([1, 2, 3], names: [:x]), Nx.tensor([4, 5, 6]), axes: [0])\n    #Nx.Tensor \n\n    iex> Nx.weighted_mean(Nx.tensor([1, 2, 3], type: :u8, names: [:x]), Nx.tensor([1, 3, 5]), axes: [:x])\n    #Nx.Tensor \n\n    iex> t = Nx.iota({3, 4})\n    iex> weights = Nx.tensor([1, 2, 3, 4])\n    iex> Nx.weighted_mean(t, weights, axes: [1])\n    #Nx.Tensor \n\n    iex> t = Nx.iota({2, 4, 4, 1})\n    iex> weights = Nx.broadcast(2, {4, 4})\n    iex> Nx.weighted_mean(t, weights, axes: [1, 2])\n    #Nx.Tensor \n\n#","ref":"Nx.html#weighted_mean/3-aggregating-over-axes"},{"type":"function","title":"Keeping axes - Nx.weighted_mean/3","doc":"iex> t = Nx.tensor(Nx.iota({2, 2, 3}), names: [:x, :y, :z])\n    iex> weights = Nx.tensor([[[0, 1, 2], [1, 1, 0]], [[-1, 1, -1], [1, 1, -1]]])\n    iex> Nx.weighted_mean(t, weights, axes: [-1], keep_axes: true)\n    #Nx.Tensor \n\n#","ref":"Nx.html#weighted_mean/3-keeping-axes"},{"type":"function","title":"Vectorized tensors - Nx.weighted_mean/3","doc":"iex> t = Nx.tensor([[1, 2, 3], [1, 1, 1]]) |> Nx.vectorize(:x)\n    #Nx.Tensor \n    iex> w = Nx.tensor([[1, 1, 1], [0, 0, 1]]) |> Nx.vectorize(:y)\n    #Nx.Tensor \n    iex> Nx.weighted_mean(t, w)\n    #Nx.Tensor","ref":"Nx.html#weighted_mean/3-vectorized-tensors"},{"type":"function","title":"Nx.window_max/3","doc":"Returns the maximum over each window of size `window_dimensions`\nin the given tensor, producing a tensor that contains the same\nnumber of elements as valid positions of the window.\n\nYou may optionally specify `:strides` which is a tuple\nof non-zero steps to take along each axis between\neach window.\n\nYou may also optionally specify `:padding` which is either\none of `:valid` (no padding) or `:same` (pad so output shape\nis the same as input shape) or a general padding configuration\nfor each dimension in the input tensor. Your padding configuration\ncannot include any negative pad values. You may only specify\npadding for the high and low edges of the given dimension. Pads\nwith the minimum value for the type of the given tensor.","ref":"Nx.html#window_max/3"},{"type":"function","title":"Examples - Nx.window_max/3","doc":"iex> Nx.window_max(Nx.tensor([[[1, 2, 3], [4, 5, 6]], [[1, 2, 3], [4, 5, 6]]]), {1, 2, 1})\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[1, 2, 3], [4, 5, 6]], [[1, 2, 3], [4, 5, 6]]])\n    iex> Nx.window_max(t, {2, 2, 1}, strides: [1, 2, 3], padding: [{0, 1}, {2, 0}, {1, 1}])\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[4.0, 2.0, 3.0], [2.0, 5.0, 6.5]], [[1.2, 2.2, 3.2], [4.0, 5.0, 6.2]]])\n    iex> Nx.window_max(t, {2, 1, 1}, strides: [2, 1, 1], padding: [{1, 1}, {0, 0}, {1, 1}])\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[4, 2, 1, 3], [4, 2, 1, 7]], [[1, 2, 5, 7], [1, 8, 9, 2]]])\n    iex> opts = [strides: [2, 1, 1], padding: :valid, window_dilations: [1, 2, 2]]\n    iex> Nx.window_max(t, {1, 1, 2}, opts)\n    #Nx.Tensor","ref":"Nx.html#window_max/3-examples"},{"type":"function","title":"Vectorized tensors - Nx.window_max/3","doc":"For vectorized tensors, the windows will slide throughout all vectorized axes,\nand all options refer to the inner shape only.\n\n    iex> t = Nx.iota({2, 1, 2, 5}) |> Nx.vectorize(:x) |> Nx.vectorize(:y)\n    #Nx.Tensor \n    iex> Nx.window_max(t, {2, 2}, strides: [1, 2], window_dilations: [1, 2])\n    #Nx.Tensor","ref":"Nx.html#window_max/3-vectorized-tensors"},{"type":"function","title":"Nx.window_mean/3","doc":"Averages over each window of size `window_dimensions` in the\ngiven tensor, producing a tensor that contains the same\nnumber of elements as valid positions of the window.\n\nYou may optionally specify `:strides` which is a tuple\nof non-zero steps to take along each axis between\neach window.\n\nYou may also optionally specify `:padding` which is either\none of `:valid` (no padding) or `:same` (pad so output shape\nis the same as input shape) or a general padding configuration\nfor each dimension in the input tensor. Your padding configuration\ncannot include any negative pad values. You may only specify\npadding for the high and low edges of the given dimension. Pads\nwith `0`.","ref":"Nx.html#window_mean/3"},{"type":"function","title":"Examples - Nx.window_mean/3","doc":"iex> t = Nx.tensor([[[1, 2, 3], [4, 5, 6]], [[1, 2, 3], [4, 5, 6]]])\n    iex> Nx.window_mean(t, {1, 2, 1})\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[1, 2, 3], [4, 5, 6]], [[1, 2, 3], [4, 5, 6]]])\n    iex> Nx.window_mean(t, {2, 2, 1}, strides: [1, 2, 3], padding: [{0, 1}, {2, 0}, {1, 1}])\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[4.0, 2.0, 3.0], [2.0, 5.0, 6.5]], [[1.2, 2.2, 3.2], [4.0, 5.0, 6.2]]])\n    iex> Nx.window_mean(t, {2, 1, 1}, strides: [2, 1, 1], padding: [{1, 1}, {0, 0}, {1, 1}])\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[4, 2, 1, 3], [4, 2, 1, 7]], [[1, 2, 5, 7], [1, 8, 9, 2]]])\n    iex> opts = [strides: [2, 1, 1], padding: :valid, window_dilations: [1, 2, 1]]\n    iex> Nx.window_mean(t, {1, 1, 2}, opts)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[4, 2, 1, 3], [4, 2, 1, 7]], [[1, 2, 5, 7], [1, 8, 9, 2]]])\n    iex> opts = [strides: [2, 1, 1], padding: :valid, window_dilations: [1, 2, 2]]\n    iex> Nx.window_mean(t, {1, 1, 2}, opts)\n    #Nx.Tensor","ref":"Nx.html#window_mean/3-examples"},{"type":"function","title":"Vectorized tensors - Nx.window_mean/3","doc":"For vectorized tensors, the windows will slide throughout all vectorized axes,\nand all options refer to the inner shape only.\n\n    iex> t = Nx.iota({2, 1, 2, 5}) |> Nx.vectorize(:x) |> Nx.vectorize(:y)\n    #Nx.Tensor \n    iex> Nx.window_mean(t, {2, 2}, strides: [1, 2], window_dilations: [1, 2])\n    #Nx.Tensor","ref":"Nx.html#window_mean/3-vectorized-tensors"},{"type":"function","title":"Nx.window_min/3","doc":"Returns the minimum over each window of size `window_dimensions`\nin the given tensor, producing a tensor that contains the same\nnumber of elements as valid positions of the window.\n\nYou may optionally specify `:strides` which is a tuple\nof non-zero steps to take along each axis between\neach window.\n\nYou may also optionally specify `:padding` which is either\none of `:valid` (no padding) or `:same` (pad so output shape\nis the same as input shape) or a general padding configuration\nfor each dimension in the input tensor. Your padding configuration\ncannot include any negative pad values. You may only specify\npadding for the high and low edges of the given dimension. Pads\nwith the maximum value for the type of the given tensor.","ref":"Nx.html#window_min/3"},{"type":"function","title":"Examples - Nx.window_min/3","doc":"iex> Nx.window_min(Nx.tensor([[[1, 2, 3], [4, 5, 6]], [[1, 2, 3], [4, 5, 6]]]), {1, 2, 1})\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[1, 2, 3], [4, 5, 6]], [[1, 2, 3], [4, 5, 6]]])\n    iex> Nx.window_min(t, {2, 2, 1}, strides: [1, 2, 3], padding: [{0, 1}, {2, 0}, {1, 1}])\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[4.0, 2.0, 3.0], [2.0, 5.0, 6.5]], [[1.2, 2.2, 3.2], [4.0, 5.0, 6.2]]])\n    iex> Nx.window_min(t, {2, 1, 1}, strides: [2, 1, 1], padding: [{1, 1}, {0, 0}, {1, 1}])\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[4, 2, 1, 3], [4, 2, 1, 7]], [[1, 2, 5, 7], [1, 8, 9, 2]]])\n    iex> opts = [strides: [2, 1, 1], padding: :valid, window_dilations: [1, 2, 2]]\n    iex> Nx.window_min(t, {1, 1, 2}, opts)\n    #Nx.Tensor","ref":"Nx.html#window_min/3-examples"},{"type":"function","title":"Vectorized tensors - Nx.window_min/3","doc":"For vectorized tensors, the windows will slide throughout all vectorized axes,\nand all options refer to the inner shape only.\n\n    iex> t = Nx.iota({2, 1, 2, 5}) |> Nx.vectorize(:x) |> Nx.vectorize(:y)\n    #Nx.Tensor \n    iex> Nx.window_min(t, {2, 2}, strides: [1, 2], window_dilations: [1, 2])\n    #Nx.Tensor","ref":"Nx.html#window_min/3-vectorized-tensors"},{"type":"function","title":"Nx.window_product/3","doc":"Returns the product over each window of size `window_dimensions`\nin the given tensor, producing a tensor that contains the same\nnumber of elements as valid positions of the window.\n\nThe rank of the input tensor and the window dimensions must\nmatch.\n\nYou may optionally specify `:strides` which is a tuple\nof non-zero steps to take along each axis between\neach window.\n\nYou may also optionally specify `:padding` which is either\none of `:valid` (no padding) or `:same` (pad so output shape\nis the same as input shape) or a general padding configuration\nfor each dimension in the input tensor. Your padding configuration\ncannot include any negative pad values. You may only specify\npadding for the high and low edges of the given dimension. Pads\nwith 1.","ref":"Nx.html#window_product/3"},{"type":"function","title":"Examples - Nx.window_product/3","doc":"iex> Nx.window_product(Nx.tensor([[[1, 2, 3], [4, 5, 6]], [[1, 2, 3], [4, 5, 6]]]), {1, 2, 1})\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[1, 2, 3], [4, 5, 6]], [[1, 2, 3], [4, 5, 6]]])\n    iex> Nx.window_product(t, {2, 2, 1}, strides: [1, 2, 3], padding: [{0, 1}, {2, 0}, {1, 1}])\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[4.0, 2.0, 3.0], [2.0, 5.0, 6.5]], [[1.2, 2.2, 3.2], [4.0, 5.0, 6.2]]])\n    iex> Nx.window_product(t, {2, 1, 1}, strides: [2, 1, 1], padding: [{1, 1}, {0, 0}, {1, 1}])\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[4, 2, 1, 3], [4, 2, 1, 7]], [[1, 2, 5, 7], [1, 8, 9, 2]]])\n    iex> opts = [strides: [2, 1, 1], padding: :valid, window_dilations: [1, 2, 2]]\n    iex> Nx.window_product(t, {1, 1, 2}, opts)\n    #Nx.Tensor","ref":"Nx.html#window_product/3-examples"},{"type":"function","title":"Vectorized tensors - Nx.window_product/3","doc":"For vectorized tensors, the windows will slide throughout all vectorized axes,\nand all options refer to the inner shape only.\n\n    iex> t = Nx.iota({2, 1, 2, 5}) |> Nx.vectorize(:x) |> Nx.vectorize(:y)\n    #Nx.Tensor \n    iex> Nx.window_product(t, {2, 2}, strides: [1, 2], window_dilations: [1, 2])\n    #Nx.Tensor","ref":"Nx.html#window_product/3-vectorized-tensors"},{"type":"function","title":"Nx.window_reduce/5","doc":"Reduces over each window of size `dimensions`\nin the given tensor, producing a tensor that contains the same\nnumber of elements as valid positions of the window.\n\nThe rank of the input tensor and the window dimensions must\nmatch.\n\nYou may optionally specify `:strides` which is a tuple\nof non-zero steps to take along each axis between\neach window.\n\nYou may also optionally specify `:padding` which is either\none of `:valid` (no padding) or `:same` (pad so output shape\nis the same as input shape) or a general padding configuration\nfor each dimension in the input tensor. Your padding configuration\ncannot include any negative pad values. You may only specify\npadding for the high and low edges of the given dimension. The\npadding value is equal to the initial value passed to `acc`.\n\nThe initial value must be a number or a scalar shaped tensor.","ref":"Nx.html#window_reduce/5"},{"type":"function","title":"Examples - Nx.window_reduce/5","doc":"iex> init_value = Nx.Constants.min_finite(:s64)\n    iex> t = Nx.tensor([[1, 2, 3, 4], [4, 5, 6, 7], [7, 8, 9, 10], [11, 12, 13, 14]])\n    iex> Nx.window_reduce(t, init_value, {2, 2}, fn x, acc -> Nx.max(x, acc) end)\n    #Nx.Tensor \n\n    iex> init_value = Nx.Constants.min_finite(:s64)\n    iex> t = Nx.tensor([[1, 2, 3], [4, 5, 6], [7, 8, 9]])\n    iex> opts = [padding: :same, strides: [1, 1]]\n    iex> Nx.window_reduce(t, init_value, {2, 2}, opts, fn x, acc -> Nx.max(x, acc) end)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[1, 2, 3], [4, 5, 6]])\n    iex> opts = [padding: :same, strides: [1, 1]]\n    iex> Nx.window_reduce(t, 0, {1, 2}, opts, fn x, acc -> Nx.add(x, acc) end)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[4, 2, 1, 3], [4, 2, 1, 7]], [[1, 2, 5, 7], [1, 8, 9, 2]]])\n    iex> opts = [padding: :valid, strides: [2, 1, 1], window_dilations: [1, 1, 2]]\n    iex> Nx.window_reduce(t, 0, {1, 1, 2}, opts, fn x, acc -> Nx.add(x, acc) end)\n    #Nx.Tensor","ref":"Nx.html#window_reduce/5-examples"},{"type":"function","title":"Vectorized tensors - Nx.window_reduce/5","doc":"The accumulator must not be vectorized. Aside from that, `window_reduce` will apply the reduction\nover each non-vectorized entry, as follows:\n\n    iex> t = Nx.tensor([[[1, 2, 3], [4, 5, 6]], [[0, -1, -2], [-3, -4, -5]]]) |> Nx.vectorize(x: 2)\n    iex> opts = [padding: [{0, 0}, {0, 1}], strides: [1, 1]]\n    iex> Nx.window_reduce(t, 0, {2, 2}, opts, fn x, acc -> Nx.add(x, acc) end)\n    #Nx.Tensor","ref":"Nx.html#window_reduce/5-vectorized-tensors"},{"type":"function","title":"Nx.window_scatter_max/5","doc":"Performs a `window_reduce` to select the maximum index in each\nwindow of the input tensor according to and scatters source tensor\nto corresponding maximum indices in the output tensor.\n\nOutput tensor is initialized as a full tensor with values\n`init_value`. If indices overlap, adds overlapping source values.\nThe shape of the source tensor must match the valid windows in the\ninput tensor. This means the shape of the source tensor must match\nthe shape of the input tensor after a `window_reduce` op with padding\n`padding` and strides `strides`.\n\nThis function is the gradient of `window_max`.","ref":"Nx.html#window_scatter_max/5"},{"type":"function","title":"Examples - Nx.window_scatter_max/5","doc":"iex> t = Nx.tensor([\n    ...>   [7, 2, 5, 3, 10, 2],\n    ...>   [3, 8, 9, 3, 4, 2],\n    ...>   [1, 5, 7, 5, 6, 1],\n    ...>   [0, 6, 2, 7, 2, 8]\n    ...> ])\n    iex> opts = [strides: [2, 3], padding: :valid]\n    iex> Nx.window_scatter_max(t, Nx.tensor([[2, 6], [3, 1]]), 0, {2, 3}, opts)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([\n    ...>   [7, 2, 5, 3, 8],\n    ...>   [3, 8, 9, 3, 4],\n    ...>   [1, 5, 7, 5, 6],\n    ...>   [0, 6, 2, 10, 2]\n    ...> ])\n    iex> opts = [strides: [2, 2], padding: :valid]\n    iex> Nx.window_scatter_max(t, Nx.tensor([[2, 6], [3, 1]]), 0, {2, 3}, opts)\n    #Nx.Tensor","ref":"Nx.html#window_scatter_max/5-examples"},{"type":"function","title":"Vectorized tensors - Nx.window_scatter_max/5","doc":"The source and target tensors can be vectorized, and will be broadcasted\nthrough `broadcast_vectors/1` for the result calculation. `init_value`\nmust not be vectorized.\n\n    iex> t = Nx.tensor([\n    ...>   [\n    ...>     [7, 2, 5, 3],\n    ...>     [3, 8, 9, 3]\n    ...>   ],\n    ...>   [\n    ...>     [1, 5, 7, 5],\n    ...>     [0, 6, 2, 8]\n    ...>   ]\n    ...> ]) |> Nx.vectorize(:x)\n    iex> opts = [strides: [1, 2], padding: :valid]\n    iex> source = Nx.tensor([[[2, 6]], [[3, 1]]]) |> Nx.vectorize(:y)\n    iex> Nx.window_scatter_max(t, source, 0, {2, 2}, opts)\n    #Nx.Tensor","ref":"Nx.html#window_scatter_max/5-vectorized-tensors"},{"type":"function","title":"Nx.window_scatter_min/5","doc":"Performs a `window_reduce` to select the minimum index in each\nwindow of the input tensor according to and scatters source tensor\nto corresponding minimum indices in the output tensor.\n\nOutput tensor is initialized as a full tensor with values\n`init_value`. If indices overlap, adds overlapping source values.\nThe shape of the source tensor must match the valid windows in the\ninput tensor. This means the shape of the source tensor must match\nthe shape of the input tensor after a `window_reduce` op with padding\n`padding` and strides `strides`.\n\nThis function is the gradient of `window_min`.","ref":"Nx.html#window_scatter_min/5"},{"type":"function","title":"Examples - Nx.window_scatter_min/5","doc":"iex> t = Nx.tensor([\n    ...>   [7, 2, 5, 3, 10, 2],\n    ...>   [3, 8, 9, 3, 4, 2],\n    ...>   [1, 5, 7, 5, 6, 1],\n    ...>   [0, 6, 2, 7, 2, 8]\n    ...> ])\n    iex> opts = [strides: [2, 3], padding: :valid]\n    iex> Nx.window_scatter_min(t, Nx.tensor([[2, 6], [3, 1]]), 0, {2, 3}, opts)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([\n    ...>   [7, 2, 5, 3, 8],\n    ...>   [3, 8, 9, 3, 4],\n    ...>   [1, 5, 7, 5, 6],\n    ...>   [0, 6, 2, 10, 2]\n    ...> ])\n    iex> opts = [strides: [2, 2], padding: :valid]\n    iex> Nx.window_scatter_min(t, Nx.tensor([[2, 6], [3, 1]]), 0, {2, 3}, opts)\n    #Nx.Tensor","ref":"Nx.html#window_scatter_min/5-examples"},{"type":"function","title":"Vectorized tensors - Nx.window_scatter_min/5","doc":"The source and target tensors can be vectorized, and will be broadcasted\nthrough `broadcast_vectors/1` for the result calculation. `init_value`\nmust not be vectorized.\n\n    iex> t = Nx.tensor([\n    ...>   [\n    ...>     [7, 2, 5, 1],\n    ...>     [3, 8, 9, 3]\n    ...>   ],\n    ...>   [\n    ...>     [1, 5, 7, 5],\n    ...>     [0, 6, 2, 8]\n    ...>   ]\n    ...> ]) |> Nx.vectorize(:x)\n    iex> opts = [strides: [1, 2], padding: :valid]\n    iex> source = Nx.tensor([[[2, 6]], [[3, 1]]]) |> Nx.vectorize(:y)\n    iex> Nx.window_scatter_min(t, source, 0, {2, 2}, opts)\n    #Nx.Tensor","ref":"Nx.html#window_scatter_min/5-vectorized-tensors"},{"type":"function","title":"Nx.window_sum/3","doc":"Sums over each window of size `window_dimensions` in the\ngiven tensor, producing a tensor that contains the same\nnumber of elements as valid positions of the window.\n\nYou may optionally specify `:strides` which is a tuple\nof non-zero steps to take along each axis between\neach window.\n\nYou may also optionally specify `:padding` which is either\none of `:valid` (no padding) or `:same` (pad so output shape\nis the same as input shape) or a general padding configuration\nfor each dimension in the input tensor. Your padding configuration\ncannot include any negative pad values. You may only specify\npadding for the high and low edges of the given dimension. Pads\nwith `0`.","ref":"Nx.html#window_sum/3"},{"type":"function","title":"Examples - Nx.window_sum/3","doc":"iex> t = Nx.tensor([[[1, 2, 3], [4, 5, 6]], [[1, 2, 3], [4, 5, 6]]])\n    iex> Nx.window_sum(t, {1, 2, 1})\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[1, 2, 3], [4, 5, 6]], [[1, 2, 3], [4, 5, 6]]])\n    iex> Nx.window_sum(t, {2, 2, 1}, strides: [1, 2, 3], padding: [{0, 1}, {2, 0}, {1, 1}])\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[4.0, 2.0, 3.0], [2.0, 5.0, 6.5]], [[1.2, 2.2, 3.2], [4.0, 5.0, 6.2]]])\n    iex> Nx.window_sum(t, {2, 1, 1}, strides: [2, 1, 1], padding: [{1, 1}, {0, 0}, {1, 1}])\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[4, 2, 1, 3], [4, 2, 1, 7]], [[1, 2, 5, 7], [1, 8, 9, 2]]])\n    iex> opts = [strides: [2, 1, 1], padding: :valid, window_dilations: [1, 2, 1]]\n    iex> Nx.window_sum(t, {1, 1, 2}, opts)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[4, 2, 1, 3], [4, 2, 1, 7]], [[1, 2, 5, 7], [1, 8, 9, 2]]])\n    iex> opts = [strides: [2, 1, 1], padding: :valid, window_dilations: [1, 2, 2]]\n    iex> Nx.window_sum(t, {1, 1, 2}, opts)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[4, 2, 1, 3], [4, 2, 1, 7]], [[1, 2, 5, 7], [1, 8, 9, 2]]])\n    iex> opts = [strides: [2, 1, 1], padding: [{2, 1}, {3, 1}, {1, 0}], window_dilations: [1, 2, 2]]\n    iex> Nx.window_sum(t, {2, 1, 2}, opts)\n    #Nx.Tensor","ref":"Nx.html#window_sum/3-examples"},{"type":"function","title":"Vectorized tensors - Nx.window_sum/3","doc":"For vectorized tensors, the windows will slide throughout all vectorized axes,\nand all options refer to the inner shape only.\n\n    iex> t = Nx.iota({2, 1, 2, 5}) |> Nx.vectorize(:x) |> Nx.vectorize(:y)\n    #Nx.Tensor \n    iex> Nx.window_sum(t, {2, 2}, strides: [1, 2], window_dilations: [1, 2])\n    #Nx.Tensor","ref":"Nx.html#window_sum/3-vectorized-tensors"},{"type":"function","title":"Nx.with_default_backend/2","doc":"Invokes the given function temporarily setting `backend` as the\ndefault backend.","ref":"Nx.html#with_default_backend/2"},{"type":"type","title":"Nx.axes/0","doc":"","ref":"Nx.html#t:axes/0"},{"type":"type","title":"Nx.axis/0","doc":"","ref":"Nx.html#t:axis/0"},{"type":"type","title":"Nx.shape/0","doc":"","ref":"Nx.html#t:shape/0"},{"type":"type","title":"Nx.t/0","doc":"Represents a numerical value.\n\nCan be a plain number, a `Complex` number or an `Nx.Tensor`.\n\nSee also: `is_tensor/1`","ref":"Nx.html#t:t/0"},{"type":"type","title":"Nx.template/0","doc":"","ref":"Nx.html#t:template/0"},{"type":"module","title":"Nx.Constants","doc":"Common constants used in computations.\n\nThis module can be used in `defn`.","ref":"Nx.Constants.html"},{"type":"function","title":"Nx.Constants.e/0","doc":"Returns $e$ in f32.","ref":"Nx.Constants.html#e/0"},{"type":"function","title":"Nx.Constants.e/2","doc":"Returns a scalar tensor with the value of $e$ for the given type.","ref":"Nx.Constants.html#e/2"},{"type":"function","title":"Options - Nx.Constants.e/2","doc":"* `:backend` - a backend to allocate the tensor on.","ref":"Nx.Constants.html#e/2-options"},{"type":"function","title":"Examples - Nx.Constants.e/2","doc":"iex> Nx.Constants.e({:f, 64})\n    #Nx.Tensor \n\n    iex> Nx.Constants.e({:f, 32})\n    #Nx.Tensor \n\n    iex> Nx.Constants.e({:f, 16})\n    #Nx.Tensor \n\n    iex> Nx.Constants.e({:bf, 16})\n    #Nx.Tensor \n\n    iex> Nx.Constants.e({:s, 32})\n    ** (ArgumentError) only floating types are supported, got: {:s, 32}","ref":"Nx.Constants.html#e/2-examples"},{"type":"function","title":"Nx.Constants.epsilon/2","doc":"Returns a scalar with the machine epsilon for the given type.\n\nThe values are compatible with a IEEE 754 floating point standard.","ref":"Nx.Constants.html#epsilon/2"},{"type":"function","title":"Options - Nx.Constants.epsilon/2","doc":"* `:backend` - a backend to allocate the tensor on.","ref":"Nx.Constants.html#epsilon/2-options"},{"type":"function","title":"Examples - Nx.Constants.epsilon/2","doc":"iex> Nx.Constants.epsilon({:f, 64})\n    #Nx.Tensor \n\n    iex> Nx.Constants.epsilon({:f, 32})\n    #Nx.Tensor \n\n    iex> Nx.Constants.epsilon({:f, 16})\n    #Nx.Tensor \n\n    iex> Nx.Constants.epsilon(:bf16)\n    #Nx.Tensor \n\n    iex> Nx.Constants.epsilon({:s, 32})\n    ** (ArgumentError) only floating types are supported, got: {:s, 32}","ref":"Nx.Constants.html#epsilon/2-examples"},{"type":"function","title":"Nx.Constants.euler_gamma/0","doc":"Returns $\\gamma$ (Euler-Mascheroni constant) in f32.","ref":"Nx.Constants.html#euler_gamma/0"},{"type":"function","title":"Nx.Constants.euler_gamma/2","doc":"Returns a scalar tensor with the value of $\\gamma$ (Euler-Mascheroni constant) for the given type.","ref":"Nx.Constants.html#euler_gamma/2"},{"type":"function","title":"Options - Nx.Constants.euler_gamma/2","doc":"* `:backend` - a backend to allocate the tensor on.","ref":"Nx.Constants.html#euler_gamma/2-options"},{"type":"function","title":"Examples - Nx.Constants.euler_gamma/2","doc":"iex> Nx.Constants.euler_gamma({:f, 64})\n    #Nx.Tensor \n\n    iex> Nx.Constants.euler_gamma({:f, 32})\n    #Nx.Tensor \n\n    iex> Nx.Constants.euler_gamma({:f, 16})\n    #Nx.Tensor \n\n    iex> Nx.Constants.euler_gamma({:bf, 16})\n    #Nx.Tensor \n\n    iex> Nx.Constants.euler_gamma({:s, 32})\n    ** (ArgumentError) only floating types are supported, got: {:s, 32}","ref":"Nx.Constants.html#euler_gamma/2-examples"},{"type":"function","title":"Nx.Constants.i/0","doc":"Returns the imaginary constant in c64","ref":"Nx.Constants.html#i/0"},{"type":"function","title":"Nx.Constants.i/2","doc":"Returns the imaginary constant.\n\nAccepts the same options as `Nx.tensor/2`","ref":"Nx.Constants.html#i/2"},{"type":"function","title":"Examples - Nx.Constants.i/2","doc":"iex> Nx.Constants.i()\n    #Nx.Tensor \n\n    iex> Nx.Constants.i(:c128)\n    #Nx.Tensor","ref":"Nx.Constants.html#i/2-examples"},{"type":"function","title":"Error cases - Nx.Constants.i/2","doc":"iex> Nx.Constants.i({:f, 32})\n    ** (ArgumentError) invalid type for complex number. Expected {:c, 64} or {:c, 128}, got: {:f, 32}","ref":"Nx.Constants.html#i/2-error-cases"},{"type":"function","title":"Nx.Constants.infinity/0","doc":"Returns infinity in f32.","ref":"Nx.Constants.html#infinity/0"},{"type":"function","title":"Nx.Constants.infinity/2","doc":"Returns infinity.","ref":"Nx.Constants.html#infinity/2"},{"type":"function","title":"Options - Nx.Constants.infinity/2","doc":"* `:backend` - a backend to allocate the tensor on.","ref":"Nx.Constants.html#infinity/2-options"},{"type":"function","title":"Examples - Nx.Constants.infinity/2","doc":"iex> Nx.Constants.infinity({:bf, 16})\n    #Nx.Tensor \n\n    iex> Nx.Constants.infinity({:f, 16})\n    #Nx.Tensor \n\n    iex> Nx.Constants.infinity({:f, 32})\n    #Nx.Tensor \n\n    iex> Nx.Constants.infinity({:f, 64})\n    #Nx.Tensor","ref":"Nx.Constants.html#infinity/2-examples"},{"type":"function","title":"Nx.Constants.max/2","doc":"Returns a scalar tensor with the maximum value for the given type.\n\nIt is infinity for floating point tensors.","ref":"Nx.Constants.html#max/2"},{"type":"function","title":"Options - Nx.Constants.max/2","doc":"* `:backend` - a backend to allocate the tensor on.","ref":"Nx.Constants.html#max/2-options"},{"type":"function","title":"Examples - Nx.Constants.max/2","doc":"iex> Nx.Constants.max({:u, 8})\n    #Nx.Tensor \n\n    iex> Nx.Constants.max({:f, 32})\n    #Nx.Tensor","ref":"Nx.Constants.html#max/2-examples"},{"type":"function","title":"Nx.Constants.max_finite/2","doc":"Returns a scalar tensor with the maximum finite value for the given type.","ref":"Nx.Constants.html#max_finite/2"},{"type":"function","title":"Options - Nx.Constants.max_finite/2","doc":"* `:backend` - a backend to allocate the tensor on.","ref":"Nx.Constants.html#max_finite/2-options"},{"type":"function","title":"Examples - Nx.Constants.max_finite/2","doc":"iex> Nx.Constants.max_finite({:u, 8})\n    #Nx.Tensor \n\n    iex> Nx.Constants.max_finite({:s, 16})\n    #Nx.Tensor \n\n    iex> Nx.Constants.max_finite({:f, 32})\n    #Nx.Tensor","ref":"Nx.Constants.html#max_finite/2-examples"},{"type":"function","title":"Nx.Constants.min/2","doc":"Returns a scalar tensor with the minimum value for the given type.\n\nIt is negative infinity for floating point tensors.","ref":"Nx.Constants.html#min/2"},{"type":"function","title":"Options - Nx.Constants.min/2","doc":"* `:backend` - a backend to allocate the tensor on.","ref":"Nx.Constants.html#min/2-options"},{"type":"function","title":"Examples - Nx.Constants.min/2","doc":"iex> Nx.Constants.min({:u, 8})\n    #Nx.Tensor \n\n    iex> Nx.Constants.min({:f, 32})\n    #Nx.Tensor","ref":"Nx.Constants.html#min/2-examples"},{"type":"function","title":"Nx.Constants.min_finite/2","doc":"Returns a scalar tensor with the minimum finite value for the given type.","ref":"Nx.Constants.html#min_finite/2"},{"type":"function","title":"Options - Nx.Constants.min_finite/2","doc":"* `:backend` - a backend to allocate the tensor on.","ref":"Nx.Constants.html#min_finite/2-options"},{"type":"function","title":"Examples - Nx.Constants.min_finite/2","doc":"iex> Nx.Constants.min_finite({:u, 8})\n    #Nx.Tensor \n\n    iex> Nx.Constants.min_finite({:s, 16})\n    #Nx.Tensor \n\n    iex> Nx.Constants.min_finite({:f, 32})\n    #Nx.Tensor","ref":"Nx.Constants.html#min_finite/2-examples"},{"type":"function","title":"Nx.Constants.nan/0","doc":"Returns NaN in f32.","ref":"Nx.Constants.html#nan/0"},{"type":"function","title":"Nx.Constants.nan/2","doc":"Returns NaN (Not a Number).","ref":"Nx.Constants.html#nan/2"},{"type":"function","title":"Options - Nx.Constants.nan/2","doc":"* `:backend` - a backend to allocate the tensor on.","ref":"Nx.Constants.html#nan/2-options"},{"type":"function","title":"Examples - Nx.Constants.nan/2","doc":"iex> Nx.Constants.nan({:bf, 16})\n    #Nx.Tensor \n\n    iex> Nx.Constants.nan({:f, 16})\n    #Nx.Tensor \n\n    iex> Nx.Constants.nan({:f, 32})\n    #Nx.Tensor \n\n    iex> Nx.Constants.nan({:f, 64})\n    #Nx.Tensor","ref":"Nx.Constants.html#nan/2-examples"},{"type":"function","title":"Nx.Constants.neg_infinity/0","doc":"Returns infinity in f32.","ref":"Nx.Constants.html#neg_infinity/0"},{"type":"function","title":"Nx.Constants.neg_infinity/2","doc":"Returns negative infinity.","ref":"Nx.Constants.html#neg_infinity/2"},{"type":"function","title":"Options - Nx.Constants.neg_infinity/2","doc":"* `:backend` - a backend to allocate the tensor on.","ref":"Nx.Constants.html#neg_infinity/2-options"},{"type":"function","title":"Examples - Nx.Constants.neg_infinity/2","doc":"iex> Nx.Constants.neg_infinity({:bf, 16})\n    #Nx.Tensor \n\n    iex> Nx.Constants.neg_infinity({:f, 16})\n    #Nx.Tensor \n\n    iex> Nx.Constants.neg_infinity({:f, 32})\n    #Nx.Tensor \n\n    iex> Nx.Constants.neg_infinity({:f, 64})\n    #Nx.Tensor","ref":"Nx.Constants.html#neg_infinity/2-examples"},{"type":"function","title":"Nx.Constants.pi/0","doc":"Returns $\\pi$ in f32.","ref":"Nx.Constants.html#pi/0"},{"type":"function","title":"Nx.Constants.pi/2","doc":"Returns a scalar tensor with the value of $\\pi$ for the given type.","ref":"Nx.Constants.html#pi/2"},{"type":"function","title":"Options - Nx.Constants.pi/2","doc":"* `:backend` - a backend to allocate the tensor on.","ref":"Nx.Constants.html#pi/2-options"},{"type":"function","title":"Examples - Nx.Constants.pi/2","doc":"iex> Nx.Constants.pi({:f, 64})\n    #Nx.Tensor \n\n    iex> Nx.Constants.pi({:f, 32})\n    #Nx.Tensor \n\n    iex> Nx.Constants.pi({:f, 16})\n    #Nx.Tensor \n\n    iex> Nx.Constants.pi({:bf, 16})\n    #Nx.Tensor \n\n    iex> Nx.Constants.pi({:s, 32})\n    ** (ArgumentError) only floating types are supported, got: {:s, 32}","ref":"Nx.Constants.html#pi/2-examples"},{"type":"function","title":"Nx.Constants.smallest_positive_normal/2","doc":"Returns a scalar tensor with the maximum finite value for the given type.","ref":"Nx.Constants.html#smallest_positive_normal/2"},{"type":"function","title":"Options - Nx.Constants.smallest_positive_normal/2","doc":"* `:backend` - a backend to allocate the tensor on.","ref":"Nx.Constants.html#smallest_positive_normal/2-options"},{"type":"function","title":"Examples - Nx.Constants.smallest_positive_normal/2","doc":"iex> Nx.Constants.smallest_positive_normal({:f, 64})\n    #Nx.Tensor \n\n    iex> Nx.Constants.smallest_positive_normal({:f, 32})\n    #Nx.Tensor \n\n    iex> Nx.Constants.smallest_positive_normal({:f, 16})\n    #Nx.Tensor \n\n    iex> Nx.Constants.smallest_positive_normal(:bf16)\n    #Nx.Tensor \n\n    iex> Nx.Constants.smallest_positive_normal({:s, 32})\n    ** (ArgumentError) only floating types are supported, got: {:s, 32}","ref":"Nx.Constants.html#smallest_positive_normal/2-examples"},{"type":"module","title":"Nx.Defn","doc":"Numerical functions.\n\nA numerical function is a subset of Elixir tailored for\nnumerical computations. For example, the following function:\n\n    defmodule MyModule do\n      import Nx.Defn\n\n      defn softmax(t) do\n        Nx.exp(t) / Nx.sum(Nx.exp(t))\n      end\n    end\n\nwill work with scalars, vector, matrices, and n-dimensional\ntensors. Depending on your compiler of choice, the code can even\nbe JIT-compiled and run either on the CPU or GPU.\n\nTo support these features, `defn` is a subset of Elixir. It\nreplaces Elixir's `Kernel` by `Nx.Defn.Kernel`. `Nx.Defn.Kernel`\nprovides tensor-aware operators, such as `+`, `-`, etc, while\nalso preserving many high-level constructs known to Elixir\ndevelopers, such as pipe operator, aliases, conditionals,\npattern-matching, the access syntax, and more:\n\nFor example, the code above can also be written as:\n\n    defmodule MyModule do\n      import Nx.Defn\n\n      defn softmax(t) do\n        t\n        |> Nx.exp()\n        |> then(& &1 / Nx.sum(&1))\n      end\n    end\n\nPlease consult `Nx.Defn.Kernel` for a complete reference.\n\nThis module can be used in `defn`.","ref":"Nx.Defn.html"},{"type":"module","title":"Operators - Nx.Defn","doc":"`defn` attempts to keep as close to the Elixir semantics as\npossible but that's not achievable. For example, mathematical\nand bitwise operators (`+`, `-`, `&&&`, `<<<`, etc.) in Elixir\nwork on numbers, which means mapping them to tensors is\nstraight-forward and they largely preserve the same semantics,\nexcept they are now multi-dimensional.\n\nOn the other hand, the logical operators `and`, `or`, and `not`\nwork with booleans in Elixir (`true` and `false`), which map\nto `0` and `1` in `defn`.\n\nTherefore, when working with logical operators inside `defn`,\n`0` is considered `false` and all other numbers are considered\n`true`, which is represented as the number `1`. For example, in\n`defn`, `0 and 1` as well as `0 and 2` return `0`, while\n`1 and 1` or `1 and -1` will return `1`.\n\nThe same semantics apply to conditional expressions inside `defn`,\nsuch as `if`, `while`, etc.","ref":"Nx.Defn.html#module-operators"},{"type":"module","title":"JIT compilers - Nx.Defn","doc":"The power of `Nx.Defn` is given by its compilers. The default\ncompiler is `Nx.Defn.Evaluator`, which evalutes the code.\nYou can use `jit/3` to compile a function on the fly using a\ndifferent compiler, such as `EXLA`:\n\n    fun = Nx.Defn.jit(&MyModule.softmax/1, compiler: EXLA)\n    fun.(my_tensor)\n\nThe above will return an anonymous function that optimizes,\ncompiles, and run `softmax` on the fly on the CPU (or the GPU)\nif available. EXLA, in particular, also exports a `EXLA.jit/2`\nfunction for convenience.\n\n`defn` functions are compiled when they are invoked, based on\nthe type and shapes of the tensors given as arguments.\nTherefore compilation may be quite time consuming on the first\ninvocation. The compilation is then cached based on the tensors\nshapes and types. Calling the same function with a tensor of\ndifferent values but same shape and type means no recompilation\nis performed.\n\nFor those interested in writing custom compilers, see `Nx.Defn.Compiler`.","ref":"Nx.Defn.html#module-jit-compilers"},{"type":"module","title":"Invoking custom Elixir code - Nx.Defn","doc":"Inside `defn` you can only call other `defn` functions and\nthe functions in the `Nx` module. However, it is possible\nto use transforms, defined with either `deftransform` or\n`deftransformp` to invoke any Elixir code.\n\nYou can call code which was defined with `deftransform` from another module:\n\n    defmodule MyRemoteModule do\n      import Nx.Defn\n\n      deftransform remote_elixir_code(value) do\n        IO.inspect(value)\n      end\n    end\n\n    defn add_and_mult(a, b, c) do\n      res = a * b + c\n      MyRemoteModule.remote_elixir_code(res)\n    end\n\nYou can also define and call a private transform defined through `deftransformp`:\n\n    defn add_and_mult(a, b, c) do\n      res = a * b + c\n      custom_elixir_code(res)\n    end\n\n    deftransformp custom_elixir_code(value), do: IO.inspect(value)\n\nThe only difference between using `deftransform` and `deftransformp`\nis wether you want to expose and share the code with other modules,\njust like `def` and `defp`.\n\nTransforms are useful to manipulate tensor expressions or\nElixir data structures without the constraints of `defn`.","ref":"Nx.Defn.html#module-invoking-custom-elixir-code"},{"type":"module","title":"Inputs and outputs types - Nx.Defn","doc":"`Nx` and `defn` expect the arguments to be numbers, tensors,\nor one composite data type that implements `Nx.LazyContainer`.\nTuples and maps implement `Nx.LazyContainer` by default.\nAs previously described, `defn` are cached based on the shape,\ntype, and names of the input tensors, but not their values.\n\n`defn` also accepts two special arguments: functions (or tuples\nof functions) and lists (most commonly as keyword lists). Those\nvalues are passed as is to numerical definitions and cached as\na whole. For this reason, you must never capture tensors in\nfunctions or pass tensors in keyword lists.\n\nWhen numbers are given as arguments, they are always immediately\nconverted to tensors on invocation. If you want to keep numbers\nas is or if you want to pass any other value to numerical definitions,\nthey must be given as keyword lists.\n\n#","ref":"Nx.Defn.html#module-inputs-and-outputs-types"},{"type":"module","title":"Default arguments - Nx.Defn","doc":"`defn` functions support default arguments. They are typically used\nas options. For example, imagine you want to create a function named\nzeros, which returns a tensor of zeroes with a given type and shape.\nIt could be implemented like this:\n\n    defn zeros(opts \\\\ []) do\n      opts = keyword!(opts, type: {:f, 32}, shape: {})\n      Nx.broadcast(Nx.tensor(0, type: opts[:type]), opts[:shape])\n    end\n\nThe function above accepts `opts` which are then validated and given\ndefault values via the `keyword!/2` function. Note that while it is\npossible to access options via the `Access` syntax, such as `opts[:shape]`,\nit is not possible to directly call functions in the `Keyword` module\ninside `defn`. To freely manipulate any Elixir value inside `defn`,\nyou have to use transforms, as described in the \"Invoking custom Elixir\ncode\" section.\n\n> **Important!** When it comes to JIT compilation, each different set of\n> options (as well as anonymous functions) will lead to a different\n> compilation of the numerical function.\n>\n> Furthermore, if tensors are given through keyword lists, they won't\n> be cached effectively. Tensors in `defn` are cached based on their shape\n> and type, not their value, but this is not true if the tensor is given\n> via a default argument or captured by an anonymous function. For this\n> reason, it is **extremely discouraged to pass tensors through anonymous\n> functions and default arguments**.\n\n#","ref":"Nx.Defn.html#module-default-arguments"},{"type":"module","title":"Working with maps and structs - Nx.Defn","doc":"While `Nx` supports maps in `defn`, you must be careful if your numerical\ndefinitions are receiving maps and returning maps. For example, imagine\nthis code:\n\n    defn update_a(map) do\n      %{map | a: Nx.add(map.a, 1)}\n    end\n\nThe following code increments the value under the key `:a`\nby 1. However, because the function receives the whole map and\nreturns the whole map, it means if the map has 120 keys, the\nwhole map will be copied to the CPU/GPU, and then brought back.\n\nHowever, if you do this instead:\n\n    defn update_a(map) do\n      Nx.add(map.a, 1)\n    end\n\nAnd then update the map on Elixir, outside of `defn`:\n\n    %{map | a: update_a(map)}\n\n`Nx` will only send the parts of the map that matters.","ref":"Nx.Defn.html#module-working-with-maps-and-structs"},{"type":"function","title":"Nx.Defn.compile/3","doc":"Compiles the given anonymous function with the given tensor shapes.\n\nWhile `jit/2` compiles a function just-in time based on the\ninput shapes, this function precompiles the given anonymous\nfunction based on the input shapes. This can be beneficial for\nlarge numerical definitions, where the cache mechanism in `jit/2`\nmay take miliseconds.\n\nFor example, take the following definition:\n\n    defn softmax(t), do: Nx.exp(t) / Nx.sum(Nx.exp(t))\n\nYou can jit and then apply it as:\n\n    fun = Nx.Defn.compile(&softmax/1, [Nx.template({3}, {:s, 64})], compiler: EXLA)\n    fun.(Nx.tensor([1, 2, 3]))\n\nYou can also pass a mixture of templates and options when\ncompiling a function. In such cases, you must only pass\nthe inputs when invoking the compiled function, as the options\nwill already be embedded in its compiled value:\n\n    fun = Nx.Defn.compile(&Nx.sum/2, [Nx.template({2, 2}, {:s, 64}), [axes: [1]]])\n    fun.(Nx.iota({2, 2}))\n\nIf the input tensors do not match the shape of the tensors\ngiven on compilation, it will raise.","ref":"Nx.Defn.html#compile/3"},{"type":"function","title":"Options - Nx.Defn.compile/3","doc":"* `:compiler` - the compiler for the JIT compilation\n\n  * `:hooks` - a map of hooks to execute. See `Nx.Defn.Kernel.hook/3`","ref":"Nx.Defn.html#compile/3-options"},{"type":"function","title":"Nx.Defn.debug_expr/2","doc":"Wraps an anonymous function to return its underlying defn expression.\n\n> ##","ref":"Nx.Defn.html#debug_expr/2"},{"type":"function","title":"Warning {: .warning} - Nx.Defn.debug_expr/2","doc":">\n> This function must be invoked for debugging purposes only.","ref":"Nx.Defn.html#debug_expr/2-warning-warning"},{"type":"function","title":"Options - Nx.Defn.debug_expr/2","doc":"* `:hooks` - a map of hooks to execute. See `Nx.Defn.Kernel.hook/3`","ref":"Nx.Defn.html#debug_expr/2-options"},{"type":"function","title":"Nx.Defn.debug_expr_apply/3","doc":"Invokes the anonymous function to return its underlying defn expression.\n\n> ##","ref":"Nx.Defn.html#debug_expr_apply/3"},{"type":"function","title":"Warning {: .warning} - Nx.Defn.debug_expr_apply/3","doc":">\n> This function must be invoked for debugging purposes only.\n\nIt accepts the same options as `debug_expr/2`.","ref":"Nx.Defn.html#debug_expr_apply/3-warning-warning"},{"type":"function","title":"Nx.Defn.default_options/0","doc":"Gets the default options for the current process.","ref":"Nx.Defn.html#default_options/0"},{"type":"function","title":"Nx.Defn.default_options/1","doc":"Sets the default options for `defn` in the current process.\n\nThe options defined here apply to all future invocations of\n`defn` done by the current process. It also applies to calls\nto the `jit/3` and `stream/3` functions in this module.\n\nThe default options are stored only in the process dictionary\nand override any global options. This means if you start a\nseparate process, such as `Task`, the default options must be\nset on the new process too.\n\nThe function returns the values that were previously set as default\noptions.\n\nThis function must be used only for scripting and testing.","ref":"Nx.Defn.html#default_options/1"},{"type":"function","title":"Examples - Nx.Defn.default_options/1","doc":"iex> Nx.Defn.default_options(compiler: EXLA, client: :cuda)\n    iex> Nx.Defn.default_options()\n    [compiler: EXLA, client: :cuda]","ref":"Nx.Defn.html#default_options/1-examples"},{"type":"macro","title":"Nx.Defn.defn/2","doc":"Defines a public numerical function.","ref":"Nx.Defn.html#defn/2"},{"type":"macro","title":"Nx.Defn.defnp/2","doc":"Defines a private numerical function.\n\nPrivate numerical functions are always inlined by\ntheir callers at compilation time. This happens to\nall local function calls within `defn`.","ref":"Nx.Defn.html#defnp/2"},{"type":"macro","title":"Nx.Defn.deftransform/1","doc":"Can be used to define bodiless clauses for multi-clause transforms.\n\nSee also: `deftransform/2`","ref":"Nx.Defn.html#deftransform/1"},{"type":"macro","title":"Examples - Nx.Defn.deftransform/1","doc":"deftransform foo(bar, baz \\ 1)\n    deftransform foo(bar, 1), do: bar\n    deftransform foo(bar, baz), do: bar + baz","ref":"Nx.Defn.html#deftransform/1-examples"},{"type":"macro","title":"Nx.Defn.deftransform/2","doc":"Defines a transform that executes the given `fun` with `arg`\nwhen building `defn` expressions.","ref":"Nx.Defn.html#deftransform/2"},{"type":"macro","title":"Example - Nx.Defn.deftransform/2","doc":"Take the following defn expression:\n\n    defn tanh_power(a, b) do\n      Nx.tanh(a) + Nx.pow(b, 2)\n    end\n\nLet's see a trivial example, which is to use `IO.inspect/1` to\nprint a tensor expression at definition time:\n\n    defn tanh_power(a, b) do\n      Nx.tanh(a) + Nx.pow(b, 2) |> my_inspect()\n    end\n\n    deftransformp my_inspect(expr), do: IO.inspect(expr)\n\nOr:\n\n    defn tanh_power(a, b) do\n      res = Nx.tanh(a) + Nx.pow(b, 2)\n      my_inspect(res)\n      res\n    end\n\nWhen invoked in both cases, it will print the expression being built\nby `defn`:\n\n    #Nx.Defn.Expr \n\nAlthough, for convenience, you might use `print_expr/2` instead.","ref":"Nx.Defn.html#deftransform/2-example"},{"type":"macro","title":"Nx.Defn.deftransformp/1","doc":"Private function version for `deftransform/1`","ref":"Nx.Defn.html#deftransformp/1"},{"type":"macro","title":"Nx.Defn.deftransformp/2","doc":"Private function version for `deftransform/2`","ref":"Nx.Defn.html#deftransformp/2"},{"type":"function","title":"Nx.Defn.global_default_options/1","doc":"Sets the default options globally.\n\nThe options defined here apply to all future invocations of\n`defn`. It also applies to calls to the `jit/3` and `stream/3`\nfunctions in this module.\n\nYou must avoid calling this function at runtime and mostly for\ntesting purposes. You may also set in your test environment using\nconfiguration:\n\n    config :nx, :default_defn_options, [compiler: EXLA, client: :cuda]\n\nThe function returns the values that were previously set as global\ndefault options.","ref":"Nx.Defn.html#global_default_options/1"},{"type":"function","title":"Nx.Defn.grad/1","doc":"Receives an anonymous function and returns a new anonymous function\nthat returns the gradient of the input function when invoked.","ref":"Nx.Defn.html#grad/1"},{"type":"function","title":"Examples - Nx.Defn.grad/1","doc":"iex> fun = Nx.Defn.grad(fn x -> Nx.sin(x) end)\n    iex> fun.(Nx.tensor(0))\n    #Nx.Tensor","ref":"Nx.Defn.html#grad/1-examples"},{"type":"function","title":"Nx.Defn.grad/2","doc":"Computes the gradient of the given `var` on `fun`.\n\nThe result of the `grad` function must be a scalar tensor.\nIf a non-scalar tensor is given, it is assumed the additional\ndimensions are batch dimensions.","ref":"Nx.Defn.html#grad/2"},{"type":"function","title":"Examples - Nx.Defn.grad/2","doc":"defn tanh_grad(t) do\n      grad(t, &Nx.tanh/&1)\n    end\n\nTo differentiate on multiple vars, pass a tuple as first argument:\n\n    defn tanh_power_grad(a, b) do\n      grad({a, b}, fn {a, b} -> Nx.tanh(a) + Nx.pow(b, 2) end)\n    end\n\n`var_or_vars` can be any `Nx.Container` with one or multiple\ntensors.","ref":"Nx.Defn.html#grad/2-examples"},{"type":"function","title":"Nx.Defn.jit/2","doc":"Wraps an anonymous function with just-in-time compilation.\n\nOnce invoked, the wrapped anonymous function will perform just\nin time compilation with the configured compiler. For example,\ntake the following definition:\n\n    defn softmax(t), do: Nx.exp(t) / Nx.sum(Nx.exp(t))\n\nYou can jit and then apply it as:\n\n    fun = Nx.Defn.jit(&softmax/1, compiler: EXLA)\n    fun.(Nx.tensor([1, 2, 3]))","ref":"Nx.Defn.html#jit/2"},{"type":"function","title":"Options - Nx.Defn.jit/2","doc":"* `:compiler` - the compiler for the JIT compilation\n\n  * `:hooks` - a map of hooks to execute. See `Nx.Defn.Kernel.hook/3`\n\n  * `:on_conflict` - what to do if a JIT compilation is already in place.\n    It may be `:raise` (the default), `:force` (forces a new JIT compilation),\n    or `:reuse` (reuses the exiting JIT compilation). It is not recommended\n    to set the `:compiler` option when reusing.","ref":"Nx.Defn.html#jit/2-options"},{"type":"function","title":"Nx.Defn.jit/3","doc":"","ref":"Nx.Defn.html#jit/3"},{"type":"function","title":"Nx.Defn.jit_apply/3","doc":"Invokes the anonymous function with just-in-time compilation.\n\nThis function is equivalent to calling `jit/2` and then applying\nthe given arguments to the anonymous function.\n\nFor example, take the following definition:\n\n    defn softmax(t), do: Nx.exp(t) / Nx.sum(Nx.exp(t))\n\nYou can `jit_apply/3` it as:\n\n    Nx.Defn.jit_apply(&softmax/1, [Nx.tensor([1, 2, 3])], compiler: EXLA)\n\nIt accepts the same options as `jit/2`.","ref":"Nx.Defn.html#jit_apply/3"},{"type":"function","title":"Nx.Defn.jit_or_apply/3","doc":"","ref":"Nx.Defn.html#jit_or_apply/3"},{"type":"function","title":"Nx.Defn.stream/3","doc":"Starts streaming the given anonymous function with just-in-time\ncompilation.\n\nAt least two arguments are expected:\n\n  1. The first argument is a tensor template of the data to\n     be streamed in\n\n  2. The second argument is a tensor with the stream initial state\n\nThe streaming function must return a two element tuple, the\nfirst element is the data to be sent and the second is the\naccumulator.\n\nFor each streamed chunk, you must call `Nx.Stream.send/2` and\n`Nx.Stream.recv/1`. You don't need to call `recv` immediately\nafter `send`, but doing so can be a useful mechanism to provide\nbackpressure. Once all chunks are sent, you must use `Nx.Stream.done/1`\nto receive the accumulated result. Let's see an example:\n\n    defmodule Streamed do\n      import Nx.Defn\n\n      defn sum(tensor, acc) do\n        {acc, tensor + acc}\n      end\n    end\n\nNow let's invoke it:\n\n    stream = Nx.Defn.stream(&Streamed.sum/2, [Nx.template({}, {:s, 64}), 0])\n\n    for i <- 1..5 do\n      Nx.Stream.send(stream, i)\n      IO.inspect {:chunk, Nx.Stream.recv(stream)}\n    end\n\n    IO.inspect {:result, Nx.Stream.done(stream)}\n\nIt will print:\n\n    {:chunk, 0}\n    {:chunk, 1}\n    {:chunk, 2}\n    {:chunk, 3}\n    {:chunk, 4}\n    {:result, 5}","ref":"Nx.Defn.html#stream/3"},{"type":"function","title":"Options - Nx.Defn.stream/3","doc":"* `:hooks` - a map of hooks to execute. See `Nx.Defn.Kernel.hook/3`","ref":"Nx.Defn.html#stream/3-options"},{"type":"function","title":"Beware: deadlocks - Nx.Defn.stream/3","doc":"Some backends (such as XLA) place locks around devices. For example,\nif you start streaming on the GPU, you cannot perform any other\noperation on the GPU until streaming is over.\n\nThis means if we modify the loop above to the following:\n\n    for i   Nx.multiply(2))\n      IO.inspect {:chunk, Nx.Stream.recv(stream)}\n    end\n\nThe loop may deadlock at the time it performs the multiplication.\nIn practice, this means you should perform the streaming on the GPU\nand the remaining operations on the CPU. If you only have a single\ndevice (i.e. only a CPU), then it may not be possible to perform the\nabove and you will have to restructure your code to manipulate the\ninput before streaming starts.","ref":"Nx.Defn.html#stream/3-beware-deadlocks"},{"type":"function","title":"Nx.Defn.value_and_grad/1","doc":"Receives an anonymous function and returns a new anonymous function\nthat returns the value and gradient of the input function when invoked.","ref":"Nx.Defn.html#value_and_grad/1"},{"type":"function","title":"Examples - Nx.Defn.value_and_grad/1","doc":"iex> fun = Nx.Defn.value_and_grad(fn x -> Nx.sin(x) end)\n    iex> {value, grad} = fun.(Nx.tensor(0))\n    iex> value\n    #Nx.Tensor \n    iex> grad\n    #Nx.Tensor","ref":"Nx.Defn.html#value_and_grad/1-examples"},{"type":"function","title":"Nx.Defn.value_and_grad/3","doc":"Computes the value and gradient of the given `var` on `fun`\nwith an optional data transformation.\n\nIt returns a tuple with the value and the gradient.","ref":"Nx.Defn.html#value_and_grad/3"},{"type":"function","title":"Examples - Nx.Defn.value_and_grad/3","doc":"defn tanh_grad(t) do\n      value_and_grad(t, &Nx.tanh/&1)\n    end\n\nTo differentiate on multiple vars, pass a tuple as first argument:\n\n    defn tanh_power_grad(a, b) do\n      value_and_grad({a, b}, fn {a, b} -> Nx.tanh(a) + Nx.pow(b, 2) end)\n    end\n\n`var_or_vars` can be any `Nx.Container` with one or multiple\ntensors.\n\n`transform` allows you to transform the expression before the gradient is\ncalculated. This enables optimizations that reuse parts of expressions. As\nan example, consider the following objective function:\n\n    defn objective(predict_fn, loss_fn, params, inputs, targets) do\n      preds = predict_fn.(params, inputs)\n      loss = loss_fn.(preds, targets)\n      {preds, loss}\n    end\n\nYou can compute the gradient with respect to just the loss function by applying\na transform:\n\n    {{preds, loss}, gradient} = value_and_grad(params, &objective(predict_fn, loss_fn, &1, inputs, targets), &elem(&1, 1))\n\n`preds` can be re-used to compute other metrics such as accuracy, absolute error,\netc. without having to do another forward pass.","ref":"Nx.Defn.html#value_and_grad/3-examples"},{"type":"module","title":"Nx.Defn.Kernel","doc":"All imported functionality available inside `defn` blocks.\n\nThis module can be used in `defn`.","ref":"Nx.Defn.Kernel.html"},{"type":"function","title":"Nx.Defn.Kernel.&&&/2","doc":"Element-wise bitwise AND operation.\n\nOnly integer tensors are supported.\nIt delegates to `Nx.bitwise_and/2` (supports broadcasting).","ref":"Nx.Defn.Kernel.html#&&&/2"},{"type":"function","title":"Examples - Nx.Defn.Kernel.&&&/2","doc":"defn and_or(a, b) do\n      {a &&& b, a ||| b}\n    end","ref":"Nx.Defn.Kernel.html#&&&/2-examples"},{"type":"function","title":"Nx.Defn.Kernel.**/2","doc":"Element-wise power operator.\n\nIt delegates to `Nx.pow/2` (supports broadcasting).","ref":"Nx.Defn.Kernel.html#**/2"},{"type":"function","title":"Examples - Nx.Defn.Kernel.**/2","doc":"defn pow(a, b) do\n      a ** b\n    end","ref":"Nx.Defn.Kernel.html#**/2-examples"},{"type":"function","title":"Nx.Defn.Kernel.*/2","doc":"Element-wise multiplication operator.\n\nIt delegates to `Nx.multiply/2` (supports broadcasting).","ref":"Nx.Defn.Kernel.html#*/2"},{"type":"function","title":"Examples - Nx.Defn.Kernel.*/2","doc":"defn multiply(a, b) do\n      a * b\n    end","ref":"Nx.Defn.Kernel.html#*/2-examples"},{"type":"macro","title":"Nx.Defn.Kernel.+/1","doc":"Element-wise unary plus operator.\n\nSimply returns the given argument.","ref":"Nx.Defn.Kernel.html#+/1"},{"type":"macro","title":"Examples - Nx.Defn.Kernel.+/1","doc":"defn plus_and_minus(a) do\n      {+a, -a}\n    end","ref":"Nx.Defn.Kernel.html#+/1-examples"},{"type":"function","title":"Nx.Defn.Kernel.+/2","doc":"Element-wise addition operator.\n\nIt delegates to `Nx.add/2` (supports broadcasting).","ref":"Nx.Defn.Kernel.html#+/2"},{"type":"function","title":"Examples - Nx.Defn.Kernel.+/2","doc":"defn add(a, b) do\n      a + b\n    end","ref":"Nx.Defn.Kernel.html#+/2-examples"},{"type":"macro","title":"Nx.Defn.Kernel.-/1","doc":"Element-wise unary plus operator.\n\nIt delegates to `Nx.negate/1`.","ref":"Nx.Defn.Kernel.html#-/1"},{"type":"macro","title":"Examples - Nx.Defn.Kernel.-/1","doc":"defn plus_and_minus(a) do\n      {+a, -a}\n    end","ref":"Nx.Defn.Kernel.html#-/1-examples"},{"type":"function","title":"Nx.Defn.Kernel.-/2","doc":"Element-wise subtraction operator.\n\nIt delegates to `Nx.subtract/2` (supports broadcasting).","ref":"Nx.Defn.Kernel.html#-/2"},{"type":"function","title":"Examples - Nx.Defn.Kernel.-/2","doc":"defn subtract(a, b) do\n      a - b\n    end","ref":"Nx.Defn.Kernel.html#-/2-examples"},{"type":"function","title":"Nx.Defn.Kernel.../0","doc":"Creates the full-slice range `0..-1//1`.\n\nThis function returns a range with the following properties:\n\n  * When enumerated, it is empty\n\n  * When used as a `slice`, it returns the sliced element as is","ref":"Nx.Defn.Kernel.html#../0"},{"type":"function","title":"Examples - Nx.Defn.Kernel.../0","doc":"iex> t = Nx.tensor([1, 2, 3])\n    iex> t[..]\n    #Nx.Tensor","ref":"Nx.Defn.Kernel.html#../0-examples"},{"type":"function","title":"Nx.Defn.Kernel.../2","doc":"Builds a range.\n\nRanges are inclusive and both sides must be integers.\n\nThe step of the range is computed based on the first\nand last values of the range.","ref":"Nx.Defn.Kernel.html#../2"},{"type":"function","title":"Examples - Nx.Defn.Kernel.../2","doc":"iex> t = Nx.tensor([1, 2, 3])\n    iex> t[1..2]\n    #Nx.Tensor","ref":"Nx.Defn.Kernel.html#../2-examples"},{"type":"function","title":"Nx.Defn.Kernel...///3","doc":"Builds a range with step.\n\nRanges are inclusive and both sides must be integers.","ref":"Nx.Defn.Kernel.html#..///3"},{"type":"function","title":"Examples - Nx.Defn.Kernel...///3","doc":"iex> t = Nx.tensor([1, 2, 3])\n    iex> t[1..2//1]\n    #Nx.Tensor","ref":"Nx.Defn.Kernel.html#..///3-examples"},{"type":"function","title":"Nx.Defn.Kernel.//2","doc":"Element-wise division operator.\n\nIt delegates to `Nx.divide/2` (supports broadcasting).","ref":"Nx.Defn.Kernel.html#//2"},{"type":"function","title":"Examples - Nx.Defn.Kernel.//2","doc":"defn divide(a, b) do\n      a / b\n    end","ref":"Nx.Defn.Kernel.html#//2-examples"},{"type":"macro","title":"Nx.Defn.Kernel.!=/2","doc":"Element-wise inequality operation.\n\nIt delegates to `Nx.not_equal/2`.","ref":"Nx.Defn.Kernel.html#!=/2"},{"type":"macro","title":"Examples - Nx.Defn.Kernel.!=/2","doc":"defn check_inequality(a, b) do\n      a != b\n    end","ref":"Nx.Defn.Kernel.html#!=/2-examples"},{"type":"macro","title":"Nx.Defn.Kernel.</2","doc":"Element-wise less than operation.\n\nIt delegates to `Nx.less/2`.","ref":"Nx.Defn.Kernel.html#%3C/2"},{"type":"macro","title":"Examples - Nx.Defn.Kernel.</2","doc":"defn check_less_than(a, b) do\n      a < b\n    end","ref":"Nx.Defn.Kernel.html#%3C/2-examples"},{"type":"function","title":"Nx.Defn.Kernel.<<</2","doc":"Element-wise left shift operation.\n\nOnly integer tensors are supported.\nIt delegates to `Nx.left_shift/2` (supports broadcasting).","ref":"Nx.Defn.Kernel.html#%3C%3C%3C/2"},{"type":"function","title":"Examples - Nx.Defn.Kernel.<<</2","doc":"defn shift_left_and_right(a, b) do\n      {a  >> b}\n    end","ref":"Nx.Defn.Kernel.html#%3C%3C%3C/2-examples"},{"type":"macro","title":"Nx.Defn.Kernel.<=/2","doc":"Element-wise less-equal operation.\n\nIt delegates to `Nx.less_equal/2`.","ref":"Nx.Defn.Kernel.html#%3C=/2"},{"type":"macro","title":"Examples - Nx.Defn.Kernel.<=/2","doc":"defn check_less_equal(a, b) do\n      a <= b\n    end","ref":"Nx.Defn.Kernel.html#%3C=/2-examples"},{"type":"macro","title":"Nx.Defn.Kernel.<>/2","doc":"Concatenates two strings.\n\nEquivalent to `Kernel. /2`.","ref":"Nx.Defn.Kernel.html#%3C%3E/2"},{"type":"macro","title":"Nx.Defn.Kernel.==/2","doc":"Element-wise equality operation.\n\nIt delegates to `Nx.equal/2`.","ref":"Nx.Defn.Kernel.html#==/2"},{"type":"macro","title":"Examples - Nx.Defn.Kernel.==/2","doc":"defn check_equality(a, b) do\n      a == b\n    end","ref":"Nx.Defn.Kernel.html#==/2-examples"},{"type":"macro","title":"Nx.Defn.Kernel.>/2","doc":"Element-wise greater than operation.\n\nIt delegates to `Nx.greater/2`.","ref":"Nx.Defn.Kernel.html#%3E/2"},{"type":"macro","title":"Examples - Nx.Defn.Kernel.>/2","doc":"defn check_greater_than(a, b) do\n      a > b\n    end","ref":"Nx.Defn.Kernel.html#%3E/2-examples"},{"type":"macro","title":"Nx.Defn.Kernel.>=/2","doc":"Element-wise greater-equal operation.\n\nIt delegates to `Nx.greater_equal/2`.","ref":"Nx.Defn.Kernel.html#%3E=/2"},{"type":"macro","title":"Examples - Nx.Defn.Kernel.>=/2","doc":"defn check_greater_equal(a, b) do\n      a >= b\n    end","ref":"Nx.Defn.Kernel.html#%3E=/2-examples"},{"type":"function","title":"Nx.Defn.Kernel.>>>/2","doc":"Element-wise right shift operation.\n\nOnly integer tensors are supported.\nIt delegates to `Nx.right_shift/2` (supports broadcasting).","ref":"Nx.Defn.Kernel.html#%3E%3E%3E/2"},{"type":"function","title":"Examples - Nx.Defn.Kernel.>>>/2","doc":"defn shift_left_and_right(a, b) do\n      {a  >> b}\n    end","ref":"Nx.Defn.Kernel.html#%3E%3E%3E/2-examples"},{"type":"macro","title":"Nx.Defn.Kernel.@/1","doc":"Reads a module attribute at compilation time.\n\nIt is useful to inject code constants into `defn`.\nIt delegates to `Kernel.@/1`.","ref":"Nx.Defn.Kernel.html#@/1"},{"type":"macro","title":"Examples - Nx.Defn.Kernel.@/1","doc":"@two_per_two Nx.tensor([[1, 2], [3, 4]])\n    defn add_2x2_attribute(t), do: t + @two_per_two","ref":"Nx.Defn.Kernel.html#@/1-examples"},{"type":"macro","title":"Nx.Defn.Kernel.alias/2","doc":"Defines an alias, as in `Kernel.SpecialForms.alias/2`.\n\nAn alias allows you to refer to a module using its aliased\nname. For example:\n\n    defn some_fun(t) do\n      alias Math.Helpers, as: MH\n      MH.fft(t)\n    end\n\nIf the `:as` option is not given, the alias defaults to\nthe last part of the given alias. For example,\n\n    alias Math.Helpers\n\nis equivalent to:\n\n    alias Math.Helpers, as: Helpers\n\nFinally, note that aliases define outside of a function also\napply to the function, as they have lexical scope:\n\n    alias Math.Helpers, as: MH\n\n    defn some_fun(t) do\n      MH.fft(t)\n    end","ref":"Nx.Defn.Kernel.html#alias/2"},{"type":"macro","title":"Nx.Defn.Kernel.and/2","doc":"Element-wise logical AND operation.\n\nZero is considered false, all other numbers\nare considered true.\n\nIt delegates to `Nx.logical_and/2` (supports broadcasting).\n\nIt does not support short-circuiting.","ref":"Nx.Defn.Kernel.html#and/2"},{"type":"macro","title":"Examples - Nx.Defn.Kernel.and/2","doc":"defn and_or(a, b) do\n      {a and b, a or b}\n    end","ref":"Nx.Defn.Kernel.html#and/2-examples"},{"type":"function","title":"Nx.Defn.Kernel.assert_keys/2","doc":"Asserts the keyword list has the given keys.\n\nIf it succeeds, it returns the given keyword list. Raises\nan error otherwise.","ref":"Nx.Defn.Kernel.html#assert_keys/2"},{"type":"function","title":"Examples - Nx.Defn.Kernel.assert_keys/2","doc":"To assert the tensor is a scalar, you can pass the empty tuple `shape`:\n\n    iex> assert_keys([one: 1, two: 2], [:one, :two])\n    [one: 1, two: 2]\n\nIf the keys are not available, an error is raised:\n\n    iex> assert_keys([one: 1, two: 2], [:three])\n    ** (ArgumentError) expected key :three in keyword list, got: [one: 1, two: 2]","ref":"Nx.Defn.Kernel.html#assert_keys/2-examples"},{"type":"function","title":"Nx.Defn.Kernel.attach_token/2","doc":"Attaches a token to an expression. See `hook/3`.","ref":"Nx.Defn.Kernel.html#attach_token/2"},{"type":"macro","title":"Nx.Defn.Kernel.case/2","doc":"Pattern matches the result of `expr` against the given clauses.\n\nFor example:\n\n    case Nx.shape(tensor) do\n      {_} -> implementation_for_rank_one(tensor)\n      {_, _} -> implementation_for_rank_two(tensor)\n      _ -> implementation_for_rank_n(tensor)\n    end\n\nOpposite to `cond/2` and `if/2`, which can execute the branching\nin the device, `case`s are always expanded when building the\nexpression, and never on the device. This allows `case/2` to work\nvery similarly to Elixir's own `Kernel.SpecialForms.case/2`,\nwith only the following restrictions in place:\n\n  * `case` inside defn only accepts structs, atoms, integers, and tuples as arguments\n  * `case` can match on struct names but not on its fields\n  * guards in `case` inside defn can only access variables defined within the pattern\n\nHere is an example of `case` with guards:\n\n    case Nx.shape(tensor) do\n      {x, y} when x > y -> implementation_for_tall(tensor)\n      {x, y} when x   implementation_for_wide(tensor)\n      {x, x} -> implementation_for_square(tensor)\n    end","ref":"Nx.Defn.Kernel.html#case/2"},{"type":"macro","title":"Nx.Defn.Kernel.cond/1","doc":"Evaluates the expression corresponding to the first\nclause that evaluates to a truthy value.\n\nIt has the format of:\n\n    cond do\n      condition1 ->\n        expr1\n\n      condition2 ->\n        expr2\n\n      true ->\n        expr3\n    end\n\nThe conditions must be a scalar. Zero is considered false,\nany other number is considered true. The booleans `false` and\n`true` are supported, but any other value will raise.\n\nAll clauses are normalized to the same type and are broadcast\nto the same shape. The last condition must always evaluate to\ntrue. All clauses are executed in the device, unless they can\nbe determined to always be true/false while building the numerical\nexpression.","ref":"Nx.Defn.Kernel.html#cond/1"},{"type":"macro","title":"Examples - Nx.Defn.Kernel.cond/1","doc":"cond do\n      Nx.all(Nx.greater(a, 0)) -> b * c\n      Nx.all(Nx.less(a, 0)) -> b + c\n      true -> b - c\n    end\n\nWhen a `defn` is invoked, all `cond` clauses are traversed\nand expanded in order to build their expressions. This means that,\n**if you attempt to raise in any clause, then it will always raise**.\nYou can only `raise` in limited situations inside `defn`, see\n`raise/2` for more information.","ref":"Nx.Defn.Kernel.html#cond/1-examples"},{"type":"function","title":"Nx.Defn.Kernel.create_token/0","doc":"Creates a token for hooks. See `hook/3`.","ref":"Nx.Defn.Kernel.html#create_token/0"},{"type":"function","title":"Nx.Defn.Kernel.custom_grad/3","doc":"Defines a custom gradient for the given expression.\n\nIt also expects a list of inputs of the gradient and a `fun`\nto compute the gradient. The function will be called with the\ncurrent gradient. It must return a list of arguments and their\nupdated gradient to continue applying `grad` on.","ref":"Nx.Defn.Kernel.html#custom_grad/3"},{"type":"function","title":"Examples - Nx.Defn.Kernel.custom_grad/3","doc":"For example, if the gradient of `cos(t)` were to be\nimplemented by hand:\n\n    def cos(t) do\n      custom_grad(Nx.cos(t), [t], fn g ->\n        [-g * Nx.sin(t)]\n      end)\n    end","ref":"Nx.Defn.Kernel.html#custom_grad/3-examples"},{"type":"function","title":"Nx.Defn.Kernel.div/2","doc":"Element-wise quotient operator.\n\nIt delegates to `Nx.quotient/2` (supports broadcasting).","ref":"Nx.Defn.Kernel.html#div/2"},{"type":"function","title":"Examples - Nx.Defn.Kernel.div/2","doc":"defn quotient(a, b) do\n      div(a, b)\n    end","ref":"Nx.Defn.Kernel.html#div/2-examples"},{"type":"function","title":"Nx.Defn.Kernel.elem/2","doc":"Gets the element at the zero-based index in tuple.\n\nIt raises ArgumentError when index is negative or it\nis out of range of the tuple elements.","ref":"Nx.Defn.Kernel.html#elem/2"},{"type":"function","title":"Examples - Nx.Defn.Kernel.elem/2","doc":"iex> tuple = {1, 2, 3}\n    iex> elem(tuple, 0)\n    1","ref":"Nx.Defn.Kernel.html#elem/2-examples"},{"type":"function","title":"Nx.Defn.Kernel.hook/2","doc":"Shortcut for `hook/3`.","ref":"Nx.Defn.Kernel.html#hook/2"},{"type":"function","title":"Nx.Defn.Kernel.hook/3","doc":"Defines a hook.\n\nHooks are a mechanism to execute an anonymous function for\nside-effects with runtime tensor values.\n\nLet's see an example:\n\n    defmodule Hooks do\n      import Nx.Defn\n\n      defn add_and_mult(a, b) do\n        add = hook(a + b, fn tensor -> IO.inspect({:add, tensor}) end)\n        mult = hook(a * b, fn tensor -> IO.inspect({:mult, tensor}) end)\n        {add, mult}\n      end\n    end\n\nNote a hook can only access the variables passed as arguments\nto the hook. It cannot access any other variable defined in\n`defn` outside of the hook.\n\nThe `defn` above defines two hooks, one is called with the\nvalue of `a + b` and another with `a * b`. Once you invoke\nthe function above, you should see this printed:\n\n    Hooks.add_and_mult(2, 3)\n    {:add, #Nx.Tensor }\n    {:mult, #Nx.Tensor }\n\nIn other words, the `hook` function accepts a tensor\nexpression as argument and it will invoke a custom\nfunction with a tensor value at runtime. `hook` returns\nthe result of the given expression. The expression can\nbe any tensor or a `Nx.Container`.\n\nNote **you must return the result of the `hook` call**.\nFor example, the code below won't inspect the `:add`\ntuple, because the hook is not returned from `defn`:\n\n    defn add_and_mult(a, b) do\n      _add = hook(a + b, fn tensor -> IO.inspect({:add, tensor}) end)\n      mult = hook(a * b, fn tensor -> IO.inspect({:mult, tensor}) end)\n      mult\n    end\n\nWe will learn how to hook into a value that is not part\nof the result in the \"Hooks and tokens\" section.","ref":"Nx.Defn.Kernel.html#hook/3"},{"type":"function","title":"Named hooks - Nx.Defn.Kernel.hook/3","doc":"It is possible to give names to the hooks. This allows them\nto be defined or overridden by calling `Nx.Defn.jit/2` or\n`Nx.Defn.stream/2`. Let's see an example:\n\n    defmodule Hooks do\n      import Nx.Defn\n\n      defn add_and_mult(a, b) do\n        add = hook(a + b, :hooks_add)\n        mult = hook(a * b, :hooks_mult)\n        {add, mult}\n      end\n    end\n\nNow you can pass the hook as argument as follows:\n\n    hooks = %{\n      hooks_add: fn tensor ->\n        IO.inspect {:add, tensor}\n      end\n    }\n\n    fun = Nx.Defn.jit(&Hooks.add_and_mult/2, hooks: hooks)\n    fun.(Nx.tensor(2), Nx.tensor(3))\n\n> **Important!** We recommend to prefix your hook names\n> by the name of your project to avoid conflicts.\n\nIf a named hook is not given, compilers can optimize\nthat away and not transfer the tensor from the device\nin the first place.\n\nYou can also mix named hooks with callbacks:\n\n    defn add_and_mult(a, b) do\n      add = hook(a + b, :hooks_add, fn tensor -> IO.inspect({:add, tensor}) end)\n      mult = hook(a * b, :hooks_mult, fn tensor -> IO.inspect({:mult, tensor}) end)\n      {add, mult}\n    end\n\nIf a hook with the same name is given to `Nx.Defn.jit/2`\nor `Nx.Defn.stream/2`, then it will override the default\ncallback.","ref":"Nx.Defn.Kernel.html#hook/3-named-hooks"},{"type":"function","title":"Hooks and tokens - Nx.Defn.Kernel.hook/3","doc":"So far, we have always returned the result of the `hook`\ncall. However, what happens if the values we want to\nhook are not part of the return value, such as below?\n\n    defn add_and_mult(a, b) do\n      _add = hook(a + b, :hooks_add, &IO.inspect({:add, &1}))\n      mult = hook(a * b, :hooks_mult, &IO.inspect({:mult, &1}))\n      mult\n    end\n\nIn such cases, you must use tokens. Tokens are used to\ncreate an ordering over hooks, ensuring hooks execute\nin a certain sequence:\n\n    defn add_and_mult(a, b) do\n      token = create_token()\n      {token, _add} = hook_token(token, a + b, :hooks_add, &IO.inspect({:add, &1}))\n      {token, mult} = hook_token(token, a * b, :hooks_mult, &IO.inspect({:mult, &1}))\n      attach_token(token, mult)\n    end\n\nThe example above creates a token and uses `hook_token/4`\nto create hooks attached to their respective tokens. By using a token,\nwe guarantee that those hooks will be invoked in the order\nin which they were defined. Then, at the end of the function,\nwe attach the token (and its associated hooks) to the result `mult`.\n\nIn fact, the `hook/3` function is implemented roughly like this:\n\n    def hook(tensor_expr, name, function) do\n      {token, result} = hook_token(create_token(), tensor_expr, name, function)\n      attach_token(token, result)\n    end\n\nNote you must attach the token at the end, otherwise the hooks\nwill be \"lost\", as if they were not defined. This also applies\nto conditionals and loops. The token must be attached within\nthe branch they are used. For example, this won't work:\n\n    token = create_token()\n\n    {token, result} =\n      if Nx.any(value) do\n        hook_token(token, some_value)\n      else\n        hook_token(token, another_value)\n      end\n\n    attach_token(token, result)\n\nInstead, you must write:\n\n    token = create_token()\n\n    if Nx.any(value) do\n      {token, result} = hook_token(token, some_value)\n      attach_token(token, result)\n    else\n      {token, result} = hook_token(token, another_value)\n      attach_token(token, result)\n    end","ref":"Nx.Defn.Kernel.html#hook/3-hooks-and-tokens"},{"type":"function","title":"Nx.Defn.Kernel.hook_token/3","doc":"Shortcut for `hook_token/4`.","ref":"Nx.Defn.Kernel.html#hook_token/3"},{"type":"function","title":"Nx.Defn.Kernel.hook_token/4","doc":"Defines a hook with an existing token. See `hook/3`.","ref":"Nx.Defn.Kernel.html#hook_token/4"},{"type":"macro","title":"Nx.Defn.Kernel.if/2","doc":"Provides if/else expressions.\n\nThe first argument must be a scalar. Zero is considered false,\nany other number is considered true. The booleans `false` and\n`true` are supported, but any other value will raise.\n\nThe second argument is a keyword list with `do` and `else`\nblocks. The sides are broadcast to return the same shape\nand normalized to return the same type.","ref":"Nx.Defn.Kernel.html#if/2"},{"type":"macro","title":"Examples - Nx.Defn.Kernel.if/2","doc":"if Nx.any(Nx.equal(t, 0)) do\n      0.0\n    else\n      1 / t\n    end\n\nIn case else is not given, it is assumed to be 0 with the\nsame as the do clause. If you want to nest multiple conditionals,\nsee `cond/1` instead.\n\nWhen a `defn` is invoked, both `do`/`else` clauses are traversed\nand expanded in order to build their expressions. This means that,\n**if you attempt to raise in any clause, then it will always raise**.\nYou can only `raise` in limited situations inside `defn`, see\n`raise/2` for more information.","ref":"Nx.Defn.Kernel.html#if/2-examples"},{"type":"macro","title":"Nx.Defn.Kernel.import/2","doc":"Imports functions and macros into the current scope,\nas in `Kernel.SpecialForms.import/2`.\n\nImports are typically discouraged in favor of `alias/2`.","ref":"Nx.Defn.Kernel.html#import/2"},{"type":"macro","title":"Examples - Nx.Defn.Kernel.import/2","doc":"defn some_fun(t) do\n      import Math.Helpers\n      fft(t)\n    end","ref":"Nx.Defn.Kernel.html#import/2-examples"},{"type":"function","title":"Nx.Defn.Kernel.inspect/2","doc":"Converts the given expression into a string.\n\n`inspect/2` is used to convert expressions into strings, typically\nto be used as part of error messages. If you want to inspect for\ndebugging, consider using `print_expr/2`, to print the underlying\nexpression, or `print_value/2` to print the value during execution.\n\n    defn square_shape(tensor) do\n      case Nx.shape(tensor) do\n        {n, n} -> n\n        shape -> raise ArgumentError, \"expected a square tensor: #{inspect(shape)}\"\n      end\n    end","ref":"Nx.Defn.Kernel.html#inspect/2"},{"type":"function","title":"Nx.Defn.Kernel.keyword!/2","doc":"Ensures the first argument is a `keyword` with the given\nkeys and default values.\n\nThe second argument must be a list of atoms, specifying\na given key, or tuples specifying a key and a default value.\nIf any of the keys in the `keyword` is not defined on\n`values`, it raises an error.\n\nThis does not validate required keys. For such, use `assert_keys/2`\ninstead.\n\nThis is equivalent to Elixir's `Keyword.validate!/2`.","ref":"Nx.Defn.Kernel.html#keyword!/2"},{"type":"function","title":"Examples - Nx.Defn.Kernel.keyword!/2","doc":"iex> keyword!([], [one: 1, two: 2]) |> Enum.sort()\n    [one: 1, two: 2]\n\n    iex> keyword!([two: 3], [one: 1, two: 2]) |> Enum.sort()\n    [one: 1, two: 3]\n\nIf atoms are given, they are supported as keys but do not\nprovide a default value:\n\n    iex> keyword!([], [:one, two: 2]) |> Enum.sort()\n    [two: 2]\n\n    iex> keyword!([one: 1], [:one, two: 2]) |> Enum.sort()\n    [one: 1, two: 2]\n\nPassing an unknown key raises:\n\n    iex> keyword!([three: 3], [one: 1, two: 2])\n    ** (ArgumentError) unknown key :three in [three: 3], expected one of [:one, :two]","ref":"Nx.Defn.Kernel.html#keyword!/2-examples"},{"type":"function","title":"Nx.Defn.Kernel.max/2","doc":"Element-wise maximum operation.\n\nIt delegates to `Nx.max/2` (supports broadcasting).","ref":"Nx.Defn.Kernel.html#max/2"},{"type":"function","title":"Examples - Nx.Defn.Kernel.max/2","doc":"defn min_max(a, b) do\n      {min(a, b), max(a, b)}\n    end","ref":"Nx.Defn.Kernel.html#max/2-examples"},{"type":"function","title":"Nx.Defn.Kernel.min/2","doc":"Element-wise minimum operation.\n\nIt delegates to `Nx.min/2` (supports broadcasting).","ref":"Nx.Defn.Kernel.html#min/2"},{"type":"function","title":"Examples - Nx.Defn.Kernel.min/2","doc":"defn min_max(a, b) do\n      {min(a, b), max(a, b)}\n    end","ref":"Nx.Defn.Kernel.html#min/2-examples"},{"type":"macro","title":"Nx.Defn.Kernel.not/1","doc":"Element-wise logical NOT operation.\n\nZero is considered false, all other numbers\nare considered true.\n\nIt delegates to `Nx.logical_not/1`.","ref":"Nx.Defn.Kernel.html#not/1"},{"type":"macro","title":"Examples - Nx.Defn.Kernel.not/1","doc":"defn logical_not(a), do: not a","ref":"Nx.Defn.Kernel.html#not/1-examples"},{"type":"macro","title":"Nx.Defn.Kernel.or/2","doc":"Element-wise logical OR operation.\n\nZero is considered false, all other numbers\nare considered true.\n\nIt delegates to `Nx.logical_or/2` (supports broadcasting).\n\nIt does not support short-circuiting.","ref":"Nx.Defn.Kernel.html#or/2"},{"type":"macro","title":"Examples - Nx.Defn.Kernel.or/2","doc":"defn and_or(a, b) do\n      {a and b, a or b}\n    end","ref":"Nx.Defn.Kernel.html#or/2-examples"},{"type":"function","title":"Nx.Defn.Kernel.print_expr/2","doc":"Prints the given expression to the terminal.\n\nIt returns the given expressions.","ref":"Nx.Defn.Kernel.html#print_expr/2"},{"type":"function","title":"Examples - Nx.Defn.Kernel.print_expr/2","doc":"defn tanh_grad(t) do\n      grad(t, &Nx.tanh/1) |> print_expr()\n    end\n\nWhen invoked, it will print the expression being built by `defn`:\n\n    #Nx.Tensor","ref":"Nx.Defn.Kernel.html#print_expr/2-examples"},{"type":"function","title":"Nx.Defn.Kernel.print_value/2","doc":"Prints the value at runtime to the terminal.\n\nThis function is implemented on top of `hook/3` and therefore\nhas the following restrictions:\n\n  * It can only inspect tensors and `Nx.Container`\n  * The return value of this function must be part of the output\n\nAll options are passed to `IO.inspect/2`.","ref":"Nx.Defn.Kernel.html#print_value/2"},{"type":"function","title":"Examples - Nx.Defn.Kernel.print_value/2","doc":"defn tanh_grad(t) do\n      grad(t, fn t ->\n        t\n        |> Nx.tanh()\n        |> print_value()\n      end)\n    end\n\n    defn tanh_grad(t) do\n      grad(t, fn t ->\n        t\n        |> Nx.tanh()\n        |> print_value(label: \"tanh\")\n      end)\n    end","ref":"Nx.Defn.Kernel.html#print_value/2-examples"},{"type":"macro","title":"Nx.Defn.Kernel.raise/1","doc":"Raises a runtime exception with the given `message`.\n\nSee `raise/2` for more information on exceptions inside `defn`.","ref":"Nx.Defn.Kernel.html#raise/1"},{"type":"macro","title":"Nx.Defn.Kernel.raise/2","doc":"Raises an `exception` with the given `arguments`.\n\n`raise/2` is invoked while building the numerical expression,\nnot inside the device. This means that `raise` may be invoked\non unexpected situations, as we build the numerical expression.\nTo better understand those cases, let's see some examples.\n\nFirst, let's start with a valid use case for `raise/2`: raise\non mismatched shapes. Inside `defn`, we know the tensor shapes\nand types, but not their values, so we can assert on the shape\nwhile building the numerical expression:\n\n    defn square_shape(tensor) do\n      case Nx.shape(tensor) do\n        {n, n} -> n\n        shape -> raise ArgumentError, \"expected a square tensor: #{inspect(shape)}\"\n      end\n    end\n\nIn the example above, only the matching branch of the case is executed,\nso if you give it a 2x2 tensor, it will return 2. However, if you give\nit a non-square tensor, it will raise.\n\nNow consider this code:\n\n    defn some_check(a, b) do\n      if a != b do\n        a * b\n      else\n        raise \"expected different tensors, got: #{inspect(a)} and #{inspect(b)}\"\n      end\n    end\n\nIn this case, both `a` and `b` are tensors and we are comparing their values.\nHowever, their values are unknown, which means we need to convert the whole\n`if` to a numerical expression and run it on the device. Therefore, once we\nconvert the `else` branch, it will execute `raise/2`, making it so the code\nabove always raises!\n\nIn such cases, there are no alternatives. We can't execute exceptions in the\nCPU/GPU, so you need to approach the problem under a different perspective.","ref":"Nx.Defn.Kernel.html#raise/2"},{"type":"function","title":"Nx.Defn.Kernel.rem/2","doc":"Element-wise remainder operation.\n\nIt delegates to `Nx.remainder/2` (supports broadcasting).","ref":"Nx.Defn.Kernel.html#rem/2"},{"type":"function","title":"Examples - Nx.Defn.Kernel.rem/2","doc":"defn divides_by_5?(a) do\n      rem(a, 5)\n      |> Nx.any()\n      |> Nx.equal(Nx.tensor(1))\n    end","ref":"Nx.Defn.Kernel.html#rem/2-examples"},{"type":"macro","title":"Nx.Defn.Kernel.require/2","doc":"Requires a module in order to use its macros, as in `Kernel.SpecialForms.require/2`.","ref":"Nx.Defn.Kernel.html#require/2"},{"type":"macro","title":"Examples - Nx.Defn.Kernel.require/2","doc":"defn some_fun(t) do\n      require NumericalMacros\n\n      NumericalMacros.some_macro t do\n        ...\n      end\n    end","ref":"Nx.Defn.Kernel.html#require/2-examples"},{"type":"function","title":"Nx.Defn.Kernel.rewrite_types/2","doc":"Rewrites the types of `expr` recursively according to `opts`","ref":"Nx.Defn.Kernel.html#rewrite_types/2"},{"type":"function","title":"Options - Nx.Defn.Kernel.rewrite_types/2","doc":"* `:max_unsigned_type` - replaces all signed tensors with size\n    equal to or greater then the given type by the given type\n\n  * `:max_signed_type` - replaces all signed tensors with size\n    equal to or greater then the given type by the given type\n\n  * `:max_float_type` - replaces all float tensors with size\n    equal to or greater then the given type by the given type","ref":"Nx.Defn.Kernel.html#rewrite_types/2-options"},{"type":"function","title":"Examples - Nx.Defn.Kernel.rewrite_types/2","doc":"rewrite_types(expr, max_float_type: {:f, 32})","ref":"Nx.Defn.Kernel.html#rewrite_types/2-examples"},{"type":"function","title":"Nx.Defn.Kernel.stop_grad/1","doc":"Stops computing the gradient for the given expression.\n\nIt effectively annotates the gradient for the given\nexpression is 1.0.","ref":"Nx.Defn.Kernel.html#stop_grad/1"},{"type":"function","title":"Examples - Nx.Defn.Kernel.stop_grad/1","doc":"expr = stop_grad(expr)","ref":"Nx.Defn.Kernel.html#stop_grad/1-examples"},{"type":"macro","title":"Nx.Defn.Kernel.tap/2","doc":"Pipes `value` to the given `fun` and returns the `value` itself.\n\nUseful for running synchronous side effects in a pipeline.","ref":"Nx.Defn.Kernel.html#tap/2"},{"type":"macro","title":"Examples - Nx.Defn.Kernel.tap/2","doc":"Let's suppose you want to inspect an expression in the middle of\na pipeline. You could write:\n\n    a\n    |> Nx.add(b)\n    |> tap(&print_expr/1)\n    |> Nx.multiply(c)","ref":"Nx.Defn.Kernel.html#tap/2-examples"},{"type":"macro","title":"Nx.Defn.Kernel.then/2","doc":"Pipes `value` into the given `fun`.\n\nIn other words, it invokes `fun` with `value` as argument.\nThis is most commonly used in pipelines, allowing you\nto pipe a value to a function outside of its first argument.","ref":"Nx.Defn.Kernel.html#then/2"},{"type":"macro","title":"Examples - Nx.Defn.Kernel.then/2","doc":"a\n    |> Nx.add(b)\n    |> then(&Nx.subtract(c, &1))","ref":"Nx.Defn.Kernel.html#then/2-examples"},{"type":"macro","title":"Nx.Defn.Kernel.while/4","doc":"Defines a `while` loop.\n\nIt expects the `initial` arguments, a `condition` expression, and\na `block`:\n\n    while initial, condition do\n      block\n    end\n\n`condition` must return a scalar tensor where 0 is false and any\nother number is true. The given `block` will be executed while\n`condition` is true. Each invocation of `block` must return a\nvalue in the same shape as `initial` arguments.\n\n`while` will return the value of the last execution of `block`.\nIf `block` is never executed because the initial `condition` is\nfalse, it returns `initial`.\n\n> Note: you must prefer to use the operations in the `Nx` module,\n> whenever available, instead of writing your own loops.","ref":"Nx.Defn.Kernel.html#while/4"},{"type":"macro","title":"Examples - Nx.Defn.Kernel.while/4","doc":"A simple loop that increments `x` until it is `10` can be written as:\n\n    while x = 0, Nx.less(x, 10) do\n      x + 1\n    end\n\nHowever, it is important to note that all variables you intend\nto use inside the \"while\" must be explicitly given as argument\nto \"while\". For example, imagine the amount we want to increment\nby in the example above is given by a variable `y`. The following\nexample is invalid:\n\n    while x = 0, Nx.less(x, 10) do\n      x + y\n    end\n\nInstead, both `x` and `y` must be passed as variables to `while`:\n\n    while {x = 0, y}, Nx.less(x, 10) do\n      {x + y, y}\n    end\n\nSimilarly, to compute the factorial of `x` using `while`:\n\n    defn factorial(x) do\n      {factorial, _} =\n        while {factorial = 1, x}, Nx.greater(x, 1) do\n          {factorial * x, x - 1}\n        end\n\n      factorial\n    end","ref":"Nx.Defn.Kernel.html#while/4-examples"},{"type":"macro","title":"Generators - Nx.Defn.Kernel.while/4","doc":"Inspired by Elixir's [for-comprehensions](`Kernel.SpecialForms.for/1`),\n`while` in `defn` supports generators. Generators may be tensors or ranges.\n\n#","ref":"Nx.Defn.Kernel.html#while/4-generators"},{"type":"macro","title":"Tensor generators - Nx.Defn.Kernel.while/4","doc":"When the generator is a tensor, Nx will traverse its highest dimension.\nFor example, you could sum a one dimensional tensor as follows:\n\n    while acc = 0, i   Note: implementing `sum` using `while`, as above, is done as an example.\n> In practice, you must prefer to use the operations in the `Nx` module,\n> whenever available, instead of writing your own loops.\n\nOne advantage of using generators is that you can also unroll the loop\nfor performance:\n\n    while acc = 0, i <- tensor, unroll: true do\n      acc + i\n    end\n\nOr unroll it in batches:\n\n    while acc = 0, i <- tensor, unroll: 4 do\n      acc + i\n    end\n\nUnrolling means that the the `while` body is automatically duplicated\na certain amount of times, as if you wrote all iterations by hand. This\nmakes the final expression larger, which causes a longer compilation\ntime, however it enables additional compile-time optimizations (such as\nfusion), improving the runtime efficiency.\n\nIn case the tensor for generator is vectorized, `:unroll` will only\naffect the non-vectorized part. For instance, if a tensor has shape `{4}`\nand vectorized axes `[x: 2][y: 3]`, `unroll: true` will only unroll\nthe `4` inner iterations.\n\n#","ref":"Nx.Defn.Kernel.html#while/4-tensor-generators"},{"type":"macro","title":"Range generators - Nx.Defn.Kernel.while/4","doc":"A range can also be given as a generator. The range may be increasing or\ndecreasing. Also remember that ranges in Elixir are inclusive on both\nbegin and end. The sum example from the previous section could also be\nwritten with ranges:\n\n    while {tensor, acc = 0}, i <- 0..Nx.axis_size(tensor, 0)-1 do\n      acc + tensor[i]\n    end","ref":"Nx.Defn.Kernel.html#while/4-range-generators"},{"type":"macro","title":"Nx.Defn.Kernel.|>/2","doc":"Pipes the argument on the left to the function call on the right.\n\nIt delegates to `Kernel.|>/2`.","ref":"Nx.Defn.Kernel.html#%7C%3E/2"},{"type":"macro","title":"Examples - Nx.Defn.Kernel.|>/2","doc":"defn exp_sum(t) do\n      t\n      |> Nx.exp()\n      |> Nx.sum()\n    end","ref":"Nx.Defn.Kernel.html#%7C%3E/2-examples"},{"type":"function","title":"Nx.Defn.Kernel.|||/2","doc":"Element-wise bitwise OR operation.\n\nOnly integer tensors are supported.\nIt delegates to `Nx.bitwise_or/2` (supports broadcasting).","ref":"Nx.Defn.Kernel.html#%7C%7C%7C/2"},{"type":"function","title":"Examples - Nx.Defn.Kernel.|||/2","doc":"defn and_or(a, b) do\n      {a &&& b, a ||| b}\n    end","ref":"Nx.Defn.Kernel.html#%7C%7C%7C/2-examples"},{"type":"function","title":"Nx.Defn.Kernel.~~~/1","doc":"Element-wise bitwise not operation.\n\nOnly integer tensors are supported.\nIt delegates to `Nx.bitwise_not/1`.","ref":"Nx.Defn.Kernel.html#~~~/1"},{"type":"function","title":"Examples - Nx.Defn.Kernel.~~~/1","doc":"defn bnot(a), do: ~~~a","ref":"Nx.Defn.Kernel.html#~~~/1-examples"},{"type":"module","title":"Nx.LinAlg","doc":"Nx conveniences for linear algebra.\n\nThis module can be used in `defn`.","ref":"Nx.LinAlg.html"},{"type":"function","title":"Nx.LinAlg.adjoint/1","doc":"Returns the adjoint of a given tensor.\n\nIf the input tensor is real it transposes it's two inner-most axes.\nIf the input tensor is complex, it additionally applies `Nx.conjugate/1` to it.","ref":"Nx.LinAlg.html#adjoint/1"},{"type":"function","title":"Examples - Nx.LinAlg.adjoint/1","doc":"iex> Nx.LinAlg.adjoint(Nx.tensor([[1, 2], [3, 4]]))\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.adjoint(Nx.tensor([[1, Complex.new(0, 2)], [3, Complex.new(0, -4)]]))\n    #Nx.Tensor","ref":"Nx.LinAlg.html#adjoint/1-examples"},{"type":"function","title":"Nx.LinAlg.cholesky/1","doc":"Performs a Cholesky decomposition of a batch of square matrices.\n\nThe matrices must be positive-definite and either Hermitian\nif complex or symmetric if real. An error is raised by the\ndefault backend if those conditions are not met. Other\nbackends may emit undefined behaviour.","ref":"Nx.LinAlg.html#cholesky/1"},{"type":"function","title":"Examples - Nx.LinAlg.cholesky/1","doc":"iex> Nx.LinAlg.cholesky(Nx.tensor([[20.0, 17.6], [17.6, 16.0]]))\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.cholesky(Nx.tensor([[[2.0, 3.0], [3.0, 5.0]], [[1.0, 0.0], [0.0, 1.0]]]))\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([\n    ...>   [6.0, 3.0, 4.0, 8.0],\n    ...>   [3.0, 6.0, 5.0, 1.0],\n    ...>   [4.0, 5.0, 10.0, 7.0],\n    ...>   [8.0, 1.0, 7.0, 25.0]\n    ...> ])\n    iex> Nx.LinAlg.cholesky(t)\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.cholesky(Nx.tensor([[1.0, Complex.new(0, -2)], [Complex.new(0, 2), 5.0]]))\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[2.0, 3.0], [3.0, 5.0]], [[1.0, 0.0], [0.0, 1.0]]]) |> Nx.vectorize(x: 2)\n    iex> Nx.LinAlg.cholesky(t)\n    #Nx.Tensor","ref":"Nx.LinAlg.html#cholesky/1-examples"},{"type":"function","title":"Error cases - Nx.LinAlg.cholesky/1","doc":"iex> Nx.LinAlg.cholesky(Nx.tensor([[1.0, 2.0], [3.0, 4.0]]))\n    ** (ArgumentError) matrix must be hermitian, a matrix is hermitian iff X = adjoint(X)","ref":"Nx.LinAlg.html#cholesky/1-error-cases"},{"type":"function","title":"Nx.LinAlg.determinant/1","doc":"Calculates the determinant of batched square matrices.","ref":"Nx.LinAlg.html#determinant/1"},{"type":"function","title":"Examples - Nx.LinAlg.determinant/1","doc":"For 2x2 and 3x3, the results are given by the closed formulas:\n\n    iex> Nx.LinAlg.determinant(Nx.tensor([[1, 2], [3, 4]]))\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.determinant(Nx.tensor([[1.0, 2.0, 3.0], [1.0, -2.0, 3.0], [7.0, 8.0, 9.0]]))\n    #Nx.Tensor \n\nWhen there are linearly dependent rows or columns, the determinant is 0:\n\n    iex> Nx.LinAlg.determinant(Nx.tensor([[1.0, 0.0], [3.0, 0.0]]))\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.determinant(Nx.tensor([[1.0, 2.0, 3.0], [-1.0, -2.0, -3.0], [4.0, 5.0, 6.0]]))\n    #Nx.Tensor \n\nThe determinant can also be calculated when the axes are bigger than 3:\n\n    iex> Nx.LinAlg.determinant(Nx.tensor([\n    ...> [1, 0, 0, 0],\n    ...> [0, 1, 2, 3],\n    ...> [0, 1, -2, 3],\n    ...> [0, 7, 8, 9.0]\n    ...> ]))\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.determinant(Nx.tensor([\n    ...> [0, 0, 0, 0, -1],\n    ...> [0, 1, 2, 3, 0],\n    ...> [0, 1, -2, 3, 0],\n    ...> [0, 7, 8, 9, 0],\n    ...> [1, 0, 0, 0, 0]\n    ...> ]))\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.determinant(Nx.tensor([\n    ...> [[2, 4, 6, 7], [5, 1, 8, 8], [1, 7, 3, 1], [3, 9, 2, 4]],\n    ...> [[2, 5, 1, 3], [4, 1, 7, 9], [6, 8, 3, 2], [7, 8, 1, 4]]\n    ...> ]))\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[1, 0], [0, 2]], [[3, 0], [0, 4]]]) |> Nx.vectorize(x: 2)\n    iex> Nx.LinAlg.determinant(t)\n    #Nx.Tensor \n\nIf the axes are named, their names are not preserved in the output:\n\n    iex> two_by_two = Nx.tensor([[1, 2], [3, 4]], names: [:x, :y])\n    iex> Nx.LinAlg.determinant(two_by_two)\n    #Nx.Tensor \n\n    iex> three_by_three = Nx.tensor([[1.0, 2.0, 3.0], [1.0, -2.0, 3.0], [7.0, 8.0, 9.0]], names: [:x, :y])\n    iex> Nx.LinAlg.determinant(three_by_three)\n    #Nx.Tensor \n\nAlso supports complex inputs:\n\n    iex> t = Nx.tensor([[1, 0, 0], [0, Complex.new(0, 2), 0], [0, 0, 3]])\n    iex> Nx.LinAlg.determinant(t)\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[0, 0, 0, 1], [0, Complex.new(0, 2), 0, 0], [0, 0, 3, 0], [1, 0, 0, 0]])\n    iex> Nx.LinAlg.determinant(t)\n    #Nx.Tensor","ref":"Nx.LinAlg.html#determinant/1-examples"},{"type":"function","title":"Nx.LinAlg.eigh/2","doc":"Calculates the Eigenvalues and Eigenvectors of batched Hermitian 2-D matrices.\n\nIt returns `{eigenvals, eigenvecs}`.","ref":"Nx.LinAlg.html#eigh/2"},{"type":"function","title":"Options - Nx.LinAlg.eigh/2","doc":"* `:max_iter` - `integer`. Defaults to `1_000`\n    Number of maximum iterations before stopping the decomposition\n\n  * `:eps` - `float`. Defaults to 1.0e-4\n    Tolerance applied during the decomposition\n\nNote not all options apply to all backends, as backends may have\nspecific optimizations that render these mechanisms unnecessary.","ref":"Nx.LinAlg.html#eigh/2-options"},{"type":"function","title":"Examples - Nx.LinAlg.eigh/2","doc":"iex> {eigenvals, eigenvecs} = Nx.LinAlg.eigh(Nx.tensor([[1, 0], [0, 2]]))\n    iex> Nx.round(eigenvals)\n    #Nx.Tensor \n    iex> eigenvecs\n    #Nx.Tensor \n\n    iex> {eigenvals, eigenvecs} = Nx.LinAlg.eigh(Nx.tensor([[0, 1, 2], [1, 0, 2], [2, 2, 3]]))\n    iex> Nx.round(eigenvals)\n    #Nx.Tensor \n    iex> eigenvecs\n    #Nx.Tensor \n\n    iex> {eigenvals, eigenvecs} = Nx.LinAlg.eigh(Nx.tensor([[[2, 5],[5, 6]], [[1, 0], [0, 4]]]))\n    iex> Nx.round(eigenvals)\n    #Nx.Tensor \n    iex> eigenvecs\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[2, 5],[5, 6]], [[1, 0], [0, 4]]]) |> Nx.vectorize(x: 2)\n    iex> {eigenvals, eigenvecs} = Nx.LinAlg.eigh(t)\n    iex> Nx.round(eigenvals)\n    #Nx.Tensor \n    iex> eigenvecs\n    #Nx.Tensor","ref":"Nx.LinAlg.html#eigh/2-examples"},{"type":"function","title":"Error cases - Nx.LinAlg.eigh/2","doc":"iex> Nx.LinAlg.eigh(Nx.tensor([[1, 2, 3], [4, 5, 6]]))\n    ** (ArgumentError) tensor must be a square matrix or a batch of square matrices, got shape: {2, 3}\n\n    iex> Nx.LinAlg.eigh(Nx.tensor([[1, 2], [3, 4]]))\n    ** (ArgumentError) matrix must be hermitian, a matrix is hermitian iff X = adjoint(X)","ref":"Nx.LinAlg.html#eigh/2-error-cases"},{"type":"function","title":"Nx.LinAlg.invert/1","doc":"Inverts a batch of square matrices.\n\nFor non-square matrices, use `pinv/2` for pseudo-inverse calculations.","ref":"Nx.LinAlg.html#invert/1"},{"type":"function","title":"Examples - Nx.LinAlg.invert/1","doc":"iex> a = Nx.tensor([[1, 2, 1, 1], [0, 1, 0, 1], [0, 0, 1, 1], [0 , 0, 0, 1]])\n    iex> a_inv = Nx.LinAlg.invert(a)\n    #Nx.Tensor \n    iex> Nx.dot(a, a_inv)\n    #Nx.Tensor \n    iex> Nx.dot(a_inv, a)\n    #Nx.Tensor \n\n    iex> a = Nx.tensor([[[1, 2], [0, 1]], [[1, 1], [0, 1]]])\n    iex> a_inv = Nx.LinAlg.invert(a)\n    #Nx.Tensor \n    iex> Nx.dot(a, [2], [0], a_inv, [1], [0])\n    #Nx.Tensor \n    iex> Nx.dot(a_inv, [2], [0], a, [1], [0])\n    #Nx.Tensor \n\nIf a singular matrix is passed, the result will silently fail.\n\n    iex> Nx.LinAlg.invert(Nx.tensor([[0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [1, 1, 1, 1]]))\n    #Nx.Tensor","ref":"Nx.LinAlg.html#invert/1-examples"},{"type":"function","title":"Error cases - Nx.LinAlg.invert/1","doc":"iex> Nx.LinAlg.invert(Nx.tensor([[3, 0, 0, 0], [2, 1, 0, 0]]))\n    ** (ArgumentError) invert/1 expects a square matrix or a batch of square matrices, got tensor with shape: {2, 4}","ref":"Nx.LinAlg.html#invert/1-error-cases"},{"type":"function","title":"Nx.LinAlg.lu/2","doc":"Calculates the A = PLU decomposition of batched square 2-D matrices A.","ref":"Nx.LinAlg.html#lu/2"},{"type":"function","title":"Options - Nx.LinAlg.lu/2","doc":"* `:eps` - Rounding error threshold that can be applied during the factorization","ref":"Nx.LinAlg.html#lu/2-options"},{"type":"function","title":"Examples - Nx.LinAlg.lu/2","doc":"iex> {p, l, u} = Nx.LinAlg.lu(Nx.tensor([[1, 2, 3], [4, 5, 6], [7, 8, 9]]))\n    iex> p\n    #Nx.Tensor \n    iex> l\n    #Nx.Tensor \n    iex> u\n    #Nx.Tensor \n    iex> p |> Nx.dot(l) |> Nx.dot(u)\n    #Nx.Tensor \n\n    iex> {p, l, u} = Nx.LinAlg.lu(Nx.tensor([[1, 0, 1], [-1, 0, -1], [1, 1, 1]]))\n    iex> p\n    #Nx.Tensor \n    iex> l\n    #Nx.Tensor \n    iex> u\n    #Nx.Tensor \n    iex> p |> Nx.dot(l) |> Nx.dot(u)\n    #Nx.Tensor \n\n    iex> {p, l, u} = Nx.LinAlg.lu(Nx.tensor([[[9, 8, 7], [6, 5, 4], [3, 2, 1]], [[-1, 0, -1], [1, 0, 1], [1, 1, 1]]]))\n    iex> p\n    #Nx.Tensor \n    iex> l\n    #Nx.Tensor \n    iex> u\n    #Nx.Tensor \n    iex> p |> Nx.dot([2], [0], l, [1], [0]) |> Nx.dot([2], [0], u, [1], [0])\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[9, 8, 7], [6, 5, 4], [3, 2, 1]], [[-1, 0, -1], [1, 0, 1], [1, 1, 1]]]) |> Nx.vectorize(x: 2)\n    iex> {p, l, u} = Nx.LinAlg.lu(t)\n    iex> p\n    #Nx.Tensor \n    iex> l\n    #Nx.Tensor \n    iex> u\n    #Nx.Tensor","ref":"Nx.LinAlg.html#lu/2-examples"},{"type":"function","title":"Error cases - Nx.LinAlg.lu/2","doc":"iex> Nx.LinAlg.lu(Nx.tensor([[1, 1, 1, 1], [-1, 4, 4, -1], [4, -2, 2, 0]]))\n    ** (ArgumentError) tensor must be a square matrix or a batch of square matrices, got shape: {3, 4}","ref":"Nx.LinAlg.html#lu/2-error-cases"},{"type":"function","title":"Nx.LinAlg.matrix_power/2","doc":"Produces the tensor taken to the given power by matrix dot-product.\n\nThe input is always a tensor of batched square matrices and an integer,\nand the output is a tensor of the same dimensions as the input tensor.\n\nThe dot-products are unrolled inside `defn`.","ref":"Nx.LinAlg.html#matrix_power/2"},{"type":"function","title":"Examples - Nx.LinAlg.matrix_power/2","doc":"iex> Nx.LinAlg.matrix_power(Nx.tensor([[1, 2], [3, 4]]), 0)\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.matrix_power(Nx.tensor([[1, 2], [3, 4]]), 6)\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.matrix_power(Nx.eye(3), 65535)\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.matrix_power(Nx.tensor([[1, 2], [3, 4]]), -1)\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.matrix_power(Nx.iota({2, 2, 2}), 3)\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.matrix_power(Nx.iota({2, 2, 2}), -3)\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.matrix_power(Nx.tensor([[1, 2], [3, 4], [5, 6]]), 1)\n    ** (ArgumentError) matrix_power/2 expects a square matrix or a batch of square matrices, got tensor with shape: {3, 2}","ref":"Nx.LinAlg.html#matrix_power/2-examples"},{"type":"function","title":"Nx.LinAlg.matrix_rank/2","doc":"Return matrix rank of input M × N matrix using Singular Value Decomposition method.\n\nApproximate the number of linearly independent rows by calculating the number\nof singular values greater than `eps * max(singular values) * max(M, N)`.\n\nThis also appears in Numerical recipes in the discussion of SVD solutions for\nlinear least squares [1].\n\n[1] W. H. Press, S. A. Teukolsky, W. T. Vetterling and B. P. Flannery,\n“Numerical Recipes (3rd edition)”, Cambridge University Press, 2007, page 795.","ref":"Nx.LinAlg.html#matrix_rank/2"},{"type":"function","title":"Options - Nx.LinAlg.matrix_rank/2","doc":"* `:eps` - Rounding error threshold used to assume values as 0. Defaults to `1.0e-7`","ref":"Nx.LinAlg.html#matrix_rank/2-options"},{"type":"function","title":"Examples - Nx.LinAlg.matrix_rank/2","doc":"iex> Nx.LinAlg.matrix_rank(Nx.tensor([[1, 2], [3, 4]]))\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.matrix_rank(Nx.tensor([[1, 1, 1, 1], [1, 1, 1, 1], [1, 2, 3, 4]]))\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.matrix_rank(Nx.tensor([[1, 1, 1], [2, 2, 2], [8, 9, 10], [-2, 1, 5]]))\n    #Nx.Tensor","ref":"Nx.LinAlg.html#matrix_rank/2-examples"},{"type":"function","title":"Error cases - Nx.LinAlg.matrix_rank/2","doc":"iex> Nx.LinAlg.matrix_rank(Nx.tensor([1, 2, 3]))\n    ** (ArgumentError) tensor must have rank 2, got rank 1 with shape {3}\n\n    iex> Nx.LinAlg.matrix_rank(Nx.tensor([[1, Complex.new(0, 2)], [3, Complex.new(0, -4)]]))\n    ** (ArgumentError) Nx.LinAlg.matrix_rank/2 is not yet implemented for complex inputs","ref":"Nx.LinAlg.html#matrix_rank/2-error-cases"},{"type":"function","title":"Nx.LinAlg.norm/2","doc":"Calculates the p-norm of a tensor.\n\nFor the 0-norm, the norm is the number of non-zero elements in the tensor.","ref":"Nx.LinAlg.html#norm/2"},{"type":"function","title":"Options - Nx.LinAlg.norm/2","doc":"* `:axes` - defines the axes upon which the norm will be calculated.\n    Applies only on 2-norm for 2-D tensors. Default: `nil`.\n  * `:keep_axes` - whether the calculation axes should be kept with\n    length 1. Defaults to `false`\n  * `:ord` - defines which norm will be calculated according to the table below. Default: `nil`.\n\n| ord          | 2-D                            | 1-D                               |\n| ------------ | -------------------------------| --------------------------------- |\n| `nil`        | Frobenius norm                 | 2-norm                            |\n| `:nuclear`   | Nuclear norm                   | -                                 |\n| `:frobenius` | Frobenius norm                 | -                                 |\n| `:inf`       | `max(sum(abs(x), axes: [1]))`  | `max(abs(x))`                     |\n| `:neg_inf`   | `min(sum(abs(x), axes: [1]))`  | `min(abs(x))`                     |\n| 0            | -                              | Number of non-zero elements       |\n| 1            | `max(sum(abs(x), axes: [0]))`  | as below                          |\n| -1           | `min(sum(abs(x), axes: [0]))`  | as below                          |\n| 2            | 2-norm                         | as below                          |\n| -2           | smallest singular value        | as below                          |\n| other        | -                              | pow(sum(pow(abs(x), p)), 1/p) |","ref":"Nx.LinAlg.html#norm/2-options"},{"type":"function","title":"Examples - Nx.LinAlg.norm/2","doc":"#","ref":"Nx.LinAlg.html#norm/2-examples"},{"type":"function","title":"Vector norms - Nx.LinAlg.norm/2","doc":"iex> Nx.LinAlg.norm(Nx.tensor([3, 4]))\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.norm(Nx.tensor([3, 4]), ord: 1)\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.norm(Nx.tensor([3, -4]), ord: :inf)\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.norm(Nx.tensor([3, -4]), ord: :neg_inf)\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.norm(Nx.tensor([3, -4, 0, 0]), ord: 0)\n    #Nx.Tensor \n\n#","ref":"Nx.LinAlg.html#norm/2-vector-norms"},{"type":"function","title":"Matrix norms - Nx.LinAlg.norm/2","doc":"iex> Nx.LinAlg.norm(Nx.tensor([[3, -1], [2, -4]]), ord: -1)\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.norm(Nx.tensor([[3, -2], [2, -4]]), ord: 1)\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.norm(Nx.tensor([[3, -2], [2, -4]]), ord: :neg_inf)\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.norm(Nx.tensor([[3, -2], [2, -4]]), ord: :inf)\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.norm(Nx.tensor([[3, 0], [0, -4]]), ord: :frobenius)\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.norm(Nx.tensor([[1, 0, 0], [0, -4, 0], [0, 0, 9]]), ord: :nuclear)\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.norm(Nx.tensor([[1, 0, 0], [0, -4, 0], [0, 0, 9]]), ord: -2)\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.norm(Nx.tensor([[3, 0], [0, -4]]))\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.norm(Nx.tensor([[3, 4], [0, -4]]), axes: [1])\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.norm(Nx.tensor([[Complex.new(0, 3), 4], [4, 0]]), axes: [0])\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.norm(Nx.tensor([[Complex.new(0, 3), 0], [4, 0]]), ord: :neg_inf)\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.norm(Nx.tensor([[0, 0], [0, 0]]))\n    #Nx.Tensor","ref":"Nx.LinAlg.html#norm/2-matrix-norms"},{"type":"function","title":"Error cases - Nx.LinAlg.norm/2","doc":"iex> Nx.LinAlg.norm(Nx.tensor([3, 4]), ord: :frobenius)\n    ** (ArgumentError) expected a 2-D tensor for ord: :frobenius, got a 1-D tensor","ref":"Nx.LinAlg.html#norm/2-error-cases"},{"type":"function","title":"Nx.LinAlg.pinv/2","doc":"Calculates the Moore-Penrose inverse, or the pseudoinverse, of a matrix.","ref":"Nx.LinAlg.html#pinv/2"},{"type":"function","title":"Options - Nx.LinAlg.pinv/2","doc":"* `:eps` - Rounding error threshold used to assume values as 0. Defaults to `1.0e-10`","ref":"Nx.LinAlg.html#pinv/2-options"},{"type":"function","title":"Examples - Nx.LinAlg.pinv/2","doc":"Scalar case:\n\n    iex> Nx.LinAlg.pinv(2)\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.pinv(0)\n    #Nx.Tensor \n\nVector case:\n\n    iex> Nx.LinAlg.pinv(Nx.tensor([0, 1, 2]))\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.pinv(Nx.tensor([0, 0, 0]))\n    #Nx.Tensor \n\nMatrix case:\n\n    iex> Nx.LinAlg.pinv(Nx.tensor([[1, 1], [3, 4]]))\n    #Nx.Tensor \n\n    iex> Nx.LinAlg.pinv(Nx.tensor([[0.5, 0], [0, 1], [0.5, 0]]))\n    #Nx.Tensor","ref":"Nx.LinAlg.html#pinv/2-examples"},{"type":"function","title":"Nx.LinAlg.qr/2","doc":"Calculates the QR decomposition of a tensor with shape `{..., M, N}`.","ref":"Nx.LinAlg.html#qr/2"},{"type":"function","title":"Options - Nx.LinAlg.qr/2","doc":"* `:mode` - Can be one of `:reduced`, `:complete`. Defaults to `:reduced`\n    For the following, `K = min(M, N)`\n\n    * `:reduced` - returns `q` and `r` with shapes `{..., M, K}` and `{..., K, N}`\n    * `:complete` - returns `q` and `r` with shapes `{..., M, M}` and `{..., M, N}`\n\n  * `:eps` - Rounding error threshold that can be applied during the triangularization. Defaults to `1.0e-10`","ref":"Nx.LinAlg.html#qr/2-options"},{"type":"function","title":"Examples - Nx.LinAlg.qr/2","doc":"iex> {q, r} = Nx.LinAlg.qr(Nx.tensor([[-3, 2, 1], [0, 1, 1], [0, 0, -1]]))\n    iex> q\n    #Nx.Tensor \n    iex> r\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[3, 2, 1], [0, 1, 1], [0, 0, 1]])\n    iex> {q, r} = Nx.LinAlg.qr(t)\n    iex> q\n    #Nx.Tensor \n    iex> r\n    #Nx.Tensor \n\n    iex> {qs, rs} = Nx.LinAlg.qr(Nx.tensor([[[-3, 2, 1], [0, 1, 1], [0, 0, -1]],[[3, 2, 1], [0, 1, 1], [0, 0, 1]]]))\n    iex> qs\n    #Nx.Tensor \n    iex> rs\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[3, 2, 1], [0, 1, 1], [0, 0, 1], [0, 0, 1]], type: :f32)\n    iex> {q, r} = Nx.LinAlg.qr(t, mode: :reduced)\n    iex> q\n    #Nx.Tensor \n    iex> r\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[3, 2, 1], [0, 1, 1], [0, 0, 1], [0, 0, 0]], type: :f32)\n    iex> {q, r} = Nx.LinAlg.qr(t, mode: :complete)\n    iex> q\n    #Nx.Tensor \n    iex> r\n    #Nx.Tensor \n\n    iex> t = Nx.tensor([[[-3, 2, 1], [0, 1, 1], [0, 0, -1]],[[3, 2, 1], [0, 1, 1], [0, 0, 1]]]) |> Nx.vectorize(x: 2)\n    iex> {qs, rs} = Nx.LinAlg.qr(t)\n    iex> qs\n    #Nx.Tensor \n    iex> rs\n    #Nx.Tensor","ref":"Nx.LinAlg.html#qr/2-examples"},{"type":"function","title":"Error cases - Nx.LinAlg.qr/2","doc":"iex> Nx.LinAlg.qr(Nx.tensor([1, 2, 3, 4, 5]))\n    ** (ArgumentError) tensor must have at least rank 2, got rank 1 with shape {5}\n\n    iex> t = Nx.tensor([[-3, 2, 1], [0, 1, 1], [0, 0, -1]])\n    iex> Nx.LinAlg.qr(t, mode: :error_test)\n    ** (ArgumentError) invalid :mode received. Expected one of [:reduced, :complete], received: :error_test","ref":"Nx.LinAlg.html#qr/2-error-cases"},{"type":"function","title":"Nx.LinAlg.solve/2","doc":"Solves the system `AX = B`.\n\n`A` must have shape `{..., n, n}` and `B` must have shape `{..., n, m}` or `{..., n}`.\n`X` has the same shape as `B`.","ref":"Nx.LinAlg.html#solve/2"},{"type":"function","title":"Examples - Nx.LinAlg.solve/2","doc":"iex> a = Nx.tensor([[1, 3, 2, 1], [2, 1, 0, 0], [1, 0, 1, 0], [1, 1, 1, 1]])\n    iex> Nx.LinAlg.solve(a, Nx.tensor([-3, 0, 4, -2])) |> Nx.round()\n    #Nx.Tensor \n\n    iex> a = Nx.tensor([[1, 0, 1], [1, 1, 0], [1, 1, 1]], type: :f64)\n    iex> Nx.LinAlg.solve(a, Nx.tensor([0, 2, 1])) |> Nx.round()\n    #Nx.Tensor \n\n    iex> a = Nx.tensor([[1, 0, 1], [1, 1, 0], [0, 1, 1]])\n    iex> b = Nx.tensor([[2, 2, 3], [2, 2, 4], [2, 0, 1]])\n    iex> Nx.LinAlg.solve(a, b) |> Nx.round()\n    #Nx.Tensor \n\n    iex> a = Nx.tensor([[[14, 10], [9, 9]], [[4, 11], [2, 3]]])\n    iex> b = Nx.tensor([[[2, 4], [3, 2]], [[1, 5], [-3, -1]]])\n    iex> Nx.LinAlg.solve(a, b) |> Nx.round()\n    #Nx.Tensor \n\n    iex> a = Nx.tensor([[[1, 1], [0, 1]], [[2, 0], [0, 2]]]) |> Nx.vectorize(x: 2)\n    iex> b = Nx.tensor([[[2, 1], [5, -1]]]) |> Nx.vectorize(x: 1, y: 2)\n    iex> Nx.LinAlg.solve(a, b)\n    #Nx.Tensor \n\nIf the axes are named, their names are not preserved in the output:\n\n    iex> a = Nx.tensor([[1, 0, 1], [1, 1, 0], [1, 1, 1]], names: [:x, :y])\n    iex> Nx.LinAlg.solve(a, Nx.tensor([0, 2, 1], names: [:z])) |> Nx.round()\n    #Nx.Tensor","ref":"Nx.LinAlg.html#solve/2-examples"},{"type":"function","title":"Error cases - Nx.LinAlg.solve/2","doc":"iex> Nx.LinAlg.solve(Nx.tensor([[1, 0], [0, 1]]), Nx.tensor([4, 2, 4, 2]))\n    ** (ArgumentError) `b` tensor has incompatible dimensions, expected {2, 2} or {2}, got: {4}\n\n    iex> Nx.LinAlg.solve(Nx.tensor([[3, 0, 0, 0], [2, 1, 0, 0], [1, 1, 1, 1]]), Nx.tensor([4]))\n    ** (ArgumentError) `a` tensor has incompatible dimensions, expected a square matrix or a batch of square matrices, got: {3, 4}","ref":"Nx.LinAlg.html#solve/2-error-cases"},{"type":"function","title":"Nx.LinAlg.svd/2","doc":"Calculates the Singular Value Decomposition of batched 2-D matrices.\n\nIt returns `{u, s, vt}` where the elements of `s` are sorted\nfrom highest to lowest.","ref":"Nx.LinAlg.html#svd/2"},{"type":"function","title":"Options - Nx.LinAlg.svd/2","doc":"* `:max_iter` - `integer`. Defaults to `100`\n    Number of maximum iterations before stopping the decomposition\n\n  * `:full_matrices?` - `boolean`. Defaults to `true`\n    If `true`, `u` and `vt` are of shape (M, M), (N, N). Otherwise,\n    the shapes are (M, K) and (K, N), where K = min(M, N).\n\nNote not all options apply to all backends, as backends may have\nspecific optimizations that render these mechanisms unnecessary.","ref":"Nx.LinAlg.html#svd/2-options"},{"type":"function","title":"Examples - Nx.LinAlg.svd/2","doc":"iex> {u, s, vt} = Nx.LinAlg.svd(Nx.tensor([[1, 0, 0], [0, 1, 0], [0, 0, -1]]))\n    iex> u\n    #Nx.Tensor \n    iex> s\n    #Nx.Tensor \n    iex> vt\n    #Nx.Tensor \n\n    iex> {u, s, vt} = Nx.LinAlg.svd(Nx.tensor([[2, 0, 0], [0, 3, 0], [0, 0, -1], [0, 0, 0]]))\n    iex> u\n    #Nx.Tensor \n    iex> s\n    #Nx.Tensor \n    iex> vt\n    #Nx.Tensor \n\n    iex> {u, s, vt} = Nx.LinAlg.svd(Nx.tensor([[2, 0, 0], [0, 3, 0], [0, 0, -1], [0, 0, 0]]), full_matrices?: false)\n    iex> u\n    #Nx.Tensor \n    iex> s\n    #Nx.Tensor \n    iex> vt\n    #Nx.Tensor","ref":"Nx.LinAlg.html#svd/2-examples"},{"type":"function","title":"Nx.LinAlg.triangular_solve/3","doc":"Solve the equation `a x = b` for x, assuming `a` is a batch of triangular matrices.\nCan also solve `x a = b` for x. See the `:left_side` option below.\n\n`b` must either be a batch of square matrices with the same dimensions as `a` or a batch of 1-D tensors\nwith as many rows as `a`. Batch dimensions of `a` and `b` must be the same.","ref":"Nx.LinAlg.html#triangular_solve/3"},{"type":"function","title":"Options - Nx.LinAlg.triangular_solve/3","doc":"The following options are defined in order of precedence\n\n* `:transform_a` - Defines `op(a)`, depending on its value. Can be one of:\n  * `:none` -> `op(a) = a`\n  * `:transpose` -> `op(a) = transpose(a)`\n  Defaults to `:none`\n* `:lower` - When `true`, defines the `a` matrix as lower triangular. If false, a is upper triangular.\n  Defaults to `true`\n* `:left_side` - When `true`, solves the system as `op(A).X = B`. Otherwise, solves `X.op(A) = B`. Defaults to `true`.","ref":"Nx.LinAlg.html#triangular_solve/3-options"},{"type":"function","title":"Examples - Nx.LinAlg.triangular_solve/3","doc":"iex> a = Nx.tensor([[3, 0, 0, 0], [2, 1, 0, 0], [1, 0, 1, 0], [1, 1, 1, 1]])\n    iex> Nx.LinAlg.triangular_solve(a, Nx.tensor([4, 2, 4, 2]))\n    #Nx.Tensor \n\n    iex> a = Nx.tensor([[1, 0, 0], [1, 1, 0], [1, 1, 1]], type: :f64)\n    iex> Nx.LinAlg.triangular_solve(a, Nx.tensor([1, 2, 1]))\n    #Nx.Tensor \n\n    iex> a = Nx.tensor([[1, 0, 0], [1, 1, 0], [0, 1, 1]])\n    iex> b = Nx.tensor([[1, 2, 3], [2, 2, 4], [2, 0, 1]])\n    iex> Nx.LinAlg.triangular_solve(a, b)\n    #Nx.Tensor \n\n    iex> a = Nx.tensor([[1, 1, 1, 1], [0, 1, 0, 1], [0, 0, 1, 2], [0, 0, 0, 3]])\n    iex> Nx.LinAlg.triangular_solve(a, Nx.tensor([2, 4, 2, 4]), lower: false)\n    #Nx.Tensor \n\n    iex> a = Nx.tensor([[1, 0, 0], [1, 1, 0], [1, 2, 1]])\n    iex> b = Nx.tensor([[0, 2, 1], [1, 1, 0], [3, 3, 1]])\n    iex> Nx.LinAlg.triangular_solve(a, b, left_side: false)\n    #Nx.Tensor \n\n    iex> a = Nx.tensor([[1, 1, 1], [0, 1, 1], [0, 0, 1]], type: :f64)\n    iex> Nx.LinAlg.triangular_solve(a, Nx.tensor([1, 2, 1]), transform_a: :transpose, lower: false)\n    #Nx.Tensor \n\n    iex> a = Nx.tensor([[1, 0, 0], [1, 1, 0], [1, 1, 1]], type: :f64)\n    iex> Nx.LinAlg.triangular_solve(a, Nx.tensor([1, 2, 1]), transform_a: :none)\n    #Nx.Tensor \n\n    iex> a = Nx.tensor([[1, 0, 0], [1, 1, 0], [1, 2, 1]])\n    iex> b = Nx.tensor([[0, 1, 3], [2, 1, 3]])\n    iex> Nx.LinAlg.triangular_solve(a, b, left_side: false)\n    #Nx.Tensor \n\n    iex> a = Nx.tensor([[1, 0, 0], [1, 1, 0], [1, 2, 1]])\n    iex> b = Nx.tensor([[0, 2], [3, 0], [0, 0]])\n    iex> Nx.LinAlg.triangular_solve(a, b, left_side: true)\n    #Nx.Tensor \n\n    iex> a = Nx.tensor([\n    ...> [1, 0, 0],\n    ...> [1, Complex.new(0, 1), 0],\n    ...> [Complex.new(0, 1), 1, 1]\n    ...>])\n    iex> b = Nx.tensor([1, -1, Complex.new(3, 3)])\n    iex> Nx.LinAlg.triangular_solve(a, b)\n    #Nx.Tensor \n\n    iex> a = Nx.tensor([[[1, 0], [2, 3]], [[4, 0], [5, 6]]])\n    iex> b = Nx.tensor([[2, -1], [3, 7]])\n    iex> Nx.LinAlg.triangular_solve(a, b)\n    #Nx.Tensor \n\n    iex> a = Nx.tensor([[[1, 1], [0, 1]], [[2, 0], [0, 2]]]) |> Nx.vectorize(x: 2)\n    iex> b = Nx.tensor([[[2, 1], [5, -1]]]) |> Nx.vectorize(x: 1, y: 2)\n    iex> Nx.LinAlg.triangular_solve(a, b, lower: false)\n    #Nx.Tensor","ref":"Nx.LinAlg.html#triangular_solve/3-examples"},{"type":"function","title":"Error cases - Nx.LinAlg.triangular_solve/3","doc":"iex> Nx.LinAlg.triangular_solve(Nx.tensor([[3, 0, 0, 0], [2, 1, 0, 0]]), Nx.tensor([4, 2, 4, 2]))\n    ** (ArgumentError) triangular_solve/3 expected a square matrix or a batch of square matrices, got tensor with shape: {2, 4}\n\n    iex> Nx.LinAlg.triangular_solve(Nx.tensor([[3, 0, 0, 0], [2, 1, 0, 0], [1, 1, 1, 1], [1, 1, 1, 1]]), Nx.tensor([4]))\n    ** (ArgumentError) incompatible dimensions for a and b on triangular solve\n\n    iex> Nx.LinAlg.triangular_solve(Nx.tensor([[0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [1, 1, 1, 1]]), Nx.tensor([4, 2, 4, 2]))\n    ** (ArgumentError) can't solve for singular matrix\n\n    iex> a = Nx.tensor([[1, 0, 0], [1, 1, 0], [1, 1, 1]], type: :f64)\n    iex> Nx.LinAlg.triangular_solve(a, Nx.tensor([1, 2, 1]), transform_a: :conjugate)\n    ** (ArgumentError) complex numbers not supported yet\n\n    iex> a = Nx.tensor([[1, 0, 0], [1, 1, 0], [1, 1, 1]], type: :f64)\n    iex> Nx.LinAlg.triangular_solve(a, Nx.tensor([1, 2, 1]), transform_a: :other)\n    ** (ArgumentError) invalid value for :transform_a option, expected :none, :transpose, or :conjugate, got: :other","ref":"Nx.LinAlg.html#triangular_solve/3-error-cases"},{"type":"module","title":"Nx.Random","doc":"Pseudo-random number generators.\n\nUnlike the stateful pseudo-random number generators (PRNGs)\nthat users of most programming languages and numerical libraries\nmay be accustomed to, Nx random functions require an explicit\nPRNG key to be passed as a first argument (see [below](#module-the-key-to-understanding-nx-random-keys) for more info). That key is defined by\nan `Nx.Tensor` composed of 2 unsigned 32-bit integers, usually\ngenerated by the `Nx.Random.key/1` function:\n\n    iex> Nx.Random.key(12)\n    #Nx.Tensor \n\nThis key can then be used in any of Nx’s random number generation\nroutines:\n\n    iex> key = Nx.Random.key(12)\n    iex> {uniform, _new_key} = Nx.Random.uniform(key)\n    iex> uniform\n    #Nx.Tensor \n\nNow, when generating a new random number, you pass the `new_key`\nto get a different number.\n\nThe function in this module also have a `*_split` variant, which\nis used when the key has been split before hand.","ref":"Nx.Random.html"},{"type":"module","title":"Design and Context - Nx.Random","doc":"In short, Nx's PRNGs are based on a Threefry counter PRNG\nassociated to a functional array-oriented splitting model.\nTo summarize, among other requirements, Nx's PRNG aims to:\n\n1. Ensure reproducibility\n\n2. Parallelize well, both in terms of vectorization\n   (generating array values) and multi-replica, multi-core\n   computation. In particular it should not use sequencing\n   constraints between random function calls.\n\n> #","ref":"Nx.Random.html#module-design-and-context"},{"type":"module","title":"The key to understanding Nx.Random keys {: .info} - Nx.Random","doc":">\n> Most Elixir users might be used to not having to keep\n> track of the PRNG state while their code executes.\n>\n> While this works fine when we're dealing with the CPU,\n> we can think of keeping track of the `Nx.Random` key as a way to\n> isolate multiple GPU users, much like the PRNG on different\n> BEAM nodes is isolated. Each key gets updated in its own\n> isolated sequence of calls, and thus we don't get different\n> results for each process using the same PRNG as we would\n> in the normal situation.\n>\n> The fact that the key is a parameter for the functions also\n> helps with the caching and operator fusion of the computational\n> graphs. Because the PRNG functions themselves are stateless,\n> compilers can take advantage of this to further improve execution times.","ref":"Nx.Random.html#module-the-key-to-understanding-nx-random-keys-info"},{"type":"function","title":"Nx.Random.choice/3","doc":"Generates random samples from a tensor.","ref":"Nx.Random.html#choice/3"},{"type":"function","title":"Options - Nx.Random.choice/3","doc":"* `:samples` - The number of samples to take\n\n  * `:axis` - The axis along which to take samples.\n    If `nil`, the tensor is flattened beforehand.\n\n  * `:replace` - a boolean that specifies if samples will\n    be taken with or without replacement. Defaults to `true`.","ref":"Nx.Random.html#choice/3-options"},{"type":"function","title":"Examples - Nx.Random.choice/3","doc":"iex> k = Nx.Random.key(1)\n    iex> t = Nx.iota({4, 3})\n    iex> {result, _key} = Nx.Random.choice(k, t, samples: 4, axis: 0)\n    iex> result\n    #Nx.Tensor \n    iex> {result, _key} = Nx.Random.choice(k, t, samples: 4, axis: 0, replace: false)\n    iex> result\n    #Nx.Tensor \n\nIf no axis is specified, the tensor is flattened:\n\n    iex> k = Nx.Random.key(2)\n    iex> t = Nx.iota({3, 2})\n    iex> {result, _key} = Nx.Random.choice(k, t)\n    iex> result\n    #Nx.Tensor \n    iex> {result, _key} = Nx.Random.choice(k, t, samples: 6, replace: false)\n    iex> result\n    #Nx.Tensor","ref":"Nx.Random.html#choice/3-examples"},{"type":"function","title":"Nx.Random.choice/4","doc":"Generates random samples from a tensor with specified probabilities.\n\nThe probabilities tensor must have the same size as the axis along\nwhich the samples are being taken. If no axis is given, the size\nmust be equal to the input tensor's size.","ref":"Nx.Random.html#choice/4"},{"type":"function","title":"Options - Nx.Random.choice/4","doc":"* `:samples` - The number of samples to take\n\n  * `:axis` - The axis along which to take samples.\n    If `nil`, the tensor is flattened beforehand.\n\n  * `:replace` - a boolean that specifies if samples will\n    be taken with or without replacement. Defaults to `true`.","ref":"Nx.Random.html#choice/4-options"},{"type":"function","title":"Examples - Nx.Random.choice/4","doc":"iex> k = Nx.Random.key(1)\n    iex> t = Nx.iota({4, 3})\n    iex> p = Nx.tensor([0.1, 0.7, 0.2])\n    iex> {result, _key} = Nx.Random.choice(k, t, p, samples: 3, axis: 1)\n    iex> result\n    #Nx.Tensor \n    iex> {result, _key} = Nx.Random.choice(k, t, p, samples: 3, axis: 1, replace: false)\n    iex> result\n    #Nx.Tensor \n\nIf no axis is specified, the tensor is flattened.\nNotice that in the first case we get a higher occurence\nof the entries with bigger probabilities, while in the\nsecond case, without replacements, we get those samples\nfirst.\n\n    iex> k = Nx.Random.key(2)\n    iex> t = Nx.iota({2, 3})\n    iex> p = Nx.tensor([0.01, 0.1, 0.19, 0.6, 0.05, 0.05])\n    iex> {result, _key} = Nx.Random.choice(k, t, p)\n    iex> result\n    #Nx.Tensor \n    iex> {result, _key} = Nx.Random.choice(k, t, p, samples: 6)\n    iex> result\n    #Nx.Tensor \n    iex> {result, _key} = Nx.Random.choice(k, t, p, samples: 6, replace: false)\n    iex> result\n    #Nx.Tensor","ref":"Nx.Random.html#choice/4-examples"},{"type":"function","title":"Nx.Random.fold_in/2","doc":"Folds in new data to a PRNG key.","ref":"Nx.Random.html#fold_in/2"},{"type":"function","title":"Examples - Nx.Random.fold_in/2","doc":"iex> key = Nx.Random.key(42)\n    iex> Nx.Random.fold_in(key, 99)\n    #Nx.Tensor \n\n    iex> key = Nx.Random.key(42)\n    iex> Nx.Random.fold_in(key, 1234)\n    #Nx.Tensor \n\n    iex> key = Nx.Random.key(42)\n    iex> Nx.Random.fold_in(key, Nx.tensor([[1, 99], [1234, 13]]))\n    #Nx.Tensor","ref":"Nx.Random.html#fold_in/2-examples"},{"type":"function","title":"Nx.Random.gumbel/2","doc":"Sample Gumbel random values with given shape and float dtype.","ref":"Nx.Random.html#gumbel/2"},{"type":"function","title":"Options - Nx.Random.gumbel/2","doc":"* `:shape` - the shape of the output tensor containing the\n    random samples. Defaults to `{}`\n\n  * `:type` - the floating-point output type. Defaults to `{:f, 32}`","ref":"Nx.Random.html#gumbel/2-options"},{"type":"function","title":"Examples - Nx.Random.gumbel/2","doc":"iex> {result, _key} = Nx.Random.gumbel(Nx.Random.key(1))\n    iex> result\n    #Nx.Tensor \n\n    iex> {result, _key} = Nx.Random.gumbel(Nx.Random.key(1), shape: {2, 3})\n    iex> result\n    #Nx.Tensor","ref":"Nx.Random.html#gumbel/2-examples"},{"type":"function","title":"Nx.Random.gumbel_split/2","doc":"Same as `gumbel/2`, but assumes the key has been split beforehand.","ref":"Nx.Random.html#gumbel_split/2"},{"type":"function","title":"Nx.Random.key/1","doc":"Create a pseudo-random number generator (PRNG) key given an integer seed.","ref":"Nx.Random.html#key/1"},{"type":"function","title":"Examples - Nx.Random.key/1","doc":"iex> Nx.Random.key(12)\n    #Nx.Tensor \n\n    iex> Nx.Random.key(999999999999)\n    #Nx.Tensor","ref":"Nx.Random.html#key/1-examples"},{"type":"function","title":"Nx.Random.multivariate_normal/4","doc":"Returns a sample from a multivariate normal distribution with given `mean` and `covariance` (matrix).\nThe function assumes that the covariance is a positive semi-definite matrix.\nOtherwise, the result will not be normally distributed.","ref":"Nx.Random.html#multivariate_normal/4"},{"type":"function","title":"Options - Nx.Random.multivariate_normal/4","doc":"* `:type` - a float type for the returned tensor\n\n  * `:shape` - batch shape of the returned tensor, i.e. the prefix of the result shape excluding the last axis\n\n  * `:names` - the names of the returned tensor\n\n  * `:method` - a decomposition method used for the covariance. Must be one of :svd, :eigh, and :cholesky.\n    Defaults to :cholesky. For singular covariance matrices, use :svd or :eigh.","ref":"Nx.Random.html#multivariate_normal/4-options"},{"type":"function","title":"Examples - Nx.Random.multivariate_normal/4","doc":"iex> key = Nx.Random.key(12)\n    iex> {multivariate_normal, _new_key} = Nx.Random.multivariate_normal(key, Nx.tensor([0]), Nx.tensor([[1]]))\n    iex> multivariate_normal\n    #Nx.Tensor \n\n    iex> key = Nx.Random.key(12)\n    iex> {multivariate_normal, _new_key} = Nx.Random.multivariate_normal(key, Nx.tensor([0, 0]), Nx.tensor([[1, 0], [0, 1]]))\n    iex> multivariate_normal\n    #Nx.Tensor \n\n    iex> key = Nx.Random.key(12)\n    iex> {multivariate_normal, _new_key} = Nx.Random.multivariate_normal(key, Nx.tensor([0]), Nx.tensor([[1]]), shape: {3, 2}, type: :f16)\n    iex> multivariate_normal\n    #Nx.Tensor \n\n    iex> key = Nx.Random.key(12)\n    iex> {multivariate_normal, _new_key} = Nx.Random.multivariate_normal(key, Nx.tensor([0, 0]), Nx.tensor([[1, 0], [0, 1]]), shape: {3, 2})\n    iex> multivariate_normal\n    #Nx.Tensor","ref":"Nx.Random.html#multivariate_normal/4-examples"},{"type":"function","title":"Nx.Random.multivariate_normal_split/4","doc":"Same as `multivariate_normal/4` but assumes the key has already been split.","ref":"Nx.Random.html#multivariate_normal_split/4"},{"type":"function","title":"Nx.Random.normal/2","doc":"Shortcut for `normal(key, 0.0, 1.0, opts)`.","ref":"Nx.Random.html#normal/2"},{"type":"function","title":"Nx.Random.normal/4","doc":"Returns a normal distribution with the given `mean` and `standard_deviation`.","ref":"Nx.Random.html#normal/4"},{"type":"function","title":"Options - Nx.Random.normal/4","doc":"* `:type` - a float or complex type for the returned tensor\n\n  * `:shape` - shape of the returned tensor\n\n  * `:names` - the names of the returned tensor","ref":"Nx.Random.html#normal/4-options"},{"type":"function","title":"Examples - Nx.Random.normal/4","doc":"iex> key = Nx.Random.key(42)\n    iex> {normal, _new_key} = Nx.Random.normal(key)\n    iex> normal\n    #Nx.Tensor \n\n    iex> key = Nx.Random.key(42)\n    iex> {normal, _new_key} = Nx.Random.normal(key, 0, 1, shape: {3, 2}, type: :f16)\n    iex> normal\n    #Nx.Tensor \n\n    iex> key = Nx.Random.key(42)\n    iex> {normal, _new_key} = Nx.Random.normal(key, 0, 1, shape: {2, 2}, type: :c64)\n    iex> normal\n    #Nx.Tensor \n\n    iex> key = Nx.Random.key(1337)\n    iex> {normal, _new_key} = Nx.Random.normal(key, 10, 5, shape: {1_000})\n    iex> Nx.mean(normal)\n    #Nx.Tensor \n    iex> Nx.standard_deviation(normal)\n    #Nx.Tensor","ref":"Nx.Random.html#normal/4-examples"},{"type":"function","title":"Nx.Random.normal_split/4","doc":"Same as `normal/4` but assumes the key has already been split.","ref":"Nx.Random.html#normal_split/4"},{"type":"function","title":"Nx.Random.randint/4","doc":"Sample uniform random integer values in the semi-open open interval `[min_value, max_value)`.","ref":"Nx.Random.html#randint/4"},{"type":"function","title":"Options - Nx.Random.randint/4","doc":"* `:type` - the integer type for the returned tensor\n  * `:shape` - shape of the returned tensor\n  * `:names` - the names of the returned tensor","ref":"Nx.Random.html#randint/4-options"},{"type":"function","title":"Examples - Nx.Random.randint/4","doc":"iex> key = Nx.Random.key(1701)\n    iex> {randint, _new_key} = Nx.Random.randint(key, 1, 100)\n    iex> randint\n    #Nx.Tensor \n\n    iex> key = Nx.Random.key(1701)\n    iex> {randint, _new_key} = Nx.Random.randint(key, 1, 100, shape: {3, 2}, type: :u32)\n    iex> randint\n    #Nx.Tensor","ref":"Nx.Random.html#randint/4-examples"},{"type":"function","title":"Nx.Random.randint_split/4","doc":"Same as `randint/4` but assumes the key has already been split.","ref":"Nx.Random.html#randint_split/4"},{"type":"function","title":"Nx.Random.shuffle/3","doc":"Randomly shuffles tensor elements along an axis.","ref":"Nx.Random.html#shuffle/3"},{"type":"function","title":"Options - Nx.Random.shuffle/3","doc":"* `:axis` - the axis along which to shuffle. Defaults to `0`\n\n  * `:independent` - a boolean that indicates wether the permutations\n    are independent along the given axis. Defaults to `false`","ref":"Nx.Random.html#shuffle/3-options"},{"type":"function","title":"Examples - Nx.Random.shuffle/3","doc":"iex> key = Nx.Random.key(42)\n    iex> {shuffled, _new_key} = Nx.Random.shuffle(key, Nx.iota({3, 4}, axis: 0))\n    iex> shuffled\n    #Nx.Tensor \n\n    iex> key = Nx.Random.key(10)\n    iex> {shuffled, _new_key} = Nx.Random.shuffle(key, Nx.iota({3, 4}, axis: 1), independent: true, axis: 1)\n    iex> shuffled\n    #Nx.Tensor","ref":"Nx.Random.html#shuffle/3-examples"},{"type":"function","title":"Nx.Random.split/2","doc":"Splits a PRNG key into `num` new keys by adding a leading axis.","ref":"Nx.Random.html#split/2"},{"type":"function","title":"Examples - Nx.Random.split/2","doc":"iex> key = Nx.Random.key(1701)\n    iex> Nx.Random.split(key)\n    #Nx.Tensor \n\n    iex> key = Nx.Random.key(999999999999)\n    iex> Nx.Random.split(key, parts: 4)\n    #Nx.Tensor","ref":"Nx.Random.html#split/2-examples"},{"type":"function","title":"Nx.Random.threefry2x32_20_concat/2","doc":"","ref":"Nx.Random.html#threefry2x32_20_concat/2"},{"type":"function","title":"Nx.Random.uniform/2","doc":"Shortcut for `uniform(key, 0.0, 1.0, opts)`.","ref":"Nx.Random.html#uniform/2"},{"type":"function","title":"Nx.Random.uniform/4","doc":"Sample uniform float values in the semi-open interval `[min_val, max_val)`.","ref":"Nx.Random.html#uniform/4"},{"type":"function","title":"Options - Nx.Random.uniform/4","doc":"* `:type` - a float type for the returned tensor\n\n  * `:shape` - shape of the returned tensor\n\n  * `:names` - the names of the returned tensor","ref":"Nx.Random.html#uniform/4-options"},{"type":"function","title":"Examples - Nx.Random.uniform/4","doc":"iex> key = Nx.Random.key(1701)\n    iex> {uniform, _new_key} = Nx.Random.uniform(key)\n    iex> uniform\n    #Nx.Tensor \n\n    iex> key = Nx.Random.key(1701)\n    iex> {uniform, _new_key} = Nx.Random.uniform(key, shape: {3, 2}, type: :f16)\n    iex> uniform\n    #Nx.Tensor \n\n    iex> key = Nx.Random.key(1701)\n    iex> {uniform, _new_key} = Nx.Random.uniform(key, shape: {2, 2}, type: :c64)\n    iex> uniform\n    #Nx.Tensor","ref":"Nx.Random.html#uniform/4-examples"},{"type":"function","title":"Nx.Random.uniform_split/4","doc":"Same as `uniform/4` but assumes the key has already been split.","ref":"Nx.Random.html#uniform_split/4"},{"type":"behaviour","title":"Nx.Serving","doc":"Serving encapsulates client and server work to perform batched requests.\n\nServings can be executed on the fly, without starting a server, but most\noften they are used to run servers that batch requests until a given size\nor timeout is reached.\n\nMore specifically, servings are a mechanism to apply a computation on a\n`Nx.Batch`, with hooks for preprocessing input from and postprocessing\noutput for the client. Thus we can think of an instance of `t:Nx.Serving.t/0`\n(a serving) as something that encapsulates batches of Nx computations.","ref":"Nx.Serving.html"},{"type":"behaviour","title":"Inline/serverless workflow - Nx.Serving","doc":"First, let's define a simple numerical definition function:\n\n    defmodule MyDefn do\n      import Nx.Defn\n\n      defnp print_and_multiply(x) do\n        x = print_value(x, label: \"debug\")\n        x * 2\n      end\n    end\n\nThe function prints the given tensor and doubles its contents.\nWe can use `new/1` to create a serving that will return a JIT\nor AOT compiled function to execute on batches of tensors:\n\n    iex> serving = Nx.Serving.new(fn opts -> Nx.Defn.jit(&MyDefn.print_and_multiply/1, opts) end)\n    iex> batch = Nx.Batch.stack([Nx.tensor([1, 2, 3])])\n    iex> Nx.Serving.run(serving, batch)\n    debug: #Nx.Tensor \n    #Nx.Tensor \n\nWe started the serving by passing a function that receives\ncompiler options and returns a JIT or AOT compiled function.\nWe called `Nx.Defn.jit/2` passing the options received as\nargument, which will customize the JIT/AOT compilation.\n\nYou should see two values printed. The former is the result of\n`Nx.Defn.Kernel.print_value/1`, which shows the tensor that was\nactually part of the computation and how it was batched.\nThe latter is the result of the computation.\n\nWhen defining a `Nx.Serving`, we can also customize how the data is\nbatched by using the `client_preprocessing` as well as the result by\nusing `client_postprocessing` hooks. Let's give it another try,\nthis time using `jit/2` to create the serving, which automatically\nwraps the given function in `Nx.Defn.jit/2` for us:\n\n    iex> serving = (\n    ...>   Nx.Serving.jit(&MyDefn.print_and_multiply/1)\n    ...>   |> Nx.Serving.client_preprocessing(fn input -> {Nx.Batch.stack(input), :client_info} end)\n    ...>   |> Nx.Serving.client_postprocessing(&{&1, &2})\n    ...> )\n    iex> Nx.Serving.run(serving, [Nx.tensor([1, 2]), Nx.tensor([3, 4])])\n    debug: #Nx.Tensor \n    {{#Nx.Tensor ,\n      :server_info},\n     :client_info}\n\nYou can see the results are a bit different now. First of all, notice that\nwe were able to run the serving passing a list of tensors. Our custom\n`client_preprocessing` function stacks those tensors into a batch of two\nentries and returns a tuple with a `Nx.Batch` struct and additional client\ninformation which we represent as the atom `:client_info`. The default\nclient preprocessing simply enforces a batch (or a stream of batches)\nwas given and returns no client information.\n\nThen the result is a triplet tuple, returned by the client\npostprocessing function, containing the result, the server information\n(which we will later learn how to customize), and the client information.\nFrom this, we can infer the default implementation of `client_postprocessing`\nsimply returns the result, discarding the server and client information.\n\nSo far, `Nx.Serving` has not given us much. It has simply encapsulated the\nexecution of a function. Its full power comes when we start running our own\n`Nx.Serving` process. That's when we will also learn why we have a `client_`\nprefix in some of the function names.","ref":"Nx.Serving.html#module-inline-serverless-workflow"},{"type":"behaviour","title":"Stateful/process workflow - Nx.Serving","doc":"`Nx.Serving` allows us to define an Elixir process to handle requests.\nThis process provides several features, such as batching up to a given\nsize or time, partitioning, and distribution over a group of nodes.\n\nTo do so, we need to start a `Nx.Serving` process with a serving inside\na supervision tree:\n\n    children = [\n      {Nx.Serving,\n       serving: Nx.Serving.jit(&MyDefn.print_and_multiply/1),\n       name: MyServing,\n       batch_size: 10,\n       batch_timeout: 100}\n    ]\n\n    Supervisor.start_child(children, strategy: :one_for_one)\n\n> Note: in your actual application, you want to make sure\n> `Nx.Serving` comes early in your supervision tree, for example\n> before your web application endpoint or your data processing\n> pipelines, as those processes may end-up hitting Nx.Serving.\n\nNow you can send batched runs to said process:\n\n    iex> batch = Nx.Batch.stack([Nx.tensor([1, 2, 3]), Nx.tensor([4, 5, 6])])\n    iex> Nx.Serving.batched_run(MyServing, batch)\n    debug: #Nx.Tensor \n    #Nx.Tensor \n\nIn the example, we pushed a batch of 2 and eventually got a reply.\nThe process will wait for requests from other processes, for up to\n100 milliseconds or until it gets 10 entries. Then it merges all\nbatches together and once the result is computed, it slices and\ndistributes those responses to each caller.\n\nIf there is any `client_preprocessing` function, it will be executed\nbefore the batch is sent to the server. If there is any `client_postprocessing`\nfunction, it will be executed after getting the response from the\nserver.\n\n#","ref":"Nx.Serving.html#module-stateful-process-workflow"},{"type":"behaviour","title":"Partitioning - Nx.Serving","doc":"You can start several partitions under the same serving by passing\n`partitions: true` when starting the serving. The number of partitions\nwill be determined according  your compiler and for which host it is\ncompiling.\n\nFor example, when creating the serving, you may pass the following\n`defn_options`:\n\n    Nx.Serving.new(computation, compiler: EXLA, client: :cuda)\n\nNow when booting up the serving:\n\n    children = [\n      {Nx.Serving,\n       serving: serving,\n       name: MyServing,\n       batch_size: 10,\n       batch_timeout: 100,\n       partitions: true}\n    ]\n\nIf you have two GPUs, `batched_run/3` will now gather batches and send\nthem to the GPUs as they become available to process requests.\n\n#","ref":"Nx.Serving.html#module-partitioning"},{"type":"behaviour","title":"Distribution - Nx.Serving","doc":"All `Nx.Serving`s are distributed by default. If the current machine\ndoes not have an instance of `Nx.Serving` running, `batched_run/3` will\nautomatically look for one in the cluster. The nodes do not need to run\nthe same code and applications. It is only required that they run the\nsame `Nx` version.\n\nThe load balancing between servings is done randomly by default, however,\nthe number of partitions are considered if the `partitions: true` option is also given.\nFor example, if you have a node with 2 GPUs and another with 4, the latter\nwill receive the double of requests compared to the former.\n\nFurthermore, the load balancing allows for assigning weights to servings.\nSimilarly to the number of partitions, when running a serving with `distribution_weight: 1`\nand another one with `distribution_weight: 2`, the latter will receive double the requests\ncompared to the former.\n\n`batched_run/3` receives an optional `distributed_preprocessing` callback as\nthird argument for preprocessing the input for distributed requests. When\nusing libraries like EXLA or Torchx, the tensor is often allocated in memory\ninside a third-party library so it is necessary to either transfer or copy\nthe tensor to the binary backend before sending it to another node.\nThis can be done by passing either `Nx.backend_transfer/1` or `Nx.backend_copy/1`\nas third argument:\n\n    Nx.Serving.batched_run(MyDistributedServing, input, &Nx.backend_copy/1)\n\nUse `backend_transfer/1` if you know the input will no longer be used.\n\nSimilarly, the serving has a `distributed_postprocessing` callback which can do\nequivalent before sending the reply to the caller.\n\nThe servings are dispatched using Erlang Distribution. You can use\n`Node.connect/1` to manually connect nodes. In a production setup, this is\noften done with the help of libraries like [`libcluster`](https://github.com/bitwalker/libcluster).","ref":"Nx.Serving.html#module-distribution"},{"type":"behaviour","title":"Advanced notes - Nx.Serving","doc":"#","ref":"Nx.Serving.html#module-advanced-notes"},{"type":"behaviour","title":"Module-based serving - Nx.Serving","doc":"In the examples so far, we have been using the default version of\n`Nx.Serving`, which executes the given function for each batch.\n\nHowever, we can also use `new/2` to start a module-based version of\n`Nx.Serving` which gives us more control over both inline and process\nworkflows. A simple module implementation of a `Nx.Serving` could look\nlike this:\n\n    defmodule MyServing do\n      @behaviour Nx.Serving\n\n      defnp print_and_multiply(x) do\n        x = print_value({:debug, x})\n        x * 2\n      end\n\n      @impl true\n      def init(_inline_or_process, :unused_arg, [defn_options]) do\n        {:ok, Nx.Defn.jit(&print_and_multiply/1, defn_options)}\n      end\n\n      @impl true\n      def handle_batch(batch, 0, function) do\n        {:execute, fn -> {function.(batch), :server_info} end, function}\n      end\n    end\n\nIt has two functions. The first, `c:init/3`, receives the type of serving\n(`:inline` or `:process`) and the serving argument. In this step,\nwe capture `print_and_multiply/1`as a jitted function.\n\nThe second function is called `c:handle_batch/3`. This function\nreceives a `Nx.Batch` and returns a function to execute.\nThe function itself must return a two element-tuple: the batched\nresults and some server information. The server information can\nbe any value and we set it to the atom `:server_info`.\n\nNow let's give it a try by defining a serving with our module and\nthen running it on a batch:\n\n    iex> serving = Nx.Serving.new(MyServing, :unused_arg)\n    iex> batch = Nx.Batch.stack([Nx.tensor([1, 2, 3])])\n    iex> Nx.Serving.run(serving, batch)\n    {:debug, #Nx.Tensor }\n    #Nx.Tensor \n\nFrom here on, you use `start_link/1` to start this serving in your\nsupervision and even customize `client_preprocessing/1` and\n`client_postprocessing/1` callbacks to this serving, as seen in the\nprevious sections.\n\nNote in our implementation above assumes it won't run partitioned.\nIn partitioned mode, `c:init/3` may receive multiple `defn_options`\nas the third argument and `c:handle_batch/3` may receive another partition\nbesides 0.\n\n#","ref":"Nx.Serving.html#module-module-based-serving"},{"type":"behaviour","title":"Streaming - Nx.Serving","doc":"`Nx.Serving` allows both inputs and outputs to be streamed.\n\nIn order to stream inputs, you only need to return a stream of `Nx.Batch`\nfrom the `client_preprocessing` callback. Serving will automatically take\ncare of streaming the inputs in, regardless if using `run/2` or `batched_run/2`.\nIt is recommended that the streaming batches have the same size as `batch_size`,\nto avoid triggering `batch_timeout` on every iteration (except for the last one\nwhich may be incomplete).\n\nTo stream outputs, you must invoke `streaming/2` with any additional\nstreaming configuration. When this is invoked, the `client_postprocessing`\nwill receive a stream which you can further manipulate lazily using the\nfunctions in the `Stream` module. `streaming/2` also allows you to configure\nhooks and stream values directly from `Nx.Defn` hooks. However, when hook\nstreaming is enabled, certain capabilities are removed: you cannot stream\ninputs nor have batches larger than the configured `batch_size`.\n\nYou can enable both input and output streaming at once.\n\n#","ref":"Nx.Serving.html#module-streaming"},{"type":"behaviour","title":"Batch keys - Nx.Serving","doc":"Sometimes it may be necessary to execute different functions under the\nsame serving. For example, sequence transformers must pad the sequence\nto a given length. However, if you are batching, the length must be\npadded upfront. If the length is too small, you have to either discard\ndata or support only small inputs. If the length is too large, then you\ndecrease performance with the extra padding.\n\nBatch keys provide a mechanism to accumulate different batches, based on\ntheir key, which execute independently. As an example, we will do a\nserving which performs different operations based on the batch key,\nbut it could also be used to perform the same operation for different\ntemplates:\n\n    iex> args = [Nx.template({10}, :s64)]\n    iex> serving = Nx.Serving.new(fn\n    ...>   :double, opts -> Nx.Defn.compile(&Nx.multiply(&1, 2), args, opts)\n    ...>   :half, opts -> Nx.Defn.compile(&Nx.divide(&1, 2), args, opts)\n    ...> end)\n    iex> double_batch = Nx.Batch.concatenate([Nx.iota({10})]) |> Nx.Batch.key(:double)\n    iex> Nx.Serving.run(serving, double_batch)\n    #Nx.Tensor \n    iex> half_batch = Nx.Batch.concatenate([Nx.iota({10})]) |> Nx.Batch.key(:half)\n    iex> Nx.Serving.run(serving, half_batch)\n    #Nx.Tensor \n\nWhen using a process-based serving, you must specify the supported\n`:batch_keys` when the process is started. The batch keys will be\navailable inside the `defn_options` passed as the third argument of\nthe `c:init/3` callback. The batch keys will also be verified\nwhen the batch is returned from the client-preprocessing callback.","ref":"Nx.Serving.html#module-batch-keys"},{"type":"function","title":"Nx.Serving.batch_size/2","doc":"Sets the batch size for this serving.\n\nThis batch size is used to split batches given to both `run/2` and\n`batched_run/2`, enforcing that the batch size never goes over a limit.\nIf you only want to batch within the serving process, you must set\n`:batch_size` via `process_options/2` (or on `start_link/1`).\n\nNote that `:batch_size` only guarantees a batch does not go over a limit.\nBatches are not automatically padded to the batch size. Such can be done\nas necessary inside your serving function by calling `Nx.Batch.pad/2`.\n\n> ##","ref":"Nx.Serving.html#batch_size/2"},{"type":"function","title":"Why batch on `run/2`? {: .info} - Nx.Serving.batch_size/2","doc":">\n> By default, `run/2` does not place a limit on its input size. It always\n> processes inputs directly within the current process. On the other hand,\n> `batched_run/2` always sends your input to a separate process, which\n> will batch and execute the serving only once the batch is full or a\n> timeout has elapsed.\n>\n> However, in some situations, an input given to `run/2` needs to be\n> broken into several batches. If we were to very large batches to our\n> computation, the computation could require too much memory. In such\n> cases, setting a batch size even on `run/2` is beneficial, because\n> Nx.Serving takes care of splitting a large batch into smaller ones\n> that do not exceed the `batch_size` value.","ref":"Nx.Serving.html#batch_size/2-why-batch-on-run-2-info"},{"type":"function","title":"Nx.Serving.batched_run/3","doc":"Runs the given `input` on the serving process given by `name`.\n\n`name` is either an atom representing a local or distributed\nserving process. First it will attempt to dispatch locally, then it\nfalls back to the distributed serving. You may specify\n`{:local, name}` to force a local lookup or `{:distributed, name}`\nto force a distributed one.\n\nThe `client_preprocessing` callback will be invoked on the `input`\nwhich is then sent to the server. The server will batch requests\nand send a response either when the batch is full or on timeout.\nThen `client_postprocessing` is invoked on the response. See the\nmodule documentation for more information. In the distributed case,\nthe callbacks are invoked in the distributed node, but still outside of\nthe serving process.\n\nNote that you cannot batch an `input` larger than the configured\n`:batch_size` in the server.","ref":"Nx.Serving.html#batched_run/3"},{"type":"function","title":"Distributed mode - Nx.Serving.batched_run/3","doc":"To run in distributed mode, the nodes do not need to run the same\ncode and applications. It is only required that they run the\nsame `Nx` version.\n\nIf the current node is running a serving given by `name` locally\nand `{:distributed, name}` is used, the request will use the same\ndistribution mechanisms instead of being handled locally, which\nis useful for testing locally without a need to spawn nodes.\n\nThis function receives an optional `distributed_preprocessing` callback as\nthird argument for preprocessing the input for distributed requests. When\nusing libraries like EXLA or Torchx, the tensor is often allocated in memory\ninside a third-party library so it may be necessary to either transfer or copy\nthe tensor to the binary backend before sending it to another node.\nThis can be done by passing either `Nx.backend_transfer/1` or `Nx.backend_copy/1`\nas third argument:\n\n    Nx.Serving.batched_run(MyDistributedServing, input, &Nx.backend_copy/1)\n\nUse `backend_transfer/1` if you know the input will no longer be used.\n\nSimilarly, the serving has a `distributed_postprocessing` callback which can do\nequivalent before sending the reply to the caller.","ref":"Nx.Serving.html#batched_run/3-distributed-mode"},{"type":"function","title":"Nx.Serving.client_postprocessing/2","doc":"Sets the client postprocessing function.\n\nThe client postprocessing receives a tuple with the\n`{output, metadata}` or a stream as first argument.\nThe second argument is always the additional information\nreturned by the client preprocessing.\n\nThe default implementation returns either the output or\nthe stream.","ref":"Nx.Serving.html#client_postprocessing/2"},{"type":"function","title":"Nx.Serving.client_preprocessing/2","doc":"Sets the client preprocessing function.\n\nThe default implementation expects a `Nx.Batch` or a stream of\nNx.Batch to be given as input and return them as is.","ref":"Nx.Serving.html#client_preprocessing/2"},{"type":"function","title":"Nx.Serving.defn_options/2","doc":"Sets the defn options of this serving.\n\nThese are the options supported by `Nx.Defn.default_options/1`.","ref":"Nx.Serving.html#defn_options/2"},{"type":"function","title":"Nx.Serving.distributed_postprocessing/2","doc":"Sets the distributed postprocessing function.\n\nThe default implementation is `Function.identity/1`.","ref":"Nx.Serving.html#distributed_postprocessing/2"},{"type":"callback","title":"Nx.Serving.handle_batch/3","doc":"Receives a batch, a partition, and returns a function to execute the batch.\n\nIn case of serving processes, the function is executed is an\nseparate process.","ref":"Nx.Serving.html#c:handle_batch/3"},{"type":"callback","title":"Nx.Serving.init/3","doc":"The callback used to initialize the serving.\n\nThe first argument reveals if the serving is executed inline,\nsuch as by calling `run/2`, by started with the process.\nThe second argument is the serving argument given to `new/2`.\nThe third argument option is a list of compiler options to be\nused to compile each partition the serving will run.\n\nIt must return `{:ok, state}`, where the `state` can be any term.","ref":"Nx.Serving.html#c:init/3"},{"type":"function","title":"Nx.Serving.jit/2","doc":"Creates a new serving by jitting the given `fun` with `defn_options`.\n\nThis is equivalent to:\n\n    new(fn opts -> Nx.Defn.jit(fun, opts) end, defn_options)","ref":"Nx.Serving.html#jit/2"},{"type":"function","title":"Nx.Serving.new/2","doc":"Creates a new function serving.\n\nIt expects a single- or double-arity function. If a single-arity\nfunction is given, it receives the compiler options and must\nreturn a JIT (via `Nx.Defn.jit/2`) or AOT compiled (via\n`Nx.Defn.compile/3`) one-arity function.\n\nIf a double-arity function is given, it receives the batch\nkey as first argument and the compiler options as second argument.\nIt must return a JIT (via `Nx.Defn.jit/2`) or AOT compiled\n(via `Nx.Defn.compile/3`) one-arity function, but in practice\nit will be a `Nx.Defn.compile/3`, since the purpose of the\nbatch key is often to precompile different versions of the\nsame function upfront. The batch keys can be given on\n`start_link/1`.\n\nThe function will be called with the arguments returned by the\n`client_preprocessing` callback.","ref":"Nx.Serving.html#new/2"},{"type":"function","title":"Nx.Serving.new/3","doc":"Creates a new module-based serving.\n\nIt expects a module and an argument that is given to its `init`\ncallback.\n\nA third optional argument called `defn_options` are additional\ncompiler options which will be given to the module. Those options\nwill be merged into `Nx.Defn.default_options/0`.","ref":"Nx.Serving.html#new/3"},{"type":"function","title":"Nx.Serving.process_options/2","doc":"Sets the process options of this serving.\n\nThese are the same options as supported on `start_link/1`,\nexcept `:name` and `:serving` itself.","ref":"Nx.Serving.html#process_options/2"},{"type":"function","title":"Nx.Serving.run/2","doc":"Runs `serving` with the given `input` inline with the current process.\n\nThe `serving` is executed immediately, without waiting or batching inputs\nfrom other processes. If a `batch_size/2` is specified, then the input may\nbe split or padded, but they are still executed immediately inline.","ref":"Nx.Serving.html#run/2"},{"type":"function","title":"Nx.Serving.start_link/1","doc":"Starts a `Nx.Serving` process to batch requests to a given serving.","ref":"Nx.Serving.html#start_link/1"},{"type":"function","title":"Options - Nx.Serving.start_link/1","doc":"All options, except `:name` and `:serving`, can also be set via\n`process_options/2`.\n\n  * `:name` - an atom with the name of the process\n\n  * `:serving` - a `Nx.Serving` struct with the serving configuration\n\n  * `:batch_keys` - all available batch keys. Batch keys allows Nx.Serving\n    to accumulate different batches with different properties. Defaults to\n    `[:default]`\n\n  * `:batch_size` - the maximum batch size. A default value can be set with\n    `batch_size/2`, which applies to both `run/2` and `batched_run/2`.\n    Setting this option only affects `batched_run/2` and it defaults to `1`\n    if none is set. Note batches received by the serving are not automatically\n    padded to the batch size, such can be done with `Nx.Batch.pad/2`.\n\n  * `:batch_timeout` - the maximum time to wait, in milliseconds,\n    before executing the batch (defaults to `100`ms)\n\n  * `:partitions` - when `true`, starts several partitions under this serving.\n    The number of partitions will be determined according to your compiler\n    and for which host it is compiling. See the module docs for more information\n\n  * `:distribution_weight` - weight used for load balancing when running\n    a distributed serving. Defaults to `1`.\n    If it is set to a higher number `w`, the serving process will receive,\n    on average, `w` times the number of requests compared to the\n    default. Note that the weight is multiplied with the number of\n    partitions, if partitioning is enabled.\n\n  * `:shutdown` - the maximum time for the serving to shutdown. This will\n    block until the existing computation finishes (defaults to `30_000`ms)\n\n  * `:hibernate_after` and `:spawn_opt` - configure the underlying serving\n    workers (see `GenServer.start_link/3`)","ref":"Nx.Serving.html#start_link/1-options"},{"type":"function","title":"Nx.Serving.streaming/2","doc":"Configure the serving to stream its results.\n\nOnce `run/2` or `batched_run/2` are invoked, it will then\nreturn a stream. The stream is must be consumed in the same\nprocess that calls `run/2` or `batched_run/2`.\n\nBatches will be streamed as they arrive. You may also opt-in\nto stream `Nx.Defn` hooks.","ref":"Nx.Serving.html#streaming/2"},{"type":"function","title":"Options - Nx.Serving.streaming/2","doc":"* `:hooks` - a list of hook names that will become streaming events","ref":"Nx.Serving.html#streaming/2-options"},{"type":"function","title":"Implementation details - Nx.Serving.streaming/2","doc":"#","ref":"Nx.Serving.html#streaming/2-implementation-details"},{"type":"function","title":"Client postprocessing - Nx.Serving.streaming/2","doc":"Once streaming is enabled, the client postprocessing callback\nwill receive a stream which will emit events for each hook\nin the shape of:\n\n    {hook_name, term()}\n\nThe stream will also receive events in the shape of\n`{:batch, output, metadata}` as batches are processed by the\nserving. The client postprocessing is often expected to call\n`Stream.transform/3` to process those events into something\nusable by callers.\n\nIf the `:hooks` option is given, only a single `:batch` event\nis emitted, at the end, as detailed next.\n\n#","ref":"Nx.Serving.html#streaming/2-client-postprocessing"},{"type":"function","title":"Batch limits - Nx.Serving.streaming/2","doc":"If you are streaming hooks, the serving server can no longer break\nbatch and you are unable to push a payload bigger than `:batch_size`.\nFor example, imagine you have a `batch_size` of 3 and you push three\nbatches of two elements (AA, BB, and CC). Without hooks, the batches\nwill be consumed as:\n\n    AAB -> BCC\n\nWith streaming, we can't break the batch `BB`, as above, so we will\nconsistently pad with zeroes:\n\n    AA0 -> BB0 -> CC0\n\nIn practice, this should not be a major problem, as you should\ngenerally avoid having a batch size that is not a multiple of the\nmost common batches.","ref":"Nx.Serving.html#streaming/2-batch-limits"},{"type":"type","title":"Nx.Serving.client_info/0","doc":"","ref":"Nx.Serving.html#t:client_info/0"},{"type":"type","title":"Nx.Serving.client_postprocessing/0","doc":"","ref":"Nx.Serving.html#t:client_postprocessing/0"},{"type":"type","title":"Nx.Serving.client_preprocessing/0","doc":"","ref":"Nx.Serving.html#t:client_preprocessing/0"},{"type":"type","title":"Nx.Serving.distributed_postprocessing/0","doc":"","ref":"Nx.Serving.html#t:distributed_postprocessing/0"},{"type":"type","title":"Nx.Serving.distributed_preprocessing/0","doc":"","ref":"Nx.Serving.html#t:distributed_preprocessing/0"},{"type":"type","title":"Nx.Serving.metadata/0","doc":"","ref":"Nx.Serving.html#t:metadata/0"},{"type":"type","title":"Nx.Serving.t/0","doc":"","ref":"Nx.Serving.html#t:t/0"},{"type":"protocol","title":"Nx.Container","doc":"A protocol that teaches `defn` how to traverse data structures.\n\nWhen you invoke a `defn`, its arguments must implement\na `Nx.LazyContainer` and return a data structure that\nimplements `Nx.Container`. Inside `defn`, you can work\nwith any container data structure, such as:\n\n  1. numbers/tensors\n  2. tuples\n  3. maps of any key\n  4. any struct that implements `Nx.Container`\n\nIn other words, LazyContainer is how you convert data\nstructures that are not meant to work inside `defn` into\na `Nx.Container`. And a `Nx.Container` is a data structure\nthat can be manipulated inside `defn` itself.\n\nThe easiest way to implement `Nx.Container` is by deriving\nit. For example:\n\n    @derive {Nx.Container,\n             containers: [:field_name, :other_field]}\n    defstruct [:field_name, :other_fields, ...]\n\nThe `:containers` option is required and it must specify a\nlist of fields that contains tensors (or other containers).\nInside `defn`, the container fields will be automatically\nconverted to tensor expressions. All other fields will be\nreset to their default value, unless you explicitly declare\nthem to be kept:\n\n    @derive {Nx.Container,\n             containers: [:field_name, :other_field],\n             keep: [:another_field]}\n    defstruct [:field_name, :other_fields, ...]\n\nNote `Nx.LazyContainer` is automatically provided for all\ndata structures that implement `Nx.Container`.\n\n> **Careful!**: If you keep a field, its value will be part\n> of the `Nx.Defn` compiler cache key (i.e. therefore if you\n> give a struct with two different values for a kept field,\n> `Nx.Defn` will have to compile and cache it twice).\n> You must only keep fields that you are certain to be used\n> inside `defn` during compilation time.","ref":"Nx.Container.html"},{"type":"protocol","title":"Serialization - Nx.Container","doc":"If you `@derive {Nx.Container, ...}`, it will automatically\ndefine a serialization function with the container and keep\nfields you declare. If you expect a struct to be serialized,\nthen you must be careful to evolve its schema over time in\na compatible way. In particular, removing fields will lead to\ncrashes. If you change the type of a field value, previously\nserialized structs may still hold the old type. And if you add\nnew fields, previously serialized structs won't have such fields\nand therefore be deserialized with its default value.","ref":"Nx.Container.html#module-serialization"},{"type":"function","title":"Nx.Container.reduce/3","doc":"Reduces non-recursively tensors in a data structure with `acc` and `fun`.\n\n`fun` is invoked with each tensor or tensor container in the\ndata structure plus an accumulator. It must return the new\naccumulator.\n\nThis function the final accumulator.\n\nGiven `fun` may receive containers, it is not recursive by default.\nSee `Nx.Defn.Composite.reduce/3` for a recursive variant.","ref":"Nx.Container.html#reduce/3"},{"type":"function","title":"Nx.Container.serialize/1","doc":"Defines how this container must be serialized to disk.\n\nIt receives the container and it must return a three element tuple\nof `{module, list_of_container_tuples, metadata}` where:\n\n  * the `module` to deserialize the container\n  * a list of tuples in the shape `{key, container}` with containers to be further serialized\n  * additional metadata for serialization/deserialization\n\nOn deserialization, `module.deserialize(list_of_container_tuples, metadata)`\nwill be invoked.","ref":"Nx.Container.html#serialize/1"},{"type":"function","title":"Nx.Container.traverse/3","doc":"Traverses non-recursively tensors in a data structure with `acc` and `fun`.\n\n`fun` is invoked with each tensor or tensor container in the\ndata structure plus an accumulator. It must return a two element\ntuple with the updated value and accumulator.\n\nThis function returns the updated container and the accumulator.\n\nGiven `fun` may receive containers, it is not recursive by default.\nSee `Nx.Defn.Composite.traverse/3` for a recursive variant.","ref":"Nx.Container.html#traverse/3"},{"type":"type","title":"Nx.Container.t/0","doc":"All the types that implement this protocol.","ref":"Nx.Container.html#t:t/0"},{"type":"protocol","title":"Nx.LazyContainer","doc":"Converts a data structure to a container lazily.\n\nWhen you invoke a `defn`, its arguments must implement\nthe `Nx.LazyContainer` protocol and return a data structure\nthat implements `Nx.Container`. In other words, this\nprotocol teaches Nx to work with additional data types\nbeyond numbers and tensors.\n\nThis module provides a single traverse implementation\nthat emits the tensor template and a function that computes\nthe tensor as two distinct values. Then a tensor is only\nallocated if necessary.\n\n`Nx.LazyContainer` is automatically provided for data\nstructures that implement `Nx.Container`.","ref":"Nx.LazyContainer.html"},{"type":"function","title":"Nx.LazyContainer.traverse/3","doc":"Traverses recursively tensors in a data structure with `acc` and `fun`.\n\nFor each tensor in the container, `fun` receives a tensor\ntemplate, an anonymous function to build the actual tensor,\nand the accumulator . It returns a two element tuple with\na non-lazy Nx.Container and the accumulator.\n\nThis function returns the updated container and the accumulator.\n\nNote this function is recursive by default. Therefore if you\nare implementing this function and one of your arguments may\nbe containers, you must call `Nx.LazyContainer.traverse/3`\non said arguments so they are recursively traversed.","ref":"Nx.LazyContainer.html#traverse/3"},{"type":"type","title":"Nx.LazyContainer.t/0","doc":"All the types that implement this protocol.","ref":"Nx.LazyContainer.html#t:t/0"},{"type":"protocol","title":"Nx.Stream","doc":"The protocol for streaming data in and out of backends.","ref":"Nx.Stream.html"},{"type":"function","title":"Nx.Stream.done/1","doc":"Returns the output of the stream.\n\nIt may be a tensor, a tuple of tensors, or a map of tensors.","ref":"Nx.Stream.html#done/1"},{"type":"function","title":"Nx.Stream.recv/1","doc":"Receives data from the stream.\n\nIt may be a tensor, a tuple of tensors, or a map of tensors.","ref":"Nx.Stream.html#recv/1"},{"type":"function","title":"Nx.Stream.send/2","doc":"Sends a tensor.\n\nReturns the given tensor.","ref":"Nx.Stream.html#send/2"},{"type":"type","title":"Nx.Stream.t/0","doc":"All the types that implement this protocol.","ref":"Nx.Stream.html#t:t/0"},{"type":"module","title":"Nx.Batch","doc":"Creates a batch of tensors (and containers).\n\nA batch is lazily traversed, concatenated, and padded upon `defn` invocation.","ref":"Nx.Batch.html"},{"type":"function","title":"Nx.Batch.__struct__/0","doc":"A Nx.Batch struct.\n\nThe `:size` field is public.","ref":"Nx.Batch.html#__struct__/0"},{"type":"function","title":"Nx.Batch.concatenate/2","doc":"Concatenates the given entries to the batch.\n\nEntries are concatenated based on their first axis.\nIf the first axis has multiple entries, each entry\nis added to the size of the batch.\n\nYou can either concatenate to an existing batch\nor skip the batch argument to create a new batch.\n\nSee `stack/2` if you want to stack entries instead\nof concatenating them.","ref":"Nx.Batch.html#concatenate/2"},{"type":"function","title":"Examples - Nx.Batch.concatenate/2","doc":"If no batch is given, one is automatically created:\n\n    iex> batch = Nx.Batch.concatenate([Nx.tensor([1]), Nx.tensor([2]), Nx.tensor([3])])\n    iex> Nx.Defn.jit_apply(&Function.identity/1, [batch])\n    #Nx.Tensor \n\nBut you can also concatenate to existing batches:\n\n    iex> batch = Nx.Batch.concatenate([Nx.tensor([1]), Nx.tensor([2])])\n    iex> batch = Nx.Batch.concatenate(batch, [Nx.tensor([3]), Nx.tensor([4])])\n    iex> Nx.Defn.jit_apply(&Function.identity/1, [batch])\n    #Nx.Tensor \n\nIf the first axis has multiple entries, each entry counts\ntowards the size of the batch:\n\n    iex> batch = Nx.Batch.concatenate([Nx.tensor([1, 2]), Nx.tensor([3, 4, 5])])\n    iex> batch.size\n    5\n    iex> Nx.Defn.jit_apply(&Function.identity/1, [batch])\n    #Nx.Tensor \n\nWhat makes batches powerful is that they can concatenate\nacross containers:\n\n    iex> container1 = {Nx.tensor([11]), Nx.tensor([21])}\n    iex> container2 = {Nx.tensor([12]), Nx.tensor([22])}\n    iex> batch = Nx.Batch.concatenate([container1, container2])\n    iex> {batched1, batched2} = Nx.Defn.jit_apply(&Function.identity/1, [batch])\n    iex> batched1\n    #Nx.Tensor \n    iex> batched2\n    #Nx.Tensor","ref":"Nx.Batch.html#concatenate/2-examples"},{"type":"function","title":"Nx.Batch.key/2","doc":"Sets the batch key for the given batch.","ref":"Nx.Batch.html#key/2"},{"type":"function","title":"Nx.Batch.merge/1","doc":"Merges a list of batches.\n\nSee `merge/2`.","ref":"Nx.Batch.html#merge/1"},{"type":"function","title":"Nx.Batch.merge/2","doc":"Merges two batches.\n\nThe tensors on the left will appear before the tensors on the right.\n\nThe size and padding of both batches are summed. The padding still\napplies only at the end of batch.\n\nIt will raise if the batch templates are incompatible.","ref":"Nx.Batch.html#merge/2"},{"type":"function","title":"Examples - Nx.Batch.merge/2","doc":"iex> batch1 = Nx.Batch.stack([Nx.tensor(1), Nx.tensor(2), Nx.tensor(3)])\n    iex> batch2 = Nx.Batch.concatenate([Nx.tensor([4, 5]), Nx.tensor([6, 7, 8])])\n    iex> batch = Nx.Batch.merge(batch1, batch2)\n    iex> batch.size\n    8\n    iex> Nx.Defn.jit_apply(&Function.identity/1, [batch])\n    #Nx.Tensor","ref":"Nx.Batch.html#merge/2-examples"},{"type":"function","title":"Nx.Batch.new/0","doc":"Returns a new empty batch.","ref":"Nx.Batch.html#new/0"},{"type":"function","title":"Nx.Batch.pad/2","doc":"Configures the batch with the given padding.\n\nThe batch will be padded when consumed:\n\n    iex> batch = Nx.Batch.stack([Nx.tensor(1), Nx.tensor(2), Nx.tensor(3)])\n    iex> Nx.Defn.jit_apply(&Function.identity/1, [Nx.Batch.pad(batch, 2)])\n    #Nx.Tensor","ref":"Nx.Batch.html#pad/2"},{"type":"function","title":"Nx.Batch.split/2","doc":"Splits a batch in two, where the first one has at most `n` elements.\n\nIf there is any padding and the batch is not full, the amount of padding\nnecessary will be moved to the first batch and the remaining stays in the\nsecond batch.","ref":"Nx.Batch.html#split/2"},{"type":"function","title":"Examples - Nx.Batch.split/2","doc":"iex> batch = Nx.Batch.concatenate([Nx.tensor([1, 2]), Nx.tensor([3, 4, 5])])\n    iex> {left, right} = Nx.Defn.jit_apply(&Function.identity/1, [Nx.Batch.split(batch, 3)])\n    iex> left\n    #Nx.Tensor \n    iex> right\n    #Nx.Tensor","ref":"Nx.Batch.html#split/2-examples"},{"type":"function","title":"Nx.Batch.stack/2","doc":"Stacks the given entries to the batch.\n\nEach entry counts exactly as a single entry.\nYou can either stack to an existing batch\nor skip the batch argument to create a new batch.\n\nSee `concatenate/2` if you want to concatenate entries\ninstead of stacking them.","ref":"Nx.Batch.html#stack/2"},{"type":"function","title":"Examples - Nx.Batch.stack/2","doc":"If no batch is given, one is automatically created:\n\n    iex> batch = Nx.Batch.stack([Nx.tensor(1), Nx.tensor(2), Nx.tensor(3)])\n    iex> batch.size\n    3\n    iex> Nx.Defn.jit_apply(&Function.identity/1, [batch])\n    #Nx.Tensor \n\nBut you can also stack an existing batch:\n\n    iex> batch = Nx.Batch.stack([Nx.tensor(1), Nx.tensor(2)])\n    iex> batch = Nx.Batch.stack(batch, [Nx.tensor(3), Nx.tensor(4)])\n    iex> batch.size\n    4\n    iex> Nx.Defn.jit_apply(&Function.identity/1, [batch])\n    #Nx.Tensor \n\nWhat makes batches powerful is that they can concatenate\nacross containers:\n\n    iex> container1 = {Nx.tensor(11), Nx.tensor(21)}\n    iex> container2 = {Nx.tensor(12), Nx.tensor(22)}\n    iex> batch = Nx.Batch.stack([container1, container2])\n    iex> {batched1, batched2} = Nx.Defn.jit_apply(&Function.identity/1, [batch])\n    iex> batched1\n    #Nx.Tensor \n    iex> batched2\n    #Nx.Tensor","ref":"Nx.Batch.html#stack/2-examples"},{"type":"type","title":"Nx.Batch.t/0","doc":"","ref":"Nx.Batch.html#t:t/0"},{"type":"module","title":"Nx.Heatmap","doc":"Provides a heatmap that is printed using ANSI colors\nin the terminal.","ref":"Nx.Heatmap.html"},{"type":"module","title":"Nx.Tensor","doc":"The tensor struct and the behaviour for backends.\n\n`Nx.Tensor` is a generic container for multidimensional data structures.\nIt contains the tensor type, shape, and names. The data itself is a\nstruct that points to a backend responsible for controlling the data.\nThe backend behaviour is described in `Nx.Backend`.\n\nThe tensor has the following fields:\n\n  * `:data` - the tensor backend and its data\n  * `:shape` - the tensor shape\n  * `:type` - the tensor type\n  * `:names` - the tensor names\n  * `:vectorized_axes` - a tuple that encodes names and sizes for vectorization\n\nIn general it is discouraged to access those fields directly. Use\nthe functions in the `Nx` module instead. Backends have to access those\nfields but it cannot update them, except for the `:data` field itself.","ref":"Nx.Tensor.html"},{"type":"type","title":"Nx.Tensor.axes/0","doc":"","ref":"Nx.Tensor.html#t:axes/0"},{"type":"type","title":"Nx.Tensor.axis/0","doc":"","ref":"Nx.Tensor.html#t:axis/0"},{"type":"type","title":"Nx.Tensor.data/0","doc":"","ref":"Nx.Tensor.html#t:data/0"},{"type":"type","title":"Nx.Tensor.name/0","doc":"","ref":"Nx.Tensor.html#t:name/0"},{"type":"type","title":"Nx.Tensor.shape/0","doc":"","ref":"Nx.Tensor.html#t:shape/0"},{"type":"type","title":"Nx.Tensor.t/0","doc":"","ref":"Nx.Tensor.html#t:t/0"},{"type":"type","title":"Nx.Tensor.t/1","doc":"","ref":"Nx.Tensor.html#t:t/1"},{"type":"type","title":"Nx.Tensor.type/0","doc":"","ref":"Nx.Tensor.html#t:type/0"},{"type":"behaviour","title":"Nx.Backend","doc":"The behaviour for tensor backends.\n\nEach backend is module that defines a struct and implements the callbacks\ndefined in this module. The callbacks are mostly implementations of the\nfunctions in the `Nx` module with the tensor output shape given as first\nargument.\n\n`Nx` backends come in two flavors: opaque backends, of which you should\nnot access its data directly except through the functions in the `Nx`\nmodule, and public ones, of which its data can be directly accessed and\ntraversed. The former typically have the `Backend` suffix.\n\n`Nx` ships with the following backends:\n\n  * `Nx.BinaryBackend` - an opaque backend written in pure Elixir\n    that stores the data in Elixir's binaries. This is the default\n    backend used by the `Nx` module. The backend itself (and its\n    data) is private and must not be accessed directly.\n\n  * `Nx.TemplateBackend` - an opaque backend written that works as\n    a template in APIs to declare the type, shape, and names of\n    tensors to be expected in the future.\n\n  * `Nx.Defn.Expr` - a public backend used by `defn` to build\n    expression trees that are traversed by custom compilers.\n\nThis module also includes functions that are meant to be shared\nacross backends.","ref":"Nx.Backend.html"},{"type":"callback","title":"Nx.Backend.abs/2","doc":"","ref":"Nx.Backend.html#c:abs/2"},{"type":"callback","title":"Nx.Backend.acos/2","doc":"","ref":"Nx.Backend.html#c:acos/2"},{"type":"callback","title":"Nx.Backend.acosh/2","doc":"","ref":"Nx.Backend.html#c:acosh/2"},{"type":"callback","title":"Nx.Backend.add/3","doc":"","ref":"Nx.Backend.html#c:add/3"},{"type":"callback","title":"Nx.Backend.all/3","doc":"","ref":"Nx.Backend.html#c:all/3"},{"type":"callback","title":"Nx.Backend.all_close/4","doc":"","ref":"Nx.Backend.html#c:all_close/4"},{"type":"callback","title":"Nx.Backend.any/3","doc":"","ref":"Nx.Backend.html#c:any/3"},{"type":"callback","title":"Nx.Backend.argmax/3","doc":"","ref":"Nx.Backend.html#c:argmax/3"},{"type":"callback","title":"Nx.Backend.argmin/3","doc":"","ref":"Nx.Backend.html#c:argmin/3"},{"type":"callback","title":"Nx.Backend.argsort/3","doc":"","ref":"Nx.Backend.html#c:argsort/3"},{"type":"callback","title":"Nx.Backend.as_type/2","doc":"","ref":"Nx.Backend.html#c:as_type/2"},{"type":"callback","title":"Nx.Backend.asin/2","doc":"","ref":"Nx.Backend.html#c:asin/2"},{"type":"callback","title":"Nx.Backend.asinh/2","doc":"","ref":"Nx.Backend.html#c:asinh/2"},{"type":"callback","title":"Nx.Backend.atan2/3","doc":"","ref":"Nx.Backend.html#c:atan2/3"},{"type":"callback","title":"Nx.Backend.atan/2","doc":"","ref":"Nx.Backend.html#c:atan/2"},{"type":"callback","title":"Nx.Backend.atanh/2","doc":"","ref":"Nx.Backend.html#c:atanh/2"},{"type":"callback","title":"Nx.Backend.backend_copy/3","doc":"","ref":"Nx.Backend.html#c:backend_copy/3"},{"type":"callback","title":"Nx.Backend.backend_deallocate/1","doc":"","ref":"Nx.Backend.html#c:backend_deallocate/1"},{"type":"callback","title":"Nx.Backend.backend_transfer/3","doc":"","ref":"Nx.Backend.html#c:backend_transfer/3"},{"type":"callback","title":"Nx.Backend.bitcast/2","doc":"","ref":"Nx.Backend.html#c:bitcast/2"},{"type":"callback","title":"Nx.Backend.bitwise_and/3","doc":"","ref":"Nx.Backend.html#c:bitwise_and/3"},{"type":"callback","title":"Nx.Backend.bitwise_not/2","doc":"","ref":"Nx.Backend.html#c:bitwise_not/2"},{"type":"callback","title":"Nx.Backend.bitwise_or/3","doc":"","ref":"Nx.Backend.html#c:bitwise_or/3"},{"type":"callback","title":"Nx.Backend.bitwise_xor/3","doc":"","ref":"Nx.Backend.html#c:bitwise_xor/3"},{"type":"callback","title":"Nx.Backend.broadcast/4","doc":"","ref":"Nx.Backend.html#c:broadcast/4"},{"type":"callback","title":"Nx.Backend.cbrt/2","doc":"","ref":"Nx.Backend.html#c:cbrt/2"},{"type":"callback","title":"Nx.Backend.ceil/2","doc":"","ref":"Nx.Backend.html#c:ceil/2"},{"type":"callback","title":"Nx.Backend.cholesky/2","doc":"","ref":"Nx.Backend.html#c:cholesky/2"},{"type":"callback","title":"Nx.Backend.clip/4","doc":"","ref":"Nx.Backend.html#c:clip/4"},{"type":"function","title":"Nx.Backend.complex_to_string/2","doc":"","ref":"Nx.Backend.html#complex_to_string/2"},{"type":"callback","title":"Nx.Backend.concatenate/3","doc":"","ref":"Nx.Backend.html#c:concatenate/3"},{"type":"callback","title":"Nx.Backend.conjugate/2","doc":"","ref":"Nx.Backend.html#c:conjugate/2"},{"type":"callback","title":"Nx.Backend.constant/3","doc":"","ref":"Nx.Backend.html#c:constant/3"},{"type":"callback","title":"Nx.Backend.conv/4","doc":"","ref":"Nx.Backend.html#c:conv/4"},{"type":"callback","title":"Nx.Backend.cos/2","doc":"","ref":"Nx.Backend.html#c:cos/2"},{"type":"callback","title":"Nx.Backend.cosh/2","doc":"","ref":"Nx.Backend.html#c:cosh/2"},{"type":"callback","title":"Nx.Backend.count_leading_zeros/2","doc":"","ref":"Nx.Backend.html#c:count_leading_zeros/2"},{"type":"callback","title":"Nx.Backend.cumulative_max/3","doc":"","ref":"Nx.Backend.html#c:cumulative_max/3"},{"type":"callback","title":"Nx.Backend.cumulative_min/3","doc":"","ref":"Nx.Backend.html#c:cumulative_min/3"},{"type":"callback","title":"Nx.Backend.cumulative_product/3","doc":"","ref":"Nx.Backend.html#c:cumulative_product/3"},{"type":"callback","title":"Nx.Backend.cumulative_sum/3","doc":"","ref":"Nx.Backend.html#c:cumulative_sum/3"},{"type":"callback","title":"Nx.Backend.determinant/2","doc":"","ref":"Nx.Backend.html#c:determinant/2"},{"type":"callback","title":"Nx.Backend.divide/3","doc":"","ref":"Nx.Backend.html#c:divide/3"},{"type":"callback","title":"Nx.Backend.dot/7","doc":"","ref":"Nx.Backend.html#c:dot/7"},{"type":"callback","title":"Nx.Backend.eigh/3","doc":"","ref":"Nx.Backend.html#c:eigh/3"},{"type":"callback","title":"Nx.Backend.equal/3","doc":"","ref":"Nx.Backend.html#c:equal/3"},{"type":"callback","title":"Nx.Backend.erf/2","doc":"","ref":"Nx.Backend.html#c:erf/2"},{"type":"callback","title":"Nx.Backend.erf_inv/2","doc":"","ref":"Nx.Backend.html#c:erf_inv/2"},{"type":"callback","title":"Nx.Backend.erfc/2","doc":"","ref":"Nx.Backend.html#c:erfc/2"},{"type":"callback","title":"Nx.Backend.exp/2","doc":"","ref":"Nx.Backend.html#c:exp/2"},{"type":"callback","title":"Nx.Backend.expm1/2","doc":"","ref":"Nx.Backend.html#c:expm1/2"},{"type":"callback","title":"Nx.Backend.eye/2","doc":"","ref":"Nx.Backend.html#c:eye/2"},{"type":"callback","title":"Nx.Backend.fft/3","doc":"","ref":"Nx.Backend.html#c:fft/3"},{"type":"callback","title":"Nx.Backend.floor/2","doc":"","ref":"Nx.Backend.html#c:floor/2"},{"type":"callback","title":"Nx.Backend.from_binary/3","doc":"","ref":"Nx.Backend.html#c:from_binary/3"},{"type":"callback","title":"Nx.Backend.gather/4","doc":"","ref":"Nx.Backend.html#c:gather/4"},{"type":"callback","title":"Nx.Backend.greater/3","doc":"","ref":"Nx.Backend.html#c:greater/3"},{"type":"callback","title":"Nx.Backend.greater_equal/3","doc":"","ref":"Nx.Backend.html#c:greater_equal/3"},{"type":"callback","title":"Nx.Backend.ifft/3","doc":"","ref":"Nx.Backend.html#c:ifft/3"},{"type":"callback","title":"Nx.Backend.imag/2","doc":"","ref":"Nx.Backend.html#c:imag/2"},{"type":"callback","title":"Nx.Backend.indexed_add/5","doc":"","ref":"Nx.Backend.html#c:indexed_add/5"},{"type":"callback","title":"Nx.Backend.indexed_put/5","doc":"","ref":"Nx.Backend.html#c:indexed_put/5"},{"type":"callback","title":"Nx.Backend.init/1","doc":"","ref":"Nx.Backend.html#c:init/1"},{"type":"callback","title":"Nx.Backend.inspect/2","doc":"","ref":"Nx.Backend.html#c:inspect/2"},{"type":"function","title":"Nx.Backend.inspect/3","doc":"Inspects the given tensor given by `binary`.\n\nNote the `binary` may have fewer elements than the\ntensor size but, in such cases, it must strictly have\nmore elements than `inspect_opts.limit`","ref":"Nx.Backend.html#inspect/3"},{"type":"function","title":"Options - Nx.Backend.inspect/3","doc":"The following must be passed through `Inspect` `:custom_options`\n\n  * `:nx_precision` - Configures the floating-point number printing precision.\n    If set, will print floating-point numbers in scientific notation using the\n    specified number of significant digits. Otherwise, default Elixir printing\n    rules are applied.","ref":"Nx.Backend.html#inspect/3-options"},{"type":"callback","title":"Nx.Backend.iota/3","doc":"","ref":"Nx.Backend.html#c:iota/3"},{"type":"callback","title":"Nx.Backend.is_infinity/2","doc":"","ref":"Nx.Backend.html#c:is_infinity/2"},{"type":"callback","title":"Nx.Backend.is_nan/2","doc":"","ref":"Nx.Backend.html#c:is_nan/2"},{"type":"callback","title":"Nx.Backend.left_shift/3","doc":"","ref":"Nx.Backend.html#c:left_shift/3"},{"type":"callback","title":"Nx.Backend.less/3","doc":"","ref":"Nx.Backend.html#c:less/3"},{"type":"callback","title":"Nx.Backend.less_equal/3","doc":"","ref":"Nx.Backend.html#c:less_equal/3"},{"type":"callback","title":"Nx.Backend.log1p/2","doc":"","ref":"Nx.Backend.html#c:log1p/2"},{"type":"callback","title":"Nx.Backend.log/2","doc":"","ref":"Nx.Backend.html#c:log/2"},{"type":"callback","title":"Nx.Backend.logical_and/3","doc":"","ref":"Nx.Backend.html#c:logical_and/3"},{"type":"callback","title":"Nx.Backend.logical_not/2","doc":"","ref":"Nx.Backend.html#c:logical_not/2"},{"type":"callback","title":"Nx.Backend.logical_or/3","doc":"","ref":"Nx.Backend.html#c:logical_or/3"},{"type":"callback","title":"Nx.Backend.logical_xor/3","doc":"","ref":"Nx.Backend.html#c:logical_xor/3"},{"type":"callback","title":"Nx.Backend.lu/3","doc":"","ref":"Nx.Backend.html#c:lu/3"},{"type":"callback","title":"Nx.Backend.map/4","doc":"","ref":"Nx.Backend.html#c:map/4"},{"type":"callback","title":"Nx.Backend.max/3","doc":"","ref":"Nx.Backend.html#c:max/3"},{"type":"callback","title":"Nx.Backend.min/3","doc":"","ref":"Nx.Backend.html#c:min/3"},{"type":"callback","title":"Nx.Backend.multiply/3","doc":"","ref":"Nx.Backend.html#c:multiply/3"},{"type":"callback","title":"Nx.Backend.negate/2","doc":"","ref":"Nx.Backend.html#c:negate/2"},{"type":"callback","title":"Nx.Backend.not_equal/3","doc":"","ref":"Nx.Backend.html#c:not_equal/3"},{"type":"callback","title":"Nx.Backend.optional/3","doc":"Invoked for execution of optional callbacks with a default implementation.\n\nFirst we will attempt to call the optional callback itself\n(one of the many callbacks defined below), then we attempt\nto call this callback (which is also optional), then we\nfallback to the default iomplementation.","ref":"Nx.Backend.html#c:optional/3"},{"type":"callback","title":"Nx.Backend.pad/4","doc":"","ref":"Nx.Backend.html#c:pad/4"},{"type":"callback","title":"Nx.Backend.phase/2","doc":"","ref":"Nx.Backend.html#c:phase/2"},{"type":"callback","title":"Nx.Backend.population_count/2","doc":"","ref":"Nx.Backend.html#c:population_count/2"},{"type":"callback","title":"Nx.Backend.pow/3","doc":"","ref":"Nx.Backend.html#c:pow/3"},{"type":"callback","title":"Nx.Backend.product/3","doc":"","ref":"Nx.Backend.html#c:product/3"},{"type":"callback","title":"Nx.Backend.put_slice/4","doc":"","ref":"Nx.Backend.html#c:put_slice/4"},{"type":"callback","title":"Nx.Backend.qr/3","doc":"","ref":"Nx.Backend.html#c:qr/3"},{"type":"callback","title":"Nx.Backend.quotient/3","doc":"","ref":"Nx.Backend.html#c:quotient/3"},{"type":"callback","title":"Nx.Backend.random_normal/4","doc":"","ref":"Nx.Backend.html#c:random_normal/4"},{"type":"callback","title":"Nx.Backend.random_uniform/4","doc":"","ref":"Nx.Backend.html#c:random_uniform/4"},{"type":"callback","title":"Nx.Backend.real/2","doc":"","ref":"Nx.Backend.html#c:real/2"},{"type":"callback","title":"Nx.Backend.reduce/5","doc":"","ref":"Nx.Backend.html#c:reduce/5"},{"type":"callback","title":"Nx.Backend.reduce_max/3","doc":"","ref":"Nx.Backend.html#c:reduce_max/3"},{"type":"callback","title":"Nx.Backend.reduce_min/3","doc":"","ref":"Nx.Backend.html#c:reduce_min/3"},{"type":"callback","title":"Nx.Backend.remainder/3","doc":"","ref":"Nx.Backend.html#c:remainder/3"},{"type":"callback","title":"Nx.Backend.reshape/2","doc":"","ref":"Nx.Backend.html#c:reshape/2"},{"type":"callback","title":"Nx.Backend.reverse/3","doc":"","ref":"Nx.Backend.html#c:reverse/3"},{"type":"callback","title":"Nx.Backend.right_shift/3","doc":"","ref":"Nx.Backend.html#c:right_shift/3"},{"type":"callback","title":"Nx.Backend.round/2","doc":"","ref":"Nx.Backend.html#c:round/2"},{"type":"callback","title":"Nx.Backend.rsqrt/2","doc":"","ref":"Nx.Backend.html#c:rsqrt/2"},{"type":"callback","title":"Nx.Backend.select/4","doc":"","ref":"Nx.Backend.html#c:select/4"},{"type":"callback","title":"Nx.Backend.sigmoid/2","doc":"","ref":"Nx.Backend.html#c:sigmoid/2"},{"type":"callback","title":"Nx.Backend.sign/2","doc":"","ref":"Nx.Backend.html#c:sign/2"},{"type":"callback","title":"Nx.Backend.sin/2","doc":"","ref":"Nx.Backend.html#c:sin/2"},{"type":"callback","title":"Nx.Backend.sinh/2","doc":"","ref":"Nx.Backend.html#c:sinh/2"},{"type":"callback","title":"Nx.Backend.slice/5","doc":"","ref":"Nx.Backend.html#c:slice/5"},{"type":"callback","title":"Nx.Backend.solve/3","doc":"","ref":"Nx.Backend.html#c:solve/3"},{"type":"callback","title":"Nx.Backend.sort/3","doc":"","ref":"Nx.Backend.html#c:sort/3"},{"type":"callback","title":"Nx.Backend.sqrt/2","doc":"","ref":"Nx.Backend.html#c:sqrt/2"},{"type":"callback","title":"Nx.Backend.squeeze/3","doc":"","ref":"Nx.Backend.html#c:squeeze/3"},{"type":"callback","title":"Nx.Backend.subtract/3","doc":"","ref":"Nx.Backend.html#c:subtract/3"},{"type":"callback","title":"Nx.Backend.sum/3","doc":"","ref":"Nx.Backend.html#c:sum/3"},{"type":"callback","title":"Nx.Backend.svd/3","doc":"","ref":"Nx.Backend.html#c:svd/3"},{"type":"callback","title":"Nx.Backend.take/4","doc":"","ref":"Nx.Backend.html#c:take/4"},{"type":"callback","title":"Nx.Backend.take_along_axis/4","doc":"","ref":"Nx.Backend.html#c:take_along_axis/4"},{"type":"callback","title":"Nx.Backend.tan/2","doc":"","ref":"Nx.Backend.html#c:tan/2"},{"type":"callback","title":"Nx.Backend.tanh/2","doc":"","ref":"Nx.Backend.html#c:tanh/2"},{"type":"callback","title":"Nx.Backend.to_batched/3","doc":"","ref":"Nx.Backend.html#c:to_batched/3"},{"type":"callback","title":"Nx.Backend.to_binary/2","doc":"","ref":"Nx.Backend.html#c:to_binary/2"},{"type":"callback","title":"Nx.Backend.top_k/3","doc":"","ref":"Nx.Backend.html#c:top_k/3"},{"type":"callback","title":"Nx.Backend.transpose/3","doc":"","ref":"Nx.Backend.html#c:transpose/3"},{"type":"callback","title":"Nx.Backend.triangular_solve/4","doc":"","ref":"Nx.Backend.html#c:triangular_solve/4"},{"type":"callback","title":"Nx.Backend.window_max/4","doc":"","ref":"Nx.Backend.html#c:window_max/4"},{"type":"callback","title":"Nx.Backend.window_min/4","doc":"","ref":"Nx.Backend.html#c:window_min/4"},{"type":"callback","title":"Nx.Backend.window_product/4","doc":"","ref":"Nx.Backend.html#c:window_product/4"},{"type":"callback","title":"Nx.Backend.window_reduce/6","doc":"","ref":"Nx.Backend.html#c:window_reduce/6"},{"type":"callback","title":"Nx.Backend.window_scatter_max/6","doc":"","ref":"Nx.Backend.html#c:window_scatter_max/6"},{"type":"callback","title":"Nx.Backend.window_scatter_min/6","doc":"","ref":"Nx.Backend.html#c:window_scatter_min/6"},{"type":"callback","title":"Nx.Backend.window_sum/4","doc":"","ref":"Nx.Backend.html#c:window_sum/4"},{"type":"type","title":"Nx.Backend.axes/0","doc":"","ref":"Nx.Backend.html#t:axes/0"},{"type":"type","title":"Nx.Backend.axis/0","doc":"","ref":"Nx.Backend.html#t:axis/0"},{"type":"type","title":"Nx.Backend.backend_options/0","doc":"","ref":"Nx.Backend.html#t:backend_options/0"},{"type":"type","title":"Nx.Backend.shape/0","doc":"","ref":"Nx.Backend.html#t:shape/0"},{"type":"type","title":"Nx.Backend.t/0","doc":"","ref":"Nx.Backend.html#t:t/0"},{"type":"type","title":"Nx.Backend.tensor/0","doc":"","ref":"Nx.Backend.html#t:tensor/0"},{"type":"module","title":"Nx.BinaryBackend","doc":"An opaque backend written in pure Elixir that stores\nthe data in Elixir's binaries.\n\nThis is the default backend used by the `Nx` module.\nThe backend itself (and its data) is private and must\nnot be accessed directly.","ref":"Nx.BinaryBackend.html"},{"type":"module","title":"Nx.TemplateBackend","doc":"An opaque backend written that is used as template\nto declare the type, shape, and names of tensors to\nbe expected in the future.\n\nIt doesn't perform any operation, it always raises.","ref":"Nx.TemplateBackend.html"},{"type":"module","title":"Nx.Type","doc":"Conveniences for working with types.\n\nA type is a two-element tuple with the name and the size.\nThe respective sizes for the types are the following:\n\n  * `:s` - signed integer (8, 16, 32, 64)\n  * `:u` - unsigned integer (8, 16, 32, 64)\n  * `:f` - float (16, 32, 64)\n  * `:bf` - a brain floating point (16)\n  * `:c` - a complex number, represented as a pair of floats (64, 128)\n\nEach type has an equivalent atom representation, for example\n`{:s, 8}` can be expressed as `:s8`. When working with user-given\ntypes make sure to call `normalize!/1` to get the canonical\nrepresentation.\n\nNote: there is a special type used by the `defn` compiler\nwhich is `{:tuple, size}`, that represents a tuple. Said types\ndo not appear on user code, only on compiler implementations,\nand therefore are not handled by the functions in this module.\n\nThis module can be used in `defn`.","ref":"Nx.Type.html"},{"type":"function","title":"Nx.Type.cast_number!/2","doc":"Casts the given number to type.\n\nIt does not handle overflow/underflow,\nreturning the number as is, but cast.","ref":"Nx.Type.html#cast_number!/2"},{"type":"function","title":"Examples - Nx.Type.cast_number!/2","doc":"iex> Nx.Type.cast_number!({:u, 8}, 10)\n    10\n    iex> Nx.Type.cast_number!({:s, 8}, 10)\n    10\n    iex> Nx.Type.cast_number!({:s, 8}, -10)\n    -10\n    iex> Nx.Type.cast_number!({:f, 32}, 10)\n    10.0\n    iex> Nx.Type.cast_number!({:bf, 16}, -10)\n    -10.0\n\n    iex> Nx.Type.cast_number!({:f, 32}, 10.0)\n    10.0\n    iex> Nx.Type.cast_number!({:bf, 16}, -10.0)\n    -10.0\n\n    iex> Nx.Type.cast_number!({:c, 64}, 10)\n    %Complex{im: 0.0, re: 10.0}\n\n    iex> Nx.Type.cast_number!({:u, 8}, -10)\n    ** (ArgumentError) cannot cast number -10 to {:u, 8}\n\n    iex> Nx.Type.cast_number!({:s, 8}, 10.0)\n    ** (ArgumentError) cannot cast number 10.0 to {:s, 8}","ref":"Nx.Type.html#cast_number!/2-examples"},{"type":"function","title":"Nx.Type.complex?/1","doc":"Returns true if the type is a complex number.","ref":"Nx.Type.html#complex?/1"},{"type":"function","title":"Examples - Nx.Type.complex?/1","doc":"iex> Nx.Type.complex?({:c, 64})\n    true\n    iex> Nx.Type.complex?({:f, 64})\n    false","ref":"Nx.Type.html#complex?/1-examples"},{"type":"function","title":"Nx.Type.e_binary/1","doc":"Returns $e$ as a binary for the given type","ref":"Nx.Type.html#e_binary/1"},{"type":"function","title":"Nx.Type.epsilon_binary/1","doc":"Returns the machine epsilon for the given type","ref":"Nx.Type.html#epsilon_binary/1"},{"type":"function","title":"Nx.Type.euler_gamma_binary/1","doc":"Returns Euler–Mascheroni constant ($\\gamma$) as a binary for the given type","ref":"Nx.Type.html#euler_gamma_binary/1"},{"type":"function","title":"Nx.Type.float?/1","doc":"Returns true if the type is a float in Elixir.","ref":"Nx.Type.html#float?/1"},{"type":"function","title":"Examples - Nx.Type.float?/1","doc":"iex> Nx.Type.float?({:f, 32})\n    true\n    iex> Nx.Type.float?({:bf, 16})\n    true\n    iex> Nx.Type.float?({:u, 64})\n    false","ref":"Nx.Type.html#float?/1-examples"},{"type":"function","title":"Nx.Type.infer/1","doc":"Infers the type of the given number.","ref":"Nx.Type.html#infer/1"},{"type":"function","title":"Examples - Nx.Type.infer/1","doc":"iex> Nx.Type.infer(1)\n    {:s, 64}\n    iex> Nx.Type.infer(1.0)\n    {:f, 32}\n    iex> Nx.Type.infer(Complex.new(1))\n    {:c, 64}","ref":"Nx.Type.html#infer/1-examples"},{"type":"function","title":"Nx.Type.infinity_binary/1","doc":"Returns infinity as a binary for the given type.","ref":"Nx.Type.html#infinity_binary/1"},{"type":"function","title":"Nx.Type.integer?/1","doc":"Returns true if the type is an integer in Elixir.","ref":"Nx.Type.html#integer?/1"},{"type":"function","title":"Examples - Nx.Type.integer?/1","doc":"iex> Nx.Type.integer?({:s, 8})\n    true\n    iex> Nx.Type.integer?({:u, 64})\n    true\n    iex> Nx.Type.integer?({:f, 64})\n    false","ref":"Nx.Type.html#integer?/1-examples"},{"type":"function","title":"Nx.Type.max_binary/1","doc":"Returns the maximum possible value for the given type.","ref":"Nx.Type.html#max_binary/1"},{"type":"function","title":"Nx.Type.max_finite_binary/1","doc":"Returns the maximum possible finite value for the given type.","ref":"Nx.Type.html#max_finite_binary/1"},{"type":"function","title":"Nx.Type.merge/2","doc":"Merges the given types finding a suitable representation for both.\n\nTypes have the following precedence:\n\n    c > f > bf > s > u\n\nIf the types are the same, they are merged to the highest size.\nIf they are different, the one with the highest precedence wins,\nas long as the size of the `max(big, small * 2))` fits under 64\nbits. Otherwise it casts to f64.\n\nIn the case of complex numbers, the maximum bit size is 128 bits\nbecause they are composed of two floats.","ref":"Nx.Type.html#merge/2"},{"type":"function","title":"Examples - Nx.Type.merge/2","doc":"iex> Nx.Type.merge({:s, 8}, {:s, 8})\n    {:s, 8}\n    iex> Nx.Type.merge({:s, 8}, {:s, 64})\n    {:s, 64}\n\n    iex> Nx.Type.merge({:s, 8}, {:u, 8})\n    {:s, 16}\n    iex> Nx.Type.merge({:s, 16}, {:u, 8})\n    {:s, 16}\n    iex> Nx.Type.merge({:s, 8}, {:u, 16})\n    {:s, 32}\n    iex> Nx.Type.merge({:s, 32}, {:u, 8})\n    {:s, 32}\n    iex> Nx.Type.merge({:s, 8}, {:u, 32})\n    {:s, 64}\n    iex> Nx.Type.merge({:s, 64}, {:u, 8})\n    {:s, 64}\n    iex> Nx.Type.merge({:s, 8}, {:u, 64})\n    {:s, 64}\n\n    iex> Nx.Type.merge({:u, 8}, {:f, 32})\n    {:f, 32}\n    iex> Nx.Type.merge({:u, 64}, {:f, 32})\n    {:f, 32}\n    iex> Nx.Type.merge({:s, 8}, {:f, 32})\n    {:f, 32}\n    iex> Nx.Type.merge({:s, 64}, {:f, 32})\n    {:f, 32}\n\n    iex> Nx.Type.merge({:u, 8}, {:f, 64})\n    {:f, 64}\n    iex> Nx.Type.merge({:u, 64}, {:f, 64})\n    {:f, 64}\n    iex> Nx.Type.merge({:s, 8}, {:f, 64})\n    {:f, 64}\n    iex> Nx.Type.merge({:s, 64}, {:f, 64})\n    {:f, 64}\n\n    iex> Nx.Type.merge({:u, 8}, {:bf, 16})\n    {:bf, 16}\n    iex> Nx.Type.merge({:u, 64}, {:bf, 16})\n    {:bf, 16}\n    iex> Nx.Type.merge({:s, 8}, {:bf, 16})\n    {:bf, 16}\n    iex> Nx.Type.merge({:s, 64}, {:bf, 16})\n    {:bf, 16}\n\n    iex> Nx.Type.merge({:f, 32}, {:bf, 16})\n    {:f, 32}\n    iex> Nx.Type.merge({:f, 64}, {:bf, 16})\n    {:f, 64}\n\n    iex> Nx.Type.merge({:c, 64}, {:f, 32})\n    {:c, 64}\n    iex> Nx.Type.merge({:c, 64}, {:c, 64})\n    {:c, 64}\n    iex> Nx.Type.merge({:c, 128}, {:c, 64})\n    {:c, 128}","ref":"Nx.Type.html#merge/2-examples"},{"type":"function","title":"Nx.Type.merge_number/2","doc":"Merges the given types with the type of a number.\n\nWe attempt to keep the original type and its size as best\nas possible.","ref":"Nx.Type.html#merge_number/2"},{"type":"function","title":"Examples - Nx.Type.merge_number/2","doc":"iex> Nx.Type.merge_number({:u, 8}, 0)\n    {:u, 8}\n    iex> Nx.Type.merge_number({:u, 8}, 255)\n    {:u, 8}\n    iex> Nx.Type.merge_number({:u, 8}, 256)\n    {:u, 16}\n    iex> Nx.Type.merge_number({:u, 8}, -1)\n    {:s, 16}\n    iex> Nx.Type.merge_number({:u, 8}, -32767)\n    {:s, 16}\n    iex> Nx.Type.merge_number({:u, 8}, -32768)\n    {:s, 16}\n    iex> Nx.Type.merge_number({:u, 8}, -32769)\n    {:s, 32}\n\n    iex> Nx.Type.merge_number({:s, 8}, 0)\n    {:s, 8}\n    iex> Nx.Type.merge_number({:s, 8}, 127)\n    {:s, 8}\n    iex> Nx.Type.merge_number({:s, 8}, -128)\n    {:s, 8}\n    iex> Nx.Type.merge_number({:s, 8}, 128)\n    {:s, 16}\n    iex> Nx.Type.merge_number({:s, 8}, -129)\n    {:s, 16}\n    iex> Nx.Type.merge_number({:s, 8}, 1.0)\n    {:f, 32}\n    iex> Nx.Type.merge_number({:u, 64}, -1337)\n    {:s, 64}\n\n    iex> Nx.Type.merge_number({:f, 32}, 1)\n    {:f, 32}\n    iex> Nx.Type.merge_number({:f, 32}, 1.0)\n    {:f, 32}\n    iex> Nx.Type.merge_number({:f, 64}, 1.0)\n    {:f, 64}","ref":"Nx.Type.html#merge_number/2-examples"},{"type":"function","title":"Nx.Type.min_binary/1","doc":"Returns the minimum possible value for the given type.","ref":"Nx.Type.html#min_binary/1"},{"type":"function","title":"Nx.Type.min_finite_binary/1","doc":"Returns the minimum possible finite value for the given type.","ref":"Nx.Type.html#min_finite_binary/1"},{"type":"function","title":"Nx.Type.nan_binary/1","doc":"Returns infinity as a binary for the given type.","ref":"Nx.Type.html#nan_binary/1"},{"type":"function","title":"Nx.Type.neg_infinity_binary/1","doc":"Returns negative infinity as a binary for the given type.","ref":"Nx.Type.html#neg_infinity_binary/1"},{"type":"function","title":"Nx.Type.normalize!/1","doc":"Validates and normalizes the given type tuple.\n\nIt returns the type tuple or raises.\n\nAccepts both the tuple format and the short atom format.","ref":"Nx.Type.html#normalize!/1"},{"type":"function","title":"Examples - Nx.Type.normalize!/1","doc":"iex> Nx.Type.normalize!({:u, 8})\n    {:u, 8}\n\n    iex> Nx.Type.normalize!(:u8)\n    {:u, 8}\n\n    iex> Nx.Type.normalize!({:u, 0})\n    ** (ArgumentError) invalid numerical type: {:u, 0} (see Nx.Type docs for all supported types)\n\n    iex> Nx.Type.normalize!({:k, 8})\n    ** (ArgumentError) invalid numerical type: {:k, 8} (see Nx.Type docs for all supported types)","ref":"Nx.Type.html#normalize!/1-examples"},{"type":"function","title":"Nx.Type.pi_binary/1","doc":"Returns $\\pi$ as a binary for the given type","ref":"Nx.Type.html#pi_binary/1"},{"type":"function","title":"Nx.Type.smallest_positive_normal_binary/1","doc":"Returns the smallest positive number as a binary for the given type","ref":"Nx.Type.html#smallest_positive_normal_binary/1"},{"type":"function","title":"Nx.Type.to_aggregate/1","doc":"Converts the given type to an aggregation precision.","ref":"Nx.Type.html#to_aggregate/1"},{"type":"function","title":"Examples - Nx.Type.to_aggregate/1","doc":"iex> Nx.Type.to_aggregate({:s, 8})\n    {:s, 64}\n    iex> Nx.Type.to_aggregate({:u, 32})\n    {:u, 64}\n    iex> Nx.Type.to_aggregate({:bf, 16})\n    {:bf, 16}\n    iex> Nx.Type.to_aggregate({:f, 32})\n    {:f, 32}\n    iex> Nx.Type.to_aggregate({:c, 64})\n    {:c, 64}","ref":"Nx.Type.html#to_aggregate/1-examples"},{"type":"function","title":"Nx.Type.to_complex/1","doc":"Converts the given type to a complex representation with\nthe minimum size necessary.","ref":"Nx.Type.html#to_complex/1"},{"type":"function","title":"Examples - Nx.Type.to_complex/1","doc":"iex> Nx.Type.to_complex({:s, 64})\n    {:c, 64}\n    iex> Nx.Type.to_complex({:bf, 16})\n    {:c, 64}\n    iex> Nx.Type.to_complex({:f, 32})\n    {:c, 64}\n    iex> Nx.Type.to_complex({:c, 64})\n    {:c, 64}\n    iex> Nx.Type.to_complex({:f, 64})\n    {:c, 128}\n    iex> Nx.Type.to_complex({:c, 128})\n    {:c, 128}","ref":"Nx.Type.html#to_complex/1-examples"},{"type":"function","title":"Nx.Type.to_floating/1","doc":"Converts the given type to a floating point representation\nwith the minimum size necessary.\n\nNote both float and complex are floating point representations.","ref":"Nx.Type.html#to_floating/1"},{"type":"function","title":"Examples - Nx.Type.to_floating/1","doc":"iex> Nx.Type.to_floating({:s, 8})\n    {:f, 32}\n    iex> Nx.Type.to_floating({:s, 32})\n    {:f, 32}\n    iex> Nx.Type.to_floating({:bf, 16})\n    {:bf, 16}\n    iex> Nx.Type.to_floating({:f, 32})\n    {:f, 32}\n    iex> Nx.Type.to_floating({:c, 64})\n    {:c, 64}","ref":"Nx.Type.html#to_floating/1-examples"},{"type":"function","title":"Nx.Type.to_real/1","doc":"Converts the given type to a real number representation\nwith the minimum size necessary.","ref":"Nx.Type.html#to_real/1"},{"type":"function","title":"Examples - Nx.Type.to_real/1","doc":"iex> Nx.Type.to_real({:s, 8})\n    {:f, 32}\n    iex> Nx.Type.to_real({:s, 64})\n    {:f, 32}\n    iex> Nx.Type.to_real({:bf, 16})\n    {:bf, 16}\n    iex> Nx.Type.to_real({:c, 64})\n    {:f, 32}\n    iex> Nx.Type.to_real({:c, 128})\n    {:f, 64}\n    iex> Nx.Type.to_real({:f, 32})\n    {:f, 32}\n    iex> Nx.Type.to_real({:f, 64})\n    {:f, 64}","ref":"Nx.Type.html#to_real/1-examples"},{"type":"function","title":"Nx.Type.to_string/1","doc":"Returns a string representation of the given type.","ref":"Nx.Type.html#to_string/1"},{"type":"function","title":"Examples - Nx.Type.to_string/1","doc":"iex> Nx.Type.to_string({:s, 8})\n    \"s8\"\n    iex> Nx.Type.to_string({:s, 16})\n    \"s16\"\n    iex> Nx.Type.to_string({:s, 32})\n    \"s32\"\n    iex> Nx.Type.to_string({:s, 64})\n    \"s64\"\n    iex> Nx.Type.to_string({:u, 8})\n    \"u8\"\n    iex> Nx.Type.to_string({:u, 16})\n    \"u16\"\n    iex> Nx.Type.to_string({:u, 32})\n    \"u32\"\n    iex> Nx.Type.to_string({:u, 64})\n    \"u64\"\n    iex> Nx.Type.to_string({:f, 16})\n    \"f16\"\n    iex> Nx.Type.to_string({:bf, 16})\n    \"bf16\"\n    iex> Nx.Type.to_string({:f, 32})\n    \"f32\"\n    iex> Nx.Type.to_string({:f, 64})\n    \"f64\"","ref":"Nx.Type.html#to_string/1-examples"},{"type":"type","title":"Nx.Type.short_t/0","doc":"","ref":"Nx.Type.html#t:short_t/0"},{"type":"type","title":"Nx.Type.t/0","doc":"","ref":"Nx.Type.html#t:t/0"},{"type":"behaviour","title":"Nx.Defn.Compiler","doc":"The specification and helper functions for custom `defn` compilers.","ref":"Nx.Defn.Compiler.html"},{"type":"callback","title":"Nx.Defn.Compiler.__compile__/4","doc":"Callback for compilation.\n\nIt receives an opaque `key` used for caching, the function\n`vars`, the function `fun` which builds a defn expression,\nand the compiler options. It must call `fun` with the `vars`\nas arguments.\n\nIt returns a function that receives a list of arguments and\nreturns a list of results.\n\nThe callback uses double underscores so it can be defined\nat root modules without affecting the module's main API.","ref":"Nx.Defn.Compiler.html#c:__compile__/4"},{"type":"callback","title":"Nx.Defn.Compiler.__jit__/5","doc":"Callback for compilation.\n\nIt receives an opaque `key` used for caching, the function\n`vars`, the function `fun` which builds a defn expression,\na list of argument list in `args_list`, and the compiler options.\n\nIt must call `fun` with the `vars` as arguments. Note the `key`\ndoes not include the `vars` in its cache. Therefore, if you want\nto cache the result of `fun.(vars)`, you likely want to include\nthe vars in the cache key. `vars` is a list of containers expressions.\n\nOnce the expression is built and compiled, it must be invoked\nfor each list of arguments in `args_list`. In a nutshell, `vars`\nare used to build the expression from `fun` which is then\ninvoked for each list of arguments in `args_list`. All lists\nin `args_list` are guaranteed to be flat lists of the same length,\ncontaining zero-arity functions that return tensors of the same type,\nshape, and name.\n\nThe callback uses double underscores so it can be defined\nat root modules without affecting the module's main API.","ref":"Nx.Defn.Compiler.html#c:__jit__/5"},{"type":"callback","title":"Nx.Defn.Compiler.__partitions_options__/1","doc":"Receives a keyword list of compiler options and\nreturns a list of compiler options, each to run\non a separate partition/device.","ref":"Nx.Defn.Compiler.html#c:__partitions_options__/1"},{"type":"callback","title":"Nx.Defn.Compiler.__stream__/7","doc":"Callback for streaming (on top of JIT compilation).\n\nIt receives the same arguments as `c:__jit__/5` with the addition\nof the streaming input and accumulator templates. If the input\nand accumulator are containers, they are kept in their container\nshapes. As in `c:__jit__/5`, both `vars` and `args_list` are flat\nlists of tensors (without their container shape).\n\nIt must return a struct that implements the `Nx.Stream` protocol.","ref":"Nx.Defn.Compiler.html#c:__stream__/7"},{"type":"function","title":"Nx.Defn.Compiler.current/0","doc":"Returns the current compiler.\n\nReturns nil if we are not inside `defn`.","ref":"Nx.Defn.Compiler.html#current/0"},{"type":"function","title":"Nx.Defn.Compiler.defn?/0","doc":"Returns if we are inside `defn` at _compilation time_.\n\nThis would be invoked inside a macro that has specific `defn` logic.","ref":"Nx.Defn.Compiler.html#defn?/0"},{"type":"module","title":"Nx.Defn.Composite","doc":"Functions to deal with composite data types.\n\nComposite data-types are traversed according to `Nx.Container`.\nIf a regular tensor is given, it is individually traversed. \nNumerical values, such as integers, floats, and complex numbers\nare not normalized before hand. Use `Nx.to_tensor/1` to do so.\n\nThe functions in this module can be used both inside and outside `defn`.\nNote that, when a value is given to `defn`, it is first converted to\ntensors and containers via `Nx.LazyContainer`. Inside `defn`, there are\nno lazy containers, only containers.","ref":"Nx.Defn.Composite.html"},{"type":"function","title":"Nx.Defn.Composite.compatible?/3","doc":"Traverses two composite types to see if they are compatible.\n\nNon-tensor values are first compared using `Nx.LazyContainer`\nand then, if not available, as `Nx.Container`.\n\nFor non-composite types, the given `fun` will be called to\ncompare numbers/tensors pairwise.","ref":"Nx.Defn.Composite.html#compatible?/3"},{"type":"function","title":"Nx.Defn.Composite.count/1","doc":"Counts the number of non-composite types in the composite type.","ref":"Nx.Defn.Composite.html#count/1"},{"type":"function","title":"Examples - Nx.Defn.Composite.count/1","doc":"iex> Nx.Defn.Composite.count(123)\n    1\n    iex> Nx.Defn.Composite.count({1, {2, 3}})\n    3\n    iex> Nx.Defn.Composite.count({Complex.new(1), {Nx.tensor(2), 3}})\n    3","ref":"Nx.Defn.Composite.html#count/1-examples"},{"type":"function","title":"Nx.Defn.Composite.flatten_list/2","doc":"Flattens recursively the given list of composite types.\n\nElements that are not tensors (i.e. numbers and `Complex` numbers) are kept as is\nunless a custom function is given.","ref":"Nx.Defn.Composite.html#flatten_list/2"},{"type":"function","title":"Examples - Nx.Defn.Composite.flatten_list/2","doc":"iex> Nx.Defn.Composite.flatten_list([1, {2, 3}])\n    [1, 2, 3]\n\n    iex> Nx.Defn.Composite.flatten_list([1, {2, 3}], [Nx.tensor(4)])\n    [1, 2, 3, Nx.tensor(4)]","ref":"Nx.Defn.Composite.html#flatten_list/2-examples"},{"type":"function","title":"Nx.Defn.Composite.reduce/3","doc":"Reduces recursively the given composite types with `acc` and `fun`.\n\nIf composite tensor expressions are given, such as a tuple,\nthe composite type is recursively traversed and returned.\n\nIf a non-composite tensor expression is given, the function\nis invoked for it but not for its arguments.","ref":"Nx.Defn.Composite.html#reduce/3"},{"type":"function","title":"Nx.Defn.Composite.traverse/2","doc":"Traverses recursively the given composite types with `fun`.\n\nIf a composite tensor is given, such as a tuple, the composite\ntype is recursively traversed and returned.\n\nOtherwise the function is invoked with the tensor (be it a\nnumber, complex, or actual tensor).","ref":"Nx.Defn.Composite.html#traverse/2"},{"type":"function","title":"Nx.Defn.Composite.traverse/3","doc":"Traverses recursively the given composite types with `acc` and `fun`.\n\nIf a composite tensor is given, such as a tuple, the composite\ntype is recursively traversed and returned.\n\nOtherwise the function is invoked with the tensor (be it a\nnumber, complex, or actual tensor).","ref":"Nx.Defn.Composite.html#traverse/3"},{"type":"module","title":"Nx.Defn.Evaluator","doc":"The default implementation of a `Nx.Defn.Compiler`\nthat evaluates the expression tree against the\ntensor backend.","ref":"Nx.Defn.Evaluator.html"},{"type":"module","title":"Options - Nx.Defn.Evaluator","doc":"The following options are specific to this compiler:\n\n  * `:garbage_collect` - when true, garbage collects\n    after evaluating each node\n\n  * `:max_concurrency` - the number of partitions to\n    start when running a `Nx.Serving` with this compiler","ref":"Nx.Defn.Evaluator.html#module-options"},{"type":"module","title":"Nx.Defn.Expr","doc":"","ref":"Nx.Defn.Expr.html"},{"type":"function","title":"Nx.Defn.Expr.__struct__/0","doc":"The expression used by `Nx.Defn.Compiler`.\n\n`Nx.Defn.Compiler` changes `Nx` default backend from `Nx.BinaryBackend`\nto `Nx.Defn.Expr`. It is a struct with the following fields:\n\n  * `:id` - a unique identifier\n  * `:op` - the operation name\n  * `:args` - the operation arguments\n  * `:context` - the context of the expression.\n    The default context is `:root`.\n\nConvenience functions for traversing expressions and composite types\ncan be found in `Nx.Defn.Composite` and `Nx.Defn.Tree`.","ref":"Nx.Defn.Expr.html#__struct__/0"},{"type":"function","title":"Syntax nodes - Nx.Defn.Expr.__struct__/0","doc":"Most nodes are created directly via the `Nx` module and\ntherefore map directly to `Nx.Tensor` callbacks. However\nthe following syntax nodes exist:\n\n  * `parameter(integer)`\n\n  * `constant(number)`\n\n  * `tensor(tensor)`\n\n  * `metadata(expr, metadata)`\n\n  * `elem(tuple, pos)` - created automatically from\n    expression that return tuples. Note it may return\n    tuples too, which means we have nested tuples\n\n  * `fun(parameters, t, mfa)` - the `mfa` is used only for\n    introspection purposes\n\n  * `cond(clauses, otherwise)`\n\n  * `while(initial, condition, body)`\n\n  * `attach_token(token(%Nx.Defn.Token{}), expr)`\n\n`defn` compilers must handle said nodes accordingly.","ref":"Nx.Defn.Expr.html#__struct__/0-syntax-nodes"},{"type":"function","title":"Nx.Defn.Expr.cond/2","doc":"Creates a `cond` tensor expression.","ref":"Nx.Defn.Expr.html#cond/2"},{"type":"function","title":"Nx.Defn.Expr.metadata/2","doc":"Creates a tensor expression metadata node wrapping the\ngiven tensor expression.\n\nThe metadata is map. If the `inspect` key is present,\nit will be used to annotate the metadata when inspected.\nOtherwise the metadata node does not appear during\ninspection.","ref":"Nx.Defn.Expr.html#metadata/2"},{"type":"function","title":"Nx.Defn.Expr.parameter/2","doc":"Creates a tensor expression parameter at `pos` based on the given tensor expression.","ref":"Nx.Defn.Expr.html#parameter/2"},{"type":"function","title":"Nx.Defn.Expr.parameter/3","doc":"Creates a tensor expression parameter at `pos` based on the given `tensor` and `context`.","ref":"Nx.Defn.Expr.html#parameter/3"},{"type":"function","title":"Nx.Defn.Expr.parameter/4","doc":"Creates a tensor expression parameter at `pos` with the given `context`, `type`,\n`shape`, and `pos`.","ref":"Nx.Defn.Expr.html#parameter/4"},{"type":"function","title":"Nx.Defn.Expr.tensor/1","doc":"Builds an tensor expression from the given tensor.","ref":"Nx.Defn.Expr.html#tensor/1"},{"type":"function","title":"Nx.Defn.Expr.tuple/2","doc":"Creates a tuple with elements in `list` that points to tuple\nexpression `expr`.\n\n`list` must be a list of tensor expressions of the same size\nas the tuple expression.","ref":"Nx.Defn.Expr.html#tuple/2"},{"type":"function","title":"Nx.Defn.Expr.while/5","doc":"Creates a `while` tensor expression.","ref":"Nx.Defn.Expr.html#while/5"},{"type":"module","title":"Nx.Defn.Token","doc":"A `defn` token used by hooks.","ref":"Nx.Defn.Token.html"},{"type":"module","title":"Documentation for compilers - Nx.Defn.Token","doc":"The token has a `hooks` field as a list of maps of the shape:\n\n    %{\n      expr: Nx.Tensor.t | Nx.Container.t,\n      name: atom(),\n      callback: (Nx.Tensor.t | Nx.Container.t -> term()) | nil\n    }\n\nThe `hooks` field must only be accessed by `defn` compilers.","ref":"Nx.Defn.Token.html#module-documentation-for-compilers"},{"type":"module","title":"Nx.Defn.Tree","doc":"Helper functions to traverse defn expressions,\neither as single nodes or in-depth.","ref":"Nx.Defn.Tree.html"},{"type":"function","title":"Nx.Defn.Tree.apply_args/4","doc":"Applies the given function to the arguments of the node,\nwith the given accumulator as a starting value.\n\nBy default, `type` is `:all`, which means all arguments\nare traversed. If `type` is `:scope`, only expressions\nthat are in the same scope are traversed. Therefore,\nexpressions such as `while`'s condition and body, \n`optional`'s default implementation, functions, and so forth\nare not traversed. Note `cond`s are always traversed because,\nwhile they introduce a new scope, they can also access its\nparents directly, so you must take `cond`s into account\naccordingly.\n\nWarning: be very careful when using this function to traverse\nthe expression recursively. If you plan to do so, you should\nconsider also storing the visited nodes to avoid multiple\ntraversals by using `tensor.data.expr.id` as cache key.","ref":"Nx.Defn.Tree.html#apply_args/4"},{"type":"function","title":"Nx.Defn.Tree.has_hooks?/2","doc":"Check if the given tree has any of the given hooks in it.","ref":"Nx.Defn.Tree.html#has_hooks?/2"},{"type":"function","title":"Nx.Defn.Tree.put_args/2","doc":"Puts new args in the given tensor expression and gives it a new id.","ref":"Nx.Defn.Tree.html#put_args/2"},{"type":"function","title":"Nx.Defn.Tree.rewrite_types/2","doc":"Rewrites the types of the given tensor expressions according to\nthe given options.","ref":"Nx.Defn.Tree.html#rewrite_types/2"},{"type":"function","title":"Options - Nx.Defn.Tree.rewrite_types/2","doc":"* `:max_float_type` - set the max float type\n  * `:max_signed_type` - set the max signed integer type\n  * `:max_unsigned_type` - set the max unsigned integer type","ref":"Nx.Defn.Tree.html#rewrite_types/2-options"},{"type":"function","title":"Nx.Defn.Tree.scope_ids/2","doc":"Gets all IDs of all elements in the same scope.\n\n`while`'s condition and body, `fun`'s body and similar are\nconsidered different scopes. When it comes to `cond`, an ID will\nonly be considered if it is used outside of the `cond` or used\nin several distinct conds. Constants are also ignored, as they\nhave global IDs based on the constants themselves.\n\nAn existing maps of `ids` can be given to accumulate on top of it.","ref":"Nx.Defn.Tree.html#scope_ids/2"},{"type":"extras","title":"Introduction to Nx","doc":"# Introduction to Nx\n\n```elixir\nMix.install([\n  {:nx, \"~> 0.5\"}\n])\n```","ref":"intro-to-nx.html"},{"type":"extras","title":"Numerical Elixir - Introduction to Nx","doc":"Elixir's primary numerical datatypes and structures are not optimized\nfor numerical programming. Nx is a library built to bridge that gap.\n\n[Elixir Nx](https://github.com/elixir-nx/nx) is a numerical computing library\nto smoothly integrate to typed, multidimensional data implemented on other\nplatforms (called tensors). This support extends to the compilers and\nlibraries that support those tensors. Nx has three primary capabilities:\n\n* In Nx, tensors hold typed data in multiple, named dimensions.\n* Numerical definitions, known as `defn`, support custom code with\n  tensor-aware operators and functions.\n* [Automatic differentiation](https://arxiv.org/abs/1502.05767), also known as\n  autograd or autodiff, supports common computational scenarios\n  such as machine learning, simulations, curve fitting, and probabilistic models.\n\nHere's more about each of those capabilities. Nx [tensors]() can hold\nunsigned integers (u8, u16, u32, u64),\nsigned integers (s8, s16, s32, s64),\nfloats (f32, f64), brain floats (bf16), and complex (c64, c128).\nTensors support backends implemented outside of Elixir, including Google's\nAccelerated Linear Algebra (XLA) and LibTorch.\n\nNumerical definitions have compiler support to allow just-in-time compilation\nthat support specialized processors to speed up numeric computation including\nTPUs and GPUs.\n\nTo know Nx, we'll get to know tensors first. This rapid overview will touch\non the major libraries. Then, future notebooks will take a deep dive into working\nwith tensors in detail, autograd, and backends. Then, we'll dive into specific\nproblem spaces like Axon, the machine learning library.","ref":"intro-to-nx.html#numerical-elixir"},{"type":"extras","title":"Nx and tensors - Introduction to Nx","doc":"Systems of equations are a central theme in numerical computing.\nThese equations are often expressed and solved with multidimensional\narrays. For example, this is a two dimensional array:\n\n$$\n\\begin{bmatrix}\n  1 & 2 \\\\\\\\\n  3 & 4\n\\end{bmatrix}\n$$\n\nElixir programmers typically express a similar data structure using\na list of lists, like this:\n\n```elixir\n[\n  [1, 2],\n  [3, 4]\n]\n```\n\nThis data structure works fine within many functional programming\nalgorithms, but breaks down with deep nesting and random access.\n\nOn top of that, Elixir numeric types lack optimization for many numerical\napplications. They work fine when programs\nneed hundreds or even thousands of calculations. They tend to break\ndown with traditional STEM applications when a typical problem\nneeds millions of calculations.\n\nIn Nx, we express multi-dimensional data using typed tensors. Simply put,\na tensor is a multi-dimensional array with a predetermined shape and\ntype. To interact with them, Nx relies on tensor-aware operators rather\nthan `Enum.map/2` and `Enum.reduce/3`.\n\nIn this section, we'll look at some of the various tools for\ncreating and interacting with tensors. The IEx helpers will assist our\nexploration of the core tensor concepts.\n\n```elixir\nimport IEx.Helpers\n```\n\nNow, everything is set up, so we're ready to create some tensors.\n\n \n\n#","ref":"intro-to-nx.html#nx-and-tensors"},{"type":"extras","title":"Creating tensors - Introduction to Nx","doc":"Start out by getting a feel for Nx through its documentation.\nDo so through the IEx helpers, like this:\n\n \n\n```elixir\nh Nx\n```\n\nImmediately, you can see that tensors are at the center of the\nAPI. The main API for creating tensors is `Nx.tensor/2`:\n\n \n\n```elixir\nh Nx.tensor\n```\n\nWe use it to create tensors from raw Elixir lists of numbers, like this:\n\n```elixir\ntensor =\n  1..4\n  |> Enum.chunk_every(2)\n  |> Nx.tensor(names: [:y, :x])\n```\n\nThe result shows all of the major fields that make up a tensor:\n\n* The data, presented as the list of lists `[[1, 2], [3, 4]]`.\n* The type of the tensor, a signed integer 64 bits long, with the type `s64`.\n* The shape of the tensor, going left to right, with the outside dimensions listed first.\n* The names of each dimension.\n\nWe can easily convert it to a binary:\n\n```elixir\nbinary = Nx.to_binary(tensor)\n```\n\nA tensor of type s64 uses eight bytes for each integer. The binary\nshows the individual bytes that make up the tensor, so you can see\nthe integers `1..4` interspersed among the zeros that make\nup the tensor. If all of our data only uses positive numbers from\n`0..255`, we could save space with a different type:\n\n```elixir\nNx.tensor([[1, 2], [3, 4]], type: :u8) |> Nx.to_binary()\n```\n\nIf you have already have binary, you can directly convert it to a tensor\nby passing the binary and the type:\n\n```elixir\nNx.from_binary( >, :u8)\n```\n\nThis function comes in handy when working with published datasets\nbecause they must often be processed. Elixir binaries make quick work\nof dealing with numerical data structured for platforms other than\nElixir.\n\nWe can get any cell of the tensor:\n\n```elixir\ntensor[0][1]\n```\n\nNow, try getting the first row of the tensor:\n\n```elixir\n# ...your code here...\n```\n\nWe can also get a whole dimension:\n\n```elixir\ntensor[x: 1]\n```\n\nor a range:\n\n```elixir\ntensor[y: 0..1]\n```\n\nNow,\n\n* create your own `{3, 3}` tensor with named dimensions\n* return a `{2, 2}` tensor containing the first two columns\n  of the first two rows\n\nWe can get information about this most recent term with\nthe IEx helper `i`, like this:\n\n \n\n```elixir\ni tensor\n```\n\nThe tensor is a struct that supports the usual `Inspect` protocol.\nThe struct has keys, but we typically treat the `Nx.Tensor`\nas an _opaque data type_ (meaning we typically access the contents and\nshape of a tensor using the tensor's API instead of the struct).\n\nPrimarily, a tensor is a struct, and the\nfunctions to access it go through a specific backend. We'll get to\nthe backend details in a moment. For now, use the IEx `h` helper\nto get more documentation about tensors. We could also open a Code\ncell, type Nx.tensor, and hover the cursor over the word `tensor`\nto see the help about that function.\n\nWe can get the shape of the tensor with `Nx.shape/1`:\n\n```elixir\nNx.shape(tensor)\n```\n\nWe can also create a new tensor with a new shape using  `Nx.reshape/2`:\n\n```elixir\nNx.reshape(tensor, {1, 9}, names: [:batches, :values])\n```\n\nThis operation reuses all of the tensor data and simply\nchanges the metadata, so it has no notable cost.\n\nThe new tensor has the same type, but a new shape.\n\nNow, reshape the tensor to contain three dimensions with\none batch, one row, and four columns.\n\n```elixir\n# ...your code here...\n```\n\nWe can create a tensor with named dimensions, a type, a shape,\nand our target data. A dimension is called an _axis_, and axes\ncan have names. We can specify the tensor type and dimension names\nwith options, like this:\n\n```elixir\nNx.tensor([[1, 2, 3]], names: [:rows, :cols], type: :u8)\n```\n\nWe created a tensor of the shape `{1, 3}`, with the type `u8`,\nthe values `[1, 2, 3]`, and two axes named `rows` and `cols`.\n\nNow we know how to create tensors, so it's time to do something with them.","ref":"intro-to-nx.html#creating-tensors"},{"type":"extras","title":"Tensor aware functions - Introduction to Nx","doc":"In the last section, we created a `s64[2][2]` tensor. In this section,\nwe'll use Nx functions to work with it. Here's the value of `tensor`:\n\n```elixir\ntensor\n```\n\nWe can use `IEx.Helpers.exports/1` or code completion to find\nsome functions in the `Nx` module that operate on tensors:\n\n \n\n```elixir\nexports Nx\n```\n\nYou might recognize that many of those functions have names that\nsuggest that they would work on primitive values, called scalars.\nIndeed, a tensor can be a scalar:\n\n```elixir\npi = Nx.tensor(3.1415, type: :f32)\n```\n\nTake the cosine:\n\n```elixir\nNx.cos(pi)\n```\n\nThat function took the cosine of `pi`.  We can also call them\non a whole tensor, like this:\n\n```elixir\nNx.cos(tensor)\n```\n\nWe can also call a function that aggregates the contents\nof a tensor. For example, to get a sum of the numbers\nin `tensor`, we can do this:\n\n```elixir\nNx.sum(tensor)\n```\n\nThat's `1 + 2 + 3 + 4`, and Nx went to multiple dimensions to get that sum.\nTo get the sum of values along the `x` axis instead, we'd do this:\n\n```elixir\nNx.sum(tensor, axes: [:x])\n```\n\nNx sums the values across the `x` dimension: `1 + 2` in the first row\nand `3 + 4` in the second row.\n\nNow,\n\n* create a `{2, 2, 2}` tensor\n* with the values `1..8`\n* with dimension names `[:z, :y, :x]`\n* calculate the sums along the `y` axis\n\n```elixir\n# ...your code here...\n```\n\nSometimes, we need to combine two tensors together with an\noperator. Let's say we wanted to subtract one tensor from\nanother. Mathematically, the expression looks like this:\n\n$$\n\\begin{bmatrix}\n  5 & 6 \\\\\\\\\n  7 & 8\n\\end{bmatrix} -\n\\begin{bmatrix}\n  1 & 2 \\\\\\\\\n  3 & 4\n\\end{bmatrix} =\n\\begin{bmatrix}\n  4 & 4 \\\\\\\\\n  4 & 4\n\\end{bmatrix}\n$$\n\nTo solve this problem, subtract each right-hand integer from the\ncorresponding left-hand integer. Unfortunately, we cannot\nuse Elixir's built-in subtraction operator as it is not tensor-aware.\nLuckily, we can use the `Nx.subtract/2` function to solve the\nproblem:\n\n```elixir\ntensor2 = Nx.tensor([[5, 6], [7, 8]])\nNx.subtract(tensor2, tensor)\n```\n\nWe get a `{2, 2}` shaped tensor full of fours, exactly as we expected.\nWhen calling `Nx.subtract/2`, both operands had the same shape.\nSometimes, you might want to process functions where the dimensions\ndon't match. To solve this problem, Nx takes advantage of\na concept called _broadcasting_.","ref":"intro-to-nx.html#tensor-aware-functions"},{"type":"extras","title":"Broadcasts - Introduction to Nx","doc":"Often, the dimensions of tensors in an operator don't match.\nFor example, you might want to subtract a `1` from every\nelement of a `{2, 2}` tensor, like this:\n\n$$\n\\begin{bmatrix}\n  1 & 2 \\\\\\\\\n  3 & 4\n\\end{bmatrix} - 1 =\n\\begin{bmatrix}\n  0 & 1 \\\\\\\\\n  2 & 3\n\\end{bmatrix}\n$$\n\nMathematically, it's the same as this:\n\n$$\n\\begin{bmatrix}\n  1 & 2 \\\\\\\\\n  3 & 4\n\\end{bmatrix} -\n\\begin{bmatrix}\n  1 & 1 \\\\\\\\\n  1 & 1\n\\end{bmatrix} =\n\\begin{bmatrix}\n  0 & 1 \\\\\\\\\n  2 & 3\n\\end{bmatrix}\n$$\n\nThat means we need a way to convert `1` to a `{2, 2}` tensor.\n`Nx.broadcast/2` solves that problem. This function takes\na tensor or a scalar and a shape.\n\n```elixir\nNx.broadcast(1, {2, 2})\n```\n\nThis broadcast takes the scalar `1` and translates it\nto a compatible shape by copying it.  Sometimes, it's easier\nto provide a tensor as the second argument, and let `broadcast/2`\nextract its shape:\n\n```elixir\nNx.broadcast(1, tensor)\n```\n\nThe code broadcasts `1` to the shape of `tensor`. In many operators\nand functions, the broadcast happens automatically:\n\n```elixir\nNx.subtract(tensor, 1)\n```\n\nThis result is possible because Nx broadcasts _both tensors_\nin `subtract/2` to compatible shapes. That means you can provide\nscalar values as either argument:\n\n```elixir\nNx.subtract(10, tensor)\n```\n\nOr subtract a row or column. Mathematically, it would look like this:\n\n$$\n\\begin{bmatrix}\n  1 & 2 \\\\\\\\\n  3 & 4\n\\end{bmatrix} -\n\\begin{bmatrix}\n  1 & 2\n\\end{bmatrix} =\n\\begin{bmatrix}\n  0 & 0 \\\\\\\\\n  2 & 2\n\\end{bmatrix}\n$$\n\nwhich is the same as this:\n\n$$\n\\begin{bmatrix}\n  1 & 2 \\\\\\\\\n  3 & 4\n\\end{bmatrix} -\n\\begin{bmatrix}\n  1 & 2 \\\\\\\\\n  1 & 2\n\\end{bmatrix} =\n\\begin{bmatrix}\n  0 & 0 \\\\\\\\\n  2 & 2\n\\end{bmatrix}\n$$\n\nThis rewrite happens in Nx too, also through a broadcast. We want to\nbroadcast the tensor `[1, 2]` to match the `{2, 2}` shape, like this:\n\n```elixir\nNx.broadcast(Nx.tensor([1, 2]), {2, 2})\n```\n\nThe `subtract` function in `Nx` takes care of that broadcast\nimplicitly, as before:\n\n```elixir\nNx.subtract(tensor, Nx.tensor([1, 2]))\n```\n\nThe broadcast worked as advertised, copying the `[1, 2]` row\nenough times to fill a `{2, 2}` tensor.  A tensor with a\ndimension of `1` will broadcast to fill the tensor:\n\n```elixir\n[[1], [2]] |> Nx.tensor() |> Nx.broadcast({1, 2, 2})\n```\n\n```elixir\n[[[1, 2, 3]]]\n|> Nx.tensor()\n|> Nx.broadcast({4, 2, 3})\n```\n\nBoth of these examples copy parts of the tensor enough\ntimes to fill out the broadcast shape. You can check out the\nNx broadcasting documentation for more details:\n\n \n\n```elixir\nh Nx.broadcast\n```\n\nMuch of the time, you won't have to broadcast yourself. Many of\nthe functions and operators Nx supports will do so automatically.\n\nWe can use tensor-aware operators via various `Nx` functions and\nmany of them implicitly broadcast tensors.\n\nThroughout this section, we have been invoking `Nx.subtract/2` and\nour code would be more expressive if we could use its equivalent\nmathematical operator. Fortunately, Nx provides a way. Next, we'll\ndive into numerical definitions using `defn`.","ref":"intro-to-nx.html#broadcasts"},{"type":"extras","title":"Numerical definitions (defn) - Introduction to Nx","doc":"The `defn` macro simplifies the expression of mathematical formulas\ncontaining tensors. Numerical definitions have two primary benefits\nover classic Elixir functions.\n\n* They are _tensor-aware_. Nx replaces operators like `Kernel.-/2`\n  with the `Defn` counterparts &mdash; which in turn use `Nx` functions\n  optimized for tensors &mdash; so the formulas we express can use\n  tensors out of the box.\n\n* `defn` definitions allow for building computation graph of all the\n  individual operations and using a just-in-time (JIT) compiler to emit\n  highly specialized native code for the desired computation unit.\n\nWe don't have to do anything special to get access to\nget tensor awareness beyond importing `Nx.Defn` and writing\nour code within a `defn` block.\n\nTo use Nx in a Mix project or a notebook, we need to include\nthe `:nx` dependency and import the `Nx.Defn` module. The\ndependency is already included, so import it in a Code cell,\nlike this:\n\n```elixir\nimport Nx.Defn\n```\n\nJust as the Elixir language supports `def`, `defmacro`, and `defp`,\nNx supports `defn`. There are a few restrictions. It allows only\nnumerical arguments in the form of primitives or tensors as arguments\nor return values, and supports only a subset of the language.\n\nThe subset of Elixir allowed within `defn` is quite broad, though. We can\nuse macros, pipes, and even conditionals, so we're not giving up\nmuch when you're declaring mathematical functions.\n\nAdditionally, despite these small concessions, `defn` provides huge benefits.\nCode in a `defn` block uses tensor aware operators and types, so the math\nbeneath your functions has a better chance to shine through. Numerical\ndefinitions can also run on accelerated numerical processors like GPUs and\nTPUs. Here's an example numerical definition:\n\n```elixir\ndefmodule TensorMath do\n  import Nx.Defn\n\n  defn subtract(a, b) do\n    a - b\n  end\nend\n```\n\nThis module has a numerical definition that will be compiled.\nIf we wanted to specify a compiler for this module, we could add\na module attribute before the `defn` clause. One of such compilers\nis [the EXLA compiler](https://github.com/elixir-nx/nx/tree/main/exla).\nYou'd add the `mix` dependency for EXLA and do this:\n\n \n\n```elixir\n@defn_compiler EXLA\ndefn subtract(a, b) do\n  a - b\nend\n```\n\nNow, it's your turn. Add a `defn` to `TensorMath`\nthat accepts two tensors representing the lengths of sides of a\nright triangle and uses the pythagorean theorem to return the\n[length of the hypotenuse](https://www.mathsisfun.com/pythagoras.html).\nAdd your function directly to the previous Code cell.\n\nThe last major feature we'll cover is called auto-differentiation, or autograd.","ref":"intro-to-nx.html#numerical-definitions-defn"},{"type":"extras","title":"Automatic differentiation (autograd) - Introduction to Nx","doc":"An important mathematical property for a function is the\nrate of change, or the gradient. These gradients are critical\nfor solving systems of equations and building probabilistic\nmodels. In advanced math, derivatives, or differential equations,\nare used to take gradients. Nx can compute these derivatives\nautomatically through a feature called automatic differentiation,\nor autograd.\n\nHere's how it works.\n\n \n\n```elixir\nh Nx.Defn.grad\n```\n\nWe'll build a module with a few functions,\nand then create another function to create the gradients of those\nfunctions. The function `grad/1` takes a function, and returns\na function returning the gradient. We have two functions: `poly/1`\nis a simple numerical definition, and `poly_slope_at/1` returns\nits gradient:\n\n$$\n  poly: f(x) = 3x^2 + 2x + 1 \\\\\\\\\n$$\n\n$$\n  polySlopeAt: g(x) = 6x + 2\n$$\n\nHere's the Elixir equivalent of those functions:\n\n```elixir\ndefmodule Funs do\n  defn poly(x) do\n    3 * Nx.pow(x, 2) + 2 * x + 1\n  end\n\n  defn poly_slope_at(x) do\n    grad(&poly/1).(x)\n  end\nend\n```\n\nNotice the second `defn`. It uses `grad/1` to take its\nderivative using autograd. It uses the intermediate `defn` AST\nand mathematical composition to compute the derivative. You can\nsee it at work here:\n\n```elixir\nFuns.poly_slope_at(2)\n```\n\nNice. If you plug the number 2 into the function $6x + 2$\nyou get 14! Said another way, if you look at the graph at\nexactly 2, the rate of increase is 14 units of `poly(x)`\nfor every unit of `x`, precisely at `x`.\n\nNx also has helpers to get gradients corresponding to a number of inputs.\nThese come into play when solving systems of equations.\n\nNow, you try. Find a function computing the gradient of a `sin` wave.\n\n```elixir\n# your code here\n```","ref":"intro-to-nx.html#automatic-differentiation-autograd"},{"type":"extras","title":"Introduction to Vectorization","doc":"# Introduction to Vectorization\n\n```elixir\nMix.install([{:nx, github: \"elixir-nx/nx\", sparse: \"nx\"}])\n```","ref":"vectorization.html"},{"type":"extras","title":"What is vectorization? - Introduction to Vectorization","doc":"Vectorization in Nx is the concept of imposing \"for-each\" semantics into leading tensor axes. This enables writing tensor operations in a simpler way, avoiding `while` loops, which also leading to more efficient code, as we'll see in this guide.\n\n \n\nThere are a few functions related to vectorization. First we'll look into `Nx.vectorize/2`, `Nx.devectorize/2`, and `Nx.revectorize/3`.\n\nFor these examples, we'll utilize the `normalize` function defined below, which only works with 1D tensors. The function subtracts the minimum value and divides the tensor by the resulting maximum.\n\n```elixir\ndefmodule Example do\n  import Nx.Defn\n\n  defn normalize(t) do\n    case Nx.shape(t) do\n      {_} -> :ok\n      _ -> raise \"invalid shape\"\n    end\n\n    min = Nx.reduce_min(t)\n    zero_min = Nx.subtract(t, min)\n    Nx.divide(zero_min, Nx.reduce_max(zero_min))\n  end\nend\n```\n\nWe can invoke it as:\n\n```elixir\nExample.normalize(Nx.tensor([1, 2, 3]))\n```\n\nHowever, if we attempt to call it for any tensor with more than one dimension, it won't work. Let's first define a 2D tensor:\n\n```elixir\nt =\n  Nx.tensor([\n    [1, 2, 3],\n    [10, 20, 30],\n    [4, 5, 6]\n  ])\n```\n\nNow if we attempt to invoke it:\n\n```elixir\nExample.normalize(t)\n```\n\nTo address this, we can vectorize the tensor:\n\n```elixir\nvec = Nx.vectorize(t, :rows)\n```\n\nAs we can see above, the newly vectorized `vec` is the same tensor as `t`, but the first axis is now a `vectorized` axis called `rows`, with size `3`. This means that for all intents and purposes, we can think of this tensor as a 1D tensor, on which our `normalize` function will now work as if we passed those 3 rows separately (thus the \"for-each\" semantics mentioned above). Let's give it a try:\n\n```elixir\nnormalized_vec = Example.normalize(vec)\n```\n\nWhile the tensor is vectorized, we can't treat it as a matrix (2D tensor):\n\n```elixir\nright =\n  Nx.tensor([\n    [1, 2, 3],\n    [2, 3, 4],\n    [3, 4, 5]\n  ])\n\n# The results might be unexpected, but they behave the same as Nx.add(Nx.tensor([1, 2, 3]), right)\n# and so on, resulting in a vectorized tensor with 3 inner matrices.\nNx.add(normalized_vec, right)\n```\n\nYou can devectorize the tensor to get its original shape:\n\n```elixir\n# If we want to keep the vectorized axes' names\nNx.devectorize(normalized_vec)\n```\n\n```elixir\n# If we want to drop the vectorized axes' names\ndevec = Nx.devectorize(normalized_vec, keep_names: false)\n```\n\nOnce devectorized, we can effectively add the two matrices together:\n\n```elixir\nNx.add(devec, right)\n```","ref":"vectorization.html#what-is-vectorization"},{"type":"extras","title":"Revectorization - Introduction to Vectorization","doc":"Now that we have the basics down, let's discuss `Nx.revectorize` with multi dimensional tensors. This is especially useful in cases where we have multiple vectorization axes which we want to collapse and then re-expand.\n\n```elixir\n# This version of vectorize also asserts on the size of the dimensions being vectorized\nt = Nx.iota({2, 1, 3, 2, 2}) |> Nx.vectorize(x: 2, y: 1, z: 3)\n```\n\nLet's imagine we want to operate on the vectorized iota `t` above, adding a specific constant for each vectorized entry. We can do this through a bit of tensor introspection in conjunction with `revectorize/3`\n\n```elixir\ncollapsed_t = Nx.revectorize(t, [vectors: :auto], target_shape: t.shape)\n\n[vectors: vectorized_size] = collapsed_t.vectorized_axes\n\nconstants =\n  Nx.iota({vectorized_size})\n  |> Nx.multiply(100)\n  |> Nx.vectorize(vectors: vectorized_size)\n\n{constants, collapsed_t}\n```\n\nFrom the output above, we can see that we have a vectorized tensor of scalars as well a vectorized tensor with the same vectorized axis, containing matrices.\n\nNow we can add them together as we discussed above, and then re-expand the vectorized axes to the original shape.\n\n```elixir\ncollapsed_t\n|> Nx.add(constants)\n|> Nx.revectorize(t.vectorized_axes)\n```\n\nBefore we move on to the last two vectorization functions, let's see how we can replace a `while` loop with vectorization. The following module defines the same function twice, once for each method.\n\n```elixir\ndefmodule WhileExample do\n  import Nx.Defn\n\n  defn while_sum_and_multiply(x, y) do\n    out =\n      case Nx.shape(x) do\n        {n, _} ->\n          Nx.broadcast(0, {n})\n\n        _ ->\n          raise \"expected x to have rank 2\"\n      end\n\n    n = Nx.axis_size(out, 0)\n\n    case Nx.shape(y) do\n      {^n} -> nil\n      _ -> raise \"expected y to have rank 1 and the same number of elements as x\"\n    end\n\n    {out, _} =\n      while {out, {x, y}}, i  \n          {n, m}\n\n        _ ->\n          raise \"expected x to have rank 2\"\n      end\n\n    case Nx.shape(y) do\n      {^n} -> nil\n      _ -> raise \"expected y to have rank 1 and the same number of elements as x\"\n    end\n\n    # this enables us to accept vectorized tensors for x and y\n    vectorized_axes = x.vectorized_axes\n\n    x = Nx.revectorize(x, [collapsed: :auto, vectors: n], target_shape: {m})\n    y = Nx.revectorize(y, [collapsed: :auto, vectors: n], target_shape: {})\n\n    out = Nx.sum(x) + y\n\n    Nx.revectorize(out, vectorized_axes, target_shape: {n})\n  end\nend\n```\n\nLet's give those definitions a try:\n\n```elixir\nx =\n  Nx.tensor([\n    [1, 2, 3],\n    [1, 0, 0],\n    [0, 1, 3]\n  ])\n\ny = Nx.tensor([4, 5, 6])\n\nx_vec = Nx.tile(x, [2, 1]) |> Nx.reshape({2, 3, 3}) |> Nx.vectorize(:rows)\n# use a different set of vectorized axes for y\ny_vec = Nx.concatenate([y, Nx.multiply(y, 2)]) |> Nx.reshape({2, 3}) |> Nx.vectorize(:cols)\n\n{\n  WhileExample.while_sum_and_multiply(x, y),\n  WhileExample.vectorized_sum_and_multiply(x, y),\n  WhileExample.vectorized_sum_and_multiply(x_vec, y),\n  WhileExample.vectorized_sum_and_multiply(x_vec, y_vec)\n}\n```\n\nThe advantage of vectorization is that the underlying compilers and hardware may do a much better job of optimizing tensor operations than `while` loops, which often run linearly.\n\nThe eagle-eyed reader will have noticed that we inadvertently threw away the vectorized axes that we received from `y`. This is because our usage of `revectorize` disregards the possibility that the axes could be different in each input tensor. Luckily, we can tackle this problem too.","ref":"vectorization.html#revectorization"},{"type":"extras","title":"Reshape and Broadcast Vectors - Introduction to Vectorization","doc":"The bug introduced in the previous section can be solved through `Nx.reshape_vectors/1` and `Nx.broadcast_vectors/1`. Both functions receive lists of tensors and will operate on their vectorized axes to ensure that the shapes are compatible in one way or the other.\n\nNx functions will natively do this for us, as we see below:\n\n```elixir\nbase = Nx.tensor([0, 1, 2, 3, 4, 5, 6, 7, 8, 9])\nvec_i = Nx.vectorize(Nx.reshape(base, {10, 1}), i: 10, j: 1)\nvec_j = Nx.vectorize(base, :j)\n\n# easy way to build a multiplication table from 0 to 9\nNx.multiply(vec_i, vec_j)\n```\n\nOk, now that we know that broadcasting will work on vectorized axes to add elements in our resulting tensor, we can look into the aforementioned functions.\n\n```elixir\na = Nx.iota({2, 1}, vectorized_axes: [x: 2, y: 1])\nb = Nx.iota({}, vectorized_axes: [z: 2, x: 1])\n\nNx.reshape_vectors([a, b], align_ranks: true)\n```\n\nIn the example above, we can see that both tensors end up containing vectorized axes `:x`, `:y`, and `:z`. Furthermore, we can also see that the `b` tensor was rearranged so that the `:x` axis comes first. All tensors end up with the same ordering of axes, but the axes aren't resized in any way.\n\nFinally, the `align_ranks: true` option is passed so that the inner shape (the non-vectorized part!) of both tensors ends up in a broadcastable shape with the same rank across all tensors. The example uses only 2 tensors, but the list can have arbitrary size.\n\n`Nx.broadcast_vectors/2` works similarly, except it also broadcasts the dimensions instead of simply reshaping:\n\n```elixir\nx = Nx.iota({2, 1}, vectorized_axes: [x: 2, y: 1])\ny = Nx.iota({}, vectorized_axes: [z: 2, x: 1])\n\nNx.broadcast_vectors([x, y])\n```\n\nThe key difference is that all vectorized axes will end up with the same size in all resulting tensors, effectively behaving the same as `Nx.broadcast` does for non-vectorized shapes. Specifically, we can see that the `x` tensor gets the new `z: 2` axis and `y` gets both `x: 2, y: 1`, where the `:x` axis was already present, but has now been resized.\n\nWith this knowledge we can rewrite our function without the bug, as follows:\n\n```elixir\ndefmodule BroadcastVectorsExample do\n  import Nx.Defn\n\n  defn vectorized_sum_and_multiply(x, y) do\n    # for the sake of equivalence, we'll keep the limitation\n    # of accepting only rank-2 tensors for x\n\n    [x, y] = Nx.broadcast_vectors([x, y])\n\n    {n, m} =\n      case Nx.shape(x) do\n        {n, m} ->\n          {n, m}\n\n        _ ->\n          raise \"expected x to have rank 2\"\n      end\n\n    case Nx.shape(y) do\n      {^n} -> nil\n      _ -> raise \"expected y to have rank 1 and the same number of elements as x\"\n    end\n\n    # this enables us to accept vectorized tensors for x and y\n    vectorized_axes = x.vectorized_axes\n\n    x = Nx.revectorize(x, [collapsed: :auto, vectors: n], target_shape: {m})\n    y = Nx.revectorize(y, [collapsed: :auto, vectors: n], target_shape: {})\n\n    out = Nx.sum(x) + y\n\n    Nx.revectorize(out, vectorized_axes, target_shape: {n})\n  end\nend\n```\n\nLet's give it once again another try:\n\n```elixir\nx =\n  Nx.tensor([\n    [1, 2, 3],\n    [1, 0, 0],\n    [0, 1, 3]\n  ])\n\ny = Nx.tensor([4, 5, 6])\n\nx_vec = Nx.tile(x, [2, 1]) |> Nx.reshape({2, 3, 3}) |> Nx.vectorize(:rows)\n# use a different set of vectorized axes for y\ny_vec = Nx.concatenate([y, Nx.multiply(y, 2)]) |> Nx.reshape({2, 3}) |> Nx.vectorize(:cols)\n\n{\n  BroadcastVectorsExample.vectorized_sum_and_multiply(x_vec, y),\n  BroadcastVectorsExample.vectorized_sum_and_multiply(x_vec, y_vec)\n}\n```","ref":"vectorization.html#reshape-and-broadcast-vectors"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.6.3 (2023-11-09) - Changelog","doc":"#","ref":"changelog.html#v0-6-3-2023-11-09"},{"type":"extras","title":"Enhancements - Changelog","doc":"* [Nx] Allow non-scalars as updates on `indexed_add` and `indexed_put`\n  * [Nx] Allow non-scalars as return of `gather`\n  * [Nx] Support the `:axes` option in `gather`, `indexed_add`, and `indexed_put`\n  * [Nx] Add `Nx.covariance`\n  * [Nx] Support `:type` in argsort\n  * [Nx] Support `:stable` option in argsort for future compatibility\n  * [Nx.Serving] Add `:weight` option for static load balancing\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* [Nx] Cast input types on slicing\n  * [Nx.Defn] Support vectorized tensors in grad\n  * [Nx.Defn] Fix bugs when diffing tensor expressions\n  * [Nx.Serving] Handle serving getting stuck on timer messages","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.6.2 (2023-09-21) - Changelog","doc":"#","ref":"changelog.html#v0-6-2-2023-09-21"},{"type":"extras","title":"Enhancements - Changelog","doc":"* [Nx.Serving] Add `Nx.Serving.batch_size/2` and perform batch splitting on run\n  * [Nx.Serving] Support input streaming","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.6.1 (2023-09-12) - Changelog","doc":"#","ref":"changelog.html#v0-6-1-2023-09-12"},{"type":"extras","title":"Enhancements - Changelog","doc":"* [Nx] Add multivariate normal distribution\n  * [Nx.Serving] Automatically split exceeding batch sizes\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* [Nx] Fix `Nx.pad/2` with different backends\n  * [Nx] Fix `Nx.clip/3` with non-finite values\n  * [Nx.Serving] Emit batches as they arrive in `Nx.Serving.streaming/2`\n  * [Nx.Serving] Ensure batch key is preserved when a batch is split","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.6.0 (2023-08-15) - Changelog","doc":"#","ref":"changelog.html#v0-6-0-2023-08-15"},{"type":"extras","title":"Enhancements - Changelog","doc":"* [Nx] Add constant creation helpers such as `u8`, `f32`, etc\n  * [Nx] Implement Bluestein's algorithm for `fft` and `ifft` in the binary backend\n  * [Nx] Support range with steps when accessing tensors\n  * [Nx] Support vectorization via `Nx.vectorize/2`, `Nx.devectorize/2`, `Nx.revectorize/2`, `Nx.reshape_vectors/2`, and `Nx.broadcast_vectors/2`\n  * [Nx] Add `Nx.logsumexp/2`\n  * [Nx] Add `Nx.split/3`\n  * [Nx] Add `Nx.tri/2`, `Nx.triu/2`, `Nx.tril/2`\n  * [Nx] Introduce a new serialization format that is more suitable to memory mapping\n  * [Nx.Defn] Consider Inspect.Opts limit when pretty printing Nx.Defn expressions\n  * [Nx.Serving] Support multiple batch keys in Nx.Serving\n  * [Nx.Serving] Support streaming in Nx.Serving\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* [Nx] Fix `from_numpy` with 1-byte width arrays\n  * [Nx] Fix cases where pretty printing large Nx.Defn expressions would take a long time\n  * [Nx] Fix `reduce_min`/`reduce_max` for non-finite values\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Deprecations - Changelog","doc":"* [Nx.Serving] The post-processing function must now be a two-arity function that receives the `{output, metadata}` as a pair or the stream\n\n#","ref":"changelog.html#deprecations"},{"type":"extras","title":"Breaking changes - Changelog","doc":"* [Nx.Serving] The `nx.serving.postprocessing` telemetry event no longer receives the serving output or serving metadata as event metadata","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"v0.5.3 (2023-04-14) - Changelog","doc":"#","ref":"changelog.html#v0-5-3-2023-04-14"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* [Nx.Defn] Fix compilation error when Elixir compiler has column tracking enabled\n  * [Nx.LinAlg] Fix cases where determinant could return NaN\n  * [Nx.LinAlg] Fix SVD when working with f16 and bf16","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.5.2 (2023-03-21) - Changelog","doc":"#","ref":"changelog.html#v0-5-2-2023-03-21"},{"type":"extras","title":"Enhancements - Changelog","doc":"* [Nx.Random] Add `stop_grad` to `Nx.Random` creation functions\n  * [Nx.Serving] Reduce references sent through serving\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* [Nx] Fix `Nx.mode` with `:axis` option","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.5.1 (2023-02-18) - Changelog","doc":"Require Elixir v1.14.\n\n#","ref":"changelog.html#v0-5-1-2023-02-18"},{"type":"extras","title":"Enhancements - Changelog","doc":"* [Nx] Support any container or lazy container in `stack`/`concatenate`\n  * [Nx] Add `Nx.top_k/2`\n  * [Nx] Add `Nx.to_list/1`\n  * [Nx] Improve shape validation in `Nx.concatenate/2`\n  * [Nx.Constants] Add `pi`, `e`, and `euler_gamma`\n  * [Nx.Random] Raise if a non-unary rank tensor is given as probabilities to `Nx.Random.choice/4`\n  * [Nx.Random] Make `samples` optional in `Nx.Random.choice/3`","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.5.0 (2023-02-10) - Changelog","doc":"#","ref":"changelog.html#v0-5-0-2023-02-10"},{"type":"extras","title":"Enhancements - Changelog","doc":"* [Nx] Support serialization of containers\n  * [Nx] Rename `Nx.power` to `Nx.pow`\n  * [Nx] Add `Nx.reflect` and `Nx.linspace`\n  * [Nx.Defn] Raise at compile time for invalid defn if/cond usage\n  * [Nx.LinAlg] Support `full_matrices?` in SVD\n  * [Nx.LinAlg] Add `Nx.LinAlg.matrix_rank`\n  * [Nx.Random] Add `Nx.Random.choice` and `Nx.Random.shuffle`\n  * [Nx.Serving] Add distributed² serving by distributing over devices (GPUs/CPUs) as well as nodes\n  * [Nx.Serving] Add telemetry to `Nx.Serving` callbacks\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Backwards incompatible changes - Changelog","doc":"* [Nx] `from_numpy` and `from_numpy_archive` have been replaced by `load_numpy!` and `load_numpy_archive!`\n  * [Nx.Defn.Evaluator] Do not force GC on evaluator","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"v0.4.2 (2023-01-13) - Changelog","doc":"#","ref":"changelog.html#v0-4-2-2023-01-13"},{"type":"extras","title":"Enhancements - Changelog","doc":"* [Nx] Allow tensors to be given on `Nx.tensor/2`\n  * [Nx] Add `Nx.with_default_backend/2`\n  * [Nx] Add `:axes` option to `Nx.flatten/2`\n  * [Nx] Add `:axes` option to `Nx.weighted_mean/2`\n  * [Nx.Defn] Warn if `Nx.tensor/2` first-argument is not constant inside defn\n  * [Nx.LinAlg] Add `Nx.LinAlg.pinv/1`\n  * [Nx.LinAlg] Optimize and handle more cases in `Nx.LinAlg.svd/1`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* [Nx] Respect fortran order in loading from numpy\n  * [Nx.Defn] Render containers in compile error type+shape mismatch\n  * [Nx.Defn] Restore pdict state after compilation","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.4.1 (2022-12-07) - Changelog","doc":"#","ref":"changelog.html#v0-4-1-2022-12-07"},{"type":"extras","title":"Enhancements - Changelog","doc":"* [Nx] Add `Nx.Batch` and `Nx.Serving`\n  * [Nx] Implement `Nx.Container` for numbers, complex, and tensors for completeness\n  * [Nx] Support batches in `Nx.eye/2`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* [Nx] Keep input tensor names on associative scan\n  * [Nx.BinaryBackend] Differentiate between complex and real output in `as_type`\n  * [Nx.BinaryBackend] Fix loss of precision in `Nx.complex/2`\n  * [Nx.BinaryBackend] Preserve NaNs in `window` and `reduce` operations\n  * [Nx.Random] Do not return infinity on `normal/2` for f16","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.4.0 (2022-10-25) - Changelog","doc":"#","ref":"changelog.html#v0-4-0-2022-10-25"},{"type":"extras","title":"Enhancements - Changelog","doc":"* [Nx] Add `Nx.rename/2`, `Nx.median/2`, `Nx.weighted_mean/3`, and `Nx.mode/2`\n  * [Nx] Implement cumulative operations using associative scan for improved performance\n  * [Nx.Constants] Add `min` and `max`\n  * [Nx.Defn] Allow lists and functions anywhere as arguments in `defn`, `jit` and `compile`\n  * [Nx.Defn] Add `Nx.LazyContainer` that allows a data-structure to lazily define tensors\n  * [Nx.Defn] Allow tensors and ranges as generators inside `while`\n  * [Nx.Defn] Add `debug_expr/2` and `debug_expr_apply/3`\n  * [Nx.Defn.Evaluator] Calculate cache lifetime to reduce memory usage on large numerical programs\n  * [Nx.LinAlg] Handle Hermitian matrices in `eigh`\n  * [Nx.LinAlg] Support batched operations in `adjoint`, `cholesky`, `determinant`, `eigh`, `invert`, `lu`, `matrix_power`, `solve`, `svd`, and `triangular_solve`\n  * [Nx.Random] Support pseudo random number generators algorithms\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* [Nx] Perform `window_reduce`/`reduce` operations from infinity and negative infinity\n  * [Nx.Defn] Ensure `defnp` emits warnings when unused\n  * [Nx.Defn] Warn on unused variables in `while`\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Deprecations - Changelog","doc":"* [Nx] Deprecate tensor as shape in `Nx.eye/2` and `Nx.iota/2`\n  * [Nx] Deprecate `Nx.random_uniform/2` and `Nx.random_normal/2`","ref":"changelog.html#deprecations"},{"type":"extras","title":"v0.3.0 (2022-08-13) - Changelog","doc":"#","ref":"changelog.html#v0-3-0-2022-08-13"},{"type":"extras","title":"Enhancements - Changelog","doc":"* [Nx] Improve support for non-finite values in `Nx.broadcast/2`, `Nx.all_close/2`, and more\n  * [Nx] Add `Nx.is_nan/1` and `Nx.is_infinite/1`\n  * [Nx] Support booleans in `Nx.tensor/2`\n  * [Nx] Add `Nx.fft/2` and `Nx.ifft/2`\n  * [Nx] Rename `Nx.logistic/1` to `Nx.sigmoid/1`\n  * [Nx] Add `Nx.put_diagonal/3` and `Nx.indexed_put/3`\n  * [Nx] Add `:reverse` to cummulative functions\n  * [Nx] Add `Nx.to_batched/3` which returns a stream\n  * [Nx] Support batched tensors in `Nx.LinAlg.qr/1`\n  * [Nx.Defn] Add `Nx.Defn.compile/3` for precompiling expressions\n  * [Nx.Defn] Add `deftransform/2` and `deftransformp/2` for easier to define transforms\n  * [Nx.Defn] Add `div/2`\n  * [Nx.Defn] Support `case/2`, `raise/1`, and `raise/2`\n  * [Nx.Defn] Support booleans in `if`, `cond`, and boolean operators\n  * [Nx.Defn] Perform branch elimitation in `if` and `cond` and execute branches lazily\n  * [Nx.Defn.Evaluator] Garbage collect after evaluation (it can be disabled by setting the `:garbage_collect` compiler option to false)\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Deprecations - Changelog","doc":"* [Nx] `Nx.to_batched_list/3` is deprecated in favor of `Nx.to_batched/3`\n  * [Nx.Defn] `transform/2` is deprecated in favor of `deftransform/2` and `deftransformp/2`\n  * [Nx.Defn] `assert_shape/2` and `assert_shape_pattern/2` are deprecated in favor of `case/2` + `raise/2`\n  * [Nx.Defn] `inspect_expr/1` and `inspect_value/1` are deprecated in favor of `print_expr/1` and `print_value/1` respectively","ref":"changelog.html#deprecations"},{"type":"extras","title":"v0.2.1 (2022-06-04) - Changelog","doc":"#","ref":"changelog.html#v0-2-1-2022-06-04"},{"type":"extras","title":"Enhancements - Changelog","doc":"* [Nx] Improve support for non-finite values in `Nx.tensor/1`\n  * [Nx] Use iovec on serialization to avoid copying binaries\n  * [Nx.BinaryBackend] Improve for complex numbers in `Nx.tensor/1`\n  * [Nx.Defn] Improve for complex numbers inside `defn`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* [Nx] Properly normalize type in `Nx.from_binary/3`\n  * [Nx.Defn] Raise on `Nx.Defn.Expr` as JIT argument\n  * [Nx.Defn.Evaluator] Handle concatenate arguments on evaluator","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.2.0 (2022-04-28) - Changelog","doc":"This version requires Elixir v1.13+.\n\n#","ref":"changelog.html#v0-2-0-2022-04-28"},{"type":"extras","title":"Enhancements - Changelog","doc":"* [Nx] Support atom notation as the type option throughout the API (for example, `:u8`, `:f64`, etc)\n  * [Nx] Add support for complex numbers (c64, c128)\n  * [Nx] Add `Nx.cumulative_sum/2`, `Nx.cumulative_product/2`, `Nx.cumulative_min/2`, `Nx.cumulative_max/2`\n  * [Nx] Add `Nx.conjugate/1`, `Nx.phase/1`, `Nx.real/1`, and `Nx.imag/1`\n  * [Nx] Initial support for NaN and Infinity\n  * [Nx] Add `:axis` option to `Nx.shuffle/2`\n  * [Nx] Add `Nx.axis_index/2`\n  * [Nx] Add `Nx.variance/2` to `Nx.standard_deviation/2`\n  * [Nx] Rename `Nx.slice_axis/3` to `Nx.slice_along_axis/4`\n  * [Nx.Backend] Add support for optional backends\n  * [Nx.Constants] Provide a convenient module to host constants\n  * [Nx.Defn] Improve error messages throughout the compiler","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.1.0 (2022-01-06) - Changelog","doc":"First release.","ref":"changelog.html#v0-1-0-2022-01-06"}],"content_type":"text/markdown"}